/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright 1998-2000 by Sygate Technologies, Inc.                     |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYGATE Techlogies, INC.            |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

#ifdef WINNT

	#include "stdio.h"
	#include "stdarg.h"
	#include "ntddk.h"
	#include "ntiologc.h"
	#include "ndis.h"

#else // !WINNT

    #define  USE_NDIS
    #include <vtoolsc.h>
    #include <ndis.h>

#endif

#include "debug.h"
#include "..\common\dbgios.h"


static PDEVICE_DBGIOS g_pdio = NULL;

int DbgiosInit(PDEVICE_DBGIOS pd)
{
	// initialize parameters
	if (pd == NULL)
		return -1;
	NdisZeroMemory(pd->header, MAX_DIOS_HEADERSIZE);
	pd->header[DIOS_HEADERSIZE] = MAX_DIOS_HEADERSIZE;
	pd->header[DIOS_VERSION]    = 0x1000;
	pd->header[DIOS_DEBUGLEVEL] = 0x0000;
	pd->flags = 0;
	pd->head = pd->tail = 0;
	pd->data_len = 0;
	// save the handle
	g_pdio = pd;
	return 0;
}

int DbgiosReadStr(char *str, int *len)
{
	unsigned int nbread, i;	

	if (*len <= 0)
		return -1;
	
	// check available data length
	nbread = *len;
	if (g_pdio->data_len < nbread)
		nbread = g_pdio->data_len;

	// read string from data buffer
	for (i=0; i<nbread; i++) {
		*str++ = g_pdio->data[g_pdio->head];
		if (++g_pdio->head >= MAX_DIOS_DATASIZE)
			g_pdio->head = 0;
	}

	// decrease data length
	g_pdio->data_len -= nbread;
	*len = nbread;
	return 0;
}

int DbgiosWriteStr(char *str, int len)
{
	if (len >= MAX_DIOS_DATASIZE)
		return -1;

	// overwite data if overflows
	if ((g_pdio->data_len + len) >= MAX_DIOS_DATASIZE) {
		g_pdio->data_len -= (MAX_DIOS_DATASIZE - len);
		g_pdio->head += (MAX_DIOS_DATASIZE - len);
		if (g_pdio->head >= MAX_DIOS_DATASIZE)
			g_pdio->head -= MAX_DIOS_DATASIZE;
	}

	// write string to data buffer
	while (*str) {
		g_pdio->data[g_pdio->tail] = *str++;
		if (++g_pdio->tail >= MAX_DIOS_DATASIZE)
			g_pdio->tail = 0;
	}

	// increase data length
	g_pdio->data_len += len;
	return 0;
}

int DbgiosSetInfo(int field, int value)
{
	if ((field < 0) || (field >= (MAX_DIOS_HEADERSIZE>>2)))
		return -1;

	g_pdio->header[field] = value;
	return 0;
}

int DbgiosGetInfo(char *str, int *len)
{
	unsigned int nbread;	

	if (*len < MAX_DIOS_HEADERSIZE)
		return -1;
	
	// read header info (512 bytes)
	NdisMoveMemory(str, g_pdio->header, MAX_DIOS_HEADERSIZE);
	
	// read debug info
	nbread = (*len - MAX_DIOS_HEADERSIZE);
	DbgiosReadStr(&str[MAX_DIOS_HEADERSIZE], &nbread);

	// return the number of bytes read
	*len = (nbread + MAX_DIOS_HEADERSIZE);
	return 0;
}

#ifdef WINNT
#else //!WINNT
char *citoa16(int value, char *str)
{
	char digits[]="0123456789ABCDEFGHIJKLMNOPQRSTUVWZYZ";
	char *dest;
	unsigned int uval, tval;

	uval = (unsigned int)value;
	dest = str;
	if (uval < 16)
		*dest++ = digits[uval];
	else {
		while (uval != 0) {
			tval = uval;
			uval >>= 4;
			tval -= (uval<<4);
			*dest++ = digits[tval];
		}
	}
	*dest = '\0';

	return strrev(str);
}
 
char *citoa10(int value, char *str)
{
	char digits[]="0123456789ABCDEFGHIJKLMNOPQRSTUVWZYZ";
	char *dest;
	unsigned int uval, tval;

	uval = ((value>=0) ? value : -value);
	dest = str;
	if (uval < 10)
		*dest++ = digits[uval];
	else {
		while (uval != 0) {
			tval = uval;
			uval /= 10;
			tval -= (uval*10);
			*dest++ = digits[tval];
		}
	}
	if (value < 0)
		*dest++ = '-';
	*dest = '\0';

	return strrev(str);
}
 
int c_vsnprintf(char *str, int count, char *format, va_list varg)
{
	char text[256];
	char *dest, *ptext;
	int state, len;

	state = 0;
	dest = str;
	while (*format) {
		switch (state) {
		case 0: // no formating
			if (*format != '%') 
				*dest++ = *format;
			else {
				state = 1;
			}
			format++;
			break;
		case 1: // format %d, %x, %c, %s
			ptext = text;
			*ptext = '\0'; // null terminated
			if (*format == 'c' || *format == 'C') {
				*dest++ = (char)va_arg(varg, int);
			}
			else if (*format == 'd' || *format == 'i' ||
				*format == 'u' || *format == 'l') {
				citoa10(va_arg(varg, int), ptext);
			}
			else if (*format == 'x' || *format == 'X') {
				citoa16(va_arg(varg, int), ptext);
			}
			else if (*format == 's') {
				ptext = (char *)va_arg(varg, char *);
				if (ptext == NULL)
					ptext = text;
			}
			else if (*format == '%') {
				*dest++ = '%';
			}
			state = 2;
			break;
		case 2: // copy string
			if (*ptext)
				*dest++ = *ptext++;
			else {
				format++;
				state = 0;
			}
			break;
		}
		if (dest >= (str + count))
			break;
	}
	*dest = '\0';
	len = (dest - str);
	return len;
}
#endif

int sysprint(char *format, ...)
{
	va_list varg;
	char str[1024];
	int len = 0;
	
	if (g_pdio->header[DIOS_DEBUGLEVEL] == 0)
		return 0;

	va_start(varg, format); 
#if WINNT
	len = _vsnprintf(str, 1023, format, varg);
#else //!WINNT
	len = c_vsnprintf(str, 1023, format, varg);
#endif
	va_end(varg);

	if (len > 0) {
		DbgiosWriteStr(str, len);
	}
	return 0;
}

