// file PktFIFO.H

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/



#ifndef TEEFERPKTFIFO_H
#define TEEFERPKTFIFO_H

#define     PACKET_CONTROL_FROM_RECEIVE              0x1
#define     PACKET_CONTROL_FROM_SEND                 0x2
#define     PACKET_CONTROL_FROM_SEND_PACKETS         0x4
#define     PACKET_CONTROL_FROM_NDIS_SEND            0x8
#define     PACKET_CONTROL_FROM_NDIS_SEND_PACKETS    0x10
#define     PACKET_CONTROL_DISCARD_PACKET            0x100
#define     PACKET_CONTROL_REPLACE_PACKET            0x200

typedef struct _PACKET_CONTROL_LIST {
    LIST_ENTRY       ListNode;
    unsigned         NdisDispatchFunctionIndex;
    unsigned         TeeferFlags;
    void*            ProtocolHandle;
    void*            MacReceiveContext;
    unsigned         dwProcessId;
}   PACKET_CONTROL_LIST, *PPACKET_CONTROL_LIST;

typedef struct _PACKET_CONTROL
{
    unsigned         NdisDispatchFunctionIndex;
    unsigned         TeeferFlags;
    void*            ProtocolHandle;
    void*            MacReceiveContext;
    void*            pPacketControlList;
    unsigned         dwProcessId;
}  PACKET_CONTROL, *PPACKET_CONTROL;

typedef struct
{
    unsigned long    PktSign;
    unsigned         PktId;
    unsigned         PktLen;
    PACKET_CONTROL   PktControl;
} SFifoPacketHead;


typedef struct
{
    unsigned long    FifoSign;
    unsigned         offset;
    unsigned         BufLen;
    unsigned         Head;
    unsigned         Tail;
    unsigned         Bottom;
    unsigned         PktId;

    unsigned long    PutBytes;        // the bytes put into FIFO, including failure
    unsigned long    GetBytes;        // the bytes get from FIFO.
    unsigned         FreeBytes;
    unsigned         MinFreeBytes;
    unsigned long    PutCount;
    unsigned long    PutSuccessCount;
    unsigned long    GetCount;
    unsigned long    GetSuccessCount;
    unsigned long    GetNoPktCount;        // the count of get a empty fifo.
    unsigned long    LastUserThreadTickCount;
    unsigned long    FifoIsBusy;
    SFifoPacketHead* LastGetPtr;
    SFifoPacketHead* LastPutPtr;
} SPacketFifo;

int PktFifoOpen(SPacketFifo *pPktFifo, unsigned BufferSize);
// return 0 if sucess
int PktFifoClose(SPacketFifo *ppf);
// return 0 if sucess
// return 1 if invalid SPacketFifo
int PktFifoPut(SPacketFifo *ppf, PPACKET_CONTROL pControl, void *pPktData, unsigned PktLen);
// return 0 if sucess
// return 1 if invalid SPacketFifo
// return 2 if no enough memory
int PktFifoPut3(SPacketFifo *ppf, PPACKET_CONTROL pControl, void *p2, unsigned p2Len, void *pPktData, unsigned PktLen);
// return 0 if sucess
// return 1 if invalid SPacketFifo
// return 2 if no enough memory
int PktFifoGet(SPacketFifo *ppf, PPACKET_CONTROL pControl, void *pBuf, unsigned bufsize, unsigned *RetLen);
// the bytes copied is placed in *RetLen.
// return 0 if sucess
// return 1 if invalid SPacketFifo
// return 2 if no packet
// return 3 if the Fifo is damaged
// return 4 if buf is too small
void PktFifoSetEventIfNotBusy(SPacketFifo *ppf, void * pEvent);
// Signal Event if fifo is not busy (is not being processed by other thread)
void PktFifoSetBusyFlag(SPacketFifo *ppf, unsigned uiFlag);
// Set busy flag
void PrintPktFifo(SPacketFifo *ppf);

void PktFifoTest(void);

BOOLEAN PktFifoIsNotExpiring(SPacketFifo *ppf);

void PktFifoUpdateUserThreadCount(SPacketFifo *ppf);

#endif

// end of PKTFIFO.H
