// file checksum.cpp
/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/
/*++
History:

04/01/1999			Created by Yadong Zhang

--*/
#ifdef WINNT

    #include "stdarg.h"
    #include "ntddk.h"

#else // WINNT

    #define  USE_NDIS
    #include <vtoolsc.h>
    #include <ndis.h>

#endif // NOT WINNT

#include "..\common\checksum.h"

typedef word			NWORD;	// Network Word
typedef dword			NDWORD; // Network DWord


// set the struct alignment=1
#ifdef _MSC_VER
#pragma pack(push,1)
#endif
typedef struct
{
	byte	DestAddr[6];
	byte	SourAddr[6];
	NWORD	Type;
} SEtherII_FrameHead;
#ifdef _MSC_VER
#pragma pack(pop)
#endif
// restore struct alignment before


// set the struct alignment=1
#ifdef _MSC_VER
#pragma pack(push,1)
#endif
typedef struct
{
	word	HardwareType;
	word	ProtocolType;
	byte	HardwareAddrLen;
	byte	ProtocolAddrLen;
	word	Operation;
	byte	SenderHardwareAddr[6];
	dword	SenderIpAddr;
	byte	TargetHardwareAddr[6];
	dword	TargetIpAddr;
} SArp_Frame;
#ifdef _MSC_VER
#pragma pack(pop)
#endif
// restore struct alignment before



// set the struct alignment=1
#ifdef _MSC_VER
#pragma pack(push,1)
#endif
typedef struct
{
	byte		Vers_Hlen;
	byte		ServiceType;
	NWORD		TotalLength;
	NWORD		Identification;
	word		Flags_FragmentOffset;
	byte		TimeToLive;
	byte		Protocol;
	NWORD		HeadChecksum;
	dword		SourAddr;
	dword		DestAddr;
	byte		Options;
	byte		OptionsLength;
} SIp4_FrameHead;
#ifdef _MSC_VER
#pragma pack(pop)
#endif
// restore struct alignment before

// set the struct alignment=1
#ifdef _MSC_VER
#pragma pack(push,1)
#endif
typedef struct
{
	NWORD		SourPort;
	NWORD		DestPort;
	NWORD		Length;
	NWORD		Checksum;
} SUdp_FrameHead;
#ifdef _MSC_VER
#pragma pack(pop)
#endif
// restore struct alignment before


// set the struct alignment=1
#ifdef _MSC_VER
#pragma pack(push,1)
#endif
typedef struct
{
	NWORD		SourPort;
	NWORD		DestPort;
	NDWORD		SeqNumber;		// Sequence Number
	NDWORD		AckNumber;		// Acknowledgment Number
	word		Offset_Flags;	// Offset is number of TCP head 32bit
	NWORD		Window;
	NWORD		Checksum;
	NWORD		UrgentPointer;
} STcpSegment_FrameHead;
#ifdef _MSC_VER
#pragma pack(pop)
#endif
// restore struct alignment before


// set the struct alignment=1
#ifdef _MSC_VER
#pragma pack(push,1)
#endif
typedef struct
{
	byte		Type;
	byte		Code;
	word		Checksum;
} SIcmp_FrameHead;
#ifdef _MSC_VER
#pragma pack(pop)
#endif
// restore struct alignment before


#define TCP_FLAGS_URG	0x20
#define TCP_FLAGS_ACK	0x10
#define TCP_FLAGS_PSH   0x8
#define TCP_FLAGS_RST	0x4
#define TCP_FLAGS_SYN	0x2
#define TCP_FLAGS_FIN	0x1


word NWORDto(NWORD nw);
dword NDWORDto(NDWORD ndw);
word Checksum(void* pf,unsigned len);
word GetIpHeaderLen(void *pIp);
word IpChecksum(void *pIp, unsigned len);
word TcpChecksum(void *pIp, unsigned len);
void CalcTcpipChecksum(void *pIp);

word UdpChecksum(void *pIp, unsigned len);

void CalcUdpChecksum(void *pIp);

int AnswerArp(void *pRawData, unsigned len, dword MyIp, byte *pMyEth);

unsigned GetTcpDataLen(SIp4_FrameHead *pip);


#define DISPLAY_DEBUG	1

static _inline word NWORDto(NWORD nw)
{
	unsigned a,b;
	a=(b=nw)&0xff;
	return((b>>8)+(a<<8));
}


static _inline dword NDWORDto(NDWORD ndw)
{
	dword aa;
	aa=(ndw>>24) + ((ndw>>8)&0xff00L) + ((ndw<<8)&0xff0000L) +
		(ndw<<24);
	return aa;
}


static _inline unsigned short Checksum(void* pf,unsigned len)
{
	word* p=(word*)pf;
	dword sum;
	len>>=1;
	for (sum=0;len>0;len--) sum+=*p++;
	sum=(sum>>16)+(sum&0xffff);
	sum+=(sum>>16);
	return (unsigned short)(~sum&0xFFFF);
}

static _inline word GetIpHeaderLen(void *pIp)
{
	SIp4_FrameHead *p;
	p = (SIp4_FrameHead*)pIp;
	return ( (p->Vers_Hlen & 0xf ) * 4 );
}

static _inline word GetIpTotalLen(void *pip)
{
	SIp4_FrameHead *p;
	p = (SIp4_FrameHead*)pip;
	return ( NWORDto(p->TotalLength) );
}


word IpChecksum(void *pIp, unsigned len)
{
	return Checksum(pIp, GetIpHeaderLen(pIp));
}


word GenPktChecksum(void *pIp, unsigned len, word type)
{
	dword sum,sum1;
	unsigned i;
	word* p;
	unsigned ipheaderlen, tcplen;

	SIp4_FrameHead *pip;
	pip = (SIp4_FrameHead*)pIp;
	
	// add address
	sum = ( (word)(pip->SourAddr & 0xFFFF) ) 
		 + ( (word)(pip->SourAddr >> 16) );
	sum += ( (word)(pip->DestAddr & 0xFFFF) )
		 + ( (word)(pip->DestAddr >> 16));
	// add PTCL
	sum += NWORDto(type);
	// add TCP length (TCP Header length plus DataLen)
	ipheaderlen = GetIpHeaderLen(pIp);
	
	// stray packet that has invalid length causes  crash,
	// do not process these packets :Mohandas
	if(ipheaderlen > len) // crazy packet..
		return 0;
	
	tcplen = len - ipheaderlen;	
	sum += ((tcplen <<8) & 0xFFFF) + (tcplen >> 8);

	p = (word*)(((byte*)pip) + ipheaderlen);
	
	// cout<<"comput sum..\n";
	sum1=0;
	for (i = 0; i < tcplen/2 ; ++i ) sum1+=*p++;
	if (tcplen & 1) sum1 += (*((byte*)p));
	// cout<<"sum1="<<sum1<<endl;
	sum += sum1;
	
	sum = (sum>>16)+(sum&0xffff);
	sum += (sum>>16);
	return (word)(~sum&0xFFFF);
}


word TcpChecksum(void *pIp, unsigned len)
{
	return GenPktChecksum(pIp, len , 0x6);
}


word IcmpChecksum(void *pIp, unsigned len)
{
	char* picmp = ((char*)pIp) + GetIpHeaderLen(pIp);

	return Checksum(picmp, GetIpTotalLen(pIp) - GetIpHeaderLen(pIp));
}



/*++
// let's use another checksum manual.
word TcpChecksum(void *pIp, unsigned len)
{
	unsigned	short	*sptr;
	unsigned	long tcksum;
	unsigned int		i;
	SIp4_FrameHead *pip;
	STcpSegment_FrameHead *ptcp;
	pip = (SIp4_FrameHead*)pIp;
	ptcp = (STcpSegment_FrameHead*) ( ((char*)pip) + GetIpHeaderLen(pip) );

	len = len - GetIpHeaderLen(pip);

	tcksum = 0;

	sptr = (unsigned short *) (&pip->SourAddr);
	// 2*IP_ALEN octets = IP_ALEN shorts... 
	// they are in net order.		
	for (i=0; i< 4; ++i)
		tcksum += *sptr++;
	sptr = (unsigned short *)ptcp;

//kprintf("tcpcksum: pep %X order %x len %u (swapped %u)\n", pep, pep->ep_order,
//len, net2hs(len));

    tcksum += (unsigned) NWORDto(((word)(6 + len)));
	if (len % 2) {
		((char *)ptcp)[len] = 0;	// pad 
		len += 1;	// for the following division 
	}
	len >>= 1;	// convert to length in shorts 

	for (i=0; i<len; ++i)
		tcksum += *sptr++;
	tcksum = (tcksum >> 16) + (tcksum & 0xffff);
	tcksum += (tcksum >> 16);

	return (short)(~tcksum & 0xffff);

}
--*/

word RawUdpChecksum(void *pIp, unsigned len)
{
	SUdp_FrameHead *pudp;
	pudp = (SUdp_FrameHead*)( ((char*)pIp) + GetIpHeaderLen(pIp) );
	
	return GenPktChecksum(pIp, len , 0x11);
}


word UdpChecksum(void *pIp, unsigned len)
{
	SUdp_FrameHead *pudp;
	pudp = (SUdp_FrameHead*)( ((char*)pIp) + GetIpHeaderLen(pIp) );
	
	// if checksum == 0, it is non-checksum
	if (pudp->Checksum == 0) return 0;

	return GenPktChecksum(pIp, len , 0x11);
}


void CalcUdpChecksum(void *pIp)
{
	SIp4_FrameHead				*pip;
	unsigned					iplen;
	SUdp_FrameHead				*pudp;

	pip = (SIp4_FrameHead*)pIp;
	iplen = NWORDto(pip->TotalLength);
	
	pudp = (SUdp_FrameHead*) ( ((char*)pip) + GetIpHeaderLen(pip) );

	// don't modify checksum when it is non-checksum
	if (pudp->Checksum == 0) return;
	
	pip->HeadChecksum = 0;
	pip->HeadChecksum = IpChecksum(pip, iplen);
	
	pudp->Checksum = 0;
	pudp->Checksum = RawUdpChecksum(pip, iplen);

	if(CHK_DEBUG)
	{
		if (IpChecksum(pip , iplen) != 0)
		{
			sysprintln("ERROR: IP check failure");
			return ;
		}

		if (UdpChecksum(pip, iplen) != 0)
		{
			sysprintln("ERROR: UDP check failure");
		}
	}
}

void CalcTcpipChecksum(void *pIp)
{
	SIp4_FrameHead				*pip;
	unsigned					iplen;
	STcpSegment_FrameHead		*ptcp;

	pip = (SIp4_FrameHead*)pIp;
	iplen = NWORDto(pip->TotalLength);
	
	ptcp = (STcpSegment_FrameHead*) ( ((char*)pip) + GetIpHeaderLen(pip) );
	
	pip->HeadChecksum = 0;
	pip->HeadChecksum = IpChecksum(pip, iplen);
	
	ptcp->Checksum = 0;
	ptcp->Checksum = TcpChecksum(pip, iplen);

	if(CHK_DEBUG)
	{
		if (IpChecksum(pip , iplen) != 0)
		{
			sysprintln("ERROR: IP check failure");
			return ;
		}

		if (TcpChecksum(pip, iplen) != 0)
		{
			sysprintln("ERROR: TCP check failure");
		}
	}
}


void CalcIcmpChecksum(void *pIp)
{
	SIp4_FrameHead				*pip;
	unsigned					iplen;
	SIcmp_FrameHead				*picmp;

	pip = (SIp4_FrameHead*)pIp;
	iplen = NWORDto(pip->TotalLength);
	
	picmp = (SIcmp_FrameHead*) ( ((char*)pip) + GetIpHeaderLen(pip) );
	
	pip->HeadChecksum = 0;
	pip->HeadChecksum = IpChecksum(pip, iplen);
	
	picmp->Checksum = 0;
	picmp->Checksum = IcmpChecksum(pip, iplen);

	if(CHK_DEBUG)
	{
		if (IpChecksum(pip , iplen) != 0)
		{
			sysprintln("ERROR: IP check failure");
			return ;
		}
		if (IcmpChecksum(pip, iplen) != 0)
		{
			sysprintln("ERROR: UDP check failure");
		}
	}
}


int AnswerArp(void *pRawData, unsigned len, dword MyIp, byte *pMyEth)
// return 0 if I am its target.
{
	SEtherII_FrameHead	*peth;
	SArp_Frame	*parp;
	
	peth = (SEtherII_FrameHead *)pRawData;
	parp = (SArp_Frame*)( peth + 1);
	if (MyIp != parp->TargetIpAddr) return 1;
	if (len < sizeof(SEtherII_FrameHead) + sizeof(SArp_Frame)) return -1;

	parp->Operation = 0x200;	// ARP reply
	
	memcpy(parp->TargetHardwareAddr, parp->SenderHardwareAddr, 6);
	parp->TargetIpAddr = parp->SenderIpAddr;

	memcpy(parp->SenderHardwareAddr, pMyEth, 6);
	parp->SenderIpAddr = MyIp;

	return 0;
}


word GetTcpHeaderLen(STcpSegment_FrameHead *ptcp)
{
	return (ptcp->Offset_Flags & 0xF0)>>2;
}

unsigned char *GetTcpData(STcpSegment_FrameHead *ptcp)
{
	return ((unsigned char*)ptcp) + GetTcpHeaderLen(ptcp);
}

unsigned GetTcpDataLen(SIp4_FrameHead *pip)
{
	STcpSegment_FrameHead *ptcp;
	
	ptcp = (STcpSegment_FrameHead*) ( ((char*)pip) + GetIpHeaderLen(pip) );
	return (unsigned)NWORDto(pip->TotalLength) - GetIpHeaderLen(pip) 
					- GetTcpHeaderLen(ptcp);
}





// ================= Increment Checksum Calcucator ===========
word IncrementChecksum(word old_checksum, word old_word, word  new_word)
{
	//return ~((~old_checksum) + (~old_word) + new_word);
	//return ((old_checksum^0xFFFF) + (old_word^0xFFFF) + new_word)^0xFFFF;
//	return -((-old_checksum) + (-old_word) + new_word);

	dword tcksum;
	tcksum = ((~old_checksum)&0xFFFF) + ((~old_word)&0xFFFF) + new_word;
	// return (old_checksum - new_word + old_word);

	tcksum = (tcksum >> 16) + (tcksum & 0xffff);
	tcksum += (tcksum >> 16);

	return (short)(~tcksum & 0xffff);

}


void RecalcChecksumWithNewDestIp(void *p_old_ip, dword new_dest_ip)
// p_old_ip points to the old ip packet.
{
	SIp4_FrameHead *pip = (SIp4_FrameHead*)p_old_ip;
	STcpSegment_FrameHead *ptcp;

	word kip[2]; *((dword*)kip) = new_dest_ip;

	pip->HeadChecksum = IncrementChecksum(pip->HeadChecksum,
						*(word*)(&pip->DestAddr),kip[0]);
	pip->HeadChecksum = IncrementChecksum(pip->HeadChecksum,
						*((word*)(&pip->DestAddr) + 1),kip[1]);
	ptcp = (STcpSegment_FrameHead*) ( ((char*)pip) + GetIpHeaderLen(pip) );	

	ptcp->Checksum = IncrementChecksum(ptcp->Checksum,
						*(word*)(&pip->DestAddr),kip[0]);
	ptcp->Checksum = IncrementChecksum(ptcp->Checksum,
						*((word*)(&pip->DestAddr) + 1),kip[1]);
	
	pip->DestAddr = new_dest_ip;

}


void RecalcChecksumWithNewSourIp(void *p_old_ip, dword new_sour_ip)
// p_old_ip points to the old ip packet.
{
	SIp4_FrameHead *pip = (SIp4_FrameHead*)p_old_ip;
	STcpSegment_FrameHead *ptcp;

	word kip[2]; *((dword*)kip) = new_sour_ip;

	pip->HeadChecksum = IncrementChecksum(pip->HeadChecksum,
						*(word*)(&pip->SourAddr),kip[0]);
	pip->HeadChecksum = IncrementChecksum(pip->HeadChecksum,
						*((word*)(&pip->SourAddr) + 1),kip[1]);
	ptcp = (STcpSegment_FrameHead*) ( ((char*)pip) + GetIpHeaderLen(pip) );	

	ptcp->Checksum = IncrementChecksum(ptcp->Checksum,
						*(word*)(&pip->SourAddr),kip[0]);
	ptcp->Checksum = IncrementChecksum(ptcp->Checksum,
						*((word*)(&pip->SourAddr) + 1),kip[1]);
	
	pip->SourAddr = new_sour_ip;
}




// end of checksum.cpp
