// file ITeeferClass.h
/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYGATE TECHNOLOGIES, INC. 1998-2003                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYGATE TECHNOLOGIES, INC.          |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
 *
 * $Author: yyeap $
 * $Date: 2003/01/15 23:15:04 $
 * $Revision: 1.1 $
 * $Workfile $
 * $Log: ITeeferClass.h,v $
 * Revision 1.1  2003/01/15 23:15:04  yyeap
 * Check in new teefer loader class
 *
 * $NoKeywords: $
 *
-*/


#ifndef ITEEFER_CLASS_H
#define ITEEFER_CLASS_H

#include "tfman.h"

class ITeeferClass
{
public:
	// constructor & destructor
	ITeeferClass(void) {}; 
	virtual ~ITeeferClass() {};

    // Set Teefer DEBUG mode
    // under DEBUG mode, more debug information will be printed in DebugOutput
    virtual void SetDebugMode(BOOL bDebugOn) = 0;

    // Set Teefer debug output call-back function.
    virtual void SetDebugOutput(FUNC_TEEFER_Dbg_CallBack pfnDebugCallBack) = 0;

    // Open Teefer Driver and bind to the given adapter
    // Return:
    // Handle               If success
    // NULL                 If fail
    virtual TEEFER_HANDLE TeeferOpen(
		BOOL bFakeDriver,
        char *szAdapterName,                          // Name of adapter
        int   nFIFOInSize,                            // Size of input receive FIFO
        int   nFIFOOutSize,                           // Size of output receive FIFO
        FUNC_TEEFER_PacketCallback pfnTeeferCallBack, // Callback for every packet
	    void *pContextPtr                             // Context for call back
        ) = 0;

    // Close the driver
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    virtual int TeeferClose(
        TEEFER_HANDLE h                              // Returns by TeeferOpen
        ) = 0;

    // Enable/Disable the hookings of Teefer Driver
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    virtual int SetMode(
        TEEFER_HANDLE   h,                           // Returns by TeeferOpen
        BOOL            bEnableReceive,              // Enable/Disable receive flag
        BOOL            bEnableSend                  // Enable/Disable send flag
        ) = 0;

    // Enable/Disable the Anti IP Spoofing by Teefer Driver
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    virtual BOOL EnableAntiIPSpoofing(
        TEEFER_HANDLE   h,                           // Returns by TeeferOpen
        BOOL            bEnable                      // Enable/Disable flag
        ) = 0;

    // Dispatch packet to Teefer
    // Return:
    // 0 if sucess
    // 1 if invalid TEEFER_HANDLE
    // 2 if Fifo is full
    // TEEFER_ERR if input len is invalid
    virtual int DispatchPacket(
        TEEFER_HANDLE   h,                           // Returns by TeeferOpen
        void           *pPacket,                     // Pointer to packet buffer
        unsigned        nLen,                        // Length of packet
        BOOL            bReplacePacket,              // TRUE if we want to replace packet
        BOOL            bDiscardPacket,              // TRUE if we want to discard packet
        BOOL            bDoNotBlock,                 // TRUE if we cannot block packet
        void           *pPacketReference             // Pointer of packet control
        ) = 0;

    // Fill the packet stats capture by Teefer driver
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    virtual int GetInfo(
        TEEFER_HANDLE h,                              // Returns by TeeferOpen
        TEEFER_STATUS *pInfo                          // Structure for info
        ) = 0;

    // Fill the all protocol id found by Teefer driver
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    // Parameters:
    // h                    Returns by TeeferOpen
    // pProtocolIDBuffer    Array of protocol id to be filled by driver
    // cbBufferSize         Total size of pProtocolIDBuffer in BYTES
    // pRetBufferSize       Size of pProtocolIDBuffer filled by driver in BYTES
    virtual int EnumProtocolDriver(
        TEEFER_HANDLE  h,                               // Returns by TeeferOpen
        PTEEFER_PROTOCOL_ID pProtocolIDBuffer,          // array of protocol id
        DWORD  cbBufferSize,                            // size of IDBuffer
        PDWORD  pRetBufferSize                          // size of IDBuffer filled
        ) = 0;

    // Get detailed info about a protocol driver
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    // Parameters:
    // h                    Returns by TeeferOpen
    // Id                   protocol id to be queries
    // pProtocolInfo        TEEFER_PROTOCOL_DRIVER_INFO to receive the info
    // pRetBufferSize       Size of pProtocolInfo in BYTES
    virtual int GetProtocolDriverInfo(
        TEEFER_HANDLE  h,                               // Returns by TeeferOpen
        TEEFER_PROTOCOL_ID  Id,                         // Protocol id to be queried
        PTEEFER_PROTOCOL_DRIVER_INFO  pProtocolInfo,    // Info for protocol
        DWORD  dwProtocolInfoSize                       // size of pProtocolInfo
        ) = 0;

    // Get a list of all interfaces opened by a protocol driver
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail 
    // Parameters:
    // h                    Returns by TeeferOpen
    // ProtocolId           Id of the Protocol be to queried
    // pInterfaceIDBuffer   Array of interface id to be filled by driver
    // cbBufferSize         Total size of pInterfaceIDBuffer in BYTES
    // pRetBufferSize       Size of pInterfaceIDBuffer filled by driver in BYTES
    virtual int EnumProtocolDriverInterface(
        TEEFER_HANDLE  h,                               // Returns by TeeferOpen
        TEEFER_PROTOCOL_ID  ProtocolId,                 // Protocol id
        PTEEFER_INTERFACE_ID  pInterfaceIDBuffer,       // Structure for interface info
        DWORD  cbBufferSize,                            // size of pInterfaceIDBuffer
        PDWORD  pRetBufferSize                          // size of pInterfaceIDBuffer
        ) = 0;
                                  
    // Get detailed info about an interface
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    // Parameters:
    // h                    Returns by TeeferOpen
    // Id                   protocol id to be queries
    // pProtocolInfo        TEEFER_PROTOCOL_DRIVER_INFO to receive the info
    // pRetBufferSize       Size of pProtocolInfo in BYTES
    virtual int GetProtocolDriverInterfaceInfo(
        TEEFER_HANDLE  h,                               // Returns by TeeferOpen
        TEEFER_PROTOCOL_ID  ProtocolId,                 // Protocol id
        TEEFER_INTERFACE_ID  InterfaceId,               // Interface id
        PTEEFER_INTERFACE_INFO  pInterfaceInfo,         // Structure for interface info
        DWORD  dwInterfaceInfoBufferSize                // size of pInterfaceInfo
        ) = 0;

    // Return the serial number of the Protocol list on the system.  Caller can
    // use this function to determine if the protocol list has been changed.
    // Return:
    // Serial number
    // Input:
    // None
    virtual DWORD GetProtocolDriverListSerial(
        void
        ) = 0;

    virtual int GetRuleTable(
        TEEFER_HANDLE h,
        PTEEFER_RULE  pRulesBuffer,
        DWORD  dwTeeferRuleBufferSize,  // byte size of rule buffer
        DWORD  dwRuleTableSize,
        PDWORD pdwRetRuleNumber         // The number of rule returned.
        ) = 0;

    virtual int SetRuleTable(
        TEEFER_HANDLE  h, 
        PTEEFER_RULE  pRules,
        DWORD  dwRuleNumber             // The number of rule that is going to write                   
        ) = 0;

    // Install the Teefer Driver service and optionally copy/upgrade the binary
    // file
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    virtual int Install(
        char               *szPath,                       // Path to install from
        BOOL               *pbNeedReboot
        ) = 0;

    // Uninstall the Teefer Driver service and optionally remove the binary file
    // Return:
    // TEEFER_OK            If success
    // TEEFER_ERR           If fail
    virtual int Uninstall(
        BOOL *pbNeedReboot
        ) = 0;

    // Check if the Teefer Driver service exist
    // Return:
    // TRUE					If teefer has been installed
    // FALSE				if teefer has not been installed
    virtual BOOL IsInstall() = 0;

    // Set the TEEFER Default RULE table. Old content in table will be overwriten.
    // rule_table		the table
    // rule_num			the number of rule
    // Return :
    // return TEEFER_OK if successfully
    // return TEEFER_ERR if operation failed.
    virtual int SetDefaultRuleTable(
        PTEEFER_RULE  pRules,
        DWORD  dwRuleNumber
        ) = 0;

    // Set the TEEFER Init RULE table. Old content in table will be overwriten.
    // rule_table		the table
    // rule_num			the number of rule
    // Return :	
    // return TEEFER_OK if successfully
    // return TEEFER_ERR if operation failed.
    // It's different from TeeferSetDefaultRuleTable for it's only effect when
    // system boot up
    virtual int SetInitRuleTable(
        PTEEFER_RULE  pRules,
        DWORD  dwRuleNumber
        ) = 0;
};


TFDRV_API ITeeferClass* TeeferCreateTeeferInstance(void);


TFDRV_API void TeeferDeleteTeeferInstance(ITeeferClass *teefer);


#endif

