#ifndef __MEMORYTEST_H
#define __MEMORYTEST_H

/********************************************************************************

  Memory test code

  Either put the two macros into DllMain (or equivalent) or have a single instance
  of class MemoryTest.

  This will ensure that memory leaks are reported in debug mode, and also will
  display peak memory use for each module on exit

********************************************************************************/
#include <windows.h>
#include <crtdbg.h>

#ifdef _DEBUG
class MemoryTest
{
public:
	MemoryTest(const char* module_name) : module_name(module_name) 
	{ 
		_CrtSetDbgFlag(_CrtSetDbgFlag( _CRTDBG_REPORT_FLAG ) | _CRTDBG_LEAK_CHECK_DF);
	}
	
	~MemoryTest() 
	{ 
		_CrtMemState mem_state;
		_CrtMemCheckpoint(&mem_state);
		char string[256];
		sprintf(string, "%s used %d bytes\n", module_name, mem_state.lHighWaterCount);
		OutputDebugString(string);
	}

private:
	const char* module_name;
};
#else
class MemoryTest
{
public:
	MemoryTest(const char*){}
};
#endif

#endif