// CheckFile.cpp: implementation of the CheckFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AppFileInfo.h"

struct LANGANDCODEPAGE {
	WORD wLanguage;
	WORD wCodePage;
};


/*
 *	CorrectFileVersionFormat() substitutes ',' by '.' and takes away ' ' 
 *  in numeral part of file version. 
 */
outputstr CorrectFileVersionFormat(const char *fvstr)
{
	outputstr sRes = "";
	DWORD dwLength, dwOriginLength;

	if (fvstr == NULL)
	{
		sysprintln("Error: sndfc::AppFileInfo::CorrectFileVersionFormat received empty file version!");
		return "";
	}
	
	dwOriginLength = strlen(fvstr);
	char *adds = strchr(fvstr, '('); // check for additional, which starts with '('
	if (adds == NULL)
		dwLength = dwOriginLength;
	else
	{	
		dwLength = adds - fvstr;
		if (*(adds - 1) == ' ')
			dwLength--;			
	}			
	
	char* sdata = new char[dwOriginLength + 2];
	if (sdata == NULL)
	{
		sysprintln("Error: sndfc::AppFileInfo::Failed to create resource!");
		return fvstr;
	}

	DWORD dwPos = 0;
	DWORD dwDotCount = 0;
	for (int i = 0; i < dwLength; i++)
	{
		if (fvstr[i] == ' ' && dwDotCount < 4)
		{
			if (dwDotCount == 3)
				dwDotCount++;
			continue;
		}

		if (fvstr[i] == ',')
			sdata[dwPos] = '.';
		else
			sdata[dwPos] = fvstr[i];

		if (sdata[dwPos++] == '.')
			dwDotCount++;
	}

	if (dwLength != dwOriginLength)
	{
		for (i = dwLength; i < dwOriginLength; i++)
			sdata[dwPos++] = fvstr[i];
	}
	sdata[dwPos] = 0;
	sRes = sdata; 

	delete[] sdata;
	return sRes;
}




int GetAppFileInfo(const char	*file_path, SAppFileInfo	*app_info)
//BOOL GetAppInfo(const char	*file_path, SAPP_INFO	*app_info)
{
	LANGANDCODEPAGE *lpTranslate;
	DWORD	dwHandle = 0;
	DWORD	dwSize = ::GetFileVersionInfoSize((LPTSTR)file_path, &dwHandle);
	if (0 == dwSize)
		return ERR;

	char	*pData = new char[dwSize];
	if (!::GetFileVersionInfo((LPTSTR)file_path, 0, dwSize, pData))
	{
		delete	pData;
		return	ERR;
	}


// Read the list of languages and code pages.
	dwSize = 0;
	if (!::VerQueryValue(pData, 
	              TEXT("\\VarFileInfo\\Translation"),
	              (LPVOID*)&lpTranslate,
	              (PUINT)&dwSize))
	{
		delete	pData;
		return	ERR;
	}

// Read the file description for each language and code page.

	char	SubBlock[256];
	wsprintf( SubBlock, 
			TEXT("\\StringFileInfo\\%04x%04x\\FileDescription"),
			lpTranslate[0].wLanguage,
			lpTranslate[0].wCodePage);

	// Retrieve file description for language and code page "i". 
	char	*desc = NULL;
	dwSize = 0;
	if ((!::VerQueryValue(pData, 
			    SubBlock, 
			    (void**)&desc, 
			    (PUINT)&dwSize))||(0 == dwSize)||(NULL == desc))
	{
		wsprintf( SubBlock, 
				TEXT("\\StringFileInfo\\040904E4\\FileDescription"),
				lpTranslate[0].wLanguage,
				lpTranslate[0].wCodePage);

		// Retrieve file description for language and code page "i". 
		desc = NULL;
		dwSize = 0;
		if ((!::VerQueryValue(pData, 
				    SubBlock, 
				    (void**)&desc, 
				    (PUINT)&dwSize))||(0 == dwSize)||(NULL == desc))
		{
			/*
			char	*p = strrchr(file_path, '\\');
			if (NULL != p)
			{
				p++;
				desc = p;
			}
			else
			{
				desc = (char*)file_path;
			}
			*/
			//error to get file description
		}
	}

	app_info->FileDescription = desc;

	wsprintf( SubBlock, 
			TEXT("\\StringFileInfo\\%04x%04x\\FileVersion"),
			lpTranslate[0].wLanguage,
			lpTranslate[0].wCodePage);

	char	*ver = NULL;
	dwSize = 0;
	if ((!::VerQueryValue(pData, 
			    SubBlock, 
			    (void**)&ver, 
			    (PUINT)&dwSize))||(0 == dwSize)||(NULL == ver))
	{
		wsprintf( SubBlock, 
				TEXT("\\StringFileInfo\\040904E4\\FileVersion"),
				lpTranslate[0].wLanguage,
				lpTranslate[0].wCodePage);

		ver = NULL;
		dwSize = 0;
		if ((!::VerQueryValue(pData, 
				    SubBlock, 
				    (void**)&ver, 
				    (PUINT)&dwSize))||(0 == dwSize)||(NULL == ver))
		{
			delete	pData;
			return	ERR;
		}
	}
// jian : I change File version to use same logic as SSHelper module fileoperations.cpp does
	//app_info->FileVersion = CorrectFileVersionFormat(ver); // Temporary fix before GA build
														   // later we plan to read file version from the root	
	VS_FIXEDFILEINFO info;
	LPVOID pver;
	unsigned verSize;

	memset(&info, 0, sizeof(info));
	app_info->FileVersion = "";
	if (VerQueryValue(pData, "\\", &pver, &verSize))
	{
		// copy VS_FIXEDFILEINFO structure
		info = *(VS_FIXEDFILEINFO*)pver;
		
		app_info->FileVersion = app_info->FileVersion + HIWORD(info.dwFileVersionMS) + "." + 
								LOWORD(info.dwFileVersionMS) + "." +
								HIWORD(info.dwFileVersionLS) + "." + 
								LOWORD(info.dwFileVersionLS);				
	}
	

	app_info->FileName = file_path;

	delete	pData;
	return OK;
}

