#ifndef __CHECKFILEMD5H__
#define __CHECKFILEMD5H__
#include <io.h>
#include "fcntl.h"
#include "stdlib.h"

#include "md5checksum.h"

/* PROTOTYPES should be set to one if and only if the compiler supports
  function argument prototyping.
  The following makes PROTOTYPES default to 0 if it has not already
  been defined with C compiler flags.
 */

#define PROTOTYPES 1

/* POINTER defines a generic pointer type */
typedef unsigned char *POINTER;

/* UINT2 defines a two byte word */
typedef unsigned short int UINT2;

/* UINT4 defines a four byte word */
typedef unsigned long int UINT4;

/* PROTO_LIST is defined depending on how PROTOTYPES is defined above.
   If using PROTOTYPES, then PROTO_LIST returns the list, otherwise it
   returns an empty list.
 */

#if PROTOTYPES
#define PROTO_LIST(list) list
#else
#define PROTO_LIST(list) ()
#endif

#define	MAX_FILE_LENGTH			MAX_PATH
#define CS_BLOCK				100 


typedef	struct _file_info2	SFILE_INFO2;
struct _file_info2
{
//	char			ExecName[MAX_FILE_LENGTH+1] ;	// file name in full path.
	long			lFileSize ;
	//unsigned char 	uaFileCheckSum[MD5_CHECKSUM_LENGTH] ;   // real size is 16byte 
	md5checksum		FileChecksum;
};

/* MD5 context. */
typedef struct {
  UINT4 state[4];             /* state (ABCD) */
  UINT4 count[2];             /* number of bits, modulo 2^64 (lsb first) */
  unsigned char buffer[64];   /* input buffer */
} MD5_CTX2;

class CheckFileMD5  
{
private:
	
public:
	CheckFileMD5();
	 ~CheckFileMD5();

	int GetFileInfo(const char* lpFileName, SFILE_INFO2* lpfi);
	/*    
	 *return :  return OK if successfully
	*			return ERR_NOT_FOUND if the file cannot be open.
	*			return ERR if other errors.
	 *          check sum result is save in lpfi->ucFileCheckSum
	 */


private:
    char*   m_InBuffer;
    char*   m_OutBuffer;
    char*   m_pbKey;

	MD5_CTX2 m_mdctx;

	void MD5Init ();
    void MD5Update PROTO_LIST ((unsigned char *, unsigned int));
    void MD5Final PROTO_LIST ((unsigned char [16]));
    void MD5Transform (UINT4 state[4], unsigned char block[64]);
    void Encode (unsigned char *output, UINT4 *input, unsigned int len);
    void Decode (UINT4 *output, unsigned char *input, unsigned int len);
	int CalcCheckSum(void* start_addr, DWORD size, SFILE_INFO2* lpfi);
	/*
	*  return : if success return 1;
	               else return 0;
	*/
};

#endif