// CheckFile.cpp: implementation of the CheckFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DScan.h"
#include "longpath.h"
#include "io.h"
#include "fcntl.h"
#include "stdlib.h"
#include "windows.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

DScan::DScan()
{
	return ;
}

DScan::~DScan()
{
	return ;
}

int DScan::GetRegisteredDLLList(PSDLL_INFO lpdi,
                   DWORD dwBufByteSize, PDWORD pdwOutDllCount,
                   PDWORD pdwTotalDllCount)
{
    char szGUIDKey[MAX_PATH+sizeof("\\InprocServer32")];
	char szVersionKey[MAX_PATH];
    char szPrevDLLName[MAX_PATH];
    char szDLLName[MAX_PATH];
    char szDLLName1[MAX_PATH];
    DWORD dwGUIDKeySize,dwVersionKeySize;
    DWORD dwDllNameLen;
    HKEY hKeyClsID, hKeyProcDLL;
    HKEY hKeyTypeLib,hKeySideBySideDLL;
    int iRet = ERR;
    int nInfoCount = dwBufByteSize/sizeof(SDLL_INFO);
    DWORD dwRegType;

    if(lpdi==NULL)
    {
        sysprintln("DScan RegisteredDLL: Invalid Parameters.");
        return ERR;
    }
    
    if (nInfoCount*sizeof(SDLL_INFO) != dwBufByteSize || !nInfoCount)
    {
        sysprintln("DScan RegisteredDLL: Invalid info size="
            + dwBufByteSize
            + ".  Expect InfoSize="
            + sizeof(SDLL_INFO
            ));
    }
    
    if (RegOpenKey(HKEY_CLASSES_ROOT, "CLSID", &hKeyClsID)!=ERROR_SUCCESS)
    {
        sysprintln("DScan RegisteredDLL: Cannot read GUID.  Registry is corrupted");
        return ERR;
    }
    
	DWORD dwAttribute;
    int dwIndex = 0;
    int i, j;

	memset(szPrevDLLName,0,sizeof(szPrevDLLName));
    memset(lpdi, 0, dwBufByteSize);
    i = 0;
    *pdwTotalDllCount = 0;
    dwGUIDKeySize = MAX_PATH;
    while (RegEnumKeyEx(hKeyClsID, dwIndex, szGUIDKey, &dwGUIDKeySize,
        NULL, NULL, NULL, NULL)==ERROR_SUCCESS)
    {
        dwRegType = REG_EXPAND_SZ;
        dwGUIDKeySize = MAX_PATH;
        dwDllNameLen = sizeof(szDLLName);
        dwIndex++;
        if (dwIndex>20000)
        {
            sysprintln("DScan: Too many DLLs.  Skipping");
            break;
        }
        if (dwGUIDKeySize > 16 && szGUIDKey[0]=='{')
        {
            strcat(szGUIDKey, "\\InprocServer32");
            if (RegOpenKey(hKeyClsID, szGUIDKey, &hKeyProcDLL)==ERROR_SUCCESS)
            {
                if (RegQueryValueEx(hKeyProcDLL, NULL, NULL, &dwRegType,
                    (LPBYTE)szDLLName, &dwDllNameLen)==ERROR_SUCCESS && dwDllNameLen>4)
                {
                    dwDllNameLen = ExpandEnvironmentStrings(szDLLName,
                        szDLLName1, sizeof(szDLLName1));
                    if (dwDllNameLen && dwDllNameLen<=sizeof(szDLLName1))
                    {
                        // Check to see if the path is enclosed by '"'.  If so, strip it
                        if (szDLLName1[0]=='"')
                        {
                            char *p;
                            strcpy(szDLLName, &szDLLName1[1]);
                            strcpy(szDLLName1, szDLLName);
                            p = strrchr(szDLLName1, '"');
                            if (p!=NULL)
                            {
                                *p = 0;
                            }
                        }
						
						/*
						// The conversion takes time. Only do it when this is a new item.

                        // If it is full path, check for file existance before converting to long path
                        if (szDLLName1[1]==':' && 
                            GetFileAttributes(szDLLName1)!=0xffffffff)
                        {
                            dwDllNameLen = ShortToLongPathName(szDLLName1, szDLLName, sizeof(szDLLName));
                        }
                        else
                        {
                            strcpy(szDLLName, szDLLName1);
                        }
						*/

						// Copy it first, even it is a short name.
                        strcpy(szDLLName, szDLLName1);
						dwDllNameLen = strlen(szDLLName);

                        if (dwDllNameLen && dwDllNameLen<=sizeof(szDLLName))
                        {
							// Most of duplicate checkings are in a row consecutive.
                            if (stricmp((const char*)szPrevDLLName, (const char*)szDLLName) == 0)
							{
								// This dll has been checked just now. skip it.
								j = 0;
							}
							else
							{
						        if ( NULL != strstr(szDLLName,"~"))   // if short name, let it go there.
									j = i;
								else
								{
		                            for (j=0; j<i; j++)
			                        {
				                        if (!strnicmp((const char*)lpdi[j].szDLLPath, (const char*)szDLLName,
					                        dwDllNameLen))
						                    break;
							        }
								}
							}

							// Remeber the previous short name.
	                        strcpy(szPrevDLLName, szDLLName1);

                            
                            // Not found on the list.  Add it
                            if (j==i)
                            {
								// If it is full path, check for file existance before converting to long path
			                    if (szDLLName1[1]==':') 
					            {
									dwAttribute = GetFileAttributes(szDLLName1);
									if (0xffffffff == dwAttribute)   // skip it if the file doesn't exist.
									{
						                RegCloseKey(hKeyProcDLL);
										continue;
									}
						            dwDllNameLen = ShortToLongPathName(szDLLName1, szDLLName, sizeof(szDLLName));
							    }

								// if really get an abbreviated name, check whether it is already in list.
								if ( dwDllNameLen > strlen(szDLLName1))
								{
		                            for (j=0; j<i; j++)
			                        {
				                        if (!strnicmp((const char*)lpdi[j].szDLLPath, (const char*)szDLLName,
					                        dwDllNameLen))
						                    break;
							        }
								}

                                if (nInfoCount > i && j == i)
                                {
                                    memmove(lpdi[i].szDLLPath, szDLLName, dwDllNameLen);
                                    i++;
                                }
                                (*pdwTotalDllCount)++;
                            }
                        }
                    }
                }
                RegCloseKey(hKeyProcDLL);
            }
        }
    }

    *pdwOutDllCount = i;
    RegCloseKey(hKeyClsID);

    BOOL bSideBySideSupported = IsPlatformSupportsSideBySide();	

	// OK if it is not supported.
	if (!bSideBySideSupported)
      return OK;

    if (RegOpenKey(HKEY_CLASSES_ROOT, "TypeLib", &hKeyTypeLib)!=ERROR_SUCCESS)
    {
        sysprintln("DScan RegisteredDLL: Cannot read TypeLib.");
        return OK;    // ignore typelib.
    }

	char strExtension[10];
	char *pDot, *pSlash;
	DWORD dwVersionIndex;

	dwIndex = 0;
    while (RegEnumKeyEx(hKeyTypeLib, dwIndex, szGUIDKey, &dwGUIDKeySize,
        NULL, NULL, NULL, NULL)==ERROR_SUCCESS)
    {
        dwRegType = REG_EXPAND_SZ;
        dwGUIDKeySize = MAX_PATH;
        dwDllNameLen = sizeof(szDLLName);
        dwIndex++;
        if (dwIndex>20000)
        {
            sysprintln("DScan: Too many DLLs.  Skipping");
            break;
        }
        if (dwGUIDKeySize > 16 && szGUIDKey[0]=='{')
        {
            if (RegOpenKey(hKeyTypeLib, szGUIDKey, &hKeySideBySideDLL)==ERROR_SUCCESS)
            {
				dwVersionIndex = 0;
				dwVersionKeySize = MAX_PATH;
				// Enum each version available.
			    while (RegEnumKeyEx(hKeySideBySideDLL, dwVersionIndex, szVersionKey, &dwVersionKeySize,
						        NULL, NULL, NULL, NULL)==ERROR_SUCCESS)
				{
					dwVersionIndex++;
					strcat(szVersionKey, "\\0\\win32");
			        if (RegOpenKey(hKeySideBySideDLL, szVersionKey, &hKeyProcDLL)==ERROR_SUCCESS)
				    {
						if (RegQueryValueEx(hKeyProcDLL, NULL, NULL, &dwRegType,
						        (LPBYTE)szDLLName, &dwDllNameLen)==ERROR_SUCCESS && dwDllNameLen>4)
			            {
							pDot = strstr(szDLLName,".");
							if (NULL != pDot)
							{
								pSlash = strstr(pDot,"\\");

								if (NULL != pSlash)    // cut \ if it appends at the end.
									pSlash[0] = 0x0;

								if (strlen(pDot) >= 10)
								{
									strncpy(strExtension,pDot,9);
									strExtension[9] = 0x0;
								}
								else
									strcpy(strExtension,pDot);
							}

							// skip .tlb and .exe item.
							if (0 != stricmp(strExtension,".dll") && 0 != stricmp(strExtension,".ocx"))
							{
				                RegCloseKey(hKeyProcDLL);
								continue;
							}

					        dwDllNameLen = ExpandEnvironmentStrings(szDLLName,
							    szDLLName1, sizeof(szDLLName1));
							if (dwDllNameLen && dwDllNameLen<=sizeof(szDLLName1))
							{
								// Check to see if the path is enclosed by '"'.  If so, strip it
								if (szDLLName1[0]=='"')
								{
		                            char *p;
				                    strcpy(szDLLName, &szDLLName1[1]);
						            strcpy(szDLLName1, szDLLName);
								    p = strrchr(szDLLName1, '"');
		                            if (p!=NULL)
				                    {
						                *p = 0;
								    }
								}
						
								// Copy it first, even it is a short name.
				                strcpy(szDLLName, szDLLName1);
								dwDllNameLen = strlen(szDLLName);

								if (dwDllNameLen && dwDllNameLen<=sizeof(szDLLName))
								{
									// Most of duplicate checkings are in a row consecutive.
				                    if (stricmp((const char*)szPrevDLLName, (const char*)szDLLName) == 0)
									{
										// This dll has been checked just now. skip it.
										j = 0;
									}
									else
									{
								        if ( NULL != strstr(szDLLName,"~"))   // if short name, let it go there.
											j = i;
										else
										{
											for (j=0; j<i; j++)
											{
												if (!strnicmp((const char*)lpdi[j].szDLLPath, (const char*)szDLLName,
													dwDllNameLen))
													break;
											}
										}
									}

									// Remeber the previous short name.
						            strcpy(szPrevDLLName, szDLLName1);
	
                            
			                        // Not found on the list.  Add it
					                if (j==i)
							        {
										// If it is full path, check for file existance before converting to long path
					                    if (szDLLName1[1]==':') 
							            {
											dwAttribute = GetFileAttributes(szDLLName1);
											if (0xffffffff == dwAttribute)   // skip it if the file doesn't exist.
											{
								                RegCloseKey(hKeyProcDLL);
												continue;
											}
								            dwDllNameLen = ShortToLongPathName(szDLLName1, szDLLName, sizeof(szDLLName));

											// if really get a an abbreviated, check whether it is already in list.
											if ( dwDllNameLen > strlen(szDLLName1))
											{	
												for (j=0; j<i; j++)
												{
													if (!strnicmp((const char*)lpdi[j].szDLLPath, (const char*)szDLLName,
														dwDllNameLen))
														break;
												}
											}

									    }

		                                if (nInfoCount > i && j == i)
				                        {
						                    memmove(lpdi[i].szDLLPath, szDLLName, dwDllNameLen);
								            i++;
		                                }
										(*pdwTotalDllCount)++;
		                            }
				                }
						    }
						}
		                RegCloseKey(hKeyProcDLL);
					}
                }
                RegCloseKey(hKeySideBySideDLL);
            }
        }
    }

    *pdwOutDllCount = i;
    RegCloseKey(hKeyTypeLib);


    return OK;
}


int DScan::GetInternalDLLList(const char* lpFileName, PSDLL_INFO lpdi,
                      DWORD dwBufByteSize, PDWORD pdwOutDllCount,
                      PDWORD pdwTotalDllCount)
{
    DWORD           dwFileSizeInByte;
    PBYTE           pStartAddress, pTmp;
    PBYTE           pDllName;
    DWORD           dwDllNameLen;
	HANDLE			hFile = INVALID_HANDLE_VALUE;
	HANDLE			hMap = INVALID_HANDLE_VALUE;
    int             nInfoCount = dwBufByteSize/sizeof(SDLL_INFO);
    int             j;


	if(lpdi==NULL || strlen(lpFileName)>=MAX_PATH)
	{
        sysprintln("DScan: Invalid Parameters.");
		return ERR;
	}

    if (nInfoCount*sizeof(SDLL_INFO) != dwBufByteSize || !nInfoCount)
    {
        sysprintln("DScan: Invalid info size="
            + dwBufByteSize
            + ".  Expect InfoSize="
            + sizeof(SDLL_INFO
            ));
    }
	
	hFile = CreateFile(
        lpFileName,GENERIC_READ,FILE_SHARE_READ,NULL,
        OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);

    if(hFile==INVALID_HANDLE_VALUE)	// we get error, can't open the file
	{
		return ERR;
	}
	
	//Record the file size
	dwFileSizeInByte = GetFileSize(hFile,NULL);
	if(dwFileSizeInByte != -1)
	{
		//No error
		lpdi->ulFileSize = dwFileSizeInByte; 
	}
	else
    {
        //On error
        lpdi->ulFileSize=0 ;
    }

    //creare file mapping
    hMap = CreateFileMapping(hFile,NULL,PAGE_READONLY,0,0,NULL);
    
    if(hMap==NULL)
	{
		CloseHandle(hFile);
		return ERR;
	}
	pStartAddress = (BYTE *)MapViewOfFile(hMap,FILE_MAP_READ,0,0,0);
	if(pStartAddress==NULL)
	{
		CloseHandle(hMap);
		CloseHandle(hFile);
		return ERR;
	}

    pTmp = pStartAddress;
    memset(lpdi, 0, dwBufByteSize);
    int i = 0;
    *pdwOutDllCount = 0;
    *pdwTotalDllCount = 0;
    while (dwFileSizeInByte > 0)
    {
        pDllName = NULL;
        _asm
        {
            push    edi
            mov     eax, '.'
            mov     edi, pTmp
            mov     ecx, dwFileSizeInByte
        again:
            repne   scasb
            jne     done
            mov     edx, [edi]
            and     edx, 0dfdfdfh    // clear 4th byte and toupper the first 3 chars
            cmp     edx, 'LLD'       // find .DLL
            je      match
            cmp     edx, 'XCO'       // find .OCX
            jne     again
        match:
            mov     pTmp, edi
            dec     edi
            mov     pDllName, edi
        done:
            mov     dwFileSizeInByte, ecx
            pop     ebx
            pop     edi
        }
        if (pDllName==NULL)
            break;
        dwDllNameLen = 4;
        while (pDllName > pStartAddress)
        {
            dwDllNameLen++;
            pDllName--;
            if (dwDllNameLen >= MAX_PATH ||
                !isalpha(*pDllName) && !isdigit(*pDllName) && *pDllName!='~'&& *pDllName!='_')
            {
                dwDllNameLen--;
                pDllName++;
                if (dwDllNameLen<=4)
                    break;
                for (j=0; j<i; j++)
                {
                    if (!strnicmp((const char*)lpdi[j].szDLLPath, (const char*)pDllName,
                        dwDllNameLen))
                        break;
                }

                // Not found on the list.  Add it
                if (j==i)
                {
                    if (nInfoCount > i)
                    {
                        memmove(lpdi[i].szDLLPath, pDllName, dwDllNameLen);
                        i++;
                    }
                    (*pdwTotalDllCount)++;
                }
                break;
            }
        }
    }
    
    *pdwOutDllCount = i;

	if ( pStartAddress )
	{
		UnmapViewOfFile( pStartAddress ); pStartAddress = NULL;
	}

    CloseHandle(hMap);
	CloseHandle(hFile);

	return OK;
}


// Windows 98SE, 2000 and XP supports side-by-side.
BOOL DScan::IsPlatformSupportsSideBySide(void)
{
    OSVERSIONINFOEX osviex ;

    osviex.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

    // If platform does not support OSVERSIONINFOEX structure, it does not 
    // have side-by-side support
    //
    if (!GetVersionEx((OSVERSIONINFO *)&osviex))
    {
        return FALSE ; // no DLL re-direction
    }

    // However, for NT, then NT4 SP4 had support for OSVERSIONINFOEX 
    // support but it did not support DLL re-direction. 
    // If DLL re-direction appears in a future NT4 SP,  this code will have 
    // to be updated.
    //
    if ( (osviex.dwPlatformId == VER_PLATFORM_WIN32_NT) && (osviex.dwMajorVersion < 5) )
    {
        return FALSE ;
    }

    // For the other platform Ids, assume has side by side support
    return TRUE ;
}
