/* $Header: /usr2/cvs/master/Develop/Products/Smc/build/sndfc/src/sndfc/snd/EspPkt.h,v 1.2 2002/01/22 22:34:15 ytian Exp $*/
/*++
*************************************************************************
+-----------------------------------------------------------------------+
| Copyright (C) SYBERGEN NETWORKS, INC. 1998,1999                       |
| All Rights Reserved.                                                  |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
 * 
 * $Author: ytian $
 * $Date: 2002/01/22 22:34:15 $
 * $Revision: 1.2 $
 * $Workfile: EspPkt.h $
 * $NoKeywords: $
 *
--*/

#ifndef ESPPKT_H
#define ESPPKT_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Packet.h"
#include "ippkt.h"


/*++
ESP packet is Encapsulating Security Payload Packet, 
which is defined at RFC2406. IPSEC uses ESP as payload 
packet format.

  
 0                   1                   2                   3
 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ ----
|               Security Parameters Index (SPI)                 | ^Auth.
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ |Cov-
|                      Sequence Number                          | |erage
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ | ----
|                    Payload Data* (variable)                   | |   ^
~                                                               ~ |   |
|                                                               | |Conf.
+               +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ |Cov-
|               |     Padding (0-255 bytes)                     | |erage*
+-+-+-+-+-+-+-+-+               +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ |   |
|                               |  Pad Length   | Next Header   | v   v
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ ------
|                 Authentication Data (variable)                |
~                                                               ~
|                                                               |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+


--*/



//////////////////////////////////////////
//         ERROR MESSAGE
#define ESP_ERR_ISNT_MINE		0x2
///////////////////////////////////////////

#define PROTO_IP_TYPE_ESP		(0x32)

#include "packon1.h"
typedef struct
{
	ndword		SecurityParamIndex;
	ndword		SequenceNumber;
} SEsp_FrameHead;
#include "packoff1.h"


// CEspPacket defines GRE v2 packet used as PPTP data stream protocol

class CEspPacket : public CPacket  
{
public:
	CIpPacket	IpPacket;

	CEspPacket(void) {};
	inline CEspPacket(CIpPacket &ipp);
	
	inline int Set(CIpPacket &ipp);

	inline SEsp_FrameHead* GetFramePtr(void)
	// return the ESP head pointer
	{ 	return (SEsp_FrameHead*)(GetPacketBinary().GetAddr()); }

	inline unsigned GetHeadLen(void);
	// return the ESP header's length

	inline unsigned GetDataLen(void)
	// return the ESP's payload data length;
	{ return GetPacketBinary().GetLen() - GetHeadLen(); }

	inline memptr GetDataSection(void)
	// return the tcp's data section
	{ memptr a(GetPacketBinary().GetAddr(GetHeadLen()), GetDataLen() ); return a;}

	inline CIpPacket& GetIpPkt(void)
	{ return IpPacket; }

	friend inline ostream& operator<<(ostream& os, CEspPacket &ep);
	

	// ============ protocol special function ===========

	inline ndword GetSecurityParmIndex(void)
	{
		return GetFramePtr()->SecurityParamIndex;
	}

	inline ndword GetSequenceNumber(void)
	{
		return GetFramePtr()->SequenceNumber;
	}

	// ==================================================

};


CEspPacket::CEspPacket(CIpPacket& ipp)
{
	Set(ipp);
}

int CEspPacket::Set(CIpPacket& ipp)
// return OK if it is our packet
// return ICMP_ERR_ISNT_MINE if not.
{
 	if (ipp.GetProtocol() != PROTO_IP_TYPE_ESP) return ESP_ERR_ISNT_MINE;

	memptr p = ipp.GetDataSection();
	if (p.GetLen() < sizeof (SEsp_FrameHead))
		return ESP_ERR_ISNT_MINE;

	CPacket::Set(ipp, p);

	IpPacket = ipp;
	
	return OK;
}


inline ostream& operator<<(ostream& os, CEspPacket &ep)
{
	os	<<"===== ESP Packet==== len:"<<dec<<ep.GetPacketBinary().GetLen()<<"====\n"
		<<dec;

	return os;
}

#endif 
