// file GenThrd.CPP		defines a Generic Thread Class
/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1999                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

02/23/1999			Created by Yadong Zhang

--*/

#include "stdafx.h"
#include "GenThrd.h"
#include "process.h"

static const char TO_STOP_EVENT_NAME_PREFIX[] = "GenericThread_ToStop";

static const char RUNNING_EVENT_NAME_PREFIX[] = "GenericThread_Running";

static const char FAIL_RUNNING_EVENT_NAME_PREFIX[] = "GenericThread_FailRunning";


static void MakeEventName( char *buf, const char *prefix, long id)
{
	strcpy(buf, prefix);

	char numstr[64];

	ltoa(id, numstr, 16);
	numstr[sizeof(numstr) - 1] = 0;
	strcat(buf, numstr);
}




unsigned __stdcall GenericThreadProc( void *obj )
{
	CGenericThread*	pgt = (CGenericThread*) obj;
	unsigned id = pgt->Id;
	CGenericThreadProcCtrl ctrl(id);
	void *reference = pgt->Reference;
	DEF_GENERIC_THREAD_ENTRY_PROC *user_entry = pgt->pEntryRoutine;
	
	char name[128];

	
	// Create to_stop event
	MakeEventName(name, TO_STOP_EVENT_NAME_PREFIX, id); name[sizeof(name) - 1] = 0;
	ctrl.hToStopEvent = CreateEvent(NULL, TRUE, FALSE, name);
	if ( ctrl.hToStopEvent == 0 )
	{
		if (GENTHRD_DEBUG) sysprintln("GenericThreadProc:ThreadProc(ERROR) failed to create tostop event.");
		goto cleanup;
	}

	// create running event
	MakeEventName(name, RUNNING_EVENT_NAME_PREFIX, id); name[sizeof(name) - 1] = 0;
	ctrl.hRunningEvent[0] = OpenEvent(EVENT_ALL_ACCESS, FALSE, name);	// manually reset
	if (ctrl.hRunningEvent[0] == 0)
	{
		ctrl.hRunningEvent[0] = CreateEvent(NULL, TRUE, FALSE, name);
		if (ctrl.hRunningEvent[0] == 0)
		{
			if (GENTHRD_DEBUG) sysprintln("GenericThreadProc:ThreadProc(ERROR) failed to create running event.");
			goto cleanup;
		}
	}
	MakeEventName(name, FAIL_RUNNING_EVENT_NAME_PREFIX, id); name[sizeof(name)-1] = 0;
	ctrl.hRunningEvent[1] = OpenEvent(EVENT_ALL_ACCESS, FALSE, name);	//manually reset
	if(ctrl.hRunningEvent[1] == 0)
	{
		ctrl.hRunningEvent[1] = CreateEvent(NULL, TRUE, FALSE, name);
		if(ctrl.hRunningEvent[1] == 0)
		{
			if(GENTHRD_DEBUG) sysprintln("GenericThreadProc: ThreadProc(ERROR) failed to create failrunning event.");
			goto cleanup;
		}
	}
	
	// call user's entry
	(*user_entry)( &ctrl, reference );
	
	if (GENTHRD_DEBUG) sysprintln("GenericThread terminated.");
	
	// reset running event
	if (ctrl.hRunningEvent[0]) ResetEvent(ctrl.hRunningEvent[0]);
	if (ctrl.hRunningEvent[1]) ResetEvent(ctrl.hRunningEvent[1]);

cleanup:
	if (ctrl.hRunningEvent[0]) { CloseHandle(ctrl.hRunningEvent[0]); ctrl.hRunningEvent[0] = 0; }
	if (ctrl.hRunningEvent[1]) { CloseHandle(ctrl.hRunningEvent[1]); ctrl.hRunningEvent[1] = 0; }
	if (ctrl.hToStopEvent) { CloseHandle(ctrl.hToStopEvent); ctrl.hToStopEvent = 0; }
	
	return 0;
}

//static long InstancesCount = 0;

CGenericThread::CGenericThread(DEF_GENERIC_THREAD_ENTRY_PROC* thread_proc, void *reference)
{
	pEntryRoutine = 0;
	hThread = hRunningEvent[0] = hRunningEvent[1] = 0;
	bThreadIsRunning = false;
	
	pEntryRoutine = thread_proc; Reference = reference;


	/*++ MODIFYED BY YADONG
	// I think it is enough for most case.
	// Sometimes, We can't get the unique id by GetCurrentProcessId and InstancesCount.
	// Like we create genericthread in an application and this application call a dll also has genericthread.
	// In this case we will get the same id at applicatin and its dll.
	while(1)
	{
		InstancesCount++;
		Id = InstancesCount + GetCurrentProcessId()*1000;

		char name[256];
		MakeEventName(name, RUNNING_EVENT_NAME_PREFIX, Id);
		HANDLE htemp = NULL;
		htemp = OpenEvent(EVENT_ALL_ACCESS, FALSE, name);
		if(htemp == NULL)break;
		CloseHandle(htemp);
	}
	--*/
	// ----------------
	// Opening event failed may be caused by lots of issue.
	// IT IS BUG, GenericThread will have same count if they are static linked in
	// DLL. For example, NETPORT.DLL and NATSRV.DLL 
	LARGE_INTEGER pc; pc.QuadPart = 0;
	QueryPerformanceCounter(&pc);
	Id = pc.HighPart + pc.LowPart + GetTickCount()*10 + GetCurrentProcessId()*1000;
}


CGenericThread::~CGenericThread(void)
{
	StopThread(180*1000);
	if (hThread) { CloseHandle(hThread); hThread = 0; }
	if ( hRunningEvent[0] ) { CloseHandle(hRunningEvent[0]); hRunningEvent[0] = 0; }
	if ( hRunningEvent[1] ) { CloseHandle(hRunningEvent[1]); hRunningEvent[1] = 0; }
}

int CGenericThread::StartThread( dword waitting_mstime, bool alert_able , unsigned priority)
// return OK if successfully
// return ERR if operation failed.
{
	if (pEntryRoutine == NULL) return ERR;
	if (bThreadIsRunning) return OK;
	
	bThreadIsRunning = false;
	hRunningEvent[0] = hRunningEvent[1] = hThread = 0;
	
	// create running event
	if (hRunningEvent[0] == 0)
	{
		char name[256];
		MakeEventName( name, RUNNING_EVENT_NAME_PREFIX, Id );
		hRunningEvent[0] = CreateEvent( NULL, TRUE, FALSE, name );	// manually reset
		
		if (hRunningEvent[0] == 0)
		{
			sysprintln("GenericThread:StartThread(ERROR) failed to create running event.");
			goto err_cleanup;
		}
	}
	if(hRunningEvent[1] == 0)
	{
		char name[256];
		MakeEventName( name, FAIL_RUNNING_EVENT_NAME_PREFIX, Id);
		hRunningEvent[1] = CreateEvent( NULL, TRUE, FALSE, name); //manually reset

		if(hRunningEvent[1] == 0)
		{
			sysprintln("GenericThread: StartThread(ERROR) failed to create failrunning event.");
			goto err_cleanup;
		}
	}

	unsigned dwIntThreadId;
	hThread = (HANDLE)_beginthreadex(
			NULL,			// void *security, 
			0,				// unsigned stack_size
			GenericThreadProc,	// unsigned ( __stdcall *start_address )( void * )
			this,				// void *arglist
			CREATE_SUSPENDED,	// unsigned initflag
			&dwIntThreadId	// unsigned *thrdaddr 
			);
	
	if (hThread == 0)
	{
		sysprintln("GenericThread:StartThread(ERROR) failed to create thread.");
		goto err_cleanup;
	}
	
    SetThreadPriority(hThread, priority);
    ResumeThread(hThread);
	
	// Wait for thread started.
	DWORD dwr;
	dwr = WaitForMultipleObjectsEx(2, hRunningEvent, FALSE, waitting_mstime, alert_able);
//	dwr = WaitForSingleObjectEx(hRunningEvent, waitting_mstime, alert_able);
	if ( dwr == WAIT_OBJECT_0 )
	{
		// we got it, thread has started.
		bThreadIsRunning = true;
		return OK;
	}
	else if ( dwr == WAIT_OBJECT_0 + 1)
	{
		sysprintln("GenericThread : StartTread(ERROR) can't start.");
		goto err_cleanup;
	}
	else if ( dwr == WAIT_TIMEOUT )
	{
		sysprintln("GenericThread:StartThread(ERROR) waitting for thread time out.");
		goto err_cleanup;
	}
	else
	{
		sysprintln("GenericThread:StartThread(ERROR) waitting for thread returned unknown error.");
		goto err_cleanup;
	}
	
	bThreadIsRunning = true;
	
	return OK;
	
err_cleanup:
	if (hThread) { CloseHandle(hThread); hThread = 0; }
	if (hRunningEvent[0]) { CloseHandle(hRunningEvent[0]); hRunningEvent[0] = 0; }
	if (hRunningEvent[1]) { CloseHandle(hRunningEvent[1]); hRunningEvent[1] = 0; }
	
	return ERR;
}


int CGenericThread::StopThread( dword waitting_mstime, bool alert_able )
// return OK if successfully
// return ERR_TIME_OUT if time was out.
{
	if (bThreadIsRunning == false) return OK;
	
	bThreadIsRunning = false;
	
	int retcode = OK;
	
	HANDLE	h_to_stop_event;
	
	char name[128];
	MakeEventName(name, TO_STOP_EVENT_NAME_PREFIX, Id);
	
	h_to_stop_event = OpenEvent(EVENT_ALL_ACCESS, FALSE, name);
	if (h_to_stop_event == 0)
	{
		// thread has been terminated.
		return OK;
	}
	
	SetEvent(h_to_stop_event);
	
	int i;
	DWORD thread_exitcode = 0;
	for (i = waitting_mstime/10; i > 0; i--)
	{
		if ( GetExitCodeThread(hThread, &thread_exitcode) == FALSE  )
		{
			sysprintln("GenericThread:StopThread(ERROR) failed to get thread code. errcode" + GetLastError());
			break;
		}	
		if ( thread_exitcode != STILL_ACTIVE) break;
		
		SleepEx(10, alert_able);
	}
	
	if ( i <= 0 )
	{
		retcode = ERR_TIMEOUT;
	}
	
	if ( h_to_stop_event) CloseHandle(h_to_stop_event);
	if (hThread) { CloseHandle(hThread); hThread = 0; }
	if ( hRunningEvent[0] ) { CloseHandle(hRunningEvent[0]); hRunningEvent[0] = 0; }
	if ( hRunningEvent[1] ) { CloseHandle(hRunningEvent[1]); hRunningEvent[1] = 0; }
	
	return retcode;
}


// Check whether the thread is still alive.
bool CGenericThread::IsThreadRunning()
// return true if running
// return false if abort or not exist.
{
	if (bThreadIsRunning == false) return false;
	
	bool bRet = true;

	DWORD thread_exitcode = 0;

	// Check its exit code.
	if ( ::GetExitCodeThread(hThread, &thread_exitcode) == FALSE  )
	{
		DWORD	dwErr = GetLastError();
		if (ERROR_INVALID_HANDLE != dwErr)		//if ERROR_INVALID_HANDLE, the thread has been ended
		{
			//sysprintln("IsThreadAlive(" + strThreadName + ")  failed to get thread code. errcode" + dwErr);
			bRet = true;   // assume it is alive.
		}
		else
			bRet = false;
	}
	else if ( STILL_ACTIVE == thread_exitcode )
		bRet = true;   // it is alive.
		
	return bRet;
}

// end of GENTHRD.H
