// file GenThrd.h		defines a Generic Thread Class
/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1999                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

02/23/1999			Created by Yadong Zhang

--*/
#ifndef SND_GENTHRD_H
#define SND_GENTHRD_H


class CGenericThreadProcCtrl
{
private:
	unsigned	Id;

public:
	HANDLE 		hToStopEvent;
	HANDLE		hRunningEvent[2];//the first event is for running .the second one is for fail to running.

	CGenericThreadProcCtrl(unsigned id) { Id = id;  hToStopEvent = hRunningEvent[0] = hRunningEvent[1] = 0; }

	//follow two function must be called after try to start the thread, otherwise the mainthread will be hold for minutes.
	inline void SetRunning(void);
	// when thread has started , call the function.
	inline void SetFailRunning(void);
	//when thread was fail to start, call this function.

	inline bool IsToStop(dword waitting_time, bool alertable = false);
	// Check if the thread shall terminate.
	// It will return right away if the thread is required to stop ( means return true)
	// If thread is not required to stop, the function will wait for
	// waitting_time (millisecond).
};

void CGenericThreadProcCtrl::SetRunning(void)
{
	if (hRunningEvent[0])
	{
		SetEvent(hRunningEvent[0]);
	}	
	else
	{
		if (GENTHRD_DEBUG) sysprintln("GenericThreadProcCtrl:SetStarted(ERROR) running event is invalid.");
	}
}

void CGenericThreadProcCtrl::SetFailRunning(void)
{
	if(hRunningEvent[1])
	{
		SetEvent(hRunningEvent[1]);
	}
	else
	{
		if (GENTHRD_DEBUG) sysprintln("GenericThreadProcCtrl : SetFailStart(ERROR) event is invalid.");
	}
}

bool CGenericThreadProcCtrl::IsToStop(dword waitting_mstime, bool alertable)
{
	DWORD dwr;
	dwr = WaitForSingleObjectEx(hToStopEvent, waitting_mstime, alertable);

	if (dwr == WAIT_OBJECT_0 ) return true;
	else if ( dwr == WAIT_TIMEOUT )
	{
		return false;
	}
	
	if (GENTHRD_DEBUG) sysprintln("GenericThread:IsToStop(ERROR) failed to wait event.");
			
	return true;
}


typedef void _cdecl DEF_GENERIC_THREAD_ENTRY_PROC(CGenericThreadProcCtrl* procctrl, void *reference);

const unsigned GENERIC_THREAD_PRIORITY_ABOVE_NORMAL = THREAD_PRIORITY_ABOVE_NORMAL;
const unsigned GENERIC_THREAD_PRIORITY_BELOW_NORMAL = THREAD_PRIORITY_BELOW_NORMAL;
const unsigned GENERIC_THREAD_PRIORITY_HIGHEST 		= THREAD_PRIORITY_HIGHEST;
const unsigned GENERIC_THREAD_PRIORITY_IDLE 		= THREAD_PRIORITY_IDLE;
const unsigned GENERIC_THREAD_PRIORITY_LOWEST 		= THREAD_PRIORITY_LOWEST;
const unsigned GENERIC_THREAD_PRIORITY_NORMAL 		= THREAD_PRIORITY_NORMAL;
const unsigned GENERIC_THREAD_PRIORITY_TIME_CRITICAL= THREAD_PRIORITY_TIME_CRITICAL;

class CGenericThread
{
private:
	DEF_GENERIC_THREAD_ENTRY_PROC	*pEntryRoutine;
	void*							Reference;
	HANDLE							hThread;
	
	long							Id;
	
	HANDLE							hRunningEvent[2];
	
	bool							bThreadIsRunning;
	
public:
	CGenericThread(DEF_GENERIC_THREAD_ENTRY_PROC* thread_proc, void *reference);

	~CGenericThread(void);
	
	int StartThread( dword waitting_mstime, bool alert_able = false , 
		unsigned priority = GENERIC_THREAD_PRIORITY_NORMAL);
	// priority:  GENERIC_THREAD_PRIORITY_XXXXXXX
	// return OK if successfully
	// return ERR if operation failed.
	
	int StopThread( dword waitting_mstime , bool alert_able = false );
	// return OK if successfully
	// return ERR_TIME_OUT if time was out.

	bool IsThreadRunning(void);
	// return true if running
	// return false if abort or not exist.

	friend unsigned __stdcall GenericThreadProc( void *obj );

};



#endif

// end of GENTHRD.H
