/* $Header: /usr2/cvs/master/Develop/Products/Smc/build/sndfc/src/sndfc/snd/IsakmPkt.h,v 1.2 2002/01/22 22:34:27 ytian Exp $*/
/*++
*************************************************************************
+-----------------------------------------------------------------------+
| Copyright (C) SYBERGEN NETWORKS, INC. 1998,1999                       |
| All Rights Reserved.                                                  |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
 * 
 * $Author: ytian $
 * $Date: 2002/01/22 22:34:27 $
 * $Revision: 1.2 $
 * $Workfile: IsakmPkt.h $
 * $NoKeywords: $
 *
--*/

#ifndef ISAKMPKT_H
#define ISAKMPKT_H

#include "sndbasex.h"
#include "UdpPkt.h"
#include "ippkt.h"


/*++
The class defines ISAKMP packet.
(Internet Security Association and Key Management Protocol, RFC 2408)

  
                         1                   2                   3
     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    !                          Initiator                            !
    !                            Cookie                             !
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    !                          Responder                            !
    !                            Cookie                             !
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    !  Next Payload ! MjVer ! MnVer ! Exchange Type !     Flags     !
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    !                          Message ID                           !
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    !                            Length                             !
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+


                 ISAKMP Header Format


--*/



//////////////////////////////////////////
//         ERROR MESSAGE
#define ISAKMP_ERR_ISNT_MINE		0x2
///////////////////////////////////////////

#define PROTO_UDP_PORT_ISAKMP		(500)

#include "packon1.h"
typedef struct
{
	UINT64			InitiatorCookie;
	UINT64			ResponderCookie;
	dword			Misc;
	dword			MessageId;
	dword			Length;
} SIsakmp_FrameHead;
#include "packoff1.h"


// CIsakmpPacket defines ISAKMP packet used in IPSEC.

class CIsakmpPacket : public CPacket  
{
public:
	CUdpPacket		UdpPacket;

	CIsakmpPacket(void) {};
	CIsakmpPacket(CUdpPacket &upp);
	
	inline int Set(CUdpPacket &upp);

	inline CUdpPacket& GetUdpPkt(void) { return UdpPacket; }

	inline SIsakmp_FrameHead* GetFramePtr(void)
	{
		return (SIsakmp_FrameHead*)(GetPacketBinary().GetAddr());
	}

	friend ostream& operator<<(ostream& os, CIsakmpPacket &tp);
	friend outputstr operator +(const outputstr ops, CIsakmpPacket &ip);
	// ==================================================
	//        Protocol Special Functions
	// ==================================================
	inline UINT64 GetInitiatorCookie(void) { return GetFramePtr()->InitiatorCookie; }

	inline UINT64 GetResponderCookie(void) { return GetFramePtr()->ResponderCookie; }

};

inline int CIsakmpPacket::Set(CUdpPacket& upp)
// return OK if it is our packet
// return ISAKMP_ERR_ISNT_MINE if not.
{
	// ISAKMP packet is UDP port 500 <--> 500
 	if (upp.GetDestPort() != tonword(PROTO_UDP_PORT_ISAKMP)
 		|| upp.GetSourPort() != tonword(PROTO_UDP_PORT_ISAKMP)
 		) return ISAKMP_ERR_ISNT_MINE;
 	
	memptr p = upp.GetDataSection();
	if (p.GetLen() < sizeof (SIsakmp_FrameHead))
		return ISAKMP_ERR_ISNT_MINE;

	CPacket::Set(upp, p);

	UdpPacket = upp;
	return OK;
}


inline CIsakmpPacket::CIsakmpPacket(CUdpPacket &upp)
{
	Set(upp);
}


inline ostream& operator<<(ostream& os, CIsakmpPacket &tp)
{
	dword ch = (dword)(tp.GetInitiatorCookie()>>32);
	dword cl = (dword)(tp.GetInitiatorCookie()&0xFFFFFFFFL);
	os	<<"===== ISAKMP Packet==== len:"<<dec<<tp.GetPacketBinary().GetLen()<<"====\n"
		<<tp.GetUdpPkt().GetIpPkt().GetSourIp()<<':'
		<<tp.GetUdpPkt().GetSourPort()
		<<"->"<<tp.GetUdpPkt().GetIpPkt().GetDestIp()<<':'
		<<tp.GetUdpPkt().GetDestPort()
		<<"\nInitiatorCookie: 0x"<<hex<<ch<<cl<<dec
		<<"\n";

	return os;
}
	
inline outputstr operator +(const outputstr ops, CIsakmpPacket &ip)
{
	outputstr out = ops + "===== ISAKMP Packet==== len:" + ip.GetPacketBinary().GetLen() + ip.GetPacketBinary().GetLen() + "====\n"
		+ip.GetUdpPkt().GetIpPkt().GetSourIp() + ':' + ip.GetUdpPkt().GetSourPort()
		+"->" + ip.GetUdpPkt().GetIpPkt().GetDestIp() + ':' + ip.GetUdpPkt().GetDestPort()
		+"\nInitiatorCookie: 0x" + HexToOutputstr((dword)(ip.GetInitiatorCookie()>>32)) + HexToOutputstr((dword)(ip.GetInitiatorCookie()&0xFFFFFFFFL))
		+"\n";

//	outbuf<<ip<<(char)0;
//	outputstr out = ops + outbuf.str();

	// unfreeze it, otherwise strstream won't release the memory 
	// and memory leak will occur.
//	outbuf.freeze(false);
	return out;
}


#endif 
