#include "stdafx.h"
#include "osutils.h"
#include "LUImpersonator.h"

CLoggedonUserImpersonator::CLoggedonUserImpersonator()
{
	m_bRevertToSelfFlag = FALSE;
}

void CLoggedonUserImpersonator::RevertToMyself()
{
	if (m_bRevertToSelfFlag)
		RevertToSelf();		
}

// The codes assumes the owner of these two processes G:\WINNT\EXPLORER.EXE and 
// G:\WINNT\System32\CMD.EXE is the current logon user if either of these process
// is running.
int CLoggedonUserImpersonator::ImpersonateLogonUser()
{
	if (IsWinNT() == FALSE)
	{
		sysprintln("LogonUserImpersonator: wrong OS type.");
		return ERR;
	}
	
	HMODULE hDll = LoadLibrary("PsApi.dll");
	if (NULL == hDll)
	{
		sysprintln("LogonUserImpersonator: load PsAPI.dll failed");
		return ERR;
	}

	ENUMPROCESS_PROC	*pEnumProcesses = NULL;
	ENUMPROCESSMODULES_PROC *pEnumProcessModules = NULL;
	GETMODULEFILENAMEEX_PROC *pGetModuleFileNameEx = NULL;
	DWORD	processid_list[256];
	DWORD	size_need;
	int		x = 0;
	pEnumProcesses = (ENUMPROCESS_PROC*)GetProcAddress(hDll, "EnumProcesses");
	pEnumProcessModules = (ENUMPROCESSMODULES_PROC*)GetProcAddress(hDll, "EnumProcessModules");
	pGetModuleFileNameEx = (GETMODULEFILENAMEEX_PROC*)GetProcAddress(hDll, "GetModuleFileNameExA");
	
	if((NULL == pEnumProcesses) || (NULL == pEnumProcessModules) || (NULL == pGetModuleFileNameEx))
	{
		if (NULL == pEnumProcesses)
			sysprintln("LogonUserImpersonator: Get EnumProcesses function failed");
		if (NULL == pEnumProcessModules)
			sysprintln("LogonUserImpersonator: Get EnumProcessModules function failed");
		if (NULL == pGetModuleFileNameEx)
			sysprintln("LogonUserImpersonator: Get GetModuleFileNameEx function failed");
		FreeLibrary(hDll);
		hDll = NULL;		
		return ERR;
	}

	BOOL res = (*pEnumProcesses)(processid_list, 256, &size_need);
	if (!res)
	{
		sysprintln("LogonUserImpersonator: EnumProcesses failed");
		FreeLibrary(hDll);
		hDll = NULL;
		return ERR;
	}

	HANDLE       hToken   = NULL;
	HANDLE hProcess = NULL;
	BOOL found = false;
	char dir[MAX_PATH];
	dir[0] = 0;
	outputstr					RefProcessName;
	outputstr					RefProcessName2;
	dword r = GetWindowsDirectory(
		dir,  // buffer for system directory
		sizeof(dir)        // size of directory buffer
		);
	if ( r > 0 && r <= sizeof(dir) )
	{			
		RefProcessName = dir;
		RefProcessName = RefProcessName +
			'\\' + SMC_USER_SHELL_REF_PROCESS_FILENAME;
	}
	
	memset(dir, 0, sizeof(dir));
	r = GetSystemDirectory(
		dir,  // buffer for Windows directory
		sizeof(dir)        // size of directory buffer
		);
	if ( r > 0 && r <= sizeof(dir) )
	{			
		RefProcessName2 = dir;
		RefProcessName2 = RefProcessName2 +
			'\\' + SMC_USER_SHELL_REF_PROCESS_FILENAME2;
	}
	for(x = 0; x < (size_need>>2); x++)		// size / 4
	{
		hToken   = NULL;
		hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, processid_list[x]);
		if (NULL == hProcess)
			continue;
		else
		{
			HMODULE hMod;
			dword dwSize2 = 0;
			char szFileName[MAX_PATH];
			szFileName[0] = 0;
			if( (*pEnumProcessModules)( hProcess, &hMod,
				sizeof( hMod ), &dwSize2 ) )
			{
				// Get Full pathname:
				if( !(*pGetModuleFileNameEx)( hProcess, hMod,
					szFileName, sizeof( szFileName ) ) )
				{
					szFileName[0] = 0 ;
					dword err = GetLastError();
					sysprintln("LogonUserImpersonator: GetModuleFileNameEx failed, err = " + err);
					CloseHandle(hProcess);
					continue;
				}
				if (0 == stricmp(szFileName, RefProcessName.GetString()))
				{
					found = true;
					break;
				}
				else
					if (0 == stricmp(szFileName, RefProcessName2.GetString()))
					{
						found = true;
						break;
					}
			}
			CloseHandle(hProcess);
		}	// else
		
	}	// for
	if (found)
	{
		
		if (OpenProcessToken(hProcess, TOKEN_QUERY|TOKEN_DUPLICATE|TOKEN_IMPERSONATE , 
			&hToken))
		{
			BOOL b1 = ImpersonateLoggedOnUser(
				hToken  // handle to token for logged-on user
				);
			if (!b1)
			{
				dword e = GetLastError();
				sysprintln("LogonUserImpersonator: ImpersonateLoggedOnUser Failed, err: " + e);
				CloseHandle(hToken);
				CloseHandle(hProcess);
				FreeLibrary(hDll);
				hDll = NULL;
				return ERR;
			}
			else
			{
				m_bRevertToSelfFlag = true;
/*				PROFILEINFO profile;
				char s[50];
				unsigned long l = sizeof(s);
				GetUserName(s, &l);
				profile.lpUserName = s;
				profile.dwFlags = PI_NOUI;
				profile.dwSize = sizeof(profile);
				profile.lpDefaultPath = NULL;
				profile.lpProfilePath = NULL;
				profile.lpServerName = NULL;
				profile.lpPolicyPath = NULL;
				
				BOOL b = LoadUserProfile(
					hToken,               // user token
					&profile  // profile
					);
				if (!b)
				{
					dword e = GetLastError();
					sysprintln("LogonUserImpersonator: LoadUserProfile Failed, err: " + e);
				}*/
			}
			CloseHandle(hToken);
		}
		else
		{
			dword e = GetLastError();
			sysprintln("LogonUserImpersonator: OpenProcessToken Failed, err: " + e);
			CloseHandle(hProcess);
			FreeLibrary(hDll);
			hDll = NULL;
			return ERR;
		}
		CloseHandle(hProcess);
	} // if (found)
	
	FreeLibrary(hDll);
	hDll = NULL;

	return OK;
}
