/* licenseBase.h */

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/
#ifndef LICENSEBASE_H
#define LICENSEBASE_H
/************************************************************************/
//
// Licenses Classes for new products
//
// The new format for serial number and registration code is:
//
// 1. Serail Number: sn1 + sn2 + sn3
//    where sn1 is 12 length alphanumeric string which is inputed when 
//				serail number is created;
//          sn2 is server checksum (2 alphanumeric characters) genarated 
//				by licmaker.exe;
//          sn3 is client checksum (2 alphanumeric characters) generated 
//				by licmaker.exe.
//			sn2 will be checked on server side. sn3 will be checked on 
//			client side.
//
// 2. Registration Code: rc -- 8 length alphanumeric string generated by 
//	  licmaker.exe. rc will be checked on client side.
//
/************************************************************************/

/*
// LICENSE_TYPE definition
// the TYPE first number
//	1 is "SyGate",
//	2 is "SyGate pro"
// the second number
//	0 is free version
//  1 is license 1, 1 users	// the license is for debug or OEM
//	2 is license 2, 3 users
//	3 is license 3, 6 users
//	4 is license 4, unlimited users.
//  5 is license 5, 10 users
//  6 is license 6, 25 users
//  7 is license 7, add one user to other licenses except license 1
*/

#define LICENSE_MIN_TYPE		LICENSE_TYPE_10
#define LICENSE_MAX_TYPE		LICENSE_TYPE_17

#define LICENSE_TYPE_FREE		LICENSE_TYPE_10
#define LICENSE_TYPE_ONE		LICENSE_TYPE_11
#define LICENSE_TYPE_THREE		LICENSE_TYPE_12
#define LICENSE_TYPE_SIX		LICENSE_TYPE_13
#define LICENSE_TYPE_UNLIMITED	LICENSE_TYPE_14
//Below two license type is added from SyGate3.0 build 511
#define LICENSE_TYPE_NINE		LICENSE_TYPE_15
#define LICENSE_TYPE_TWENTYFOUR	LICENSE_TYPE_16
#define	LICENSE_TYPE_ADDONE		LICENSE_TYPE_17
#define LICENSE_TYPE_NO_LICENSE LICENSE_TYPE_18
#define LICENSE_TYPE_EXPIRED	LICENSE_TYPE_19

#define LICENSE_TYPE_10			0x0
#define LICENSE_TYPE_11			0x1
#define LICENSE_TYPE_12			0x2
#define LICENSE_TYPE_13			0x3
#define LICENSE_TYPE_14			0x4
#define LICENSE_TYPE_15			0x5
#define LICENSE_TYPE_16			0x6
#define LICENSE_TYPE_17			0x7
#define LICENSE_TYPE_18			0x8
#define LICENSE_TYPE_19			0x9


#define LICENSE_OK						0
#define LICENSE_ERR_SERIALNUM_INVALID	1
#define LICENSE_ERR_REGCODE_INVALID		2
#define LICENSE_ERR_CANNOT_ADDUSER		4
#define LICENSE_ERR_DUPLICATECODE		8

/************************************************************************/
//
// Base Class for future new product.
//
// Pure virtual function forces unique interface.
//
/************************************************************************/
class LicenseBase
{
public:
	LicenseBase() {}

	virtual void SnCsByClient(char *sn, char *sn3) = 0;
	virtual void SnRc(char *sn, char *rc) = 0;

	virtual int VerifyByClient(char *strSN, unsigned slen, char *strRegCode, unsigned rlen) = 0;
	/* return LICENSE_OK if passed */
	/* return LICENSE_ERR_SERIALNUM_INVALID if sn doesn't passed. */
	/* return LICENSE_ERR_REGCODE_INVALID if sn passed, but rc doesn't pass. */

	virtual int GetLicenseType(char *strSN, unsigned slen, char *strRegCode, unsigned rlen) = 0;
	/* return LICENSE_TYPE_XXXX */
	/* return LICENSE_TYPE_FREE if s/n or regcode 's verification failed. */
};

/************************************************************************/
//
// Class for current structure of serial number and registration code.
//
/************************************************************************/
class LicenseNewProduct : public LicenseBase
{
public:
	LicenseNewProduct();

	// Generate client checksum from Serial number(SN)
	virtual void SnCsByClient(char *sn, char *sn3);

	// Generate registration code from Serial number(SN)
	virtual void SnRc(char *sn, char *rc);

	// Verify client checksum and registration code
	virtual int VerifyByClient(char *strSN, unsigned slen, char *strRegCode, unsigned rlen);
	/* return LICENSE_OK if passed */
	/* return LICENSE_ERR_SERIALNUM_INVALID if sn doesn't passed. */
	/* return LICENSE_ERR_REGCODE_INVALID if sn passed, but rc doesn't pass. */

	// Get license type based on sn3 and rc
	virtual int GetLicenseType(char *strSN, unsigned slen, char *strRegCode, unsigned rlen);
	/* return LICENSE_TYPE_XXXX */
	/* return LICENSE_TYPE_FREE if s/n or regcode 's verification failed. */

	// Adjust character to guarantee that it is in following range
	// '0'-'9' or 'A'-'Z'
	static char AdjustedChar(char ch);

};

#endif

