#include "stdafx.h"
#include "netaid.h"
CNetAid::CNetAid()
{
	bInit=FALSE;
	dwErrCode=NO_ERR;
	pIPAddrTable=NULL;
	pIPRouteTableBuffer=NULL;
	dwLastRefreshTime=0;
}
CNetAid::~CNetAid()
{
	if (pIPAddrTable)
	{
		free (pIPAddrTable);
		pIPAddrTable=NULL;
	}
	if (pIPRouteTableBuffer)
	{
		free (pIPRouteTableBuffer);
		pIPRouteTableBuffer=NULL;
	}
	DeleteCriticalSection(&NetAid_CriticalSection);
	TDIQ_Cleanup();
}
void CNetAid::Init()
{
	BOOL b_ret=FALSE;
	InitializeCriticalSection(&NetAid_CriticalSection);

	TDIQ_Startup();
	 //
   // Confirm That TCP/IP Is Installed
   //
   b_ret = TDIQ_IsIPInstalled();

   if( !b_ret )
   {
      TDIQ_Cleanup();
	  dwErrCode=ERR_NOT_INSTALL_IP;
	  bInit=FALSE;
      return ;
   }
   bInit=TRUE;
   return ;
}

int CNetAid::RefreshNetAidInfo()
{
	 //get ip snmp info
   if (!bInit) return ERR_NOT_INIT;
 
   //first enterCriticalSection
   EnterCriticalSection(&NetAid_CriticalSection);

   DWORD ipSnmpInfoSize = sizeof(ipSnmpInfo);

   int i_ret = TDIQ_GetIPSNMPInfo( &ipSnmpInfo, &ipSnmpInfoSize );

   if (i_ret)
   {
      LeaveCriticalSection(&NetAid_CriticalSection);
      return ERR_GETIPSNMPINFO;
   }

   DWORD ipAddrTableSize = sizeof(IPAddrEntry) * ipSnmpInfo.ipsi_numaddr;
   if (pIPAddrTable) 
   {
	   free (pIPAddrTable);
	   pIPAddrTable=NULL;

   }
   pIPAddrTable = (IPAddrEntry *)(malloc(ipAddrTableSize));

   i_ret = TDIQ_GetIPAddrTable(
               (PUCHAR )pIPAddrTable,
               &ipAddrTableSize
               );

   if (i_ret)
   {
	  LeaveCriticalSection(&NetAid_CriticalSection);	
      return ERR_GETIPADDRTABLE;
   }

   //
   // Get IP Route Table
   //
   if (pIPRouteTableBuffer)	
   {
	   free (pIPRouteTableBuffer);
	   pIPRouteTableBuffer=NULL;
   }
   DWORD ipRouteTableBufSize = sizeof(IPRouteEntry) * ipSnmpInfo.ipsi_numroutes;
   pIPRouteTableBuffer = (PUCHAR )(malloc (ipRouteTableBufSize));

   i_ret = TDIQ_GetIPRouteTable(
               pIPRouteTableBuffer,
               &ipRouteTableBufSize
               );

   if (i_ret)
   {
	   LeaveCriticalSection(&NetAid_CriticalSection);
       return ERR_GETIPROUTETABLE;
   }
	dwLastRefreshTime=GetTickCount();
   LeaveCriticalSection(&NetAid_CriticalSection);
   return NO_ERR;

}


//this function only support win2000 and winXP an Ethernet
int CNetAid::QueryNicCableConnectionStatusByMacAddress(
		const byte* pMacAddr,
		NicCableStatus * out_eNicCableStatus)
//int CNetAid::QueryNicCableConnectionStatusByMacAddress(const PBYTE pMacAddr,NicCableStatus & eNicCableStatus)   

{
	if (!bInit) return ERR_NOT_INIT;
	DWORD current_time=GetTickCount();		
	DWORD ret_val=NO_ERR;
	if (current_time-dwLastRefreshTime>MIN_REFRESH_TIME) ret_val=RefreshNetAidInfo();
	if (NO_ERR!=ret_val) return ERR_REFRESHNETAIDINFO;
	*out_eNicCableStatus=NICCABLESTATUS_CABLE_UNKNOWN ;
	EnterCriticalSection(&NetAid_CriticalSection);
	IPSNMPInfo *pIPSNMPInfo=&ipSnmpInfo;

   for(DWORD i = 0; i < pIPSNMPInfo->ipsi_numaddr; i++ )
   {
	  ULONG IFEntryBufferSize;
	  UCHAR IFEntryBuffer[ sizeof( IFEntry ) + MAX_IFDESCR_LEN ];
	  IFEntry *pIFEntry;
	  ULONG InterfaceInfoSize;
	  UCHAR InterfaceInfo[ sizeof( IPInterfaceInfo ) + MAX_PHYSADDR_SIZE ];
	  IPInterfaceInfo *pIPInterfaceInfo;
//	  struct in_addr inaddrScratch;
//	  IPRouteEntry *pIPRouteTableEntry;
//	  IPRouteEntry *pIPRouteEntry;

	  //
	  // Get IPInterfaceInfo For Address
	  //
	  InterfaceInfoSize = sizeof( InterfaceInfo );
	  pIPInterfaceInfo = (IPInterfaceInfo * )InterfaceInfo;

	  ret_val = TDIQ_GetIPInterfaceInfoForAddr(
				  pIPAddrTable[i].iae_addr,
				  (PUCHAR )pIPInterfaceInfo,
				  &InterfaceInfoSize
				  );

	  if( ret_val )
	  {
		 pIPInterfaceInfo = NULL;
	  }

	  //
	  // Get IFEntry For Address
	  //
	  IFEntryBufferSize = sizeof( IFEntryBuffer );
	  pIFEntry = (IFEntry * )IFEntryBuffer;

	  ret_val = TDIQ_GetIFEntryForIFIndex(
				  pIPAddrTable[i].iae_index,
				  (PUCHAR )pIFEntry,
				  &IFEntryBufferSize
				  );

	  if( ret_val )
	  {
		 continue;
	  }
	  if (pIFEntry->if_type !=IF_TYPE_ETHERNET)
	  { 
		 continue;			  
	  }
	  if (memcmp(pMacAddr,pIFEntry->if_physaddr,6)!=0)
	  {
		  continue;
	  }

	  if (pIPInterfaceInfo) 	*out_eNicCableStatus=NICCABLESTATUS_CABLE_CONNECTED;
	  else *out_eNicCableStatus=NICCABLESTATUS_CABLE_DISCONNECTED;
   }
	LeaveCriticalSection(&NetAid_CriticalSection);
	return NO_ERR;

}


int CNetAid::QueryGatewayIpAddress(
			DWORD* gateway_list_buf,	// support NULL.
			DWORD buf_size,			// size in byte
			DWORD* out_gw_Num,		// number of gateway returned
			DWORD* out_total_gw_num	// total number of gateway detected
			)
//int CNetAid::QueryGatewayIpAddress(PDWORD pdwGatewayIPArray,PDWORD pdwGatewaypIpNum)
{
	if (!bInit) return ERR_NOT_INIT;
	DWORD current_time=GetTickCount();		
	DWORD ret_val=NO_ERR;
	if (current_time-dwLastRefreshTime>MIN_REFRESH_TIME) ret_val=RefreshNetAidInfo();
	if (NO_ERR!=ret_val) return ERR_REFRESHNETAIDINFO;
	EnterCriticalSection(&NetAid_CriticalSection);
	*out_gw_Num=0;
	*out_total_gw_num=0;
	IPSNMPInfo *pIPSNMPInfo=&ipSnmpInfo;
	IPRouteEntry *pIPRouteEntry= (IPRouteEntry * )pIPRouteTableBuffer;
	for(DWORD i = 0; i < pIPSNMPInfo->ipsi_numroutes; i++ )
	{
      IPAddrEntry *pIPAddrEntry = NULL;

      for(DWORD j = 0; j < pIPSNMPInfo->ipsi_numaddr; j++ )
      {
         if( pIPAddrTable[j].iae_index == pIPRouteEntry->ire_index )
         {
            pIPAddrEntry = &pIPAddrTable[j];
            break;
         }
      }

      if( !pIPAddrEntry )
      {
         //
         // Move To Next Route
         //
         pIPRouteEntry = TDIQ_GetNextIPRouteTableEntry( pIPRouteEntry );
         continue;
      }

	  if (pIPRouteEntry->ire_dest==0)
	  {
		  		
		  *out_total_gw_num+=1;
		  if (gateway_list_buf && (*out_gw_Num< (buf_size/4)) )
		  {
			  gateway_list_buf[*out_gw_Num]=pIPRouteEntry->ire_nexthop;
			  *out_gw_Num+=1;
		  }
		   
	  }
      //
      // Move To Next Route
      //
      pIPRouteEntry = TDIQ_GetNextIPRouteTableEntry( pIPRouteEntry );
   }
	
	LeaveCriticalSection(&NetAid_CriticalSection);
	return NO_ERR;
	
}