// netutils.cpp

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/
/*++
History:

04/01/1999			Created by Yadong Zhang

--*/


#include "stdafx.h"

#include "netutils.h"
#include "osutils.h"

ipaddr GetLocalIP(ipaddr *pIpList, unsigned IpListSizeByByte)
// No ip list returned if pIpList == NULL.
// IpListSizeByByte will be ignore if pIpList == NULL.
// GetLocalIP will copy 16 IP address at most.
// NOTICE: for history reason, the return value of 
// GetLocalIP() is Net-Ordered.
// the pIpList is Local-machine-Ordered.
{
	DWORD   dwIP=0;
	HOSTENT *phent;
	char	localhostname[128];
	unsigned	i;

	if (pIpList) memset(pIpList, 0, IpListSizeByByte);
	
	if (gethostname(localhostname, 128)) {
		return toipaddr(0); 
	} 

	phent = gethostbyname(localhostname);
	if (phent == 0) return toipaddr(0);
	dwIP = *(DWORD*)phent->h_addr_list[0];

	if (pIpList)
	{
		memset(pIpList, 0, IpListSizeByByte);
		i = 0;
		while (i < IpListSizeByByte/4)
		{
			if (phent->h_addr_list[i] == 0) break;
			pIpList[i] = toipaddr(*(DWORD*)phent->h_addr_list[i]);
			++i;
		}
	}

	return toipaddr(dwIP);
}


int SendAUdp(DWORD sourip, DWORD destip, WORD destport, char * Message, int MessLen)
// Send a udp packet for probe
// myiptoa: source IP, if you want to use local IP which is assigned by TCP/IP stack, 
// set sourip to ADDR_ANY.
// return 0 if success
{
	SOCKET sock;
	struct sockaddr_in ia;
	int err;
  
	char achMessage[100];
  
	// Variable to set the broadcast option with setsockopt ().
	BOOL fBroadcast = TRUE;
    
    sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

	if (sock == INVALID_SOCKET) return 1;

    // bind to a local socket and an interface.
    ia.sin_family = AF_INET;
    ia.sin_addr.s_addr = sourip;
	ia.sin_port = htons(0);
//    ia.sin_port = htons ( destport );

    err = bind ( sock, (SOCKADDR *) &ia, sizeof (SOCKADDR_IN) );

    if ( SOCKET_ERROR == err )
    {
		sysprintln("GetLocalIp(ERR): SendAUdp bind error. ret=" + WSAGetLastError());
		return 1;
    }

	ia.sin_family = AF_INET;
    ia.sin_addr.s_addr = destip;
	ia.sin_port = htons ( destport );


    lstrcpy ( achMessage, "Hi, Anybody here?" );
  
	if(Message == NULL || MessLen == 0)
	{
		err = sendto ( sock, achMessage, strlen ( achMessage ), 0, (SOCKADDR *) &ia, sizeof ( SOCKADDR_IN ));
	}
	else
	{
		err = sendto ( sock, Message, MessLen,0, (SOCKADDR *)&ia,sizeof(SOCKADDR_IN));
	}
    if ( SOCKET_ERROR == err )
    {
		sysprintln("SendAUdp sendto error. ret=" + WSAGetLastError());
		return 1;
    }

   	closesocket(sock);

	return 0;
}

int RunNcpa(bool wait_for_end)
// if wait_for_end is true, the function won't return until the program executed finished
// return OK if success
{
	// disable the function.
	return OK;

#if 0
	char sysdir[300];

	if (GetSystemDirectory(sysdir, sizeof(sysdir)) == 0)
	{
		sysprintln("RunNcpa: can't get system directory.");
		return ERR;
	}
	
	strcat(sysdir, "\\control.exe");
	
	int r;
	r = Exec(sysdir, " ncpa.cpl", wait_for_end, true);
	SleepEx(7000, TRUE);

	return r;
#endif
}

int SetAutoDial(bool enable)
// enable/disable Auto-Dial. The setting is same as 
// control panel -> Internet -> Connection -> Connect to Internet using XXXX
// NOTICE: It is supported only by Internet Explorer 3.0 or above.
// return OK if successful
// return ERR if failed.
{
	int retcode = ERR;
	HKEY skey = 0;
	DWORD status;
	DWORD regtype;
	char valbuf[64];
	DWORD valbuflen;
	DWORD newdata;

	const char reg_path[] = "Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings";
	const char val_autodial[] = "EnableAutoDial";
	
	status = RegOpenKeyEx(
			HKEY_CURRENT_USER,
			reg_path,
			0,
			KEY_READ | KEY_WRITE,
			&skey
			);

	if (status != ERROR_SUCCESS) goto cleanup;

	regtype = REG_BINARY;

	valbuflen = sizeof(valbuf);
	status = RegQueryValueEx(skey, val_autodial, 
					NULL, &regtype,
					(BYTE*)valbuf, &valbuflen);

	// if (status != ERROR_SUCCESS) goto cleanup;

	// if it is not a DWORD
	if (status == ERROR_SUCCESS && valbuflen != 4) goto cleanup;

	newdata = enable;

	status = RegSetValueEx(skey, val_autodial, 
							NULL, regtype,
							(BYTE*)&newdata, sizeof(DWORD));

	if (status != ERROR_SUCCESS) goto cleanup;
		
	retcode = OK;

cleanup:
	if (skey) RegCloseKey(skey);

	return retcode;
}

// ==========================================
// FOR COMPATIBILITY WITH SYGATE
// DON'T USE IT
// ===========================================
DWORD GetLocalIP(DWORD *pIpList, unsigned IpListSizeByByte)
// No ip list returned if pIpList == NULL.
// IpListSizeByByte will be ignore if pIpList == NULL.
// GetLocalIP will copy 16 IP address at most.
// NOTICE: for history reason, the return value of 
// GetLocalIP() is Net-Ordered.
// the pIpList is Local-machine-Ordered.
{
	DWORD   dwIP=0;
	HOSTENT *phent;
	char	localhostname[128];
	unsigned	i;

	if (pIpList) memset(pIpList, 0, IpListSizeByByte);
	
	if (gethostname(localhostname, 128)) {
		return 0; 
	} 

	phent = gethostbyname(localhostname);
	if (phent == 0) return 0;
	dwIP = *(DWORD*)phent->h_addr_list[0];

	if (pIpList)
	{
		memset(pIpList, 0, IpListSizeByByte);
		i = 0;
		while (i < IpListSizeByByte/4)
		{
			if (phent->h_addr_list[i] == 0) break;
			pIpList[i] = *(DWORD*)phent->h_addr_list[i];
			++i;
		}
	}

	return dwIP;
}


int OpenNetworkAppletOnControlPanel_NT(void)
// return OK if success
{
	char sysdir[300];

	if (GetSystemDirectory(sysdir, sizeof(sysdir)) == 0)
	{
		//sysprintln("OpenNetworkAppletOnControlPanel: can't get system directory."<<endl;
		return ERR;
	}
	
	strcat(sysdir, "\\control.exe");
	
	int r;
	r = Exec(sysdir, " ncpa.cpl", false, true);
	//SleepEx(7000, TRUE);

	return r;
}

int OpenNetworkAppletOnControlPanel_95(void)
// return OK if success
{
	char sysdir[300];
	char windir[300];

	if (GetWindowsDirectory(windir, sizeof(windir)) == 0)
	{
		//sysprintln("OpenNetworkAppletOnControlPanel: can't get system directory."<<endl;
		return ERR;
	}
	if (GetSystemDirectory(sysdir, sizeof(windir)) == 0)
	{
		//sysprintln("OpenNetworkAppletOnControlPanel: can't get system directory."<<endl;
		return ERR;
	}
	
	strcat(windir, "\\control.exe");
	char parm[300];
	sprintf(" %s\\netcpl.cpl", parm, sysdir);
	
	int r;
	r = Exec(windir, parm, false, true);
	//SleepEx(7000, TRUE);

	return r;
}

int OpenNetworkAppletOnControlPanel(void)
{
	if (IsWinNT()) return OpenNetworkAppletOnControlPanel_NT();
	return OpenNetworkAppletOnControlPanel_95();
}

int SetProxySetting(bool enable)
// enable/disable the proxy setting. The setting is same as 
// control panel -> Internet -> Connection -> LAN Settings -> check "Proxy Server"
// return OK if successful
// return ERR if failed.
{
	int retcode = ERR;
	HKEY skey = 0;
	DWORD status;
	DWORD regtype;
	char valbuf[64];
	DWORD valbuflen;
	DWORD newdata;

	const char reg_path[] = "Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings";
	const char val_proxy[] = "ProxyEnable";
	
	status = RegOpenKeyEx(
			HKEY_CURRENT_USER,
			reg_path,
			0,
			KEY_READ | KEY_WRITE,
			&skey
			);

	if (status != ERROR_SUCCESS) goto cleanup;

	regtype = REG_BINARY;

	valbuflen = sizeof(valbuf);
	status = RegQueryValueEx(skey, val_proxy, 
					NULL, &regtype,
					(BYTE*)valbuf, &valbuflen);

	// if (status != ERROR_SUCCESS) goto cleanup;

	// if it is not a DWORD
	if (status == ERROR_SUCCESS && valbuflen != 4) goto cleanup;

	newdata = enable;

	status = RegSetValueEx(skey, val_proxy, 
							NULL, regtype,
							(BYTE*)&newdata, sizeof(DWORD));

	if (status != ERROR_SUCCESS) goto cleanup;
		
	retcode = OK;

cleanup:
	if (skey) RegCloseKey(skey);

	return retcode;
}



// end of Netutils.cpp
