// file NicIp.cpp

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/
/*++
History:

04/02/1998			Merged into SNDFC by Yadong Zhang
11/15/1998			Created by Yadong Zhang

--*/

#include "stdafx.h"
#include "nicip.h"
#include "regutil.h"
#include "osutils.h"
#include "NicUtils.h"
#include "NicIpHlp.h"


static char *cov_inet_addr_2str(long ip, char *out_str)	
{
	unsigned char *p = (unsigned char*)&ip;
	sprintf(out_str, "%u.%u.%u.%u", (unsigned)(*p), (unsigned)(*(p+1)),
				(unsigned)(*(p+2)), (unsigned)(*(p+3)));
	return out_str;
}

static int GetNetDevUnderOne(char *reg_path, char *nic_name, char *out_buf, unsigned buf_len)
// NOTICE: reg_path Does NOT end with '\';
// the path of nic found will be placed into out_buf
// return 0 if successful
// return -1 if failed.
{
    HKEY		syskey, subkey;
    DWORD       status;
	FILETIME	dummy;

	DWORD		regtype;

	DWORD		index = 0;
	char		namebuf[256];
	DWORD		name_len;

	char		keypath[256];

	int			retcode = -1;
	bool		igotit = false;

	if (buf_len == 0) return -1;
	out_buf[0] = 0;

	status = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			reg_path,
			0,
			KEY_READ,
			&syskey
			);

	if (status != ERROR_SUCCESS) return -1;

	for (index = 0; 1 ; index++)
	{
		name_len = sizeof(namebuf);
		status = RegEnumKeyEx(syskey, index, (LPSTR) namebuf, &name_len, 
						NULL, NULL, NULL, &dummy);
		if (status != ERROR_SUCCESS) break;

		// we got a key, let's see 
		// ==========================================================
		strcpy(keypath, reg_path); strcat(keypath, "\\");
		strcat(keypath, namebuf);
		// cout<<"testing  "<<keypath<<endl;

		status = RegOpenKeyEx(
						HKEY_LOCAL_MACHINE,
						(char*) keypath, 0, KEY_READ, &subkey 
						);
		if (status != ERROR_SUCCESS) continue;	// if failed.

		igotit = false;
		do { // only one cycle
			char valbuf[256];
			DWORD valbuflen;

			// check "Class Net"
			valbuflen = sizeof(valbuf);
			status = RegQueryValueEx(subkey, "Class", 
									NULL, &regtype,
									(BYTE*)valbuf, &valbuflen);
			if (status != ERROR_SUCCESS) break;
			valbuf[sizeof(valbuf) - 1] = 0;
			if (stricmp(valbuf, "Net") != 0) break;

			// check "Driver...."
			valbuflen = sizeof(valbuf);
			status = RegQueryValueEx(subkey, "Driver", 
									NULL, &regtype,
									(BYTE*)valbuf, &valbuflen);
			if (status != ERROR_SUCCESS) break;
			valbuf[sizeof(valbuf) - 1] = 0;

			char cmpbuf[64]; strcpy(cmpbuf, "Net\\");
			strcat(cmpbuf, nic_name); 
			if (stricmp(valbuf, cmpbuf) != 0) break;

			// OK, we got one.
			// cout<<"We got one, "<<keypath<<endl;
			strncpy(out_buf, keypath, buf_len);
			out_buf[buf_len - 1] = 0;
			
			igotit = true;

		} while (0);

		RegCloseKey(subkey);
		// ===========================================================

		if (igotit) 
		{
			retcode = 0; break;
		}
		else
		{
			if (GetNetDevUnderOne(keypath, nic_name, out_buf, buf_len) == 0)
			{
				retcode = 0;
				break;
			}
		}

//		cout<<index<<"  "<<namebuf<<endl;

	}


	RegCloseKey(syskey);

	return retcode;
}


static int GetNetTransFromMsTcp(char *mstcp_num, char *outbuf, unsigned buflen)
// return 0 if successful
// return -1 if failed.
{
	int retcode = -1;
	HKEY subkey;
	char mstcp[128];
	DWORD status;
	DWORD		regtype;
	
	if (buflen == 0) return -1;
	outbuf[0] = 0;
	
	strcpy(mstcp, "Enum\\Network\\MSTCP\\");
	strcat(mstcp, mstcp_num);

	status = RegOpenKeyEx(
				HKEY_LOCAL_MACHINE,
				(char*) mstcp, 0, KEY_READ, &subkey 
					);
	if (status != ERROR_SUCCESS) -1;	// if failed.

	do // only once
	{
		// check "Driver"
		char valbuf[256];
		DWORD valbuflen;
		valbuflen = sizeof(valbuf);
		status = RegQueryValueEx(subkey, "Driver", 
						NULL, &regtype,	(BYTE*)valbuf, &valbuflen);
		if (status != ERROR_SUCCESS) break;

		valbuf[sizeof(valbuf) - 1] = 0;
		char cmpbuf[] = "NetTrans\\";
		if (strnicmp(cmpbuf, valbuf, strlen(cmpbuf)) == 0) 
		{
			retcode = 0;

			strncpy(outbuf, &valbuf[strlen(cmpbuf)], buflen);
			outbuf[buflen - 1] = 0;
		}
	} while (0);

	RegCloseKey(subkey);
	
	return retcode;
}


static int GetNetTrans1(char *reg_path, char *nic_name, char *outbuf, unsigned buflen)
// return 0 if success
{
    HKEY		syskey;
    DWORD       status;
	DWORD		valtype;
	char		valdata[64];

	DWORD		index = 0;
	char		namebuf[256];
	DWORD		name_len;

	int retcode = -1;

	char keypath[256];

	if (buflen == 0) return -1;
	outbuf[0] = 0;

	if (GetNetDevUnderOne(reg_path, nic_name, keypath, sizeof(keypath)) != 0)
			return -1;
		
	strcat(keypath, "\\Bindings");

	status = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			keypath,		// reg_path,
			0,
			KEY_READ,
			&syskey
			);

	if (status != ERROR_SUCCESS) return -1;

	for (index = 0; 1 ; index++)
	{
		name_len = sizeof(namebuf);
		DWORD valdatalen = sizeof(valdata);
		status = RegEnumValue( 
					syskey,
					index,				// DWORD dwIndex,          // index of value to query
					(LPSTR) namebuf,	// LPTSTR lpValueName,     // address of buffer for value string
					&name_len,			// LPDWORD lpcbValueName,  // address for size of value buffer
					NULL,				// LPDWORD lpReserved,     // reserved
					&valtype,			// LPDWORD lpType,         // address of buffer for type code
					(LPBYTE)valdata,	//  LPBYTE lpData,          // address of buffer for value data
					&valdatalen			// LPDWORD lpcbData        // address for size of data buffer
					);
		if (status != ERROR_SUCCESS) break;

		namebuf[sizeof(namebuf) - 1] = 0;

		char cmpbuf[] = "MSTCP\\";
		if (strnicmp(cmpbuf, namebuf, strlen(cmpbuf)) != 0) continue;
		
		// found "MSTCP\"
		char *tcpnum = &namebuf[strlen(cmpbuf)];

		if (GetNetTransFromMsTcp(tcpnum, outbuf, buflen) == 0)
		{
			retcode = 0;		break;
		}
	}

	RegCloseKey(syskey);

	return retcode;
}


static int GetNetTransFromNicName(char *nicname, char *outbuf, unsigned buflen)
// return 0 if success
{
	if (GetNetTrans1("Enum\\PCI", nicname, outbuf, buflen) == 0) return 0;

	if (GetNetTrans1("Enum\\ISAPNP", nicname, outbuf, buflen) == 0) return 0;

	if (GetNetTrans1("Enum\\PCMCIA", nicname, outbuf, buflen) == 0) return 0;

	if (GetNetTrans1("Enum\\ROOT", nicname, outbuf, buflen) == 0) return 0;
	

	// WOW!! We still cannot find it. Try to search whole enum tree.
	if (GetNetTrans1("Enum", nicname, outbuf, buflen) == 0) return 0;

	return -1;
}


int GetNicNetParam_95(char *nicname, SNicNetParam* nnp,etheraddr *mac)
// return 0 if successful
// return -1 if failed
{
	char 		nettrans[128];
    HKEY		subkey;
    DWORD       status;
	DWORD		valtype;
	char		valdata[64];
	int retcode = 0;
	
	memset(nnp, 0, sizeof(*nnp));
	if (GetNetTransFromNicName(nicname, nettrans, sizeof(nettrans)) != 0) return -1;

	char keypath[256];
	strcpy(keypath, "System\\CurrentControlSet\\Services\\Class\\NetTrans\\");
	strcat(keypath, nettrans);
	// cout<<nettrans<<endl;

	status = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			keypath,		// reg_path,
			0,
			KEY_READ,
			&subkey
			);
	if (status != ERROR_SUCCESS) return -1;

	{
		DWORD valdatalen = sizeof(valdata);
		status = RegQueryValueEx(subkey, "IPAddress", 
							NULL, &valtype,
							(BYTE*)valdata, &valdatalen);
		if (status == ERROR_SUCCESS) 
		{
			valdata[sizeof(valdata) - 1] = 0;
			nnp->IpAddress = cov_inet_addr(valdata);

			if (nnp->IpAddress == 0) nnp->DhcpEnabled = true;
		}
		
		valdatalen = sizeof(valdata);
		status = RegQueryValueEx(subkey, "IPMask", 
							NULL, &valtype,
							(BYTE*)valdata, &valdatalen);
		if (status == ERROR_SUCCESS) 
		{
			valdata[sizeof(valdata) - 1] = 0;
			nnp->SubnetMask = cov_inet_addr(valdata);
		}

		valdatalen = sizeof(valdata);
		status = RegQueryValueEx(subkey, "DefaultGateway", 
							NULL, &valtype,
							(BYTE*)valdata, &valdatalen);
		if (status == ERROR_SUCCESS) 
		{
			valdata[sizeof(valdata) - 1] = 0;
			nnp->DefaultGateway = cov_inet_addr(valdata);
		}

		retcode = 0;
	}
	RegCloseKey(subkey);
	
	if(nnp->DhcpEnabled)
	{
	    HKEY		Mainkey;

		strcpy(keypath, "System\\CurrentControlSet\\Services\\Vxd\\DHCP");
		status = RegOpenKeyEx(
				HKEY_LOCAL_MACHINE,
				keypath,		// reg_path,
				0,
				KEY_READ,
				&Mainkey
				);
		if (status == ERROR_SUCCESS)
		{
			char	strTemp[128];
			FILETIME	dwDummy;
			DWORD		dwIndex = 0;
			DWORD		dwTemp;
			DWORD		dwTime = 0;

			dwTemp = sizeof(strTemp);
			status = RegEnumKeyEx(Mainkey, dwIndex, strTemp, &dwTemp, 
						NULL, NULL, NULL, &dwDummy);
			while(status == ERROR_SUCCESS)
			{
				BYTE tempbuf[200];
				
				wsprintf((char*) tempbuf, "%s%s%s", keypath,"\\",strTemp);
				status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,(char*) tempbuf,
							0, KEY_READ, &subkey );
				if (status == ERROR_SUCCESS) 
				{
					DWORD	RegType;

					RegType = REG_BINARY;
					dwTemp = sizeof(tempbuf);
					status=RegQueryValueEx(subkey, "DhcpInfo", NULL,&RegType,(BYTE *)tempbuf, &dwTemp);
					if(status == ERROR_SUCCESS) // Get the DHCP info from Win95
					{
						struct DHCPINFO{
							DWORD	DhcpIndex;
							DWORD	DhcpIpAddress;
							DWORD	DhcpSubnetMask;
							DWORD	DhcpServer;
							DWORD	DhcpDesiredIpAddress;
							DWORD	Lease;
							DWORD	LeaseObtainedTime;
							DWORD	T1;
							DWORD	T2;
							DWORD	LeaseTerminatesTime;
							DWORD	Unknow;
							char	HardwareType;
							etheraddr	HardwareAdd;
						};

						DHCPINFO * pDhcpInfo;
						pDhcpInfo = (DHCPINFO *)tempbuf;
						if(memcmp(&(pDhcpInfo->HardwareAdd), mac, sizeof(etheraddr)) == 0 &&
								pDhcpInfo->LeaseObtainedTime > dwTime)
						{
							BYTE	dwOptions[128];

							dwTime = pDhcpInfo->LeaseObtainedTime;
							nnp->IpAddress = pDhcpInfo->DhcpIpAddress;
							nnp->SubnetMask = pDhcpInfo->DhcpSubnetMask;
							status = RegQueryValueEx(subkey,"OptionInfo",NULL,&RegType,
										dwOptions,&dwTemp);
							if (status == ERROR_SUCCESS)
							{
								int ipoint = 0;
								while(dwOptions[ipoint]!= 0x03 && dwOptions[ipoint] != 0xFF)
								{
									ipoint++;
									ipoint += dwOptions[ipoint];
									ipoint++;
								}
								if(dwOptions[ipoint] == 0x03)
								{
									ipoint++;
									ipoint++;
									nnp->DefaultGateway = dwOptions[ipoint+3];
									nnp->DefaultGateway = ((nnp->DefaultGateway)<<8) + dwOptions[ipoint+2];
									nnp->DefaultGateway = ((nnp->DefaultGateway)<<8) + dwOptions[ipoint+1];
									nnp->DefaultGateway = ((nnp->DefaultGateway)<<8) + dwOptions[ipoint];
								}
							}
						}
							
					}

					
					etheraddr HardwareAdd;
					
					dwTemp = sizeof(HardwareAdd);
					RegType = REG_SZ ;

					status=RegQueryValueEx(subkey, "HardwareAddress", NULL, &RegType,
	                           (BYTE *)&HardwareAdd,  &dwTemp);
					if (status == ERROR_SUCCESS)
					{
						if(memcmp(&HardwareAdd,mac,sizeof(etheraddr)) ==0)
						//match the Mac Address , Read IP Address form it.
						{
							DWORD	dwvalue;
							RegType = REG_DWORD;
							dwTemp = sizeof(DWORD);
							status = RegQueryValueEx(subkey,"LeaseObtainedTime",NULL,&RegType,
									(BYTE *)&dwvalue,&dwTemp);
							if(dwvalue > dwTime)
							{
								dwTime = dwvalue;
								BYTE	dwOptions[128];
								dwTemp = sizeof(DWORD);
								status = RegQueryValueEx(subkey,"DhcpIPAddress",NULL,&RegType,
										(BYTE *)&(nnp->IpAddress),&dwTemp);
								dwTemp = sizeof(DWORD);
								status = RegQueryValueEx(subkey,"DhcpSubnetMask",NULL,&RegType,
										(BYTE *)&(nnp->SubnetMask),&dwTemp);
								RegType = REG_BINARY;
								dwTemp = sizeof(dwOptions);
								status = RegQueryValueEx(subkey,"OptionInfo",NULL,&RegType,
											dwOptions,&dwTemp);
								if (status == ERROR_SUCCESS)
								{
									int ipoint = 0;
									while(dwOptions[ipoint]!= 0x03 && dwOptions[ipoint] != 0xFF)
									{
										ipoint++;
										ipoint += dwOptions[ipoint];
										ipoint++;
									}
									if(dwOptions[ipoint] == 0x03)
									{
										ipoint++;
										ipoint++;
										nnp->DefaultGateway = dwOptions[ipoint+3];
										nnp->DefaultGateway = ((nnp->DefaultGateway)<<8) + dwOptions[ipoint+2];
										nnp->DefaultGateway = ((nnp->DefaultGateway)<<8) + dwOptions[ipoint+1];
										nnp->DefaultGateway = ((nnp->DefaultGateway)<<8) + dwOptions[ipoint];
									}
								}
							}
						}
					}
					RegCloseKey(subkey);
				}
				dwTemp = sizeof(strTemp);
				dwIndex ++;
				status = RegEnumKeyEx(Mainkey, dwIndex, strTemp, &dwTemp, 
							NULL, NULL, NULL, &dwDummy);
			}
			RegCloseKey(Mainkey);
		}
	}
	
	return retcode;
}



int SetNicNetParam_95(char *nicname, SNicNetParam* nnp)
// return 0 if successful
// return -1 if failed
{
	char		nettrans[128];
    HKEY		subkey;
    DWORD       status;
	char		valdata[64];
	DWORD		valdatalen;

	int retcode = -1;
	
	if (GetNetTransFromNicName(nicname, nettrans, sizeof(nettrans)) != 0) return -1;

	char keypath[256];
	strcpy(keypath, "System\\CurrentControlSet\\Services\\Class\\NetTrans\\");
	strcat(keypath, nettrans);
	// cout<<nettrans<<endl;

	status = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			keypath,		// reg_path,
			0,
			KEY_WRITE,
			&subkey
			);
	if (status != ERROR_SUCCESS) return -1;

	retcode = 0;
	
	{
		if (nnp->DhcpEnabled) 
			cov_inet_addr_2str(0, valdata);
		else
			cov_inet_addr_2str(nnp->IpAddress, valdata);

		valdatalen = strlen(valdata) + 1;
		status = RegSetValueEx(subkey, "IPAddress", 
							NULL, REG_SZ,
							(BYTE*)valdata, valdatalen);
		if (status != ERROR_SUCCESS)  retcode = -1;
		
		if (nnp->DhcpEnabled) 
			cov_inet_addr_2str(0, valdata);
		else
			cov_inet_addr_2str(nnp->SubnetMask, valdata);

		valdatalen = strlen(valdata) + 1;
		status = RegSetValueEx(subkey, "IPMask", 
							NULL, REG_SZ,
							(BYTE*)valdata, valdatalen);
		if (status != ERROR_SUCCESS)  retcode = -1;

		if (nnp->DefaultGateway && nnp->DhcpEnabled == false)
		{
			cov_inet_addr_2str(nnp->DefaultGateway, valdata);
			valdatalen = strlen(valdata) + 1;
			status = RegSetValueEx(subkey, "DefaultGateway", 
								NULL, REG_SZ,
								(BYTE*)valdata, valdatalen);
			if (status != ERROR_SUCCESS)  retcode = -1;
		}
		else
		{
			// set Gateway to 0.0.0.0
			// delete it.
			RegDeleteValue(subkey, "DefaultGateway");
			// don't care if it succeeds.
		}

	}

	RegCloseKey(subkey);
	
	return retcode;
}

int GetNicNetParam_2000(char *nicname, SNicNetParam* nnp, etheraddr *mac)
{
	CNicIpHlpApi	iphlp;

	if(iphlp.ItExist() == false)
		return ERR;

	unsigned long	buflen;
	PIP_ADAPTER_INFO pAdapterInfo;
	PIP_ADDR_STRING	 pAddrStr;
	DWORD	dwret;
	int		iret;

	iret = ERR;
    buflen = 0;
	dwret = iphlp.IphlpGetAdaptersInfo(NULL, &buflen);
	if(dwret != ERROR_BUFFER_OVERFLOW)
		return iret;
	pAdapterInfo = 	(PIP_ADAPTER_INFO) new char[buflen];

	if(pAdapterInfo == NULL) return iret; 

	dwret = iphlp.IphlpGetAdaptersInfo(pAdapterInfo,&buflen);
	if(dwret == ERROR_SUCCESS)
	{
		PIP_ADAPTER_INFO	pNext;
		pNext = pAdapterInfo;
		while(pNext != NULL)
		{
			if(strcmp(pNext->AdapterName,nicname) == 0)
			{
				if(pNext->DhcpEnabled == 1)
					nnp->DhcpEnabled = true;
				else
					nnp->DhcpEnabled = false;
				pAddrStr = &(pNext->IpAddressList);
				nnp->IpAddress = cov_inet_addr(pAddrStr->IpAddress.String);
				nnp->SubnetMask = cov_inet_addr(pAddrStr->IpMask.String);
				pAddrStr = &(pNext->GatewayList);
				nnp->DefaultGateway = cov_inet_addr(pAddrStr->IpAddress.String);
				if(pNext->AddressLength == 6)
					*mac = pNext->Address;
				iret = OK;
				break;
			}
			pNext = pNext->Next;
		}
	}
	
	delete pAdapterInfo;

	return iret;
}

int GetNicNetParam_NT(char *nicname, SNicNetParam* nnp,etheraddr *mac)
// return 0 if successful
// return -1 if failed
{

    HKEY		subkey;
    DWORD       status;
	DWORD		valtype;
	char		valdata[128];

	int retcode = 0;
	
	memset(nnp, 0, sizeof(*nnp));

	char keypath[256];
	strcpy(keypath, "System\\CurrentControlSet\\Services\\");
	strcat(keypath, nicname);
	strcat(keypath, "\\Parameters\\Tcpip");

	// cout<<nettrans<<endl;

	status = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			keypath,		// reg_path,
			0,
			KEY_READ,
			&subkey
			);
	if (status != ERROR_SUCCESS) return -1;

	{
		
		DWORD valdatalen = sizeof(valdata);
		valdatalen = sizeof(valdata);
		status = RegQueryValueEx(subkey, "EnableDHCP", 
							NULL, &valtype,
							(BYTE*)valdata, &valdatalen);
		if (status == ERROR_SUCCESS) 
		{
			nnp->DhcpEnabled = (*((DWORD*)valdata) != 0);
		}

		//DhcpEnabled flag doestnot reflect the correct status. :-(
		//We use the ipaddress. if ipaddress = 0 and DhcpIPAddress is not empty. then Dhcp enabled.
		//We get the ip parameters from DHCPIPAddress , DhcpSubnetMask and DhcpDefaultGateway
		valdatalen = sizeof(valdata);
		status = RegQueryValueEx(subkey, "IPAddress", 
							NULL, &valtype,
							(BYTE*)valdata, &valdatalen);
		if (status == ERROR_SUCCESS) 
		{
			valdata[sizeof(valdata) - 1] = 0;
			nnp->IpAddress = cov_inet_addr(valdata);
		}

		if(nnp->IpAddress == 0)
		{
			valtype = REG_SZ;
			valdatalen = sizeof(valdata);
			status = RegQueryValueEx(subkey,"DhcpIPAddress",NULL,
							&valtype, (BYTE *)valdata, &valdatalen);
			if(status == ERROR_SUCCESS)
				nnp->IpAddress = cov_inet_addr(valdata);
			
			valdatalen = sizeof(valdata);
			status = RegQueryValueEx(subkey,"DhcpSubnetMask",NULL,
							&valtype, (BYTE *)valdata, &valdatalen);
			if(status == ERROR_SUCCESS)
				nnp->SubnetMask = cov_inet_addr(valdata);
			
			valdatalen = sizeof(valdata);
			valtype = REG_BINARY;
			status = RegQueryValueEx(subkey,"DhcpDefaultGateway",NULL,
							&valtype, (BYTE *)valdata, &valdatalen);
			if(status == ERROR_SUCCESS)
			{
				valdata[sizeof(valdata) - 1] = 0;
				nnp->DefaultGateway = cov_inet_addr(valdata);
			}
		}
		else
		{
			valdatalen = sizeof(valdata);
			status = RegQueryValueEx(subkey, "SubnetMask", 
								NULL, &valtype,
								(BYTE*)valdata, &valdatalen);
			if (status == ERROR_SUCCESS) 
			{
				valdata[sizeof(valdata) - 1] = 0;
				nnp->SubnetMask = cov_inet_addr(valdata);
			}

			valdatalen = sizeof(valdata);
			status = RegQueryValueEx(subkey, "DefaultGateway", 
								NULL, &valtype,
								(BYTE*)valdata, &valdatalen);
			if (status == ERROR_SUCCESS) 
			{
				valdata[sizeof(valdata) - 1] = 0;
				nnp->DefaultGateway = cov_inet_addr(valdata);
			}
		}

		retcode = 0;
	}


	RegCloseKey(subkey);
	
	return retcode;
}


int SetNicNetParam_NT(char *nicname, SNicNetParam* nnp)
// return 0 if successful
// return -1 if failed
{
    HKEY		subkey;
    DWORD       status;
	char		valdata[64];
	DWORD		valdatalen;

	int retcode = -1;
	
	char keypath[256];
	if(IsWinNT2000())
	{
		sprintf(keypath,"%s\\%s","SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interfaces",
			nicname);

	}
	else
	{
		strcpy(keypath, "System\\CurrentControlSet\\Services\\");
		strcat(keypath, nicname);
		strcat(keypath, "\\Parameters\\Tcpip");
	}

	status = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			keypath,		// reg_path,
			0,
			KEY_WRITE,
			&subkey
			);
	if (status != ERROR_SUCCESS) return -1;

	retcode = 0;
	
	{
		// write IpAddress

		memset(valdata, 0, sizeof(valdata));
/*	4/12/01 Waine: if I set nnp->IpAddress to 0, then the conversion will skipped.
//		if (nnp->IpAddress)
//			cov_inet_addr_2str(nnp->IpAddress, valdata);
*/
		cov_inet_addr_2str(nnp->IpAddress, valdata);
		valdatalen = strlen(valdata) + 2;
		status = RegSetValueEx(subkey, "IPAddress", 
							NULL, REG_MULTI_SZ ,
							(BYTE*)valdata, valdatalen);
		if (status != ERROR_SUCCESS)  retcode = -1;
		
		// write SubnetMask
		
		memset(valdata, 0, sizeof(valdata));
/* 4/12/01 Waine:
//		if (nnp->SubnetMask)
//			cov_inet_addr_2str(nnp->SubnetMask, valdata);
*/
		cov_inet_addr_2str(nnp->SubnetMask, valdata);

		valdatalen = strlen(valdata) + 2;
		
		status = RegSetValueEx(subkey, "SubnetMask", 
							NULL, REG_MULTI_SZ,
							(BYTE*)valdata, valdatalen);
		if (status != ERROR_SUCCESS)  retcode = -1;

		// write DefaultGateway
		
		memset(valdata, 0, sizeof(valdata));
/* 4/12/01 Waine:
//		if (nnp->DefaultGateway)
//			cov_inet_addr_2str(nnp->DefaultGateway, valdata);
*/
		cov_inet_addr_2str(nnp->DefaultGateway, valdata);

		valdatalen = strlen(valdata) + 2;
		
		status = RegSetValueEx(subkey, "DefaultGateway", 
								NULL, REG_MULTI_SZ,
								(BYTE*)valdata, valdatalen);
		if (status != ERROR_SUCCESS)  retcode = -1;

		// Write EnableDhcp

		DWORD ed = (nnp->DhcpEnabled != 0);
		valdatalen = sizeof(ed);
		status = RegSetValueEx(subkey, "EnableDHCP", 
								NULL, REG_DWORD,
								(BYTE*)&ed, valdatalen);

		if (status != ERROR_SUCCESS)  retcode = -1;
	}

	RegCloseKey(subkey);
	
	return retcode;
}


int GetNicNetParam(char *nicname, SNicNetParam* nnp,etheraddr *mac)
// return 0 if successful
{
	if (IsWinNT2000())
		return GetNicNetParam_2000(nicname, nnp, mac);
	else if (IsWinNT())
		return GetNicNetParam_NT(nicname, nnp, mac);
	else
		return GetNicNetParam_95(nicname, nnp, mac);
}


int SetNicNetParam(char *nicname, SNicNetParam* nnp)
// return 0 if successful
// return -1 if failed
{
	if (IsWinNT2000())
		return SetNicNetParam_NT(nicname, nnp);
	else if (IsWinNT())
		return SetNicNetParam_NT(nicname, nnp);
	else
		return SetNicNetParam_95(nicname, nnp);
}

int GetDHCPNicNetParam_NT(SNicNetParam* nnp, char *nicName)
// return 0 if successful
// return -1 if failed
{
	char subkey[128] = "SYSTEM\\CurrentControlSet\\Services\\";
	char val[128];
	dword outlen;
	int r;

	strcat(subkey, nicName);
	strcat(subkey, "\\Parameters\\Tcpip");

	r = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,
			subkey, 
			"DhcpIPAddress",
			val,
			sizeof(val),
			&outlen);

	if (r != OK) return -1; 

	nnp->IpAddress = cov_inet_addr(val);

	r = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,
			subkey, 
			"DhcpServer",
			val,
			sizeof(val),
			&outlen);

	if (r != OK) return -1;

	nnp->DefaultGateway = cov_inet_addr(val);

	r = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,
			subkey, 
			"DhcpSubnetMask",
			val,
			sizeof(val),
			&outlen);

	if (r != OK) return -1;

	nnp->SubnetMask = cov_inet_addr(val);

	return 0;
}

int GetDHCPNicNetParam_95(SNicNetParam* nnp, etheraddr *mac)
// return 0 if successful
// return -1 if failed
{
	int			retcode = -1;
	char		strDHCPKey[11];
	HKEY		subkey;
	DWORD       status;
	DWORD		valtype;
	byte		valdata[6];
	etheraddr	hwAddress;
	DWORD		dwIP;
	char		keypath[256];
	DWORD		valdatalen; 
	
	memset(nnp, 0, sizeof(*nnp));

	for (int i = 0; i < MAX_DHCPINFO; i++)
	{
		sprintf(strDHCPKey, "%s%.2d", "DhcpInfo", i);
		strcpy(keypath, "System\\CurrentControlSet\\Services\\VXD\\DHCP\\");
		strcat(keypath, strDHCPKey);
			
		status = RegOpenKeyEx(
				HKEY_LOCAL_MACHINE,
				keypath,		// reg_path,
				0,
				KEY_READ,
				&subkey
				);
		if (status != ERROR_SUCCESS) continue;

		valdatalen = sizeof(valdata);
		status = RegQueryValueEx(subkey, "HardwareAddress", 
							NULL, &valtype,
							(BYTE*)valdata, &valdatalen);
		if (status != ERROR_SUCCESS) 
		{
			RegCloseKey(subkey);
			continue;
		}

		hwAddress = valdata;
		if ((*mac) == hwAddress)	//got it
		{
			valdatalen = sizeof(dwIP);
			status = RegQueryValueEx(subkey, "DhcpIPAddress", 
								NULL, &valtype,
								(BYTE*)&dwIP, &valdatalen);
			if (status != ERROR_SUCCESS) 
			{
				RegCloseKey(subkey);
				break;
			}

			nnp->IpAddress = dwIP;

			valdatalen = sizeof(dwIP);
			status = RegQueryValueEx(subkey, "DhcpSubnetMask", 
								NULL, &valtype,
								(BYTE*)&dwIP, &valdatalen);
			if (status != ERROR_SUCCESS) 
			{
				RegCloseKey(subkey);
				break;
			}

			nnp->SubnetMask = dwIP;

			valdatalen = sizeof(dwIP);
			status = RegQueryValueEx(subkey, "DhcpServer", 
								NULL, &valtype,
								(BYTE*)&dwIP, &valdatalen);
			if (status != ERROR_SUCCESS) 
			{
				RegCloseKey(subkey);
				break;
			}

			nnp->DefaultGateway = dwIP;

			retcode = 0;
			nnp->DhcpEnabled = true;
			RegCloseKey(subkey);
			break;
		}
	}

	return	retcode;
}

int GetDHCPNicNetParam(char *nicName, SNicNetParam* nnp,etheraddr *mac)
// return 0 if successful
{
	if (IsWinNT2000())
		return GetDHCPNicNetParam_NT(nnp, nicName);
	else if (IsWinNT())
		return GetDHCPNicNetParam_NT(nnp, nicName);
	else
		return GetDHCPNicNetParam_95(nnp, mac);
}

bool VerifyNicDev_95(char *nicname,char *path)
// sometimes, some NicName under Net\XXXX have NOT really Device.
// the function return true if here is a really device for this nicname
// return false if no device
{
	char outbuf[300];
	unsigned buflen = sizeof(outbuf);

	if (GetNetDevUnderOne("Enum\\PCI", nicname, outbuf, buflen) == 0) 
	{
		if(path) strcpy(path, outbuf);
		return true;
	}

	if (GetNetDevUnderOne("Enum\\ISAPNP", nicname, outbuf, buflen) == 0) 
	{
		if(path) strcpy(path, outbuf);
		return true;
	}

	if (GetNetDevUnderOne("Enum\\PCMCIA", nicname, outbuf, buflen) == 0)
	{
		if(path) strcpy(path, outbuf);
		return true;
	}

	if (GetNetDevUnderOne("Enum\\ROOT", nicname, outbuf, buflen) == 0)
	{
		if(path) strcpy(path, outbuf);
		return true;
	}

	// WOW!! We still cannot find it. Try to search whole enum tree. God save me.
	if (GetNetDevUnderOne("Enum", nicname, outbuf, buflen) == 0)
	{
		if(path) strcpy(path, outbuf);
		return true;
	}

	return false;
}

bool GetNicDevPath(char * nicname, char* path)
{
	if (!IsWinNT()) return VerifyNicDev_95(nicname,path);
	return true;
}

bool VerifyNicDev(char *nicname)
{
	return GetNicDevPath(nicname, NULL);
}

int IsNetDevUnderDyn_95(char *reg_path)
// NOTICE: reg_path Does NOT end with '\';
// return 0 if succeeded
// return -1 if failed.
//this function get the dynamic info to reflect the existence of one network card
{
    HKEY		syskey, subkey;
    DWORD       status;
	FILETIME	dummy;

	DWORD		regtype;

	DWORD		index = 0;
	char		namebuf[256];
	DWORD		name_len;

	char		keypath[256];

	int			retcode = -1;
	bool		igotit = false;

	if (reg_path == NULL) return -1;
	if (reg_path[0] == 0) return -1;
	
	status = RegOpenKeyEx(
			HKEY_DYN_DATA,
			"Config Manager\\Enum",
			0,
			KEY_READ,
			&syskey
			);

	if (status != ERROR_SUCCESS) return -1;

	for (index = 0; 1 ; index++)
	{
		name_len = sizeof(namebuf);
		status = RegEnumKeyEx(syskey, index, (LPSTR) namebuf, &name_len, 
						NULL, NULL, NULL, &dummy);
		if (status != ERROR_SUCCESS) break;

		// we got a key, let's see 
		// ==========================================================
		strcpy(keypath, "Config Manager\\Enum"); strcat(keypath, "\\");
		strcat(keypath, namebuf);
		// cout<<"testing  "<<keypath<<endl;

		status = RegOpenKeyEx(
						HKEY_DYN_DATA,
						(char*) keypath, 0, KEY_READ, &subkey 
						);
		if (status != ERROR_SUCCESS) continue;	// if failed.

		igotit = false;
		do { // only one cycle
			char valbuf[256];
			DWORD valbuflen;

			// check "HardwareKey"
			valbuflen = sizeof(valbuf);
			status = RegQueryValueEx(subkey, "HardwareKey", 
									NULL, &regtype,
									(BYTE*)valbuf, &valbuflen);
			if (status != ERROR_SUCCESS) break;
			valbuf[sizeof(valbuf) - 1] = 0;
			char h[256];
			strcpy(h, "Enum\\");
			strcat(h, valbuf);
			if (stricmp(h, reg_path) == 0)
			// OK, we got one.
			// cout<<"We got the nicpath in dyn_data registry "<<keypath<<endl;
			igotit = true;

		} while (0);

		RegCloseKey(subkey);
		// ===========================================================

		if (igotit) 
		{
			retcode = 0; break;
		}
//		cout<<index<<"  "<<namebuf<<endl;

	}

	RegCloseKey(syskey);

	return retcode;
}

int IsNetDevUnderDyn(char *reg_path)
{
	if(!IsWinNT()) return IsNetDevUnderDyn_95(reg_path);
	return OK;
}


DWORD GetAllDnsServer_95(ipaddr *piplist, dword *plistnum)
{
	char str[256];
	char *pstr;
	dword outlen;
	dword buffernum;
	int r;

	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const  char *szSubKey, 
		"NameServer",		// const char* szValueName,
		str,				// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	buffernum = *plistnum;
	*plistnum = 0;

	if (r != OK) return OK;
	pstr = str;
	while(*pstr == ' ') pstr++;
	while(strlen(pstr) != 0)
	{
		if(*plistnum < buffernum)
			piplist[*plistnum] = pstr;
		(*plistnum)++;
		pstr = strstr(pstr, " ");
		if(pstr == NULL) break;
		while(*pstr == ' ')pstr++;
	}

	if( *plistnum >= buffernum) return ERR;

	return OK;
}


int SetDnsServer_95(ipaddr ip)
// return 0 if successful
// return 1 if successful and it will take effect after reboot.
// return -1 if failed.
{
//	ostrstream ostr;
//	ostr<<ip<<(char)0;

	int r1,r2;
	char str[256];

	cov_inet_addr_2str(ip.getraw(), str);

	r1 = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const char *szKey, 
		"NameServer",			// const char *szValueName,
		str
		);

	char one[] = "1";
	
	r2 = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const char *szKey, 
		"EnableDNS",			// const char *szValueName,
		one
		);

	if (r1 != OK || r2 != OK) return -1;

	// 
	int r;
	DWORD outlen = sizeof(str);
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const  char *szSubKey, 
		"HostName",			// const char* szValueName,
		str,				// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	// Hostname exists, return.
	if (r == OK) return 1;

	
	// we need to write hostname.
	char buf[MAX_COMPUTERNAME_LENGTH + 1];	
	outlen = sizeof(buf);

	if (GetComputerName(buf, &outlen) == 0) return -1;

//	cout<<"hostname = "<<buf<<endl;

	buf[sizeof(buf) - 1] = 0;

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",
		"HostName",			// const char *szValueName,
		buf
		);

	if (r != OK) return -1;


	return 1;
}

DWORD GetAllDnsServer_2000(ipaddr *piplist, dword *plistnum)
{
	CNicIpHlpApi iphlp;
	dword	buffernum = 0;
	dword	dwret;
	

	buffernum = *plistnum;
	*plistnum = 0;
	if(iphlp.ItExist())
	{ 
		PFIXED_INFO	pInfo;
		DWORD	buflen;

		buflen = 0;
		dwret = iphlp.IphlpGetNetworkParams(NULL,&buflen);
		if(dwret != ERROR_BUFFER_OVERFLOW)	return OK;

		pInfo = (PFIXED_INFO)new char[buflen];
		dwret = iphlp.IphlpGetNetworkParams(pInfo,&buflen);

		if(dwret != ERROR_SUCCESS)
		{
			delete pInfo;
			return OK;
		}

		PIP_ADDR_STRING	pdns;
		pdns = &pInfo->DnsServerList;
		while(pdns != NULL)
		{
			if(*plistnum < buffernum)
			{
				piplist[*plistnum] = pdns->IpAddress.String;
			}
			(*plistnum)++;
			pdns = pdns->Next;
		}
		delete pInfo;
		if(*plistnum >= buffernum) 	return ERR;
	}
	
	return OK;
}

DWORD GetAllDnsServer_NT(ipaddr *piplist, dword *plistnum)
{
	char str[256];
	char *pstr;
	dword outlen,bknum;
	dword buffernum;
	int r;
	bknum = *plistnum;
	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
		"NameServer",		// const char* szValueName,
		str,				// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	buffernum = *plistnum;
	*plistnum = 0;

	if (r != OK) return OK;
	pstr = str;
	while(*pstr == ' ') pstr++;
	while(strlen(pstr) != 0)
	{
		if(*plistnum < buffernum)
			piplist[*plistnum] = pstr;
		(*plistnum)++;
		pstr = strstr(pstr, " ");
		if(pstr == NULL) break;
		while(*pstr == ' ')pstr++;
	}

	r = ReadBinaryFromRegistry(HKEY_LOCAL_MACHINE, "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Transient"
		,"NameServer", str, sizeof(str), &outlen);
	if(r == OK)
	{
		pstr = str;
		while(*pstr == ' ') pstr++;
		while(strlen(pstr)!= 0)
		{
			if(*plistnum < buffernum)
				piplist[*plistnum] = pstr;
			(*plistnum) ++;
			pstr = strstr(pstr, " ");
			if(pstr == NULL)break;
			while(*pstr == ' ') pstr++;
		}
	}
	if( *plistnum > buffernum) return ERR;
	if (*plistnum==0)
	{
		outlen = 0;
		r = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
			"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
			"DhcpNameServer",		// const char* szValueName,
			str,				// void *p, 
			sizeof(str),		// const unsigned buflen, 
			&outlen				// DWORD* outlen
			);

		buffernum = bknum;
		*plistnum = 0;

		if (r != OK) return OK;
		pstr = str;
		while(*pstr == ' ') pstr++;
		while(strlen(pstr) != 0)
		{
			if(*plistnum < buffernum)
				piplist[*plistnum] = pstr;
			(*plistnum)++;
			pstr = strstr(pstr, " ");
			if(pstr == NULL) break;
			while(*pstr == ' ')pstr++;
		}

	}

	if( *plistnum > bknum) return ERR;


	return OK;
}

int SetDnsServer_2000(ipaddr ip, char * nicname)
// return 0 if successful
// return 1 if successful and it will take effect after reboot.
// return -1 if failed.
{
	if(nicname == NULL) return -1;

	SNic	niclist[16];
	int	nicnum;
	char	Ipstr[64];

	sprintf(Ipstr,"%u.%u.%u.%u",ip.byVal[0],ip.byVal[1],ip.byVal[2],ip.byVal[3]);
	nicnum = GetEtherNicList(niclist,sizeof(niclist));
	
	for(int i = 0; i<nicnum; i++)
	{
		int r;
		DWORD	outlen;
		char RegKey[256];
		char DnsString[256];

		if(strcmp(niclist[i].NicName, nicname) != 0)
			continue;

		sprintf(RegKey,"%s\\%s","SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interfaces",
			niclist[i].NicName);
		memset(DnsString,0,sizeof(DnsString));
		r = ReadStringFromRegistry(HKEY_LOCAL_MACHINE,
					RegKey, "NameServer",DnsString,sizeof(DnsString),&outlen);
		if(r != OK)//There is not this key. 
			break;
		if(strstr(DnsString, Ipstr) != NULL)
			return 0; //This Dns exists. return success.
		else
		{
			char	strTemp[300];
			memset(strTemp, 0, sizeof(strTemp));
			if(ip.dwVal != 0)
			{
				if(strlen(DnsString) == 0)
					strcpy(strTemp,Ipstr);
				else if(strlen(DnsString) + strlen(Ipstr) + 1 < sizeof(strTemp))
					sprintf(strTemp,"%s,%s",DnsString, Ipstr);
				else
					break;
			}
			if(WriteStringToRegistry(HKEY_LOCAL_MACHINE, RegKey, "NameServer", strTemp) == OK)
				return 1;
		}
	}

	return -1;
}

int SetDnsServer_NT(ipaddr ip)
// return 0 if successful
// return 1 if successful and it will take effect after reboot.
// return -1 if failed.
{
	char ipstr[IP_ADDRESS_STRING_MAX_LENGTH + 1];

	memset(ipstr, 0, sizeof(ipstr));
	if(ip.dwVal != 0)
		ip.getstr(ipstr, sizeof(ipstr));
	
	int r;
	
	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
		"NameServer",			// const char *szValueName,
		ipstr
		);

	if (r != OK) return -1;

	// 
	char str[256];
	DWORD outlen = sizeof(str);
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
		"HostName",			// const char* szValueName,
		str,				// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	// Hostname exists, return.
	if (r == OK) return 1;

	char buf[MAX_COMPUTERNAME_LENGTH + 1];
	outlen = sizeof(buf);

	if (GetComputerName(buf, &outlen) == 0) return -1;

//	cout<<"hostname = "<<buf<<endl;

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
		"Hostname",			// const char *szValueName,
		buf
		);

	if (r != OK) return -1;

	return 1;
}

int SetDnsServer(ipaddr ip, char * nicname)
{
	if (IsWinNT2000())
		return SetDnsServer_2000(ip, nicname);
	else if (IsWinNT())
		return SetDnsServer_NT(ip);
	else
		return SetDnsServer_95(ip);
}


ipaddr GetDnsServer(char * nicname)
{
	ipaddr	iptemp;
	dword	dwtemp;

	dwtemp = 1;
	iptemp.dwVal = 0;
	if (IsWinNT2000() || IsWin98() || IsWinME())
	{
		GetAllDnsServer_2000(&iptemp, &dwtemp);
		return iptemp;
	}
	else if(IsWinNT())
	{
		GetAllDnsServer_NT(&iptemp, &dwtemp);
		return iptemp;
	}
	else
	{
		GetAllDnsServer_95(&iptemp, &dwtemp);
		return iptemp;
	}
}

int GetAllDnsServer(ipaddr * piplist, dword * plistnum)
{
	if(IsWinNT2000() || IsWin98() || IsWinME())
		return GetAllDnsServer_2000(piplist, plistnum);
	else if(IsWinNT())
		return GetAllDnsServer_NT(piplist, plistnum);
	else
		return GetAllDnsServer_95(piplist, plistnum);


}

bool IsFixedIp_NT(char *nicname)
{
    HKEY		subkey;
    DWORD       status;
	DWORD		valtype;
	char		valdata[128];
	int			retcode = 0;
	char		keypath[256];

	strcpy(keypath, "System\\CurrentControlSet\\Services\\");
	strcat(keypath, nicname);
	strcat(keypath, "\\Parameters\\Tcpip");

	// cout<<nettrans<<endl;

	status = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			keypath,		// reg_path,
			0,
			KEY_READ,
			&subkey
			);
	if (status == ERROR_SUCCESS)
	{
		DWORD valdatalen = sizeof(valdata);
		status = RegQueryValueEx(subkey, "IPAddress", 
							NULL, &valtype,
							(BYTE*)valdata, &valdatalen);
		RegCloseKey(subkey);

		if (status == ERROR_SUCCESS) 
		{
			valdata[sizeof(valdata) - 1] = 0;
			if (cov_inet_addr(valdata) != 0)
				return true;
		}
	}

	return false;
}

bool IsFixedIp_Win95(char *nicname)
{
	char 		nettrans[128];
    HKEY		subkey;
    DWORD       status;
	DWORD		valtype;
	char		valdata[64];
	
	if (GetNetTransFromNicName(nicname, nettrans, sizeof(nettrans)) != 0) 
		return true;

	char keypath[256];
	strcpy(keypath, "System\\CurrentControlSet\\Services\\Class\\NetTrans\\");
	strcat(keypath, nettrans);
	// cout<<nettrans<<endl;

	status = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			keypath,		// reg_path,
			0,
			KEY_READ,
			&subkey
			);
	if (status == ERROR_SUCCESS)
	{
		DWORD valdatalen = sizeof(valdata);
		status = RegQueryValueEx(subkey, "IPAddress", 
							NULL, &valtype,
							(BYTE*)valdata, &valdatalen);
		RegCloseKey(subkey);

		if (status == ERROR_SUCCESS) 
		{
			valdata[sizeof(valdata) - 1] = 0;

			if (cov_inet_addr(valdata) != 0)
				return true; // fixed Ip is used
		}
	}
	
	return false;
}

bool IsFixedIp_Win2000(char *nicname)
{
	CNicIpHlpApi	iphlp;

	if(iphlp.ItExist() == false)
		return true; // ok, lets assume it is fixed Ip

	unsigned long	buflen;
	PIP_ADAPTER_INFO pAdapterInfo;
	DWORD	dwret;

    buflen = 0;
	dwret = iphlp.IphlpGetAdaptersInfo(NULL, &buflen);
	if(dwret != ERROR_BUFFER_OVERFLOW)
		return true;

	bool	iret = false;
	pAdapterInfo = (PIP_ADAPTER_INFO) new char[buflen];

	if(pAdapterInfo == NULL) return true; 

	dwret = iphlp.IphlpGetAdaptersInfo(pAdapterInfo,&buflen);
	if(dwret == ERROR_SUCCESS)
	{
		PIP_ADAPTER_INFO	pNext;
		pNext = pAdapterInfo;
		while(pNext != NULL)
		{
			if(strcmp(pNext->AdapterName,nicname) == 0)
			{
				if(!pNext->DhcpEnabled)
					iret = true;
				break;
			}
			pNext = pNext->Next;
		}
	}
	
	delete pAdapterInfo;

	return iret;
}

bool IsFixedIp(char *nicname)
{
	if (IsWinNT2000())
		return IsFixedIp_Win2000(nicname);
	else if (IsWinNT()) 
		return IsFixedIp_NT(nicname);
	else
		return IsFixedIp_Win95(nicname);
}

DWORD GetNicMTU_2000(char * nicname)
{
	return 0;
}

DWORD GetNicMTU_95(char * nicname)
{
	char		nettrans[128];
    DWORD       ret = 0;
	
	if (GetNetTransFromNicName(nicname, nettrans, sizeof(nettrans)) != 0) return 0;

	char keypath[256];
	strcpy(keypath, "System\\CurrentControlSet\\Services\\Class\\NetTrans\\");
	strcat(keypath, nettrans);

	char	MTUValue[128];
	DWORD	outlen;
	if(ReadStringFromRegistry(HKEY_LOCAL_MACHINE, keypath, "MaxMTU" , MTUValue, sizeof(MTUValue), &outlen) == OK)
	{
		sscanf(MTUValue, "%ld", &ret);
	}

	if(ret == 0)
	{
		sprintf(keypath, "%s%s","System\\CurrentControlSet\\Services\\Class\\Net\\", nicname );
		if(ReadStringFromRegistry(HKEY_LOCAL_MACHINE, keypath, "IPMTU", MTUValue, sizeof(MTUValue), &outlen) == OK)
		{
			sscanf(MTUValue, "%ld", &ret);
		}
	}

	return ret;
}

DWORD GetNicMTU_NT(char * nicname)
{
    DWORD       ret;

	char keypath[256];
	strcpy(keypath, "System\\CurrentControlSet\\Services\\");
	strcat(keypath, nicname);
	strcat(keypath, "\\Parameters\\Tcpip");

	if(ReadDwordFromRegistry(HKEY_LOCAL_MACHINE, keypath, "MTU", & ret) != OK)
		ret = 0;

	return ret;
}

DWORD GetNicMTU(char * nicname)
{
	if(IsWinNT2000())
		return GetNicMTU_2000(nicname);
	else if(IsWinNT())
		return GetNicMTU_NT(nicname);
	else
		return GetNicMTU_95(nicname);
}
#define REGKEY_RECWINDOW_NT "System\\CurrentControlSet\\Services\\Tcpip\\Parameters"
#define REGKEY_NICRECWINDOW_2000 "System\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interface"
#define NEWREGVALUE_RECWINDOW_2000 "GlobalMaxTcpWindowSize"
#define REGVALUE_RECWINDOW_NT "TcpWindowSize"

DWORD GetNicRecWindow_2000(char * nicname)
{
	DWORD ret;
	if(ReadDwordFromRegistry(HKEY_LOCAL_MACHINE, REGKEY_RECWINDOW_NT, NEWREGVALUE_RECWINDOW_2000, &ret )!= OK)
	{
		if(ReadDwordFromRegistry(HKEY_LOCAL_MACHINE, REGKEY_RECWINDOW_NT, REGVALUE_RECWINDOW_NT, &ret) != OK)
			ret = 0;
	}

	if(nicname != NULL)
	{
		char regvalue[256];
		sprintf(regvalue, "%s\\%s", REGKEY_NICRECWINDOW_2000, nicname);
		ReadDwordFromRegistry(HKEY_LOCAL_MACHINE, regvalue, REGVALUE_RECWINDOW_NT, &ret);
	}
	return ret;
}

DWORD GetNicRecWindow_NT(void)
{
	DWORD ret;
	if(ReadDwordFromRegistry(HKEY_LOCAL_MACHINE, REGKEY_RECWINDOW_NT, REGVALUE_RECWINDOW_NT, &ret) != OK)
		ret = 0;

	return ret;
}

#define REGKEY_RECWINDOW_95 "System\\CurrentControlSet\\Services\\VxD\\MSTCP"
#define REGVALUE_RECWINDOW_95 "DefaultRcvWindow"
DWORD GetNicRecWindow_95(void)
{
	DWORD ret;
	DWORD outlen;
	char keystring[256];

	if(ReadStringFromRegistry(HKEY_LOCAL_MACHINE, REGKEY_RECWINDOW_95, REGVALUE_RECWINDOW_95, keystring, sizeof(keystring), &outlen) != OK)
		ret = 0;
	else
		sscanf(keystring, "%d", &ret);

	return ret;
}
DWORD GetNicRecieveWindow(char * nicname)
{
	if(IsWinNT2000())
		return GetNicRecWindow_2000(nicname);
	else if(IsWinNT())
		return GetNicRecWindow_NT();
	else
		return GetNicRecWindow_95();
}
int	GetDnsData_95(SDNSData	*lpDNSData)
//get SDNSData from register
//return 0 if successful
//return -1 if failed
{
	//Get Domain, if failed -1
	char	str[DOMAIN_LEN];
	DWORD outlen;
	int	r;

	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const  char *szSubKey, 
		"Domain",		// const char* szValueName,
		str,			// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if (r != OK)	return -1;

	memcpy(lpDNSData->strDomain, str, outlen);

	//Get HostName, if failed return -1
	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const  char *szSubKey, 
		"HostName",		// const char* szValueName,
		str,			// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if ((r != OK)||(outlen > HOSTNAME_LEN))
		return	-1;

	memcpy(lpDNSData->strHostName, str, outlen);

	//get NameServer, if failed return -1
	ipaddr	ipdns;
	char	strdns[MAX_DNS][IP_ADDRESS_STRING_MAX_LENGTH + 1];
	char	*lpstart, *lpend;
	int		i, idnslen;

	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const  char *szSubKey, 
		"NameServer",		// const char* szValueName,
		str,				// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if (r != OK)
		return	-1;

	lpstart = str;
	memset(strdns, 0, (MAX_DNS*IP_ADDRESS_STRING_MAX_LENGTH + 1));

	for (i = 0; i < MAX_DNS; i++)
	{
		lpend = strchr(lpstart, ',');
		if (lpend != NULL)
		{
			idnslen = lpend - lpstart;
			memcpy(strdns[i], lpstart, idnslen); 
			lpstart = lpend + 1;
			ipdns = strdns[i];
			lpDNSData->DNSServer[i] = todword(ipdns);
		}
		else
		{
			idnslen = outlen - (lpstart - str);
			memcpy(strdns[i], lpstart, idnslen); 
			ipdns = strdns[i];
			lpDNSData->DNSServer[i] = todword(ipdns);
			break;
		}
	}

	//get EnableDNS, if failed return -1
	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const  char *szSubKey, 
		"EnableDNS",		// const char* szValueName,
		str,				// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if (r != OK)
		return	-1;

	if (str[0] == '0')
		lpDNSData->bDNSEnabled = false;
	else
		lpDNSData->bDNSEnabled = true;

	//get SearchList, if failed return -1;
	char	strDomainSuffix[MAX_DOMAINSUFFIX*DOMAINSUFFIX_LEN];
	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const  char *szSubKey, 
		"SearchList",		// const char* szValueName,
		strDomainSuffix,	// void *p, 
		sizeof(strDomainSuffix),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if (r != OK)
		return	-1;

	lpstart = strDomainSuffix;

	for (i = 0; i < MAX_DOMAINSUFFIX; i++)
	{
		lpend = strchr(lpstart, ',');
		if (lpend != NULL)
		{
			memcpy(lpDNSData->strSearchList[i], lpstart, lpend - lpstart); 
			lpstart = lpend + 1;
		}
		else
		{
			memcpy(lpDNSData->strSearchList[i], lpstart, outlen - (lpstart - strDomainSuffix)); 
			break;
		}
	}

	return 0;
}

int GetDNSDomain_NT(char * domainname, DWORD bufsize)
//return OK if successful
//return ERR if failed
{
	int ret = ERR;
	if(domainname != NULL)
	{
		DWORD outlen = 0;
		ret = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
			"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
			"Domain",		// const char* szValueName,
			domainname,			// void *p, 
			bufsize,		// const unsigned buflen, 
			&outlen				// DWORD* outlen
			);

	}
	return ret;
}

int GetDNSDomain_2000(char * domainname, DWORD bufsize)
//We don't support this function right now.
//return OK if successful
//return ERR if failed
{
	return ERR;
}

int GetHostName_NT(char * hostname, DWORD bufsize)
//return OK if successful
//return ERR if failed
{
	int ret = ERR;
	if(hostname != NULL)
	{
		DWORD outlen = 0;
		ret = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
			"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
			"Hostname",		// const char* szValueName,
			hostname,				// void *p, 
			bufsize,		// const unsigned buflen, 
			&outlen				// DWORD* outlen
			);

	}
	return ret;
}

int GetHostName_2000(char * hostname, DWORD bufsize)
//return OK if successful
//return ERR if failed
{
	int ret = ERR;
	if(hostname != NULL)
	{
		DWORD outlen = bufsize;
		if(GetComputerName(hostname, &bufsize) == 0)
			return ret;
		if (outlen > bufsize) 
			hostname[bufsize] = 0;
		else 
			hostname[outlen - 1] = 0;
		ret = OK;
	}
	return ret;
}

int GetDomainSuffix_NT(char * suffix, DWORD bufsize)
//return OK if successful
//return ERR if failed
{
	int ret = ERR;
	if(suffix != NULL)
	{
		DWORD outlen;
		ret = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
			"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
			"SearchList",		// const char* szValueName,
			suffix,	// void *p, 
			bufsize,		// const unsigned buflen, 
			&outlen				// DWORD* outlen
			);
	}

	return ret;
}

int GetDomainSuffix_2000(char * suffix, DWORD bufsize)
//We don't support this function right now.
//return OK if successful
//return ERR if failed
{
	return ERR;
}

ipaddr GetDnsServer_2000(char * nicname)
//this function just return the first DNS in the list
{
	CNicIpHlpApi iphlp;
	ipaddr ip;
	
	ip.zero();
	if(nicname == NULL)
	{
		if(iphlp.ItExist())
		{ 
			PFIXED_INFO	pInfo;
			DWORD	buflen;
			DWORD	dwret;

			dwret = iphlp.IphlpGetNetworkParams(NULL,&buflen);
			if(dwret == ERROR_BUFFER_OVERFLOW)
			{
				pInfo = (PFIXED_INFO)new char[buflen];
				dwret = iphlp.IphlpGetNetworkParams(pInfo,&buflen);
				if(dwret == ERROR_SUCCESS)
					ip = pInfo->DnsServerList.IpAddress.String;
				delete pInfo;
			}
		}
	}
	else
	{
		DWORD outlen;
		
		char RegKey[256];
		char DnsString[256];

		sprintf(RegKey,"%s\\%s","SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interfaces",
			nicname);
		memset(DnsString,0,sizeof(DnsString));
		if(ReadStringFromRegistry(HKEY_LOCAL_MACHINE, RegKey, "NameServer",DnsString,sizeof(DnsString),&outlen) == OK)
		{
			DnsString[sizeof(DnsString) -1] = 0;
			ip = DnsString;
		}
	}
	return ip;
}

int GetDnsData_2000(SDNSData *lpDNSData)
//this function can only get one DNS. after merge with NatServer sndfc. A DNS list could be getted.
//get current DNS setting from register
//return 0 if successful
//return -1 if failed
{
	if(GetDNSDomain_2000(lpDNSData->strDomain, sizeof(lpDNSData->strDomain)) != OK)
	{
		#pragma message("Function GetDnsData_2000 wasn't implemented fully at " __FILE__)
		//right now we ignore this return value.
		// return -1;
	}
	

	if(GetHostName_2000(lpDNSData->strHostName, sizeof(lpDNSData->strHostName)) != OK)
	{
		#pragma message("Function GetDnsData_2000 wasn't implemented fully at " __FILE__)
		//right now we ignore this return value.
		//return -1;
	}

	ipaddr	ipdns;

	if(strlen(lpDNSData->strNicName) == 0)
		ipdns = GetDnsServer_2000(NULL);
	else
		ipdns = GetDnsServer_2000(lpDNSData->strNicName);

	if(ipdns.dwVal == 0)
		lpDNSData->bDNSEnabled = false;
	else
		lpDNSData->bDNSEnabled = true;

	lpDNSData->DNSServer[0] = todword(ipdns);

	char	strDomainSuffix[MAX_DOMAINSUFFIX*DOMAINSUFFIX_LEN];
	if(GetDomainSuffix_2000(strDomainSuffix, sizeof(strDomainSuffix)) != OK)
	{
		#pragma message("Function GetDnsData_2000 wasn't implemented fully at " __FILE__)
		//right now we ignore this return value.
		//return -1;
	}
/*	char * lpstart;
	char * lpend;

	lpstart = strDomainSuffix;
	for (int i = 0; i < MAX_DOMAINSUFFIX; i++)
	{
		lpend = strchr(lpstart, ' ');
		if (lpend != NULL)
		{
			memcpy(lpDNSData->strSearchList[i], lpstart, lpend - lpstart); 
			lpstart = lpend + 1;
		}
		else
		{
			memcpy(lpDNSData->strSearchList[i], lpstart, strlen(strDomainSuffix) - (lpstart - strDomainSuffix)); 
			break;
		}
	}*/

	return 0;
}

int	GetDnsData_NT(SDNSData	*lpDNSData)
//get current DNS setting from register
//return 0 if successful
//return -1 if failed
{
	DWORD outlen;

	int r;
	
	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
		"Domain",		// const char* szValueName,
		lpDNSData->strDomain,				// void *p, 
		sizeof(lpDNSData->strDomain),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if (r != OK) return -1;

	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
		"Hostname",		// const char* szValueName,
		lpDNSData->strHostName,				// void *p, 
		sizeof(lpDNSData->strHostName),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if (r != OK) return -1;

	char	str[MAX_DNS*(IP_ADDRESS_STRING_MAX_LENGTH + 1)];
	char	strdns[MAX_DNS][IP_ADDRESS_STRING_MAX_LENGTH + 1];
	char	*lpstart, *lpend;
	ipaddr	ipdns;
	int		idnslen;

	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
		"NameServer",		// const char* szValueName,
		str,				// void *p, 
		sizeof(str),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if (r != OK) return -1;

	lpstart = str;
	memset(strdns, 0, (MAX_DNS*IP_ADDRESS_STRING_MAX_LENGTH + 1));

	for (int i = 0; i < MAX_DNS; i++)
	{
		lpend = strchr(lpstart, ' ');
		if (lpend != NULL)
		{
			idnslen = lpend - lpstart;
			memcpy(strdns[i], lpstart, idnslen); 
			lpstart = lpend + 1;
			ipdns = strdns[i];
			lpDNSData->DNSServer[i] = todword(ipdns);
		}
		else
		{
			idnslen = outlen - (lpstart - str);
			memcpy(strdns[i], lpstart, idnslen); 
			ipdns = strdns[i];
			lpDNSData->DNSServer[i] = todword(ipdns);
			break;
		}
	}

	char	strDomainSuffix[MAX_DOMAINSUFFIX*DOMAINSUFFIX_LEN];
	outlen = 0;
	r = ReadBinaryFromRegistry(
		HKEY_LOCAL_MACHINE,		//  HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",	// const  char *szSubKey, 
		"SearchList",		// const char* szValueName,
		strDomainSuffix,	// void *p, 
		sizeof(strDomainSuffix),		// const unsigned buflen, 
		&outlen				// DWORD* outlen
		);

	if (r != OK)
		return	-1;

	lpstart = strDomainSuffix;

	for (i = 0; i < MAX_DOMAINSUFFIX; i++)
	{
		lpend = strchr(lpstart, ' ');
		if (lpend != NULL)
		{
			memcpy(lpDNSData->strSearchList[i], lpstart, lpend - lpstart); 
			lpstart = lpend + 1;
		}
		else
		{
			memcpy(lpDNSData->strSearchList[i], lpstart, outlen - (lpstart - strDomainSuffix)); 
			break;
		}
	}

	return 0;
}

int	SetDnsData_95(SDNSData	*lpDNSData)
//save SDNSData structure to register
//return 0 if successful
//return -1 if failed
{
	//Set Domain, if failed -1
	int	r;
	DWORD outlen;

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",
		"Domain",			// const char *szValueName,
		lpDNSData->strDomain
		);

	if (r != OK)	return -1;


	//Set HostName, if failed return -1
	char buf[HOSTNAME_LEN];

	if (lpDNSData->strHostName[0] != 0)
		memcpy(buf, lpDNSData->strHostName, HOSTNAME_LEN);
	else
	{
		outlen = sizeof(buf);
		if (GetComputerName(buf, &outlen) == 0) return -1;
		buf[sizeof(buf) - 1] = 0;
	}

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",
		"HostName",			// const char *szValueName,
		buf
		);


	if (r != OK)
		return	-1;

	//Set NameServer, if failed return -1
	char	dnsbuf[MAX_DNS * (IP_ADDRESS_STRING_MAX_LENGTH + 1)];
	char	strdns[IP_ADDRESS_STRING_MAX_LENGTH + 1];
	ipaddr	ipdns;

	char	*lppos = dnsbuf; 
	memset(dnsbuf, 0, sizeof(dnsbuf));
	for (int i = 0; i < MAX_DNS; i++)
	{
		if (lpDNSData->DNSServer[i] == 0)
			break;
		ipdns = toipaddr(lpDNSData->DNSServer[i]);
		ipdns.getstr(strdns, sizeof(strdns));
		memcpy(lppos, strdns, strlen(strdns));
		lppos += strlen(strdns);
		*lppos = ',';
		lppos++;
	}

	*(--lppos) = 0;

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const char *szKey, 
		"NameServer",			// const char *szValueName,
		dnsbuf
		);

	if (r != OK)
		return	-1;

	//Set EnableDNS, if failed return -1
	char	one[] = "1";
	if (lpDNSData->DNSServer[0] == 0)
		one[0] = '0';

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const char *szKey, 
		"EnableDNS",			// const char *szValueName,
		one
		);

	if (r != OK)
		return	-1;

	//Set SearchList, if failed return -1
	char	strDomainSuffix[MAX_DOMAINSUFFIX*DOMAINSUFFIX_LEN];

	lppos = strDomainSuffix;
	memset(strDomainSuffix, 0, sizeof(strDomainSuffix));
	for (i = 0; i < MAX_DOMAINSUFFIX; i++)
	{
		if (strlen(lpDNSData->strSearchList[i]) == 0)
			break;
		memcpy(lppos, lpDNSData->strSearchList[i], strlen(lpDNSData->strSearchList[i]));
		lppos += strlen(lpDNSData->strSearchList[i]);
		*lppos = ',';
		lppos++;
	}
	*(--lppos) = 0;

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"System\\CurrentControlSet\\Services\\VxD\\MSTCP",	// const char *szKey, 
		"SearchList",			// const char *szValueName,
		strDomainSuffix
		);

	if (r != OK)
		return -1;

	return 0;
}

int SetDNSDomain_NT(char * domainname)
//return OK if successful
//return ERR if failed
{
	int ret = ERR;
	if(domainname != NULL)
	{
		ret =  WriteStringToRegistry(
				HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
				"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
				"Domain",			// const char *szValueName,
				domainname
				);
	}
	return ret;
}

int SetDNSDomain_2000(char *domainname)
//We don't support this funciton right now.
//return OK if successful
//return ERR if failed
{
	return ERR;
}

int SetHostName_NT(char * hostname)
//return OK if successful
//return ERR if failed
{
	int ret = ERR;
	char buf[HOSTNAME_LEN];
	DWORD outlen;
	
	memset(buf, 0, sizeof(buf));
	outlen = sizeof(buf);
	if(hostname == NULL )
	{
		if(GetComputerName(buf, &outlen) == 0)
			return ret;
		buf[sizeof(buf) -1] = 0;
	}

	ret =  WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
		"Hostname",			// const char *szValueName,
		buf
		);
	return ret;
}

int SetHostName_2000(char * domainname)
//We don't support this funciton right now.
//return OK if successful
//return ERR if failed
{
	return ERR;
}

int SetDomainSuffix_NT(char * suffix)
//return OK if successful
//return ERR if failed
{
	int ret = ERR;
	if(suffix != NULL)
	{

		char	strDomainSuffix[MAX_DOMAINSUFFIX*DOMAINSUFFIX_LEN];
		memset(strDomainSuffix, 0, sizeof(strDomainSuffix));
		
		if(GetDomainSuffix_NT(strDomainSuffix, sizeof(strDomainSuffix)) == OK)
		{
			if(strstr(suffix, strDomainSuffix) != NULL)
			{
				if(strlen(strDomainSuffix) + strlen(suffix) + 2 < sizeof(strDomainSuffix))
				{
					strcat(strDomainSuffix, " ");
					strcat(strDomainSuffix, suffix);
					ret = WriteStringToRegistry(
						HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
						"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
						"SearchList",			// const char *szValueName,
						strDomainSuffix);
				}
			}
			else
				ret = OK;
		
		};
	}

	return ret;
}

int SetDomainSuffix_2000(char * suffix)
//We don't support this funciton right now.
//return OK if successful
//return ERR if failed
{
	return ERR;
}

int SetDnsData_2000(SDNSData * lpDNSData)
//save SDNSData structure to register
//return 0 if successful
//return -1 if failed
{
	if(strlen(lpDNSData->strNicName) == 0) return -1;

	//Set Domain, if failed -1
	if(SetDNSDomain_2000(lpDNSData->strDomain) != OK)
	{
		#pragma message("Function SetDnsData_2000 wasn't implemented fully at " __FILE__)
		//We ignore this return value.
		//return -1;
	}
	//Set HostName, if failed return -1
	if(strlen(lpDNSData->strHostName) == 0)
	{
		DWORD outlen = sizeof(lpDNSData->strHostName);
		if(GetComputerName(lpDNSData->strHostName, &outlen) == 0)
			return -1;
	}
	if(SetHostName_2000(lpDNSData->strHostName) != OK)
	{
		#pragma message("Function GetDnsData_2000 wasn't implemented fully at " __FILE__)
		//We ignore this return value.
		//return -1;
	}

	//Set NameServer, if failed return -1
	
	ipaddr	ipdns;
	
	ipdns.zero();
	if(SetDnsServer_2000(ipdns, lpDNSData->strNicName) == -1) //empty dns setting;
		return -1;

	if(lpDNSData->bDNSEnabled)
	{
		for (int i = 0; i < MAX_DNS; i++)
		{
			if (lpDNSData->DNSServer[i] == 0)
				break;
			ipdns = toipaddr(lpDNSData->DNSServer[i]);
			if(SetDnsServer_2000(ipdns, lpDNSData->strNicName) == -1)
				return -1;
		}
	}

	//Set SearchList, if failed return -1
	for (int i = 0; i < MAX_DOMAINSUFFIX; i++)
	{
		if (strlen(lpDNSData->strSearchList[i]) == 0 && i != 0)
			break;
		if(SetDomainSuffix_2000(lpDNSData->strSearchList[i]) != OK)
		{
			#pragma message("Function GetDnsData_2000 wasn't implemented fully at " __FILE__)
			//We ignore this return value.
			//return -1;
		}
	}

	return 0;
}

int	SetDnsData_NT(SDNSData	*lpDNSData)
//save SDNSData structure to register
//return 0 if successful
//return -1 if failed
{
	//Set Domain, if failed -1
	int	r;
	DWORD outlen;

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
		"Domain",			// const char *szValueName,
		lpDNSData->strDomain
		);

	if (r != OK)	return -1;


	//Set HostName, if failed return -1
	char buf[HOSTNAME_LEN];

	if (lpDNSData->strHostName[0] != 0)
		memcpy(buf, lpDNSData->strHostName, HOSTNAME_LEN);
	else
	{
		outlen = sizeof(buf);
		if (GetComputerName(buf, &outlen) == 0) return -1;
		buf[sizeof(buf) - 1] = 0;
	}

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
		"Hostname",			// const char *szValueName,
		buf
		);


	if (r != OK)
		return	-1;

	//Set NameServer, if failed return -1
	char	dnsbuf[MAX_DNS * (IP_ADDRESS_STRING_MAX_LENGTH + 1)];
	char	strdns[IP_ADDRESS_STRING_MAX_LENGTH + 1];
	ipaddr	ipdns;

	char	*lppos = dnsbuf; 
	memset(dnsbuf, 0, sizeof(dnsbuf));
	for (int i = 0; i < MAX_DNS; i++)
	{
		if (lpDNSData->DNSServer[i] == 0)
			break;
		ipdns = toipaddr(lpDNSData->DNSServer[i]);
		ipdns.getstr(strdns, sizeof(strdns));
		memcpy(lppos, strdns, strlen(strdns));
		lppos += strlen(strdns);
		*lppos = ' ';
		lppos++;
	}

	*(--lppos) = 0;

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
		"NameServer",			// const char *szValueName,
		dnsbuf
		);

	if (r != OK)
		return	-1;

	//Set SearchList, if failed return -1
	char	strDomainSuffix[MAX_DOMAINSUFFIX*DOMAINSUFFIX_LEN];

	lppos = strDomainSuffix;
	memset(strDomainSuffix, 0, sizeof(strDomainSuffix));
	for (i = 0; i < MAX_DOMAINSUFFIX; i++)
	{
		if (strlen(lpDNSData->strSearchList[i]) == 0)
			break;
		memcpy(lppos, lpDNSData->strSearchList[i], strlen(lpDNSData->strSearchList[i]));
		lppos += strlen(lpDNSData->strSearchList[i]);
		*lppos = ' ';
		lppos++;
	}
	*(--lppos) = 0;

	r = WriteStringToRegistry(
		HKEY_LOCAL_MACHINE,		// HKEY hRootKey, 
		"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
		"SearchList",			// const char *szValueName,
		strDomainSuffix
		);

	if (r != OK)
		return -1;

	return 0;
}

int SetDnsData(SDNSData	*lpDNSData)
{
	//we should merge SetDnsData_2000 and SetDnsData_NT in future. They use too many same codes.
	if (IsWinNT2000())
		return SetDnsData_2000(lpDNSData);
	else if (IsWinNT())
		return SetDnsData_NT(lpDNSData);
	else
		return SetDnsData_95(lpDNSData);
}


int GetDnsData(SDNSData	*lpDNSData)
{
	if(lpDNSData == NULL)
		return -1;
//	memset(lpDNSData, 0, sizeof(SDNSData));
	if (IsWinNT2000())
		return GetDnsData_2000(lpDNSData);
	else if (IsWinNT()) 
		return GetDnsData_NT(lpDNSData);
	else
		return GetDnsData_95(lpDNSData);
}


static DWORD GetNicIpAddr_2000(char *nicname,etheraddr *mac)
{
	CNicIpHlpApi	iphlp;

	if(iphlp.ItExist() == false)
		return ERR;

	unsigned long	buflen;
	PIP_ADAPTER_INFO pAdapterInfo;
	PIP_ADDR_STRING	 pAddrStr;
	DWORD	dwret;

    buflen = 0;
	dwret = iphlp.IphlpGetAdaptersInfo(NULL, &buflen);
	if(dwret != ERROR_BUFFER_OVERFLOW)
		return 0;
	pAdapterInfo = 	(PIP_ADAPTER_INFO) new char[buflen];

	if(pAdapterInfo == NULL) return 0; 

	dwret = iphlp.IphlpGetAdaptersInfo(pAdapterInfo,&buflen);
	if(dwret == ERROR_SUCCESS)
	{
		PIP_ADAPTER_INFO	pNext;
		pNext = pAdapterInfo;
		while(pNext != NULL)
		{
			if(strcmp(pNext->AdapterName,nicname) == 0)
			{
				pAddrStr = &(pNext->IpAddressList);
				dwret = cov_inet_addr(pAddrStr->IpAddress.String);
				if(pNext->AddressLength == 6)
					*mac = pNext->Address;
				break;
			}
			pNext = pNext->Next;
		}
	}
	else
		dwret = 0;
	
	delete pAdapterInfo;

	return dwret;
}

static DWORD GetNicIpAddr_NT(char *nicname,etheraddr *mac)
{
	char subkey[128] = "SYSTEM\\CurrentControlSet\\Services\\";
	char val[128];
	dword outlen;
	dword ip;
	int r;

	strcat(subkey, nicname);
	strcat(subkey, "\\Parameters\\Tcpip");

	r = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,
			subkey, 
			"DhcpIPAddress",
			val,
			sizeof(val),
			&outlen);

	if (r == OK) 
	{
		ip = cov_inet_addr(val);
		if (ip) return ip;
	}

	r = ReadBinaryFromRegistry(
			HKEY_LOCAL_MACHINE,
			subkey, 
			"IPAddress",
			val,
			sizeof(val),
			&outlen);

	if (r != OK) return 0;
	return cov_inet_addr(val);
}


static DWORD GetNicIpAddr_95(char *nicname,etheraddr *mac)
{
	// serr<<"GetNicIpAddr_95 isn't supported"<<endl;
	SNicNetParam nnp;
	if (GetNicNetParam(nicname, &nnp,mac) == 0)
	{
		// if (nnp.DhcpEnabled) return 0;
		return nnp.IpAddress;
	}

	return 0;
}


DWORD GetNicIpAddr(char *nicname,etheraddr *mac)
// it support 95 and NT
{
	if (IsWinNT2000())
		return GetNicIpAddr_2000(nicname,mac);
	else if(IsWinNT())
		return GetNicIpAddr_NT(nicname,mac);
	else
		return GetNicIpAddr_95(nicname,mac);
}



// End of NicIP.cpp
