// Tpmio.cpp : Implements the TPM device functions.
//

#include "stdafx.h"
#include <stdio.h>
#include "luimpersonator.h"
#include "tpmio.h"

#define	DEFAULT_SYLINK_REG_PATH	"SOFTWARE\\Sygate Technologies, Inc.\\Sygate Personal Firewall\\SYLINK"

#define MS_CRYPTOB	"Microsoft Base Cryptographic Provider v1.0"
#define MS_CRYPTOS	"Microsoft Strong Cryptographic Provider"
#define IBM_ESSCSP	"IBM Embedded Security Subsystem Enhanced CSP"
#define HPQ_IFNCSP	"Infineon TPM Cryptographic Provider"

#define TPM_SUPPORT_ENUMPROV	0
#define NUM_OF_TPMCSP			2
#define ENCRYPT_ALGORITHM		CALG_RC4

typedef struct TPMPC_ST
{
	int  tpmvid;
	char csp[128];
} TPMPCST;

TPMPCST gTpmPc[NUM_OF_TPMCSP] = {
	{1, IBM_ESSCSP},
	{2, HPQ_IFNCSP}
};

HWND hMainWnd = NULL;
HCRYPTPROV ghProv = NULL;
HCRYPTKEY ghKeySess = NULL;
HCRYPTKEY ghKeySign = NULL;
HCRYPTKEY ghKeyXchg = NULL;
DWORD gTpmVendorId = 0;
char szCmpuName[256] = "";
char szUserName[256] = "";
char szApplName[256] = "Sygate SSA";
char szStr[256];

///////////////////////////////////////////////
//
// Function declaration
//
int TpmEnumDevices();
int TpmGetDeviceInfo(HCRYPTPROV hProv);
int TpmOpenKey(HCRYPTPROV hProv, ALG_ID algid, HCRYPTKEY *hKey);
int TpmNewSSKey(HCRYPTPROV hProv, ALG_ID algid, HCRYPTKEY *hKey);
int TpmNewPPKey(HCRYPTPROV hProv, ALG_ID algid, HCRYPTKEY *hKey);
int TpmCloseKey(HCRYPTKEY *hKey);

///////////////////////////////////////////////
//
// Output string to the debug window
//
int OutString(char *msg)
{
	char str[256] = "TPMIO: ";
	if (msg == NULL)
		return -1;

	strcat(str, msg);
	//if (hMainWnd) {
	//	::SendMessage(hMainWnd, WM_COMMAND, WM_USER, (LPARAM)str);
	//}
	sysprintln(str);
	return 0;
}

///////////////////////////////////////////////
//
// TpmIoControl Interface function
//
TPMWINAPI  
BOOL TpmIoControl(DWORD IoCtlCode,
	DWORD *inBuf, DWORD inBufLen,
	DWORD *outBuf, DWORD *outBufLen)
{
	DWORD errCode = 0;
	BOOL bRet = FALSE;

	switch (IoCtlCode) {
	case TPM_OPENDEVICE: 
		{			
			char cspProv[256];
			BOOL bOkay; 
			DWORD cbSize;
			cbSize = 256;
			
			OutString("TPM_OPENDEVICE");
			GetComputerName(szCmpuName, &cbSize);
			sprintf(szStr, "Computer Name = %s\n", szCmpuName);
			OutString(szStr);
			GetUserName(szUserName, &cbSize);
			sprintf(szStr, "User Name = %s\n", szUserName);
			OutString(szStr);
			if (ghProv == NULL) {
				// enumerate CryptoAPI service providers
				//TpmEnumDevices();
			}
			else {
				// opened already
				break;
			}
			// obtain a handle to the CSP provider
			strcpy(cspProv, MS_CRYPTOS);
			//strcpy(cspProv, IBM_ESSCSP);
			//strcpy(cspProv, HP_INFNCSP);			
			//bOkay = CryptAcquireContext(&ghProv, 
			//	NULL, cspProv, PROV_RSA_FULL, 0);
			//bOkay = CryptAcquireContext(&ghProv, 
			//	NULL, cspProv, PROV_RSA_FULL, CRYPT_MACHINE_KEYSET);

			for (int i=0; i<NUM_OF_TPMCSP; i++) {
				bOkay = CryptAcquireContext(&ghProv, 
					szApplName, gTpmPc[i].csp, 
					PROV_RSA_FULL, 0);
				if (bOkay) {
					bRet = TRUE;
					break;
				}
				else {
					errCode = GetLastError();
					sprintf(szStr, "GetLastError() = 0x%x", errCode);
					OutString(szStr);				
					// generate a new key container if not existed
					if ((NTE_KEYSET_NOT_DEF == errCode) ||
						(NTE_BAD_KEYSET == errCode)) {
						bOkay = CryptAcquireContext(&ghProv, 
							szApplName, gTpmPc[i].csp, 
							PROV_RSA_FULL, CRYPT_NEWKEYSET);
						if (bOkay) {
							bRet = TRUE;
							break;
						}
						else {
							errCode = GetLastError();
							sprintf(szStr, "GetLastError() = 0x%x", errCode);
							OutString(szStr);
						}
					}
				}
			}

			// generate a new session key
			if (bRet == TRUE) {
				gTpmVendorId = gTpmPc[i].tpmvid;
				TpmNewSSKey(ghProv, ENCRYPT_ALGORITHM, &ghKeySess);
			}
		}
		break;

	case TPM_CLOSEDEVICE:
		{
			OutString("TPM_CLOSEDEVICE");
			// release the handle to the provider
			if (ghProv != NULL) {
				// close the session key
				if (ghKeySess != NULL) {
					TpmCloseKey(&ghKeySess);
				}
				// release the CSP provider context
				BOOL bOkay = CryptReleaseContext(ghProv, 0);
				if (FALSE == bOkay) {
					errCode = GetLastError();
					break;
				}
				ghProv = NULL;
			}
			bRet = TRUE;
		}
		break;

	case TPM_DEVICEINFO:
		{
			OutString("TPM_DEVICEINFO");
			// show info of the CSP provider
			TpmGetDeviceInfo(ghProv);
			bRet = TRUE;
		}
		break;

	case TPM_NEWSSSKEY:
		{
			OutString("TPM_NEWSSSKEY");
			// generate new session key
			if (TpmNewSSKey(ghProv, ENCRYPT_ALGORITHM, &ghKeySess) == 0) {
				bRet = TRUE;
			}
		}
		break;
	
	case TPM_NEWPKIKEY:
		{
			OutString("TPM_NEWPKIKEY");
			// generate new AT_SIGNATURE key
			//if (TpmNewPPKey(ghProv, AT_SIGNATURE, &ghKeySign) == 0) {
			//	bRet = TRUE;
			//}
			// generate new AT_KEYEXCHANGE key
			if (TpmNewPPKey(ghProv, AT_KEYEXCHANGE, &ghKeyXchg) == 0) {
				bRet = TRUE;
			}
		}
		break;
	
	case TPM_OPENPKIKEY:
		{
			OutString("TPM_OPENPKIKEY");
			// open AT_SIGNATURE key
			//if (TpmOpenKey(ghProv, AT_SIGNATURE, &ghKeySign) == 0) {
			//	bRet = TRUE;
			//}
			// open AT_KEYEXCHANGE key
			if (TpmOpenKey(ghProv, AT_KEYEXCHANGE, &ghKeyXchg) == 0) {
				bRet = TRUE;
			}
		}
		break;
	
	case TPM_CLOSEKEY:
		{
			OutString("TPM_CLOSEKEY");
			// close session key
			if (ghKeySess != NULL) {
				TpmCloseKey(&ghKeySess);
				bRet = TRUE;
			}
			// close AT_SIGNATURE key
			if (ghKeySign != NULL) {
				TpmCloseKey(&ghKeySign);
				bRet = TRUE;
			}
			// close AT_KEYEXCHANGE key
			if (ghKeyXchg != NULL) {
				TpmCloseKey(&ghKeyXchg);
				bRet = TRUE;
			}
		}
		break;
	
	case TPM_RESETDEVICE:
		{
			HCRYPTPROV thProv = NULL;
			BOOL bOkay; 

			OutString("TPM_RESETDEVICE");
			// release the CSP provider context if any
			if (ghProv != NULL) {
				// close the session key
				if (ghKeySess != NULL) {
					TpmCloseKey(&ghKeySess);
				}
				bOkay = CryptReleaseContext(ghProv, 0);
				if (FALSE == bOkay) {
					errCode = GetLastError();
					sprintf(szStr, "GetLastError() = 0x%x", errCode);
					OutString(szStr);
					break;
				}
				ghProv = NULL;
			}
			
			// obtain a handle to the CSP provider
			for (int i=0; i<NUM_OF_TPMCSP; i++) {
				bOkay = CryptAcquireContext(&thProv, 
					szApplName, gTpmPc[i].csp, 
					PROV_RSA_FULL, 0);
				if (bOkay) {
					// release the CSP provider context
					CryptReleaseContext(thProv, 0);
					thProv = NULL;
					// delete the key set
					bOkay = CryptAcquireContext(&thProv, 
						szApplName, gTpmPc[i].csp, 
						PROV_RSA_FULL, CRYPT_DELETEKEYSET);
					if (bOkay) {
						bRet = TRUE;
					}
					else {
						errCode = GetLastError();
						sprintf(szStr, "GetLastError() = 0x%x", errCode);
						OutString(szStr);
					}
					break;
				}
				else {
					errCode = GetLastError();
				}
			}

		}
		break;
	
	case TPM_ENCRYPT:
		break;
	
	case TPM_DECRYPT:
		break;

	case TPM_ENUMDEVICE:
		{
			OutString("TPM_ENUMDEVICE");
			TpmEnumDevices();
		}
		break;

	case TPM_DEBUGWND:
		{
			hMainWnd = (HWND)*inBuf;
		}
		break;
	}

	return bRet;
}

///////////////////////////////////////////////
//
// TpmEnumDevice Interface function
//
int TpmEnumDevices(void)
{
	int nTpmDev = 0;
#if TPM_SUPPORT_ENUMPROV
	LPTSTR pszName;
	DWORD dwIndex;
	DWORD dwType;
	DWORD cbName;
	DWORD errCode = 0;

	//--------------------------------------------------------------
	//   Print header lines for provider types.
	// Loop through enumerating provider types.
	dwIndex = 0;
	while(CryptEnumProviderTypes(dwIndex, 
		   NULL, 0, &dwType, NULL, &cbName)) {
		//--------------------------------------------------------------------
		//  cbName is the length of the name of the next provider type.
		//  Allocate memory in a buffer to retrieve that name.

		if (!(pszName = (LPTSTR)LocalAlloc(LMEM_ZEROINIT, cbName+1)))	{
		   return -1;
		}
		
		//--------------------------------------------------------------------
		//  Get the provider type name.
		if (CryptEnumProviderTypes(dwIndex,
			   NULL, NULL, &dwType, pszName, &cbName)) {
			sprintf(szStr, "     %4.0d        %s\n",dwType, pszName);
			OutString(szStr);
		}
		else {
			errCode = GetLastError();
		}
		LocalFree(pszName);

		// next provider
		dwIndex++;
	}

	//---------------------------------------------------------------- 
	// Loop through enumerating providers.
	dwIndex = 0;
	while(CryptEnumProviders(dwIndex, 
		NULL, 0, &dwType, NULL, &cbName)) {
		//--------------------------------------------------------------------
		//  cbName is the length of the name of the next provider.
		//  Allocate memory in a buffer to retrieve that name.

		if (!(pszName = (LPTSTR)LocalAlloc(LMEM_ZEROINIT, cbName+1))) {
		   return -1;
		}

		//--------------------------------------------------------------------
		//  Get the provider name.
		if (CryptEnumProviders(dwIndex,
			   NULL, 0, &dwType, pszName, &cbName)) {
			sprintf (szStr, "     %4.0d        %s\n",dwType, pszName);
			OutString(szStr);
		}
		else {
			errCode = GetLastError();
		}
		LocalFree(pszName);

		// next provider
		dwIndex++;

	} // End of while loop
#endif
	return nTpmDev;
}

///////////////////////////////////////////////
//
// TpmGetDeviceInfo Interface function
//
int TpmGetDeviceInfo(HCRYPTPROV hProv)
{
	BYTE pbData[1024];
	DWORD cbData;
	DWORD errCode = 0;

	// read CSP provider name
	cbData = 1000;
	if(CryptGetProvParam(hProv, 
		PP_NAME, pbData, &cbData, 0)) {
		OutString("CryptGetProvParam succeeded. \n");
		sprintf(szStr, "CSP provider name: %s", (char *)pbData);
		OutString(szStr);
	}
	else {
		OutString("Error reading CSP provider name. \n");
	}

	// enumerate CSP algorithms 
	OutString("List all algorithms: \n");
	PROV_ENUMALGS_EX *pAlg;
	cbData = 1000;
	if (CryptGetProvParam(hProv, 
		PP_ENUMALGS_EX, pbData, &cbData, CRYPT_FIRST)) {
		pAlg = (PROV_ENUMALGS_EX *)pbData;
		sprintf(szStr, "Algorithm name: %s", (char *)pAlg->szLongName);
		OutString(szStr);
		while (CryptGetProvParam(hProv, 
			PP_ENUMALGS_EX, pbData, &cbData, 0)) {
			pAlg = (PROV_ENUMALGS_EX *)pbData;
			sprintf(szStr, "Algorithm name: %s", (char *)pAlg->szLongName);
			OutString(szStr);
		}
		errCode = GetLastError();
		if (errCode != ERROR_NO_MORE_ITEMS) {
			sprintf(szStr, "GetLastError() = 0x%x", errCode);
			OutString(szStr);
		}
	}
	else {
		errCode = GetLastError();
		sprintf(szStr, "GetLastError() = 0x%x", errCode);
		OutString(szStr);
	}

	// read CSP default key container name
	cbData = 1000;
	if(CryptGetProvParam(hProv, 
		PP_CONTAINER, pbData, &cbData, 0)) {
		OutString("CryptGetProvParam succeeded. \n");
		sprintf(szStr, "Default Key Container name: %s", (char *)pbData);
		OutString(szStr);
	}
	else {
		OutString("Error reading key container name. \n");
	}

	// enumerate key container names
	OutString("List all key containers: \n");
	cbData = 1000;
	if (CryptGetProvParam(hProv, 
		PP_ENUMCONTAINERS, pbData, &cbData, CRYPT_FIRST)) {
		sprintf(szStr, "Key Container name: %s", (char *)pbData);
		OutString(szStr);
		while (CryptGetProvParam(hProv, 
			PP_ENUMCONTAINERS, pbData, &cbData, 0)) {
			sprintf(szStr, "Key Container name: %s", (char *)pbData);
			OutString(szStr);
		}
		errCode = GetLastError();
		if (errCode != ERROR_NO_MORE_ITEMS) {
			sprintf(szStr, "GetLastError() = 0x%x", errCode);
			OutString(szStr);
		}
	}
	else {
		errCode = GetLastError();
		sprintf(szStr, "GetLastError() = 0x%x", errCode);
		OutString(szStr);
	}

	return 0;
}

///////////////////////////////////////////////
//
// TpmOpenKey Interface function
//
int TpmOpenKey(HCRYPTPROV hProv, ALG_ID algid, HCRYPTKEY *hKey)
{
	DWORD errCode = 0;

	// get signature key
	if (CryptGetUserKey(hProv, algid, hKey)) {
		OutString("The key has been acquired. \n");
	}
	else {
		errCode = GetLastError();
		sprintf(szStr, "GetLastError() = 0x%x", errCode);
		OutString(szStr);
		OutString("Error during CryptGetUserKey.\n");
		return -1;
	}

	return 0;
}

///////////////////////////////////////////////
//
// TpmNewSSKey Interface function
//
int TpmNewSSKey(HCRYPTPROV hProv, ALG_ID algid, HCRYPTKEY *hKey)
{
	char szClearText[256] = "SS1S-M5MM-XXX9-W2WW-I3II-YY9Y-DDD7-ZZ8Z-QQQ9-1AAA-UUU6-G0GG-PPP4";
	HCRYPTHASH hHash;
	DWORD szLength;
	DWORD errCode = 0;

	// create a hash function
	if (CryptCreateHash(hProv, CALG_MD5, 0, 0, &hHash) == FALSE) {
		errCode = GetLastError();
		sprintf(szStr, "GetLastError() = 0x%x", errCode);
		OutString(szStr);
		OutString("Error during CryptCreateHash.\n");
		return -1;
	}
	
	// hash the password string
	szLength = strlen(szClearText);
	for (DWORD i=0; i<szLength; i++) {
		szClearText[i] = (0xFF - szClearText[i]);
	}
	if (CryptHashData(hHash, (BYTE *)szClearText, szLength, 0) == FALSE) {
		errCode = GetLastError();
		sprintf(szStr, "GetLastError() = 0x%x", errCode);
		OutString(szStr);
		OutString("Error during CryptHashData.\n");
		CryptDestroyHash(hHash);
		return -1;
	}

	// create a new session key based on the hash of the password
	if(CryptDeriveKey(hProv, algid, hHash, 
		CRYPT_EXPORTABLE, hKey) == FALSE) {
		errCode = GetLastError();
		sprintf(szStr, "GetLastError() = 0x%x", errCode);
		OutString(szStr);
		OutString("Error during CryptDeriveKey!");
		CryptDestroyHash(hHash);
		return -1;
	}

	// Destroy the hash object.
	CryptDestroyHash(hHash);

	return 0;
}

///////////////////////////////////////////////
//
// TpmNewPPKey Interface function
//
int TpmNewPPKey(HCRYPTPROV hProv, ALG_ID algid, HCRYPTKEY *hKey)
{
	DWORD errCode = 0;

	// get signature key
	if (CryptGetUserKey(hProv, algid, hKey)) {
		OutString("The key has been acquired. \n");
	}
	else {
		errCode = GetLastError();
		if (errCode == NTE_NO_KEY) {
			if (CryptGenKey(hProv, algid, 0, hKey)) {
				OutString("The new key has been generated. \n");
			}
			else {
				sprintf(szStr, "GetLastError() = 0x%x", errCode);
				OutString(szStr);
				OutString("Error during CryptGenKey.\n");
				return -1;
			}
		}
		else {
			sprintf(szStr, "GetLastError() = 0x%x", errCode);
			OutString(szStr);
			OutString("Error during CryptGetUserKey.\n");
			return -1;
		}
	}

	return 0;
}

///////////////////////////////////////////////
//
// TpmCloseKey Interface function
//
int TpmCloseKey(HCRYPTKEY *hKey)
{
	CryptDestroyKey(*hKey);
	OutString("Destroyed key.\n");
	*hKey = NULL;
	return 0;
}

///////////////////////////////////////////////
//
// TpmImpersonateLoggedOnUser helper function
//
/*
BOOL TpmImpersonateLoggedOnUser(BOOL bSet)
{
	extern BOOL FindExplorerEXEPid(PDWORD pdwProcessId);
    static HANDLE hToken = NULL;
    static HANDLE hProcess;
    static DWORD dwPidExplorer;

	if (bSet) {
		if (IsWinNT()) {
			if (FindExplorerEXEPid(&dwPidExplorer)) {
				hProcess = OpenProcess(PROCESS_QUERY_INFORMATION|PROCESS_VM_READ, FALSE, dwPidExplorer);
				if (hProcess != NULL) {
					if (OpenProcessToken(hProcess, 
						TOKEN_QUERY|TOKEN_DUPLICATE, &hToken)) {
						ImpersonateLoggedOnUser(hToken);
						OutString("begin impersonate loggedon user");
					}
				}
			}
		}
	}
	else {
		RevertToSelf();  
		if (hToken) {
			CloseHandle(hToken);
			hToken = NULL;
			OutString("end impersonate loggedon user");
		}
	}
	return TRUE;
}
*/
///////////////////////////////////////////////
//
// IsTpmDevice Interface function
//
BOOL InitTpmDevice(void)
{
	DWORD OutBufLen;
	BOOL bOK, bRet;

	OutString("InitTpmDevice() called");

    // begin impersonate Logged On User
	//TpmImpersonateLoggedOnUser(TRUE);
	CLoggedonUserImpersonator lui;
	int retc = lui.ImpersonateLogonUser();
	if (retc != OK) {
		sysprintln("TPMIO: ImpersonateLogonUser() = " + retc);
	}
    
	// initialize TPM device
	bRet = FALSE;
	OutBufLen = 0;
	bOK = TpmIoControl(TPM_OPENDEVICE, NULL, 0, NULL, &OutBufLen);
	if (bOK) {
		bOK = TpmIoControl(TPM_NEWPKIKEY, NULL, 0, NULL, &OutBufLen);
		if (bOK) {
			bRet = TRUE;
		}
		TpmIoControl(TPM_CLOSEKEY, NULL, 0, NULL, &OutBufLen);
		TpmIoControl(TPM_CLOSEDEVICE, NULL, 0, NULL, &OutBufLen);
	}

    // end impersonate Logged On User
	//TpmImpersonateLoggedOnUser(FALSE);
	lui.RevertToMyself();


	OutString("InitTpmDevice() returned");
	return bRet;
}

///////////////////////////////////////////////
//
// CheckTpmDevice Interface function
//
BOOL CheckTpmDevice(int *TpmVendorId)
{
	DWORD OutBufLen;
	BOOL bOK, bRet;

	OutString("CheckTpmDevice() called");

    // begin impersonate Logged On User
	//TpmImpersonateLoggedOnUser(TRUE);
	CLoggedonUserImpersonator lui;
	int retc = lui.ImpersonateLogonUser();
	if (retc != OK) {
		sysprintln("TPMIO: ImpersonateLogonUser() = " + retc);
	}


	// check TPM device info
	bRet = FALSE;
	OutBufLen = 0;
	bOK = TpmIoControl(TPM_OPENDEVICE, NULL, 0, NULL, &OutBufLen);
	if (bOK) {
		bOK = TpmIoControl(TPM_OPENPKIKEY, NULL, 0, NULL, &OutBufLen);
		if (bOK) {
			*TpmVendorId = gTpmVendorId;
			bRet = TRUE;
		}
		TpmIoControl(TPM_CLOSEKEY, NULL, 0, NULL, &OutBufLen);
		TpmIoControl(TPM_CLOSEDEVICE, NULL, 0, NULL, &OutBufLen);
	}

    // end impersonate Logged On User
	//TpmImpersonateLoggedOnUser(FALSE);
	lui.RevertToMyself();

	OutString("CheckTpmDevice() returned");
	return bRet;
}

///////////////////////////////////////////////
//
// GetTpmVendorId Interface function
//
int GetTpmVendorId(void)
{
	HKEY h_key;
	DWORD tpmVendorId = 0;
	DWORD dwType = REG_DWORD;
	DWORD dwSize = sizeof(DWORD);

	// get the ID from the SSA registry key
	if (ERROR_SUCCESS == RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
		DEFAULT_SYLINK_REG_PATH"\\SyLink", 0, KEY_ALL_ACCESS, &h_key)) {
		if (ERROR_SUCCESS == RegQueryValueEx(h_key, 
			"TpmVendorID", NULL, &dwType, 
			(LPBYTE)&tpmVendorId, &dwSize)) {
			// verify the TPM Vendor ID value
			if ((tpmVendorId >= 0) && (tpmVendorId <= NUM_OF_TPMCSP))
				gTpmVendorId = tpmVendorId;
		}
		RegCloseKey(h_key);	
	}

	return gTpmVendorId;
}

