// ArpPkt: implementation of the CArpPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

08/12/1998			Created by Yadong Zhang

--*/


#include "stdafx.h"
#include "ArpPkt.h"

ostream& operator<<(ostream& os, CArpPacket &ep)
{
	os	<<"===== ARP Packet==== len:"<<dec<<ep.GetPacketBinary().GetLen()<<"====\n"
		<<ep.GetEthiiPkt().GetSourAddr()<<" --> "
		<<ep.GetEthiiPkt().GetDestAddr()
		<<"\nSender: "<<ep.GetSenderEtherAddr()<<"  "<<ep.GetSenderIpAddr()
		<<"\nTarget: "<<ep.GetTargetEtherAddr()<<"  "<<ep.GetTargetIpAddr()
		;

	return os;
}

outputstr operator+(const outputstr& ops, CArpPacket &ap)
{
	ostrstream outbuf;
	outbuf<<ap<<(char)0;
	outputstr out = ops + outbuf.str();

	// unfreeze it, otherwise strstream won't release the memory 
	// and memory leak will occur.
	outbuf.freeze(false);
	return out;
}

unsigned MakeArpQuery(byte *outbuf, unsigned buflen, etheraddr myeth, ipaddr myip, ipaddr targetip)
// outbuf points to a ethernet packet buffer.
// return the length of ARP packet (includes ethernet head)
// return 0 if failed.
{
	if (buflen < sizeof(SEtherii_FrameHead) + sizeof(SArp_FrameHead)) return 0;
	
	SEtherii_FrameHead* eth = (SEtherii_FrameHead*)outbuf;
	SArp_FrameHead*	arp = (SArp_FrameHead*)(outbuf + sizeof(SEtherii_FrameHead));
	
	byte ea[6]; memset(ea, 0xFF, sizeof(ea));
	eth->DestAddr = ea;
	eth->SourAddr = myeth;
	byte *pt=((byte *)eth)+12;
	pt[0]=0x08;
	pt[1]=0x06;
//	eth->Type = tonword(PROTO_ETHER_TYPE_ARP);;

	
	memset(ea, 0, sizeof(ea));
	
	arp->HardwareType = tonword(ARP_HARDWARE_TYPE_ETH);
	arp->ProtocolType = tonword(ARP_PROTO_TYPE_IP);
	arp->HardwareAddrLen = 6;
	arp->ProtocolAddrLen = 4;
	arp->Operation = tonword(ARP_OPERATION_REQUEST);
	arp->SenderHardwareAddr = myeth;
	arp->SenderIpAddr = myip;
	arp->TargetHardwareAddr = ea;
	arp->TargetIpAddr = targetip;
	
	return sizeof(SEtherii_FrameHead) + sizeof(SArp_FrameHead);
}


int MakeArpRespondence(CArpPacket* arp, etheraddr host_ethaddr, ipaddr host_ip)
{
	// we just care Ethernet and IP protocol.
	if (arp->GetFramePtr()->HardwareType != tonword(ARP_HARDWARE_TYPE_ETH))
		return ERR_UNKNOWN;
	
	if (arp->GetFramePtr()->ProtocolType != tonword(ARP_PROTO_TYPE_IP))
		return ERR_UNKNOWN;
	
	if (arp->GetFramePtr()->Operation != tonword(ARP_OPERATION_REQUEST))
		return ERR_UNKNOWN;
	
	// the MAC of host sent query.
	etheraddr client_mac = arp->GetFramePtr()->SenderHardwareAddr;
	ipaddr client_ip = arp->GetFramePtr()->SenderIpAddr;
	
	arp->GetEthiiPkt().GetFramePtr()->DestAddr = arp->GetEthiiPkt().GetFramePtr()->SourAddr;
	arp->GetEthiiPkt().GetFramePtr()->SourAddr = host_ethaddr;
	
	arp->GetFramePtr()->Operation = tonword(ARP_OPERATION_RESPONSE);
	
	arp->GetFramePtr()->SenderHardwareAddr = host_ethaddr;
	arp->GetFramePtr()->SenderIpAddr = host_ip;
	
	arp->GetFramePtr()->TargetHardwareAddr = client_mac;
	arp->GetFramePtr()->TargetIpAddr = client_ip;
	
	if (DEBUG)
	{
		sysprintln("ARP respond: IP=" + host_ip + ", mac=" + host_ethaddr +
			"  from Client <IP=" + client_ip + ", mac=<" + client_mac + ">");
	}
	
	return OK;
		
}


// End of ArpPkt.cpp
