// ArpPkt.h: interface for the CArpPkt class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

08/12/1998			Created by Yadong Zhang

--*/


#ifndef ARPPKT_H
#define ARPPKT_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "sndbasex.h"
#include "memptr.h"
#include "packet.h"
#include "ethpkt.h"
#include "ipaddr.h"


//////////////////////////////////////////
//         ERROR MESSAGE

///////////////////////////////////////////


#define PROTO_ETHER_TYPE_ARP		0x806

#define ARP_HARDWARE_TYPE_ETH		0x1
#define ARP_PROTO_TYPE_IP			0x800
#define ARP_OPERATION_REQUEST		0x1
#define ARP_OPERATION_RESPONSE		0x2


#include "packon1.h"
struct SArp_FrameHead
{
	nword		HardwareType;
	nword		ProtocolType;
	byte		HardwareAddrLen;
	byte		ProtocolAddrLen;
	nword		Operation;
	etheraddr	SenderHardwareAddr;
	ipaddr		SenderIpAddr;
	etheraddr	TargetHardwareAddr;
	ipaddr		TargetIpAddr;
};
#include "packoff1.h"


unsigned MakeArpQuery(byte *outbuf, unsigned buflen, etheraddr myeth, ipaddr myip, ipaddr targetip);
// outbuf points to a ethernet packet buffer.
// return the length of ARP packet (includes ethernet head)
// return 0 if failed.

////////////////////////////////////////////////////
// CArpPacket define a ARP packet class ( for IP version 4)
// 

class CArpPacket:public CPacket
{
public:
	CEtheriiPacket	EthiiPacket;

	inline CArpPacket(void) {};
	inline CArpPacket(CEtheriiPacket &ep);
	
	inline int Set(CEtheriiPacket &ep);

	inline SArp_FrameHead* GetFramePtr(void);

	// return the ip4_frame pointer

	inline CEtheriiPacket& GetEthiiPkt(void);

	inline memptr GetRealEthiiPktBinary(void);
	// the function return a memptr pointing a ethernet packet, whose length 
	// of ethernet head plus arp packet.
	// BTW: a ethernet packet may contain some padding bytes after IP.

	CArpPacket& operator= (CArpPacket& a) { CopyFrom(a); EthiiPacket = a.EthiiPacket; return *this; }
	
	friend ostream& operator<<(ostream& os, CArpPacket &ep);
	friend outputstr operator +(const outputstr& ops, CArpPacket &ap);
	
	// ==================================================
	inline bool	IsReponse(void) 
	{ return (GetFramePtr()->Operation == tonword(ARP_OPERATION_RESPONSE)); }

	inline ipaddr GetTargetIpAddr(void) { return GetFramePtr()->TargetIpAddr; }
	inline ipaddr GetSenderIpAddr(void) { return GetFramePtr()->SenderIpAddr; }
	inline etheraddr GetTargetEtherAddr(void) { return GetFramePtr()->TargetHardwareAddr; }
	inline etheraddr GetSenderEtherAddr(void) { return GetFramePtr()->SenderHardwareAddr; }
	
};


CArpPacket::CArpPacket(CEtheriiPacket &ep)
{
	Set(ep);
}


int CArpPacket::Set(CEtheriiPacket &ep)
// return OK if it is our packet
// return ERR if not.
{
 	if (ep.GetProtocol() != tonword(PROTO_ETHER_TYPE_ARP)) return ERR;
 
	memptr p = ep.GetDataSection();
	if (p.GetLen() < sizeof (SArp_FrameHead))
		return ERR;

	CPacket::Set(ep, p);
	
	if (GetFramePtr()->HardwareType != tonword(ARP_HARDWARE_TYPE_ETH)) return ERR;
	if (GetFramePtr()->ProtocolType != tonword(ARP_PROTO_TYPE_IP)) return ERR;
	if (GetFramePtr()->HardwareAddrLen != 6) return ERR;
	if (GetFramePtr()->ProtocolAddrLen != 4) return ERR;

	EthiiPacket = ep;

	return OK;
}


SArp_FrameHead* CArpPacket::GetFramePtr(void)
{
	return (SArp_FrameHead*)(GetPacketBinary().GetAddr());
}


CEtheriiPacket& CArpPacket::GetEthiiPkt(void)
{
	return EthiiPacket;
}


int MakeArpRespondence(CArpPacket* arp, etheraddr host_eth, ipaddr host_ip);
// return OK if successfully
// return others if protocols is not supported or unknown error occurred.


#endif
