// file DhcpPkt.CPP
/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

08/14/1998			Created by Yadong Zhang

--*/

#include "stdafx.h"
#include "DhcpPkt.h"

ostream& operator<<(ostream& os, CDhcpPacket &ep)
{
	os	<<"===== DHCP Packet==== len:"<<dec<<ep.GetPacketBinary().GetLen()<<"====\n"
		<<"OpCode: "<<(unsigned)ep.GetFramePtr()->OpCode
		<<", yiaddr:"<<ep.GetFramePtr()->YourIpAddr

		<<endl;

	return os;
}

outputstr operator+(const outputstr& ops, CDhcpPacket &dp)
{
	ostrstream outbuf;
	outbuf<<dp<<(char)0;
	outputstr out = ops + outbuf.str();

	// unfreeze it, otherwise strstream won't release the memory 
	// and memory leak will occur.
	outbuf.freeze(false);
	return out;
}


unsigned CDhcpPacket::GetNextOption(unsigned option_id, byte *option_type,
				void *val_buf, unsigned buflen, unsigned *val_len)
// get the option after the option pointed by option_id.
// return DHCP_INVALID_OPTION_ID if no more;
{
	byte *p = GetOptionsPtr(); p += option_id;
	unsigned i, vlen;
	
	for (i = option_id; i < GetOptionsLen(); (i++, p++))
	{
		if (*p == DHCP_OPTION_TYPE_PAD) continue;
		if (*p == DHCP_OPTION_TYPE_END) return DHCP_INVALID_OPTION_ID;
		
		// we got a option.
		*option_type = *p;
		vlen = *(p + 1);
		*val_len = vlen;

		memcpy(val_buf, p + 2, min(vlen, buflen));	//.. to replace below
//		if (buflen >= vlen) memcpy(val_buf, p + 2, vlen);
//				else memcpy(val_buf, p + 2, buflen);
					
		return (p - GetOptionsPtr()) + 2 + vlen;	//to fix a bug below
//		return option_id + 2 + vlen;
	}
	return DHCP_INVALID_OPTION_ID;
}

unsigned
CDhcpPacket::GetNextOption(
					unsigned offset,			//.. offset to the "OPTION head"
				OUT byte *option_type,			//.. the next OPTION byte
					void *val_buf,				//.. ptr to buffer
					unsigned buflen,			//.. buffer size
				OUT unsigned *val_len,			//.. buffer output size
				OUT byte *(&option_ptr))		//.. ptr to the "currently found" OPTION byte	
// get the next OPTION after the OPTION offseted by option_id.
// return DHCP_INVALID_OPTION_ID if no more;
//.. if successful, return the Offset to the next Option
{
	option_ptr = GetOptionsPtr();
	option_ptr += offset;		//.. offset
	unsigned vlen = 0;
	
	for (option_ptr; (DHCP_OPTION_TYPE_END != *option_ptr); option_ptr++)
	{
		if (*option_ptr == DHCP_OPTION_TYPE_PAD) continue;
		
		// we got an option.
		*option_type = *option_ptr;
		vlen = *(option_ptr + 1);
		*val_len = vlen;

		memcpy(val_buf, (option_ptr + 2), min(vlen, buflen));
					
		return (option_ptr - GetOptionsPtr()) + 2 + vlen;		//.. offset to next Option_type
	}

	return DHCP_INVALID_OPTION_ID;
}



int CDhcpPacket::GetSpecOption(byte option_type, 
				void *val_buf, unsigned buflen, unsigned *val_len)
// get a specified option, code = option_code
// the value of option will be placed into *val_buf
// the value's length will be placed into *val_len
// return OK if found;
// return ERR_NOT_FOUND if not found.
{
	unsigned optid;
	byte	opt_type;
	optid = GetFirstOption(
			&opt_type,		// byte 		*option_type,	// output the type of option
			val_buf,		// void 		*val_buf, 		// pointer a out buffer of  the value of option
			buflen,			// unsigned 	buflen, 		// buf's length
			val_len		// unsigned 	*val_len		// output the value's length.
			);
	
	while (optid != DHCP_INVALID_OPTION_ID)
	{
		if (opt_type == option_type)
		{
			return OK;
		}
		
		optid = GetNextOption(
				optid,			// unsigned		optid
				&opt_type,		// byte 		*option_type,	// output the type of option
				val_buf,		// void 		*val_buf, 		// pointer a out buffer of  the value of option
				buflen,			// unsigned 	buflen, 		// buf's length
				val_len		// unsigned 	*val_len		// output the value's length.
				);
	};

	return ERR_NOT_FOUND;
}


int
CDhcpPacket::GetSpecOption(
					byte option_to_find,		//.. OPTION to find
					void *val_buf,					
					unsigned buflen,
				OUT unsigned *val_len,
				OUT byte *(&option_ptr))			//.. ptr to the "current" OPTION byte
// get a specified option, code = option_code
// the value of option will be placed into *val_buf
// the value's length will be placed into *val_len
// return ERR_NOT_FOUND if not found.
{
	unsigned nOffsetToNext = 0;			//.. offset to next OPTION
	byte opt_type;
	nOffsetToNext = GetFirstOption(	&opt_type,
									val_buf,
									buflen,
									val_len,
									option_ptr);
	if(option_to_find == opt_type)
	{
		if(DHCP_INVALID_OPTION_ID == nOffsetToNext)			//.. already reach the END
			return ERR_NOT_FOUND;
		else
			return (nOffsetToNext - 2 - *val_len);		//.. offset to the requested OPTION
	}
	else
	{
		while(DHCP_INVALID_OPTION_ID != nOffsetToNext)
		{
			//.. keep searching
			nOffsetToNext = GetNextOption(	nOffsetToNext,
											&opt_type,
											val_buf,
											buflen,
											val_len,
											option_ptr);
			if(option_to_find == opt_type)
			{
				return (nOffsetToNext - 2 - *val_len);		//.. offset to the requested OPTION
			}
		}
	}
	return ERR_NOT_FOUND;
}

				
unsigned MakeEthDhcpReply(
		byte				*outbuf, 
		unsigned			buflen,

		ipaddr				myip,
		etheraddr			myeth,

		CDhcpPacket			&req,
			
		ipaddr				yiaddr,
			
		byte				*addition_options,
		unsigned			addition_opt_len
		)
// buflen MUST be above on 1514.
{
	byte *buf = outbuf;
	CEtheriiPacket	eiip;
	CIpPacket		ipp;
	CUdpPacket		udp;
	CDhcpPacket		dhcp;
	
	CDhcpPacket		&dhcp0 = req;
	CUdpPacket		&udp0 = dhcp0.GetUdpPkt();
	CIpPacket		&ipp0 = udp0.GetIpPkt();
	CEtheriiPacket	&eiip0 = ipp0.GetEthiiPkt();


	memptr rawpkt(buf, buflen);
	eiip.Set(rawpkt, eiip0.GetNicId());
	if(eiip0.GetPacketBinary().GetLen() > buflen)
	{	return 0 ;	}
	memcpy(buf, eiip0.GetPacketBinary().GetAddr(),
			eiip0.GetPacketBinary().GetLen());

	if (ipp.Set(eiip) != OK) return 0;
	if (udp.Set(ipp) != OK) return 0;

	// set ethernet address
	if (eiip0.GetSourAddr().is_zero() || eiip0.GetSourAddr().is_broadcast())
	{
		// sender use 00-00-00-00-00-00 or ff-ff-ff-ff-ff-ff
		eiip.GetFramePtr()->DestAddr.set_broadcast();
	}
	else
	{
		eiip.GetFramePtr()->DestAddr = eiip0.GetSourAddr();
	}

	eiip.GetFramePtr()->SourAddr = myeth;
	
	// set ip address
	ipp.GetFramePtr()->SourAddr = myip;
	if ( ipp0.GetSourIp() != toipaddr(0) &&
			ipp0.GetSourIp() != toipaddr(0xFFFFFFFFL))
		ipp.GetFramePtr()->DestAddr = ipp0.GetSourIp();
	else
		ipp.GetFramePtr()->DestAddr = toipaddr(0xFFFFFFFFL);
	
	// set udp port
	swap(&(udp.GetFramePtr()->SourPort), &(udp.GetFramePtr()->DestPort));
	
	if (dhcp.Set(udp) != OK) return 0;
	
	dhcp.GetFramePtr()->OpCode = DHCP_OP_BOOT_REPLY;
	dhcp.GetFramePtr()->HardwareType = DHCP_HARDWARETYPE_10METH;		
	dhcp.GetFramePtr()->HardwareAddrLen = 6;	// 6 for 10mb ethernet
	dhcp.GetFramePtr()->Hops = 0;				// optionally used in booting via a relay agent
	// don't change it. 	// dword		TransactionId;
	// don't change it.		// nword		Seconds;
	dhcp.GetFramePtr()->Flags = 0;
	dhcp.GetFramePtr()->ClientIpAddr = toipaddr(0);		// client Ip addr, only filled if BOUND, RENEW or REBINDING.
	dhcp.GetFramePtr()->YourIpAddr = yiaddr;			// 'your' (client) IP address
	dhcp.GetFramePtr()->NextServerIpAddr = toipaddr(0);	// reutrned in DHCPOFFER, DHCPACK by server
	dhcp.GetFramePtr()->RelayAgentIpAddr = toipaddr(0);	// used in booting via a relay agent
	dhcp.GetFramePtr()->ClientHardwareAddr = eiip0.GetSourAddr();
	memset(dhcp.GetFramePtr()->ServerHostName, 0, sizeof(dhcp.GetFramePtr()->ServerHostName));
	memset(dhcp.GetFramePtr()->BootFileName, 0, sizeof(dhcp.GetFramePtr()->BootFileName));
	
	dhcp.GetFramePtr()->VendorSpecInfo = dhcp0.GetFramePtr()->VendorSpecInfo;

	// write options.
	byte *p = dhcp.GetOptionsPtr();
	memcpy(p, addition_options, addition_opt_len);
	p += addition_opt_len;
	
	unsigned nowlen = p - eiip.GetPacketBinary().GetAddr();
	int c1 = nowlen - eiip0.GetPacketBinary().GetLen();
	short c = (short)c1;
	
	// let's change length;
	ipp.GetFramePtr()->TotalLength = tonword(
			toword(ipp.GetFramePtr()->TotalLength) + c
					);
	udp.GetFramePtr()->Length = tonword(
			toword(udp.GetFramePtr()->Length) + c
					);
	
	ipp.RefillIpChecksum();
	udp.RefillUdpChecksum();

	return nowlen;	
}

unsigned MakeEthDhcpDiscoverPkt(	byte			*OutBuffer, 
								unsigned		OutBufferLen,
								etheraddr	MyEtherAddr,
								byte			*DhcpExtension,
								unsigned		DhcpExtensionLen )
// buflen MUST be above on 1514.
{
	CEtheriiPacket	eiip;
	CIpPacket		ipp;
	CUdpPacket		udp;
	CDhcpPacket		dhcp;
	ipaddr			ZeroIP ;		ZeroIP.zero() ;

		//// setup Ethernet packet header
	memset(OutBuffer, 0, OutBufferLen) ;
	memptr RawPkt( OutBuffer, OutBufferLen) ;
	if(	eiip.Set(RawPkt, ETHERNET_NICID) != OK )
	{	return 0 ;	}
	eiip.GetFramePtr()->DestAddr.set_broadcast() ;
	eiip.GetFramePtr()->SourAddr = MyEtherAddr ;
	byte *bt=((byte *)eiip.GetFramePtr())+12;
	bt[0]=0x08;
	bt[1]=0x00;
	//eiip.GetFramePtr()->Type = tonword(0x0800) ;


		//// setup IP packet header
	if( 	ipp.Set(eiip) != OK )
	{	return 0 ;	}
	ipp.GetFramePtr()->Vers_Hlen			= 0x45 ;
	ipp.GetFramePtr()->ServiceType			= 0x00 ;
	ipp.GetFramePtr()->TotalLength			= tonword(0) ;	//// need to recalculate later
	ipp.GetFramePtr()->Identification		= tonword(555);	//// random number
	ipp.GetFramePtr()->Flags_FragmentOffset	= tonword(0) ;
	ipp.GetFramePtr()->TimeToLive			= 32 ;			//// default value
	ipp.GetFramePtr()->Protocol			= 0x11 ;			//// UDP
	ipp.GetFramePtr()->HeadChecksum			= 1 ;			//// need to recalculate later
	ipp.GetFramePtr()->SourAddr			= ZeroIP ;
	ipp.GetFramePtr()->DestAddr			= toipaddr(0xFFFFFFFFL) ;		//// broadcast

		//// setup UDP packet header
	if(udp.Set(ipp) != OK) 
	{	return 0 ;	}
	udp.GetFramePtr()->SourPort	= tonword(PROTO_UDP_PORT_DHCP_CLIENT) ;
	udp.GetFramePtr()->DestPort	= tonword(PROTO_UDP_PORT_DHCP_SERVER) ;
	udp.GetFramePtr()->Length		= tonword(0);		//// need to recalculate later
	udp.GetFramePtr()->Checksum	= 0 ;			//// need to recalculate later

		//// setup DHCP packet header
	if(dhcp.Set(udp) != OK)
	{	return 0 ;	}
		//// fill the DISCOVER packet
	dhcp.GetFramePtr()->OpCode				= DHCP_OP_BOOT_REQUEST;
	dhcp.GetFramePtr()->HardwareType		= DHCP_HARDWARETYPE_10METH;		
	dhcp.GetFramePtr()->HardwareAddrLen		= DHCP_HARDWARE_LEN_ETHER_ADDR;		// 6 Bytes for 10mb ethernet
	dhcp.GetFramePtr()->Hops				= 0;			// optionally used in booting via a relay agent
	dhcp.GetFramePtr()->TransactionId		= 0x55555555;
	dhcp.GetFramePtr()->Seconds			= tonword(0);
	dhcp.GetFramePtr()->Flags				= 0;			//// not broadcast
	dhcp.GetFramePtr()->ClientIpAddr		= ZeroIP;	// client Ip addr, only filled if BOUND, RENEW or REBINDING.
	dhcp.GetFramePtr()->YourIpAddr			= ZeroIP;	// 'your' (client) IP address
	dhcp.GetFramePtr()->NextServerIpAddr	= ZeroIP;	// reutrned in DHCPOFFER, DHCPACK by server
	dhcp.GetFramePtr()->RelayAgentIpAddr	= ZeroIP;	// used in booting via a relay agent
	dhcp.GetFramePtr()->ClientHardwareAddr	= MyEtherAddr;
	memset(dhcp.GetFramePtr()->ClientHardwareAddr_add, 0, sizeof(dhcp.GetFramePtr()->ClientHardwareAddr_add) ) ;
	memset(dhcp.GetFramePtr()->ServerHostName, 0, sizeof(dhcp.GetFramePtr()->ServerHostName));
	memset(dhcp.GetFramePtr()->BootFileName, 0, sizeof(dhcp.GetFramePtr()->BootFileName));
	dhcp.GetFramePtr()->VendorSpecInfo		= DHCP_VENDOR_SPECIFIC_AREA_HEADER;
		//// write options.
	byte *pEnd = dhcp.GetOptionsPtr();
	memcpy(pEnd, DhcpExtension, DhcpExtensionLen);
	pEnd += DhcpExtensionLen;

		//// recalculate the Length and Checksum
	ipp.GetFramePtr()->TotalLength = tonword(pEnd - eiip.GetPacketBinary().GetAddr()) ;
	udp.GetFramePtr()->Length		= tonword(pEnd - ipp.GetPacketBinary().GetAddr()) ;

	ipp.RefillIpChecksum();
	udp.RefillUdpChecksum();

	return (pEnd - eiip.GetPacketBinary().GetAddr()) ;
}

// End of DhcpPkt.CPP
