// DhcpPkt.h: interface for the CDhcpPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

08/14/1998			Created by Yadong Zhang

--*/


#ifndef DhcpPkt_H
#define DhcpPkt_H

#include "Packet.h"
#include "udppkt.h"

//////////////////////////////////////////
//         ERROR MESSAGE
#define DHCP_ERR_ISNT_MINE		0x2
///////////////////////////////////////////

#define DHCP_INVALID_OPTION_ID	0

// define Options
// ===========================================================

//The value of the length octet does not include the two
//   octets specifying the tag and length.  The length octet is followed
//   by "length" octets of data.
// ---------- only 'END' and 'PAD' option types are fixed length.---
// one byte.
#define DHCP_OPTION_TYPE_PAD					0		//(0x00)
#define DHCP_OPTION_TYPE_END					255		//(0xff)
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_SUBNET_MASK			1		//(0x01)
#define DHCP_OPTION_LEN_SUBNET_MASK				4		
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_ROUTER					3		//(0x03)
// length is not fixed
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_DOMAIN_NAME_SERVER		6		//(0x06)
// length is not fixed
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_HOST_NAME				12		//(0x0c)
// its length is not a fixed value.
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_DOMAIN_NAME			15		//(0x0f)
// length is not fixed
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_NETBIOS_NAME_SERVER	44		//(0x2c)
// length is not fixed
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_NETBIOS_NODETYPE		46		//(0x2e)
// length is not fixed
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_REQUESTED_IP			50		//(0x32)
#define DHCP_OPTION_LEN_REQUESTED_IP			4
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_IP_LEASE_TIME			51		//(0x33)
#define DHCP_OPTION_LEN_IP_LEASE_TIME			4
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_OVERLOAD				52		//(0x34)
#define DHCP_OPTION_LEN_OVERLOAD				1
// ---------- ------------------------------------------------------
//
#define DHCP_OPTION_TYPE_MESSAGE_TYPE			53		//(0x35)
#define DHCP_OPTION_LEN_MESSAGE_TYPE			1
// 1 byte, may be one of following
	#define DHCP_OPTION_MESSAGE_TYPE_DHCPDISCOVER	1
	#define DHCP_OPTION_MESSAGE_TYPE_DHCPOFFER		2
	#define DHCP_OPTION_MESSAGE_TYPE_DHCPREQUEST		3
	#define DHCP_OPTION_MESSAGE_TYPE_DHCPDECLINE		4
	#define DHCP_OPTION_MESSAGE_TYPE_DHCPACK			5
	#define DHCP_OPTION_MESSAGE_TYPE_DHCPNAK			6
	#define DHCP_OPTION_MESSAGE_TYPE_RELEASE			7
	#define DHCP_OPTION_MESSAGE_TYPE_DHCPINFORM		8

// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_SERVER_ID				54		//(0x36)
#define DHCP_OPTION_LEN_SERVER_ID				4
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_REQUEST_LIST			55		//(0x37)
// Its length is not a fixed value.
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_MESSAGE				56		//(0x38)
// Its length is not a fixed value.
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_MESSAGE_MAX_SIZE		57		//(0x39)
#define DHCP_OPTION_LEN_MESSAGE_MAX_SIZE			2
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_RENEWAL_T1_TIME			58	//(0x3a)
#define DHCP_OPTION_LEN_RENEWAL_T1_TIME			4
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_REBINDING_T2_TIME		59		//(0x3b)
#define DHCP_OPTION_LEN_REBINDING_T2_TIME		4
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_VENDOR_CLASS_ID			60	//(0x3c)
// Its length is not a fixed value.
// ------------------------------------------------------------
#define DHCP_OPTION_TYPE_CLIENT_IDENTIFIER		61		//(0x3d)
// Its length is not a fixed value.
// ------------------------------------------------------------


// ==========================================================

//
#define PROTO_UDP_PORT_DHCP_SERVER	67
#define PROTO_UDP_PORT_DHCP_CLIENT	68

// define Message OP code.
#define DHCP_OP_BOOT_REQUEST		0x1
#define DHCP_OP_BOOT_REPLY			0x2

// define HardwareType
#define DHCP_HARDWARETYPE_10METH		0x1
#define DHCP_HARDWARE_LEN_TYPE			1	//// 1 Byte
#define DHCP_HARDWARE_LEN_ETHER_ADDR	6	//// 6 Bytes

//// define NICID
#define ETHERNET_NICID		1

//// Vendor Specific Area
#define	DHCP_VENDOR_SPECIFIC_AREA_HEADER	0x63538263	//// 99,130,83,99 (tricky)

// For the detail of packet, see also RFC2131
#include "packon1.h"
typedef struct
{
	byte			OpCode;		
	byte			HardwareType;	
	byte			HardwareAddrLen;			// '6' for 10mb ethernet 
	byte			Hops;					// optionally used in booting via a relay agent
	dword		TransactionId;
	nword		Seconds;
	word			Flags;
	ipaddr		ClientIpAddr;				// client Ip addr, only filled if BOUND, RENEW or REBINDING.
	ipaddr		YourIpAddr;				// 'your' (client) IP address
	ipaddr		NextServerIpAddr;			// returned in DHCPOFFER, DHCPACK by server
	ipaddr		RelayAgentIpAddr;			// used in booting via a relay agent
	
	etheraddr	ClientHardwareAddr;
	byte			ClientHardwareAddr_add[10]; // client hardware address is 16 bytes.
	
	char			ServerHostName[64];		// Optional, null terminated string
	char			BootFileName[128];		// null terminated string
	dword		VendorSpecInfo;			//// 99, 130, 83, 99 (RFC 1497)
} SDhcp_FrameHead;
// Dhcp Options following header.
#include "packoff1.h"


class CDhcpPacket : public CPacket  
{
public:
	CUdpPacket		UdpPacket;
	unsigned		OptionsLen;

	CDhcpPacket(void) {};
	inline CDhcpPacket(CUdpPacket &upp);
	
	inline int Set(CUdpPacket &upp);

	inline CUdpPacket& GetUdpPkt(void) { return UdpPacket; }

	inline SDhcp_FrameHead* GetFramePtr(void);
	// return the Dns datagram head pointer

	friend ostream& operator<<(ostream& os, CDhcpPacket &tp);
	friend outputstr operator+(const outputstr& ops, CDhcpPacket &dp);
	
	// ==================================================

public:	
	inline unsigned GetFirstOption(
			byte 		*option_type,	// output the type of option
			void 		*val_buf, 		// pointer a out buffer of  the value of option
			unsigned 	buflen, 		// buf's length
			unsigned 	*val_len		// output the value's length.
			);
	inline unsigned GetFirstOption(
		OUT	byte 		*option_type,		// output the type of option
			void 		*val_buf, 			// pointer a out buffer of  the value of option
			unsigned 	buflen, 			// buf's length
		OUT	unsigned 	*val_len,			// output the value's length.
		OUT	byte		*(&ptrOption)		// ptr to the Option_type
			);
	// return OptionID;
	// return DHCP_INVALID_OPTION_ID if no option.
	
	unsigned GetNextOption(unsigned option_id, byte *option_type,
				void *val_buf, unsigned buflen, unsigned *val_len);
	unsigned GetNextOption(	unsigned offset,					//.. offset
							OUT byte *option_type,
							void *val_buf,
							unsigned buflen,
							OUT unsigned *val_len,
							OUT byte *(&option_ptr));
	// get the option after the option pointed by option_id.
	// return DHCP_INVALID_OPTION_ID if no more;
	
	inline GetOpCode(void) { return GetFramePtr()->OpCode;	}
	// return DHCP_OP_BOOT_XXXXX
	
	inline byte *GetOptionsPtr(void);
	inline unsigned GetOptionsLen(void);
	
	int GetSpecOption(byte option_type, 
				void *val_buf, unsigned buflen, unsigned *val_len);
	int GetSpecOption(	byte option_to_find, 
						void *val_buf,
						unsigned buflen, 
						OUT unsigned *val_len,
						OUT byte *(&option_ptr));
	//.. get specific option

	inline int GetOptionMsgType(byte *msg_type);
	// output: msg_type,   DHCP_OPTION_MESSAGE_TYPE_XXXXXX
	// return OK if successful
	// return ERR_NOT_FOUND if msg_type wasn't found.
	// return ERR_UNKNOWN if we found the msg_type, but it's length is invalid.
	
	inline int GetOptionIpRequested(ipaddr *ip);
	// return OK if successful
	// return ERR_NOT_FOUND if msg_type wasn't found.
	// return ERR_UNKNOWN if we found the msg_type, but it's length is invalid.

	inline int GetOptionHostName(char *outbuf, unsigned buflen);
	// return OK if successful
	// return ERR_NOT_FOUND if msg_type wasn't found.
	// return ERR_UNKNOWN if we found the msg_type, but it's length is invalid.
	
	inline int GetOptionServerIP(ipaddr *ip);
	// return OK if successful
	// return ERR_NOT_FOUND if msg_type wasn't found.
	// return ERR_UNKNOWN if we found the msg_type, but it's length is invalid.

};


CDhcpPacket::CDhcpPacket(CUdpPacket& upp)
{
	Set(upp);
}

int CDhcpPacket::Set(CUdpPacket& upp)
// return OK if it is our packet
// return UDP_ERR_ISNT_MINE if not.
{
 	if (upp.GetDestPort() != tonword(PROTO_UDP_PORT_DHCP_SERVER)
 		&& upp.GetDestPort() != tonword(PROTO_UDP_PORT_DHCP_CLIENT)
 		) return DHCP_ERR_ISNT_MINE;
 	
	memptr p = upp.GetDataSection();
	if (p.GetLen() < sizeof (SDhcp_FrameHead))
		return DHCP_ERR_ISNT_MINE;

	CPacket::Set(upp, p);

	UdpPacket = upp;
	
	OptionsLen = GetPacketBinary().GetLen() - sizeof(SDhcp_FrameHead);
	
	return OK;
}


SDhcp_FrameHead* CDhcpPacket::GetFramePtr(void)
// return the TcpSegment pointer
{
	return (SDhcp_FrameHead*)(GetPacketBinary().GetAddr());
}

// =======================================================

byte *CDhcpPacket::GetOptionsPtr(void)
{
	return ( ((byte*)GetFramePtr()) + sizeof(SDhcp_FrameHead) );	
}

unsigned CDhcpPacket::GetOptionsLen(void)
{
	return OptionsLen;
}

unsigned CDhcpPacket::GetFirstOption(
			byte 		*option_type,	// output the type of option
			void 		*val_buf, 		// pointer a out buffer of  the value of option
			unsigned 	buflen, 		// buf's length
			unsigned 	*val_len		// output the value's length.
			)
// return OptionID;
// return DHCP_INVALID_OPTION_ID if no useful option.
{
	return GetNextOption(0, option_type, val_buf, buflen, val_len);
}
unsigned CDhcpPacket::GetFirstOption(
			byte 		*option_type,	// output the type of option
			void 		*val_buf, 		// pointer a out buffer of  the value of option
			unsigned 	buflen, 		// buf's length
			unsigned 	*val_len,		// output the value's length.
			byte		*(&ptrOption)	)
// return OptionID;
// return DHCP_INVALID_OPTION_ID if no useful option.
{
	return GetNextOption(0, option_type, val_buf, buflen, val_len, ptrOption);
}


int CDhcpPacket::GetOptionMsgType(byte *msg_type)
// output: msg_type,   DHCP_OPTION_MESSAGE_TYPE_XXXXXX
// return OK if successful
// return ERR_NOT_FOUND if msg_type wasn't found.
// return ERR_UNKNOWN if we found the msg_type, but it's length is invalid.
{
	byte opt_val[512];
	unsigned val_len;
	byte *ptrOption;
	
	*msg_type = 0;
	
	if ( GetSpecOption(	DHCP_OPTION_TYPE_MESSAGE_TYPE,
						opt_val,
						sizeof(opt_val),
						&val_len,
						ptrOption) == ERR_NOT_FOUND)
		return ERR_NOT_FOUND;

	if (val_len != 1) return ERR_UNKNOWN;
	
	*msg_type = opt_val[0];
	
	return OK;
}


int CDhcpPacket::GetOptionIpRequested(ipaddr *ip)
// return OK if successful
// return ERR_NOT_FOUND if msg_type wasn't found.
// return ERR_UNKNOWN if we found the msg_type, but it's length is invalid.
{
	unsigned val_len; 	byte opt_val[512];
	byte *ptrOption;
	
	if ( GetSpecOption(	DHCP_OPTION_TYPE_REQUESTED_IP,
						opt_val,
						sizeof(opt_val),
						&val_len,
						ptrOption) == ERR_NOT_FOUND)
		return ERR_NOT_FOUND;

	if (val_len != 4) return ERR_UNKNOWN;
	
	memcpy(ip, opt_val, 4);
	
	return OK;
}


int CDhcpPacket::GetOptionHostName(char *outbuf, unsigned buflen)
// return OK if successful
// return ERR_NOT_FOUND if msg_type wasn't found.
// return ERR_NO_ENOUGH_SPACE if buf is too small
{
	unsigned val_len;
	outbuf[0] = 0;
	byte *ptrOption;
	
	if ( GetSpecOption(	DHCP_OPTION_TYPE_HOST_NAME,
						outbuf,
						buflen,
						&val_len,
						ptrOption) == ERR_NOT_FOUND)
		return ERR_NOT_FOUND;

	if (val_len >= buflen  ) return ERR_NO_ENOUGH_SPACE;
	
	if (buflen) outbuf[buflen - 1] = 0;
	
	return OK;
}

unsigned MakeEthDhcpReply(	byte			*outbuf, 
						unsigned		buflen,
						ipaddr		myip,
						etheraddr	myeth,
						CDhcpPacket	&req,
						ipaddr		yiaddr,
						byte			*addition_options,
						unsigned		addition_opt_len );
// buflen MUST be above on 1514.
// outbuf points to a ethernet packet buffer.
// MakeEthDhcpReply() will generate MESSAGE_TYPE option, if 
// caller want some additional options, use addition_options
// notice: addition_option should end with "0xFF"(OPTION_END)
		
// return the length of DHCP packet (includes ethernet head)
// return 0 if failed.


int CDhcpPacket::GetOptionServerIP(ipaddr *ip)
// return OK if successful
// return ERR_NOT_FOUND if msg_type wasn't found.
// return ERR_UNKNOWN if we found the msg_type, but it's length is invalid.
{
	unsigned OptionLength;
	byte OptionBuffer[512];
	byte *ptrOption;
	
	if ( GetSpecOption( DHCP_OPTION_TYPE_SERVER_ID,
						OptionBuffer ,
						sizeof(OptionBuffer),
						&OptionLength,
						ptrOption) == ERR_NOT_FOUND )
	{	return ERR_NOT_FOUND;	}

	if (OptionLength != DHCP_OPTION_LEN_SERVER_ID)
	{	return ERR_UNKNOWN;	}
	
	memcpy(ip, OptionBuffer, DHCP_OPTION_LEN_SERVER_ID);
	
	return OK;
}

unsigned MakeEthDhcpDiscoverPkt(	byte			*OutBuffer, 
								unsigned		OutBufferLen,
								etheraddr	MyEtherAddr,
								byte			*DhcpExtension,
								unsigned		DhcpExtensionLen ) ;
	//// 1. OutBuffer MUST NOT be above  1514, and OutBuffer points
	////		to an ethernet packet buffer.
	//// 2. MakeEthDhcpDiscoverPkt() will generate DHCP_OPTION_EXTENSIONS, if caller
	////		wants some additional options, use DhcpExtension
	//// 3. Notice: DhcpExtension should end with "0xFF"(OPTION_END)
	//// 4. Return the length of DHCP packet (includes ethernet head)
	//// 5. Return 0 if failed.

#endif
