/* $Header: /usr2/cvs/master/Develop/Products/Smc/build/sndfc/src/sndfc/snd/dnspkt.h,v 1.1.1.1 2001/06/29 19:30:08 herbert Exp $*/
/*++
*************************************************************************
+-----------------------------------------------------------------------+
| Copyright (C) SYBERGEN NETWORKS, INC. 1998,1999 |
| All Rights Reserved. |
| Reproduction, adaptation, or translation without prior written |
| permission is prohibited, except as allowed under Copyright laws. |
| |
| N O T I C E |
| |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC. |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED. |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
 * 
 * $Author: herbert $
 * $Date: 2001/06/29 19:30:08 $
 * $Revision: 1.1.1.1 $
 * $Workfile: dnspkt.h $
 * $Log: dnspkt.h,v $
 * Revision 1.1.1.1  2001/06/29 19:30:08  herbert
 * Migrate from VSS to CVS.
 *
 * 
 * 5     2/13/01 2:48p Vhsieh
 * Bug fix: When DNS packets is invalid, NoError function return false;
 * 
 * 4     8/02/00 5:50p Herbertw
 * Merge with SSD sndfc
 * 
 * 3     5/16/00 5:46p Herbertw
 * Merge SNDFC from  NatSrv and Netport
 * 
 * 1     1/28/00 10:58a Ydz
 * 
 * 1     1/26/00 2:07p Herbertw
 * 
 * 4     10/14/99 11:23a Jchuang
 * 
 * 3     8/12/99 6:15p Jchuang
 * fix crash on query DNS
 * 
 * 2     8/06/99 12:22p Herbertw
 * $NoKeywords: $
 *
--*/

 
/*++
History:

08/14/1998			Created by Yadong Zhang

--*/


#ifndef DNSPKT_H
#define DNSPKT_H

#include "Packet.h"
#include "udppkt.h"

//////////////////////////////////////////
//         ERROR MESSAGE
#define DNS_ERR_ISNT_MINE		0x2
///////////////////////////////////////////

#define DNS_QNAME_MAX_LENGTH	256
#define DNS_MAX_ANCOUNT			16

#define PROTO_UDP_PORT_DNS		0x35

#define DNS_MSG_TYPE_QUERY		0x0
#define DNS_MSG_TYPE_IQUERY		0x1
#define DNS_MSG_TYPE_STATUS		0x2

#define DNS_RREC_TYPE_ADDR		0x1
#define DNS_RREC_TYPE_PTR		0xc

#define DNS_RREC_TYPE_ADDR_NET	0x0100
#define DNS_RREC_TYPE_PTR_NET	0x0c00
#define DNS_RREC_TYPE_SRV		0x2100
#define DNS_RREC_TYPE_CNAME		0x0005

#define RESOURCE_TYPE_A			0x1
#define RESOURCE_TYPE_CNAME		0x5

#include "packon1.h"
typedef struct
{
	nword		Id;
	word		Misc;
	nword		QDCount;	// Question
	nword		ANCount;
	nword		NSCount;
	nword		ARCount;
} SDns_FrameHead;

typedef struct
{
	nword		Type;
	nword		Class;
	byte		TTL[4];
	nword		RDLength;
} SDns_ResourceRecord;

#include "packoff1.h"


class CDnsPacket : public CPacket  
{
private:
	nword	GetANCount(void){ return GetFramePtr()->ANCount;}
	nword	GetQDCount(void){ return GetFramePtr()->QDCount;}

	ipaddr			AnswerIP[DNS_MAX_ANCOUNT];
	unsigned		ANCount;
	unsigned		QDCount;

	unsigned short	ResourceType;
	int				m_lenOfQuestionSections;

	int		m_iNoError;
	
	void	Initial(void);

public:
	CUdpPacket		UdpPacket;
	

	CDnsPacket(void);
	CDnsPacket(CUdpPacket &upp);
	
	int Set(CUdpPacket &upp);

	inline CUdpPacket& GetUdpPkt(void) { return UdpPacket; }

	inline SDns_FrameHead* GetFramePtr(void);

		;
	// return the Dns datagram head pointer

	friend ostream& operator<<(ostream& os, CDnsPacket &tp);
	friend outputstr operator +(const outputstr ops, CDnsPacket &dp);

	// New functionality with reserve dns processing
	inline unsigned short GetResourceType(void);
	// ==================================================
	
	inline unsigned GetMsgType(void);
	inline bool IsStardandQueryPkt(void);

	//Merge from SSD
	inline unsigned GetQueryMessageType(void);
	inline word CDnsPacket::GetFlags();
	inline BOOL IsQuery(void);
	inline BOOL NoError(void);
	int GetAnswerAddresses(dword *outbuf, unsigned buflen);
	// get the array of IP addresses which are include in a DNS
	// answer section.
	// return OK if successful.
	//============
	word	GetId(void){return toword(GetFramePtr()->Id);}
	int GetQName(char *outbuf, unsigned buflen);
	// get the name queried.( DNS_QNAME_MAX_LENGTH max.)
	// format likes "xxx.xxxx.xxx", nor "xxx.xxx.xxx."
	// The function supports ONLY ONE QName.
	// The second and more QName will be ignored.
	// return OK if successful.
	unsigned int GetAnswerIP (ipaddr *ip, unsigned ipBufCount);

	unsigned int GetQIP(unsigned long *ip);
	unsigned int GetAnswerName(char *outbf, unsigned buflen);
	unsigned int GetAnswerTTL(unsigned int *ttl);

	inline bool IsAnswer(void){ return !IsQuery();}
};

SDns_FrameHead* CDnsPacket::GetFramePtr(void)
// return the TcpSegment pointer
{
	return (SDns_FrameHead*)(GetPacketBinary().GetAddr());
}

word CDnsPacket::GetFlags()
{
	word flags = GetFramePtr()->Misc;
	return flags;
}

unsigned short CDnsPacket::GetResourceType(void)
{
	return ResourceType;
}
unsigned CDnsPacket::GetQueryMessageType(void)
{
	// Get Opcode
	return (GetFlags() & 0x78);
}

unsigned CDnsPacket::GetMsgType(void)
{
	return ((GetFramePtr()->Misc >> 4) && 0xF);
}

bool CDnsPacket::IsStardandQueryPkt(void)
{
	return (GetMsgType() == DNS_MSG_TYPE_QUERY);
}

BOOL CDnsPacket::IsQuery()
{
	return ((GetFlags() & 0xA0) == 0);
}

BOOL CDnsPacket::NoError()
{
	// Check RCode
//	int RCode = ( GetFlags() & 0xF00);
//	return (RCode == 0);
	return (m_iNoError == OK);
}
#endif
