// ETHPKT.h: interface for the Ethernet's class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

04/02/1999			Merged into SNDFC by Yadong Zhang.
04/10/1998			Created by Yadong Zhang

--*/



#ifndef SND_ETHPKT_H
#define SND_ETHPKT_H


#include "sndbasex.h"
#include "packet.h"
#include "memptr.h"
#include "ethaddr.h"
#include "netword.h"



// =====================================================
// TEMPORARY, it is for compatibility with SyGate Code
const unsigned NICID_TOB = 1;
// =====================================================



#include "packon1.h"
struct SEtherii_FrameHead
{
	friend class CEtheriiPacket;
	etheraddr	DestAddr;
	etheraddr	SourAddr;
private:                              //I put it to private , bcz it may be 802.3 packet,so use GetProtocol() to access it
	nword		Type;
};

struct SEther802_3_SNAP_FrameHead  //802.3 SNAP 
{
	etheraddr	DestAddr;
	etheraddr	SourAddr;
	word		Length;
	byte		Dsap;
	byte		Ssap;
	byte		uframe;
	byte		Snap_head[3];
	nword		Type;
};

#include "packoff1.h"

#define MAX_ETHERNET_PACKET_LENGTH		1514


////////////////////////////////////////////////////
// CEtheriiPacket define a EtherII packet class
// 

class CEtheriiPacket;
odstream& operator<<(odstream& os, CEtheriiPacket &ep);

class CEtheriiPacket:public CPacket
{
private:
	unsigned		NicId;
public:
	inline CEtheriiPacket(void) {};
	inline CEtheriiPacket(void *p, unsigned len, unsigned nicid);
	inline CEtheriiPacket(memptr p, unsigned nicid);
	
	inline int Set(memptr p, unsigned nicid);
	// return OK if successful
	
	inline unsigned GetNicId(void) { return NicId; }

	inline SEtherii_FrameHead* GetFramePtr(void);
	inline memptr GetDataSection(void);
	inline unsigned GetDataLen(void);
	
	inline unsigned GetHeadLen(void) ;

	inline nword GetProtocol(void);
	inline etheraddr GetSourAddr(void);
	inline etheraddr GetDestAddr(void);
	inline BOOL IsEther802_3_SNAP_Packet();
	friend odstream& operator<<(odstream& os, CEtheriiPacket &ep);
	friend outputstr operator+(const outputstr& ops, CEtheriiPacket &ep);
};


int CEtheriiPacket::Set(memptr p, unsigned nicid)
{
	CPacket::Set(p);
	NicId = nicid;
	return OK;
}


CEtheriiPacket::CEtheriiPacket(memptr p, unsigned nicid)
{ 
	Set(p, nicid);
}


SEtherii_FrameHead* CEtheriiPacket::GetFramePtr(void)
{
	return (SEtherii_FrameHead*)(GetPacketBinary().GetAddr());
}


unsigned CEtheriiPacket::GetDataLen(void)
{
	return GetPacketBinary().GetLen() - GetHeadLen();
}


memptr CEtheriiPacket::GetDataSection(void)
{
	return GetPacketBinary().GetSubset(GetHeadLen()); 
}


nword CEtheriiPacket::GetProtocol(void)
{
	if (IsEther802_3_SNAP_Packet())
	{
		SEther802_3_SNAP_FrameHead *p=(SEther802_3_SNAP_FrameHead *)GetFramePtr();
		return p->Type;
	}
	return GetFramePtr()->Type;
}

etheraddr CEtheriiPacket::GetSourAddr(void)
{	
	return GetFramePtr()->SourAddr;
}

etheraddr CEtheriiPacket::GetDestAddr(void)
{	
	return GetFramePtr()->DestAddr;
}

inline BOOL CEtheriiPacket::IsEther802_3_SNAP_Packet()
{
	if (toword(GetFramePtr()->Type)<=0x5ee)
	{
		SEther802_3_SNAP_FrameHead *p=(SEther802_3_SNAP_FrameHead *)GetFramePtr();
		if (p->Dsap==0xaa && p->Ssap==0xaa  ) return TRUE;
	}
	return FALSE;
}

inline unsigned CEtheriiPacket::GetHeadLen(void)
{
	if (IsEther802_3_SNAP_Packet())
	{
		return sizeof(SEther802_3_SNAP_FrameHead);
	}
	
	return sizeof(SEtherii_FrameHead);
}

#endif
// end of ethpkt.h

