// GrePkt.h: interface for the CGrePkt class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/
/*++
History:

08/24/1998			Created by Yadong Zhang

--*/


#ifndef GREPKT_H
#define GREPKT_H

#include "Packet.h"
#include "ippkt.h"

//////////////////////////////////////////
//         ERROR MESSAGE
#define GRE_ERR_ISNT_MINE		0x2
///////////////////////////////////////////

#define PROTO_IP_TYPE_GRE		(47)

#define GRE_FLAG_Checksum_Present				(0x0080)
#define GRE_FLAG_Routing_Present				(0x0040)
#define GRE_FLAG_Key_Present					(0x0020)
#define GRE_FLAG_SeqNum_Present					(0x0010)
#define GRE_FLAG_StrictSourceRoute_Present		(0x0008)
#define GRE_FLAG_Recursion_Control				(0x0007)
#define GRE_FLAG_AckNum_Present					(0x8000)
#define GRE_FLAG_Mbz							(0x7800)
#define GRE_FLAG_VER							(0x0700)

#include "packon1.h"
typedef struct
{
	word		Flags_Ver;
	nword		ProtocolType;
	nword		PayloadLength;
	nword		CallId;
	ndword		SeqNumber;			// optional
	ndword		AckNumber;			// optional
} SGre_FrameHead;
#include "packoff1.h"


// CGrePacket defines GRE v2 packet used as PPTP data stream protocol

class CGrePacket : public CPacket  
{
public:
	CIpPacket	IpPacket;

	CGrePacket(void) {};
	inline CGrePacket(CIpPacket &ipp);
	
	inline int Set(CIpPacket &ipp);

	inline SGre_FrameHead* GetFramePtr(void)
	// return the ICMP head pointer
	{ 	return (SGre_FrameHead*)(GetPacketBinary().GetAddr()); }

	inline unsigned GetHeadLen(void);
	// return the ICMP header's length

	inline unsigned GetDataLen(void)
	// return the tcp's data length;
	{ return GetPacketBinary().GetLen() - GetHeadLen(); }

	inline memptr GetDataSection(void)
	// return the tcp's data section
	{ memptr a(GetPacketBinary().GetAddr(GetHeadLen()), GetDataLen() ); return a;}

	inline CIpPacket& GetIpPkt(void)
	{ return IpPacket; }

	friend ostream& operator<<(ostream& os, CGrePacket &ep);
	friend outputstr operator +(const outputstr& ops, CGrePacket &gp);
	// ============ protocol special function ===========
	
	inline unsigned GetVersion(void);
	
	inline word GetFlags(word flag_mask);
	
	inline nword GetProtocolType(void)
	{ return GetFramePtr()->ProtocolType; }
	
	inline unsigned GetPayloadLength(void)
	{ return toword(GetFramePtr()->PayloadLength) ; }

	inline nword GetCallId(void)
	{ return GetFramePtr()->CallId; }
	
	inline ndword GetSeqNumber(void)
	{ return GetFramePtr()->SeqNumber; }

	inline ndword GetAckNumber(void)
	{ return GetFramePtr()->AckNumber; }

};


CGrePacket::CGrePacket(CIpPacket& ipp)
{
	Set(ipp);
}

int CGrePacket::Set(CIpPacket& ipp)
// return OK if it is our packet
// return ICMP_ERR_ISNT_MINE if not.
{
 	if (ipp.GetProtocol() != PROTO_IP_TYPE_GRE) return GRE_ERR_ISNT_MINE;

	memptr p = ipp.GetDataSection();
	if (p.GetLen() < sizeof (SGre_FrameHead))
		return GRE_ERR_ISNT_MINE;

	CPacket::Set(ipp, p);

	IpPacket = ipp;
	
	// we only process GRE v2.
	if (GetVersion() != 1) return GRE_ERR_ISNT_MINE;
	
	return OK;
}



word CGrePacket::GetFlags(word flag_mask)
{
	return (GetFramePtr()->Flags_Ver & flag_mask);
}

unsigned CGrePacket::GetVersion(void)
{
	return (GetFramePtr()->Flags_Ver & 	GRE_FLAG_VER) >> 8;
}

#endif 
