// IcmpPkt.h: interface for the CIcmpPkt class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/
/*++
History:

04/02/1999			Merged into SNDFC by Yadong Zhang
05/10/1998			Created by Yadong Zhang

--*/



#ifndef SND_ICMPPKT_H
#define SND_ICMPPKT_H

#include "sndbasex.h"
#include "Packet.h"
#include "ippkt.h"

//////////////////////////////////////////
//         ERROR MESSAGE
#define ICMP_ERR_ISNT_MINE		0x2
///////////////////////////////////////////

#define PROTO_IP_TYPE_ICMP		0x1

#include "packon1.h"
typedef struct
{
	byte		Type;
	byte		Code;
	word		Checksum;
} SIcmp_FrameHead;
#include "packoff1.h"


class CIcmpPacket;
odstream& operator<<(odstream& os, CIcmpPacket &ep);

class CIcmpPacket : public CPacket  
{
public:
	CIpPacket	IpPacket;

	CIcmpPacket(void) {};
	inline CIcmpPacket(CIpPacket &ipp);
	
	inline int Set(CIpPacket &ipp);

	inline SIcmp_FrameHead* GetFramePtr(void)
	// return the ICMP head pointer
	{ 	return (SIcmp_FrameHead*)(GetPacketBinary().GetAddr()); }

	inline unsigned GetHeadLen(void) { return sizeof(SIcmp_FrameHead); }
	// return the ICMP header's length

	inline unsigned GetDataLen(void)
	// return the tcp's data length;
	{ return GetPacketBinary().GetLen() - GetHeadLen(); }

	inline memptr GetDataSection(void)
	// return the tcp's data section
	{ memptr a(GetPacketBinary().GetAddr(GetHeadLen()), GetDataLen() ); return a;}

	inline CIpPacket& GetIpPkt(void)
	{ return IpPacket; }

	inline byte GetType(void)
	// return the ICMP's type section
	{ return GetFramePtr()->Type; }

	inline byte GetCode(void)
	// return the ICMP's code section, even if always zero.
	{ return GetFramePtr()->Code; }

	bool IsValidIcmpChecksum(void);
	int RefillIcmpChecksum(void);

	friend odstream& operator<<(odstream& os, CIcmpPacket &ep);
	friend outputstr operator +(const outputstr & ops, CIcmpPacket &ip);
};


CIcmpPacket::CIcmpPacket(CIpPacket& ipp)
{
	Set(ipp);
}

int CIcmpPacket::Set(CIpPacket& ipp)
// return OK if it is our packet
// return ICMP_ERR_ISNT_MINE if not.
{
 	if (ipp.GetProtocol() != PROTO_IP_TYPE_ICMP) return ICMP_ERR_ISNT_MINE;

	memptr p = ipp.GetDataSection();
	if (p.GetLen() < sizeof (SIcmp_FrameHead))
		return ICMP_ERR_ISNT_MINE;

	CPacket::Set(ipp, p);

	IpPacket = ipp;
	if ( GetPacketBinary().GetLen() >= GetHeadLen())
		return OK;
	else 
		return ICMP_ERR_ISNT_MINE;
}


#endif // !defined(AFX_ICMPPKT_H__20367882_9D45_11D1_946D_0020AF923BD8__INCLUDED_)
