// IPPKT.h: interface for the CIpPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

04/02/1999			Merged into SNDFC by Yadong Zhang.
03/29/1998			Created by Yadong Zhang

--*/


#ifndef SND_IPPKT_H
#define SND_IPPKT_H

#include "sndbasex.h"
#include "packet.h"
#include "ethpkt.h"
#include "ipaddr.h"




//////////////////////////////////////////
//         ERROR MESSAGE
#define IP_ERR_INVALID_CHECKSUM	0x2


///////////////////////////////////////////


#define PROTO_ETHER_TYPE_IP4		0x800

/*++
  IP Header format:
                                    
    0                   1                   2                   3   
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |Version|  IHL  |Type of Service|          Total Length         |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |         Identification        |Flags|      Fragment Offset    |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |  Time to Live |    Protocol   |         Header Checksum       |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                       Source Address                          |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                    Destination Address                        |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |                    Options                    |    Padding    |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

  Flags:  3 bits   Various Control Flags.
      Bit 0: reserved, must be zero
      Bit 1: (DF) 0 = May Fragment,  1 = Don't Fragment.
      Bit 2: (MF) 0 = Last Fragment, 1 = More Fragments.
          0   1   2
        +---+---+---+
        |   | D | M |
        | 0 | F | F |
        +---+---+---+


--*/
#include "packon1.h"
struct  SIp4_FrameHead
{
	byte		Vers_Hlen;
	byte		ServiceType;
	nword		TotalLength;
	nword		Identification;
	nword		Flags_FragmentOffset;
	byte		TimeToLive;
	byte		Protocol;
	word		HeadChecksum;	// we don't need nword.
	ipaddr		SourAddr;
	ipaddr		DestAddr;
//	byte		Options;
//	byte		OptionsLength;
};
#include "packoff1.h"

////////////////////////////////////////////////////
// CIpPacket define a IP packet class ( for IP version 4)
// 

class CIpPacket;
odstream& operator<<(odstream& os, CIpPacket &ep);

class CIpPacket:public CPacket
{
public:
	CEtheriiPacket	EthiiPacket;

	inline CIpPacket(void) {};
	inline CIpPacket(CEtheriiPacket &ep);
	
	inline int Set(CEtheriiPacket &ep);
	inline int Set(SIp4_FrameHead* fh, unsigned len);

	inline SIp4_FrameHead* GetFramePtr(void);
	// return the ip4_frame pointer

	inline CEtheriiPacket& GetEthiiPkt(void);

	inline unsigned GetHeadLen(void);
	// return the ip header's length

	inline unsigned GetDataLen(void);
	// return the ip's data length;

	inline memptr GetDataSection(void);
	// return the ip's data section

	inline byte GetProtocol(void);
	// return the ip's protocol segment

	inline byte GetTimeToLive(void);

	inline ipaddr GetSourIp(void);

	inline ipaddr GetDestIp(void);

	inline void SetSourIp(ipaddr a);

	inline void SetDestIp(ipaddr a);

	// ---- fragment extersion ------------

	inline bool IsDoNotFragment(void);
	inline void SetDoNotFragment(bool enable_it);
	inline bool IsMoreFragmentFollow(void);
	inline void SetMoreFragmentFollow(bool enable_it);
	inline word GetFragmentOffset(void);
	inline void SetFragmentOffset(word offset);
	// ------------------------------------

    inline word GetChecksum(void) { return GetFramePtr()->HeadChecksum;}
    
    word CalculatedChecksum(void);
    
	bool IsValidIpChecksum(void);
	// return true if the checksum is valid

	int RefillIpChecksum(void);
	// re-calculate the ipheader's checksum and write to ipheader.
	// return OK if success,,, I think it should success always..  :-)

	inline memptr GetRealEthiiPktBinary(void);
	// the function return a memptr pointing a ethernet packet, whose length 
	// of ethernet head plus ip packet.
	// BTW: a ethernet packet may contain some padding bytes after IP.

	CIpPacket& operator= (CIpPacket& a) { CopyFrom(a); EthiiPacket = a.EthiiPacket; return *this; }
	
	friend ostream& operator<<(ostream& os, CIpPacket &ep);
	friend outputstr operator +(const outputstr &ops, CIpPacket &ep);
};


CIpPacket::CIpPacket(CEtheriiPacket &ep)
{
	Set(ep);
}


int CIpPacket::Set(SIp4_FrameHead* fh, unsigned len)
// the function set a special IP packet not included by ethernet packet,
// which contains only IP part.
// Its main function is for ICMP's TTL_EXCEEDED protocol, see "TtlExPkt.h"
// for this type of CIpPacket, GetEtheriiPkt() is invalid, Do NOT use it.
// or anything about the member EthiiPacket;
{
	memptr a;
	a.Set(fh, len);

	CPacket::Set(a);
	return OK;
}


int CIpPacket::Set(CEtheriiPacket &ep)
// return OK if it is our packet
// return ERR if not.
{
 	if (ep.GetProtocol() != tonword(PROTO_ETHER_TYPE_IP4)) return ERR;
	memptr a;
	a.Set(
		ep.GetDataSection().GetAddr(), 
		toword(((SIp4_FrameHead*)ep.GetDataSection().GetAddr())->TotalLength) 
		) ;

	if(ep.GetDataLen() < a.GetLen())
	{
		SIp4_FrameHead *ptemp;

		ptemp = (SIp4_FrameHead*)ep.GetDataSection().GetAddr();
		ptemp->TotalLength = tonword(ep.GetDataLen());
		// Pin: Removed the dump because we might flood the logs
        //sysprintln("Change the length of the packet from " + a.GetLen() + " to " + ep.GetDataLen());
	}

	if (ep.GetDataLen() < sizeof(SIp4_FrameHead))
		return ERR;

	CPacket::Set(ep, a);

	EthiiPacket = ep;
	if (GetPacketBinary().GetLen() >= GetHeadLen())
		return OK;
	else
		return ERR;
}


SIp4_FrameHead* CIpPacket::GetFramePtr(void)
{
	return (SIp4_FrameHead*)(GetPacketBinary().GetAddr());
}


CEtheriiPacket& CIpPacket::GetEthiiPkt(void)
{
	return EthiiPacket;
}


unsigned CIpPacket::GetHeadLen(void)
{
	return ( (GetFramePtr()->Vers_Hlen & 0xf ) * 4 );
}


unsigned CIpPacket::GetDataLen(void)
{
	return GetPacketBinary().GetLen() - GetHeadLen();
}


memptr CIpPacket::GetDataSection(void)
{
//	memptr a(GetPacketBinary().GetAddr(GetHeadLen()), GetDataLen() );
//	return a;
	memptr a;
	a.Set(	GetPacketBinary().GetAddr(GetHeadLen()), GetDataLen() );

	return a;
}


byte CIpPacket::GetProtocol(void)
{
	return GetFramePtr()->Protocol;
}


byte CIpPacket::GetTimeToLive(void)
{
	return GetFramePtr()->TimeToLive;
}


ipaddr CIpPacket::GetSourIp(void)
{	
	return GetFramePtr()->SourAddr;
}

ipaddr CIpPacket::GetDestIp(void)
{	
	return GetFramePtr()->DestAddr;
}

void CIpPacket::SetSourIp(ipaddr a)
{	
	GetFramePtr()->SourAddr = a;
}

void CIpPacket::SetDestIp(ipaddr a)
{	
	GetFramePtr()->DestAddr = a;
}

bool CIpPacket::IsDoNotFragment(void)
{
	if((toword(GetFramePtr()->Flags_FragmentOffset) & 0x4000) != 0)
		return true;
	else
		return false;
}

void CIpPacket::SetDoNotFragment(bool enable_it)
{
	GetFramePtr()->Flags_FragmentOffset = tonword(toword(GetFramePtr()->Flags_FragmentOffset) & 0xBFFF);
	if(enable_it)
		GetFramePtr()->Flags_FragmentOffset = tonword(toword(GetFramePtr()->Flags_FragmentOffset) + 0x4000);
}

bool CIpPacket::IsMoreFragmentFollow(void)
{
	if((toword(GetFramePtr()->Flags_FragmentOffset) & 0x2000) != 0)
		return true;
	else
		return false;
}

void CIpPacket::SetMoreFragmentFollow(bool enable_it)
{
	GetFramePtr()->Flags_FragmentOffset = tonword(toword(GetFramePtr()->Flags_FragmentOffset) & 0xDFFF);
	if(enable_it)
		GetFramePtr()->Flags_FragmentOffset = tonword(toword(GetFramePtr()->Flags_FragmentOffset) + 0x2000);
}

word CIpPacket::GetFragmentOffset(void)
{
	nword ntemp;
	ntemp = GetFramePtr()->Flags_FragmentOffset;
	return toword(ntemp) & 0x1FFF;
}

void CIpPacket::SetFragmentOffset(word offset)
{
	nword ntemp;
	ntemp = GetFramePtr()->Flags_FragmentOffset;
	ntemp = tonword(toword(ntemp) & 0x1FFF + offset & 0x1FFF);
	GetFramePtr()->Flags_FragmentOffset = ntemp;
}

memptr CIpPacket::GetRealEthiiPktBinary(void)
// the function return a memptr pointing a ethernet packet, whose length 
// of ethernet head plus ip packet.
// BTW: a ethernet packet may contain some padding bytes after IP.
{
	memptr a(EthiiPacket.GetPacketBinary().GetAddr(),
			GetPacketBinary().GetLen() + EthiiPacket.GetHeadLen());
	return a;
}

#endif 
// end of ippkt.h

