// MiniGWNet.cpp
#include "stdafx.h"
#include "netutils.h"
#include "miniGNet.h"
#include "osutils.h"
#include "sndtime.h"

static DWORD s_GWNetDialingFailedCount = 0;

static EDialingDevLineStatus s_LineStatus = ELS_OFFLINE;
static dword s_GWNetLastDailTime = 0;
static bool s_GWNetStartedByUs = false;
static bool *s_UrgentExitFlagPtr = 0;
static DWORD s_MaxIdleTime = 180000;
static DWORD s_LastestPacketTime = 0;

static const unsigned LocalIpListMaxNum = 16;
static DWORD s_LocalIpList[LocalIpListMaxNum];
static bool s_bLocalIpListValid = false;

static BOOL IsFileExist(const char* filepath, const char* filename)
{
	char strFilename[256];
	sprintf(strFilename,"%s\\%s", filepath, filename);

	HANDLE file;
	WIN32_FIND_DATA filedata;
	memset(&filedata, 0, sizeof(WIN32_FIND_DATA));
	
	file = FindFirstFile(strFilename, &filedata);
	
	if(file == INVALID_HANDLE_VALUE)
		return FALSE;

	FindClose(file);
	return TRUE;
}

static bool IsUrgentExit(void)
{
	if (s_UrgentExitFlagPtr) 
	{
		if (*s_UrgentExitFlagPtr) 
		{
			return true;
		};
	}
	return false;
}

static HWND GWNetGetMainFrame(void)
// get Gateway.net main frame handle
// return non-null if successfully
{
	
	SSearchWnd	srchwnd;
	static	DWORD	processID = 0xFFFFFFFF;
	static  HWND	hwnd = NULL;
	
	if(!IsWinNT() || IsWinNT2000())
	{
		DWORD	idtemp;
		if(!IsProcessRunning("WGW.EXE",idtemp))
		{
			hwnd = NULL;
			return hwnd;
		}
		if(processID == idtemp && hwnd != NULL)
			return hwnd;
		processID = idtemp;
	}
	
	sysprint("Find Gateway.net handle...");
	memset(&srchwnd, 0, sizeof(SSearchWnd));
	strcpy(srchwnd.szTitle, "Gateway.net");
	SearchWindow(srchwnd);

	hwnd = srchwnd.hWnd;
	if(hwnd == NULL && processID != 0xFFFFFFFF)
		sysprintln("Failed.");
	else
		sysprintln("End.");
	return hwnd;
}


static HWND GWNetGetSignOnDlg(void)
{
	HWND hGWNetMain = 0;
	HWND hGWNetMdiClient = 0;
	HWND hGWNetSignOnDlg = 0;


	hGWNetMain = GWNetGetMainFrame();

	if (hGWNetMain == 0) return 0;

	hGWNetMdiClient = FindWindowEx(
		hGWNetMain,			// handle to parent window
		NULL,				// handle to a child window
		"MDIClient",		// pointer to class name
		NULL				// pointer to window name
		);

	if (hGWNetMdiClient == 0) return 0;

	// NOTICE: the dlg name is not fixed, sometimes, it is "Sign On",
	// sometimes is "Good bye..."
	hGWNetSignOnDlg = FindWindowEx(
		hGWNetMdiClient,		// handle to parent window
		NULL,				// handle to a child window
		"AOL Child",		// pointer to class name
		NULL				// pointer to window name
		);


	if(GWNET_DEBUG)
	{
		sysprintln("Main = " + hGWNetMain);
		sysprintln("MdiClient = " + hGWNetMdiClient);
		sysprintln("hGWNetSignOnDlg = " + hGWNetSignOnDlg);
	}	
		
	return hGWNetSignOnDlg;
}


static HWND GWNetGetSignOnButton(void)
{
	HWND hSignOnButton;
	HWND hGWNetSignOnDlg = GWNetGetSignOnDlg();
	
	if (hGWNetSignOnDlg == 0) return 0;
	
	int i;
	const int ICONS_NUMBER = 16;
	HWND hButtons[ICONS_NUMBER];
	memset(hButtons, 0, sizeof(hButtons));

	for (i = 0; i < ICONS_NUMBER ; i++)
	{
		hButtons[i] = FindWindowEx(
			hGWNetSignOnDlg,		// handle to parent window
			(i == 0)? 0: hButtons[i - 1],				// handle to a child window
			"_AOL_Icon",		// pointer to class name
			NULL				// pointer to window name
			);

		if (hButtons[i] == 0) break;
		if(GWNET_DEBUG) sysprintln("No." + i + "  " +hButtons[i]);
	}
	

	int rpos = -1;
	long rleft = -1, rbottom = -1;

	for (i = 0; i < ICONS_NUMBER; i++)
	{
		if (hButtons[i] == 0) break;
		
		RECT rt;
		if (GetWindowRect(hButtons[i], &rt) != 0) 
		{
			if (rt.left > rleft && rt.bottom >= rbottom)
			{
				rpos = i; rleft = rt.left; rbottom = rt.bottom;
			}
		
		}

	}

	if (GWNET_DEBUG) sysprintln("rpos = " + rpos);

	// we got the button
	hSignOnButton = hButtons[rpos];

	return hSignOnButton;
}

// this is workaround function to avoid too excessive file accessing
DWORD GWNetGetLocalIP(DWORD *pIpList, unsigned IpListSizeByByte, bool force_to_call)
{
	static int my_index = 0;
	static DWORD my_iplist[LocalIpListMaxNum];
	static DWORD my_ipcount;

	if (force_to_call || my_index == 0)
	{
		memset(my_iplist, 0, sizeof(my_iplist));
		// calling GetLocalIP will access internet cache index file
		my_ipcount = GetLocalIP(my_iplist, sizeof(my_iplist));
		my_index = 0;
	}

	// approximately this will reduce file accessing to every 4 seconds
	my_index = (my_index + 1) % 32;
	memcpy (pIpList, my_iplist, IpListSizeByByte);
	return min(my_ipcount, IpListSizeByByte / sizeof(DWORD));
}

bool GWNetIsDialingReady(void)
{
	if (GWNetGetSignOnButton() != 0) return true;

	return false;
}


static int GWNetClickSignOnButton(void)
{
	HWND hSignOnButton = GWNetGetSignOnButton();

	if (hSignOnButton == 0)
	{
		sysprintln("GWNet: failed to get signon button.");
		return ERR;
	}


	RECT rect_signon;
	GetWindowRect(hSignOnButton, &rect_signon);

	SetActiveWindow(hSignOnButton);
	
	SetCursorPos(rect_signon.right -10, rect_signon.bottom - 10);
	
	if (PostMessage(hSignOnButton, 
		WM_LBUTTONDOWN ,
		MK_LBUTTON,
//		(LPARAM)((rect_signon.left + 10) + ((rect_signon.top + 10)<<16))
		(LPARAM)((rect_signon.right- rect_signon.left)/2 + (((rect_signon.bottom - rect_signon.top)/2)<<16))
		) == 0) return ERR;


	PostMessage(hSignOnButton, 
		WM_LBUTTONUP ,
		0,
		(LPARAM)((rect_signon.right- rect_signon.left)/2 + (((rect_signon.bottom - rect_signon.top)/2)<<16))
		);
	
	return OK;

}


int GWNetSelectLocation(bool to_dial)
{
	HWND hLocationCombo = 0;
	DWORD dr;

	HWND hGWNetSignOnDlg = GWNetGetSignOnDlg();
	if (hGWNetSignOnDlg == 0) return ERR;
	
	int i;
	const int ICONS_NUMBER = 16;
	HWND hCombos[ICONS_NUMBER];
	memset(hCombos, 0, sizeof(hCombos));

	for (i = 0; i < ICONS_NUMBER ; i++)
	{
		hCombos[i] = FindWindowEx(
			hGWNetSignOnDlg,		// handle to parent window
			(i == 0)? 0: hCombos[i - 1],				// handle to a child window
			"_AOL_Combobox",		// pointer to class name
			NULL				// pointer to window name
			);

		if (hCombos[i] == 0) break;
		if (GWNET_DEBUG) sysprintln("No." + i + "  " + hCombos[i]);

	}
	

	int rpos = -1;
	long rbottom = -1;

	for (i = 0; i < ICONS_NUMBER; i++)
	{
		if (hCombos[i] == 0) break;
		
		RECT rt;
		if (GetWindowRect(hCombos[i], &rt) != 0) 
		{
			if (rt.bottom > rbottom)
			{
				rpos = i; rbottom = rt.bottom;
			}
		
		}

	}
	if(GWNET_DEBUG) sysprintln("rpos = " + rpos);

	// we got the hLocationCombo
	hLocationCombo = hCombos[rpos];
	
	int n = SendMessage(hLocationCombo, CB_GETCOUNT, 0, 0);
	if (n <= 0)
	{
		sysprintln("GWNet: invalid speciation.");
		return ERR;
	}

	if (n == 1) return OK;	// only one item on the combo box, so we don't need to choose.

	// In LocationCombo, here are two items, 
	// "HOME" or "ISP/LAN Connection", but we are not able to get the text
	// from it by LB_GETLBTEXT because it is "Owner_Drawing" style
	// and "HASSTRING" is not available.
	
	// We assume: the first item is "HOME", second one is "ISPXXX"
	// So it is a little rough, but we have no better choice so far.

	dr = SendMessage(hLocationCombo, 
		CB_SETCURSEL,
		!to_dial,
		0
		);


	dr = SendMessage(hGWNetSignOnDlg, 
		WM_COMMAND,
		((DWORD)CBN_SELCHANGE)<<16 ,
		(LPARAM)hLocationCombo
		);

	return OK;

}

BOOL GWNetGetSignOffId(UINT & hSignOffId)
{
	char strbuf[128];

	HWND hGWNetMain;

	hGWNetMain = GWNetGetMainFrame();

	if (hGWNetMain == 0) return FALSE;
	HMENU hGWNetMenu = GetMenu(hGWNetMain);
	
	if (hGWNetMenu == 0) return FALSE;

	// Try to get "Access"
	const int signoff_pos = 2;
	strbuf[0] = 0;
	GetMenuString(hGWNetMenu, signoff_pos, strbuf, sizeof(strbuf), MF_BYPOSITION);
	if(GWNET_DEBUG)
	{
		sysprintln("The submenu name string: " + strbuf);
	}

	if (stricmp(strbuf, "&Access") != 0) 
	{
		if(GWNET_DEBUG)	sysprintln("failed to locate signoff submenu --- " + strbuf);
		return FALSE;
	}

	HMENU hSignoffSubMenu = GetSubMenu(hGWNetMenu, signoff_pos);

	if (hSignoffSubMenu == 0) return FALSE;

	
	const int signoff_item_pos = 0;

	hSignOffId = GetMenuItemID(hSignoffSubMenu, signoff_item_pos);

	strbuf[0] = 0;
	GetMenuString(hSignoffSubMenu, hSignOffId, strbuf, sizeof(strbuf), MF_BYCOMMAND);
	if(GWNET_DEBUG)
	{
		sysprintln("The menu item string: " + strbuf);
	}
	
	if (stricmp(strbuf, "&Disconnect") != 0)
	{
		if(GWNET_DEBUG)	sysprintln("failed to locate signoff item --- " + strbuf);
		return FALSE;
	}

	return TRUE;
}

static int GWNetChooseSignOffMenu(void)
{
	HWND hGWNetMain;

	hGWNetMain = GWNetGetMainFrame();

	if (hGWNetMain == 0) return ERR;

	UINT hSignOffId;

	if(GWNetGetSignOffId(hSignOffId) == FALSE) return ERR;

	if (PostMessage(hGWNetMain, WM_COMMAND, hSignOffId, 0)) return OK;

	return ERR;
}


int GWNetSetMiniWindow(bool enable_mini)
{
	HWND hGWNetMain;

	hGWNetMain = GWNetGetMainFrame();

	if (hGWNetMain == 0) return ERR;

	if (enable_mini)
	{
		Sleep (500);
		ShowWindow(hGWNetMain, SW_MINIMIZE);
	}
	else
		ShowWindow(hGWNetMain, SW_RESTORE); 

	return OK;
}


int GWNetMinimizeWindow(void)
{
	HWND hGWNetMain;

	hGWNetMain = GWNetGetMainFrame();

	if (hGWNetMain == 0) return ERR;

	SendMessage(hGWNetMain, WM_SYSCOMMAND, SC_MINIMIZE, 0);

	return OK;
}



int GWNetGetAppPath(char *strbuf, DWORD len)
{
	HKEY	hKey;
	DWORD	dwStatus, RegType;

	dwStatus = RegOpenKeyEx(
				HKEY_LOCAL_MACHINE,		// handle of open key
				"SOFTWARE\\Gateway.net\\Gateway.net\\CurrentVersion",		// address of name of subkey to open
				0,				// reserved  
				KEY_READ,		// security access mask
				&hKey	// address of handle of open key
				);
	if (dwStatus == ERROR_SUCCESS)
	{
		dwStatus = RegQueryValueEx(
						hKey, 
						"AppPath",
						NULL,
						&RegType,
						(BYTE*) strbuf,
						&len
						);
		RegCloseKey(hKey);
		if (dwStatus == ERROR_SUCCESS && len != 0)
		{
			if (IsFileExist(strbuf, "GW.exe"))
				return OK;
		}
	}

	return ERR;
}


bool GWNetIsInstalled(void)
{
	char strbuf[300];
	if (GWNetGetAppPath(strbuf, sizeof(strbuf)) == OK) return true;
	return false;
}

bool GWNetIsRunning(void)
{
	if (GWNetGetMainFrame() != 0) return true;
	return false;
}


int GWNetStartProgram(bool wait_for_signon_button)
{
	if ( ! GWNetIsInstalled() )
	{
		sysprintln("GWNet:(ERROR) Gateway.net is not installed.");
		return ERR;
	}

	if ( GWNetIsRunning() ) return OK;

	char path[300];
	if (GWNetGetAppPath(path, sizeof(path)) != OK) return ERR;

	char GWNet_BINARY_NAME[] = "gw.exe";
	strcat(path, "\\");
	strcat(path, GWNet_BINARY_NAME);

	if (Exec(path, GWNet_BINARY_NAME, false, true) != OK)
	{
		sysprintln("GWNet: failed to start Gateway.net, path=" + path);
		return ERR;
	}

	int i;
	for (i = 10000/200; i > 0; i--)
	{
		sysprint(".");
		if (GWNetIsRunning()) break;
		if (IsUrgentExit()) break;
		SleepEx(200, TRUE);
	}

	if ( GWNetIsRunning() == false ) 
	{
		sysprintln("GWNet: start Gateway.net timeout.");
		return ERR;
	}

	if(GWNET_DEBUG) sysprintln("GWNet: Gateway.net started, waiting for signon.");
	
	if ( ! wait_for_signon_button) return OK;

	for (i = 20000/200; i > 0; i-- )
	{
		if (GWNetIsDialingReady()) return OK;
		if (GWNetIsRunning() == false)
		{
			sysprintln("GWNet: Gateway.net was started but stopped again for unknown reason.");
			return ERR;
		}
		if (IsUrgentExit()) break;
		SleepEx(200, TRUE);
	}

	if ( i <= 0) 
		sysprintln("GWNet:Start(ERR) Gateway.net ready timeout.");

	return ERR;
}


int GWNetStopProgram(bool wait_for_end)
{
	HWND hGWNetMain;

	hGWNetMain = GWNetGetMainFrame();

	if (hGWNetMain == 0) return OK;

	PostMessage(hGWNetMain, WM_CLOSE, 0, 0);

	if (wait_for_end == false) return OK;

	int i;
	for (i = 0; i < 7000/100; i++)
	{
		if (GWNetGetMainFrame() == 0)
		{
			SleepEx(200, TRUE);
			if(GWNET_DEBUG)
				sysprintln("GWNet: Gateway.net stopped.");
			return OK;
		}

		if (IsUrgentExit() && wait_for_end == false) break;

		SleepEx(100, TRUE);
	}

	return ERR;
}


bool GWNetIsOnlineByGui(void)
{
	char strbuf[128];

	HWND hGWNetMain;

	hGWNetMain = GWNetGetMainFrame();

	if (hGWNetMain == 0) return false;

	HMENU hGWNetMenu = GetMenu(hGWNetMain);
	
	if (hGWNetMenu == 0) return false;

	// Try to get "Access"
	const int signoff_pos = 2;
	strbuf[0] = 0;
	GetMenuString(hGWNetMenu, signoff_pos, strbuf, sizeof(strbuf), MF_BYPOSITION);
	if(GWNET_DEBUG)
	{
		sysprintln("The submenu name string: " + strbuf);
	}
	
	if (stricmp(strbuf, "&Access") == 0) 
	{
		// try to find "Disconnect" sub-menu item.
		
		HMENU hGWNetSignOffMenu = GetSubMenu(hGWNetMenu, signoff_pos);
		if (hGWNetSignOffMenu == 0) return false;
		
		int n = GetMenuItemCount(hGWNetSignOffMenu);
		if (n < 0) return false;
		
		int i;
		for (i = 0; i < n; i++)
		{
			strbuf[0] = 0;
			GetMenuString(hGWNetSignOffMenu, i, strbuf, sizeof(strbuf), MF_BYPOSITION);
			if (stricmp(strbuf, "&Disconnect") == 0) return true;
		}
		
		return false;
	}

	if(GWNET_DEBUG)sysprintln("GWNet: IsOnline(UNKNOWN) unknown sign.");

	return false;
}


bool GWNetIsOnlineWithLocalIpList(void)
{
	if (s_bLocalIpListValid == false)
	{
		sysprintln("GWNet: invalid enviroment for local");
		return false;
	}
	
	DWORD iplist[16];
	memset(iplist, 0, sizeof(iplist));
	GWNetGetLocalIP(iplist, sizeof(iplist), false);
	
	bool gotit = false;
	int i = -1;
	int j;
	while (iplist[++i])
	{
		j = -1;
		bool oldone = false;
		while (s_LocalIpList[++j])
		{
			if (iplist[i] == s_LocalIpList[j])
			{
				oldone = true;
				break;
			}
		}
		
		if (!oldone)
		{
			gotit = true;
			break;		
		}
	};
	
	if (gotit)
	{
		return true;
	}
	
	return false;
	
	
}

bool GWNetIsOnline(void)
{
	UINT hSignOffId;
	if(GWNetGetSignOffId(hSignOffId) == FALSE) return false;
	
	if (s_bLocalIpListValid) return GWNetIsOnlineWithLocalIpList();
	
	if (GWNetIsOnlineByGui()) return true;
	
	DWORD iplist[16];
	memset(iplist, 0, sizeof(iplist));
	GWNetGetLocalIP(iplist, sizeof(iplist), false);
	
	bool gotit = false;
	int i = -1;
	while (iplist[++i])
	{
		if ((iplist[i] & 0x0000FFFFL) == 0x0000a8c0) continue;
		if ((iplist[i] & 0x000000FFL) == 0x0000000AL ) continue;
		if ((iplist[i] & 0x000000FFL) == 0x000000A9L ) continue;
		gotit = true;
		break;		
	};
	
	if (gotit)
	{
		return true;
	}

	return false;
}


EDialingDevLineStatus GWNetGetLineStatus(void)
// return EALS_XXXXXXX
{
	if (GWNetIsOnline())
	{
		if (s_LineStatus != ELS_ONLINE && s_LineStatus != ELS_DISCONNECTING)
		{
			s_LineStatus = ELS_ONLINE;
		}
	}
	else
	{
		if (s_LineStatus != ELS_OFFLINE && s_LineStatus != ELS_DIALING)
		{
			s_LineStatus = ELS_OFFLINE;
		}

		if (GWNetIsRunning() == false || ((GetTimeCount() - s_GWNetLastDailTime) > 60000)) s_LineStatus = ELS_OFFLINE;
	}

	return s_LineStatus;
}


int GWNetDial1(bool bAsyncDial = false)
// return OK if successfully
{
	GWNetSetMiniWindow(false);
	
	if (GWNetIsOnline())
	{
		GWNetReportActivity();
		return OK;
	}
	
	s_LineStatus = ELS_OFFLINE;

	// check if Gateway.net has been installed.
	if ( ! GWNetIsInstalled() ) return ERR;

	// Gateway.net is offline.

	if ( GWNetIsRunning() ) 
	{
		s_GWNetStartedByUs = false;
	}
	else
	{
		if(GWNET_DEBUG) sysprintln("Starting Gateway.net...");
		if (GWNetStartProgram(true) != OK)
		{
			sysprintln("GWNet: Dial1(ERROR) failed to start Gateway.net.");
			GWNetStopProgram();
			return ERR;
		}

		s_GWNetStartedByUs = true;
	}
	

	GWNetSelectLocation(true);
	sysprintln("Gateway.net Dialing...");

	if ( GWNetClickSignOnButton() != OK)
	{
		sysprintln("GWNet: Dial(ERROR) failed to sign on, close Gateway.net.");
		GWNetStopProgram();
		return ERR;
	}

	if (s_GWNetStartedByUs) GWNetSetMiniWindow(true);

	s_LineStatus = ELS_DIALING;
	s_GWNetLastDailTime = GetTimeCount();
	
	if(bAsyncDial) return OK;

	int iret = ERR;
	int i;
	for (i = 120*1000/500; i > 0; i--)
	{
		if (GWNetIsOnline())
		{
			if(GWNET_DEBUG) sysprintln("Gateway.net is online.");
			
			SleepEx(1000, TRUE);	// for more reliablity
			if (IsUrgentExit()) break;
			SleepEx(1000, TRUE);	// for more reliablity
			if (IsUrgentExit()) break;
			SleepEx(1000, TRUE);	// for more reliablity
			if (IsUrgentExit()) break;
			
			s_LineStatus = ELS_ONLINE;

			GWNetReportActivity();

			if (s_GWNetStartedByUs) GWNetSetMiniWindow(true);
			iret = OK;
			break;
		}

		if (IsUrgentExit()) break;
		
		SleepEx(500, TRUE);
	}

	if(i == 0)
		s_LineStatus = ELS_OFFLINE;

	return iret;
}

int GWNetDial(bool bAsyncDial)
{
	const int r = GWNetDial1(bAsyncDial);
	
	if (r != OK) s_GWNetDialingFailedCount++;
	else s_GWNetDialingFailedCount = 0;
	
	return r;
}


int GWNetHangup(bool always_close_GWNet, bool wait_for_end)
// return OK if successfully
{
	if (s_GWNetStartedByUs || always_close_GWNet)
	{
powerful_close:
		// minimize window.
		GWNetSetMiniWindow(true);

		sysprintln("Gateway.net hangup...");

		s_LineStatus = ELS_DISCONNECTING;
		
		GWNetStopProgram(wait_for_end);
		
		s_GWNetStartedByUs = false;

		s_LineStatus = ELS_OFFLINE;
		return OK;
	}
	
	// Gateway.net is not started by us, so we should sign off only.
	if (GWNetIsOnline() == false) return OK;

	sysprintln("Gateway.net hangup...");
	
	if (GWNetChooseSignOffMenu() != OK)
	{
		sysprintln("GWNet: Hangup(ERROR) failed to signoff.");
		
		s_LineStatus = ELS_OFFLINE;
		sysprintln("GWNet: Exec generic signoff.");
		goto powerful_close;
		// return ERR;
	}

	s_LineStatus = ELS_DISCONNECTING;

	int i;
	for (i = 20*1000/200; i > 0; i--)
	{
		if ( GWNetIsOnline() == false) break;

		SleepEx(200, TRUE);
	}

	if (i <= 0) 
	{
		sysprintln("GWNet: Hangup(ERROR) sign off timeout.");
		sysprintln("GWNet: Exec generic signoff.");
		goto powerful_close;
	}

	s_LineStatus = ELS_OFFLINE;

	// force to update the cached iplist
	DWORD iplist[16];
	memset(iplist, 0, sizeof(iplist));
	GWNetGetLocalIP(iplist, sizeof(iplist), true);

	return OK;
}


int GWNetInit(int autohangup_max_idletime, bool *urgent_exit_flag_ptr)
// return OK if successfully
{
	s_UrgentExitFlagPtr = urgent_exit_flag_ptr;
	s_MaxIdleTime = autohangup_max_idletime;

	sysprintln("Check Gateway.net Process running...");
	if (GWNetIsRunning())
	{
		s_bLocalIpListValid = false;
		sysprintln("Gateway.net is running");
	}
	else
	{
		GWNetGetLocalIP(s_LocalIpList, sizeof(s_LocalIpList), true);
		s_bLocalIpListValid = true;
		if(GWNET_DEBUG) sysprintln("GWNet: Local IP is valid.");
	}
	sysprintln("end of check...");
	GWNetReportActivity();

	return OK;
}


int GWNetShut(void)
{
	s_UrgentExitFlagPtr = 0;

	memset(s_LocalIpList, 0, sizeof(s_LocalIpList));
	s_bLocalIpListValid = false;
	GWNetReportActivity();
	
	return OK;
}


void GWNetReportActivity(void)
{
	s_LastestPacketTime = GetTimeCount();
}


DWORD GWNetGetIdleTime(void)
{
	if (s_LineStatus == ELS_ONLINE && s_GWNetStartedByUs)
		return (GetTimeCount() - s_LastestPacketTime);

	return 0;
}



int GWNetProcMaintain(void)
{
	DWORD idletime;
	idletime = GWNetGetIdleTime();

	if (idletime > s_MaxIdleTime)
	{
		sysprintln("MaxIdleTime: " + s_MaxIdleTime/1000 +"(s) out, hang up Gateway.net.");

		// do not hang up Gateway.net at all since we do not know Gateway.net 's activities on server machine
		/*
		if (GWNetHangup() != OK)
		{
			serr<<"GWNet: ProcMaintain(ERROR) failed to hang up."<<endl;
		}
		*/
	}
	
	return OK;
}

DWORD GWNetGetDialingFailedCount(void)
/* return the failure count since the lastest success. */
{
	return s_GWNetDialingFailedCount;
}


// end of miniGWNet.cpp
