// miniaol.cpp
#include "stdafx.h"
#include "netutils.h"
#include "miniaol.h"
#include "osutils.h"
#include "sndtime.h"

static DWORD s_AolDialingFailedCount = 0;

static EDialingDevLineStatus s_LineStatus = ELS_OFFLINE;
static dword s_AolLastDialTime = 0;
static bool s_AolStartedByUs = false;
static bool *s_UrgentExitFlagPtr = 0;
static DWORD s_MaxIdleTime = 180000;
static DWORD s_LastestPacketTime = 0;

static const unsigned LocalIpListMaxNum = 16;
static DWORD s_LocalIpList[LocalIpListMaxNum];
static bool s_bLocalIpListValid = false;

static BOOL IsFileExist(const char* filepath, const char* filename)
{
	char strFilename[256];
	sprintf(strFilename,"%s\\%s", filepath, filename);

	HANDLE file;
	WIN32_FIND_DATA filedata;
	memset(&filedata, 0, sizeof(WIN32_FIND_DATA));
	
	file = FindFirstFile(strFilename, &filedata);
	
	if(file == INVALID_HANDLE_VALUE)
		return FALSE;

	FindClose(file);
	return TRUE;
}

static bool IsUrgentExit(void)
{
	if (s_UrgentExitFlagPtr) 
	{
		if (*s_UrgentExitFlagPtr) 
		{
			return true;
		};
	}
	return false;
}

static HWND AolGetMainFrame(void)
// get AOL main frame handle
// return non-null if successfully
{
	SSearchWnd srchwnd;
	static	DWORD	processID = 0xFFFFFFFF;
	static  HWND	hwnd = NULL;
	
	if(!IsWinNT() || IsWinNT2000())
	{
		DWORD	idtemp;
		if(!IsProcessRunning("WAOL.EXE",idtemp))
		{
			hwnd = NULL;
			return hwnd;
		}
		if(processID == idtemp && hwnd != NULL)
			return hwnd;
		processID = idtemp;
	}

	sysprint("Finding AOL handle...");
	memset(&srchwnd, 0, sizeof(SSearchWnd));
	strcpy(srchwnd.szTitle, "America  Online");
//We don't use that because Gateway.net use that also.
//	strcpy(srchwnd.szWindowClass, "AOL Frame25");
	SearchWindow(srchwnd);

	hwnd = srchwnd.hWnd;
	if(hwnd == NULL && processID != 0xFFFFFFFF)
		sysprintln("Failed!");
	else
		sysprintln("End");
	return hwnd;
}


static HWND AolGetSignOnDlg(void)
{
	HWND hAolMain = 0;
	HWND hAolMdiClient = 0;
	HWND hAolSignOnDlg = 0;


	hAolMain = AolGetMainFrame();

	if (hAolMain == 0) return 0;

	hAolMdiClient = FindWindowEx(
		hAolMain,			// handle to parent window
		NULL,				// handle to a child window
		"MDIClient",		// pointer to class name
		NULL				// pointer to window name
		);

	if (hAolMdiClient == 0) return 0;

	// NOTICE: the dlg name is not fixed, sometimes, it is "Sign On",
	// sometimes is "Good bye..."
	hAolSignOnDlg = FindWindowEx(
		hAolMdiClient,		// handle to parent window
		NULL,				// handle to a child window
		"AOL Child",		// pointer to class name
		NULL				// pointer to window name
		);


	if(AOL_DEBUG)
	{
		sysprintln("Main = " + hAolMain);
		sysprintln("MdiClient = " + hAolMdiClient);
		sysprintln("hAolSignOnDlg = " + hAolSignOnDlg);
	}	
		
	return hAolSignOnDlg;
}


//HWND AolGetSignOnButton(HWND h_signon_dlg)
//{
//}

static HWND AolGetSignOnButton(void)
{
	HWND hSignOnButton;
	HWND hAolSignOnDlg = AolGetSignOnDlg();
	
	if (hAolSignOnDlg == 0) return 0;
	
	int i;
	const int ICONS_NUMBER = 16;
	HWND hButtons[ICONS_NUMBER];
	memset(hButtons, 0, sizeof(hButtons));

	for (i = 0; i < ICONS_NUMBER ; i++)
	{
		hButtons[i] = FindWindowEx(
			hAolSignOnDlg,		// handle to parent window
			(i == 0)? 0: hButtons[i - 1],				// handle to a child window
			"_AOL_Icon",		// pointer to class name
			NULL				// pointer to window name
			);

		if (hButtons[i] == 0) break;
		if(AOL_DEBUG) sysprintln("No." + i + "  " +hButtons[i]);
	}
	

	int rpos = -1;
	long rleft = -1, rbottom = -1;

	for (i = 0; i < ICONS_NUMBER; i++)
	{
		if (hButtons[i] == 0) break;
		
		RECT rt;
		if (GetWindowRect(hButtons[i], &rt) != 0) 
		{
			if (rt.left > rleft && rt.bottom >= rbottom)
			{
				rpos = i; rleft = rt.left; rbottom = rt.bottom;
			}
		
		}

	}

	if (AOL_DEBUG) sysprintln("rpos = " + rpos);

	// we got the button
	hSignOnButton = hButtons[rpos];

	return hSignOnButton;
}

// this is workaround function to avoid too excessive file accessing
DWORD AolGetLocalIP(DWORD *pIpList, unsigned IpListSizeByByte, bool force_to_call)
{
	static int my_index = 0;
	static DWORD my_iplist[LocalIpListMaxNum];
	static DWORD my_ipcount;

	if (force_to_call || my_index == 0)
	{
		memset(my_iplist, 0, sizeof(my_iplist));
		// calling GetLocalIP will access internet cache index file
		my_ipcount = GetLocalIP(my_iplist, sizeof(my_iplist));
		my_index = 0;
	}

	// approximately this will reduce file accessing to every 4 seconds
	my_index = (my_index + 1) % 32;
	memcpy (pIpList, my_iplist, IpListSizeByByte);
	return min(my_ipcount, IpListSizeByByte / sizeof(DWORD));
}

bool AolIsDialingReady(void)
{
	if (AolGetSignOnButton() != 0) return true;

	return false;
}


static int AolClickSignOnButton(void)
{
	HWND hSignOnButton = AolGetSignOnButton();

	if (hSignOnButton == 0)
	{
		sysprintln("ACOM: failed to get signon button.");
		return ERR;
	}


	RECT rect_signon;
	GetWindowRect(hSignOnButton, &rect_signon);

	SetActiveWindow(hSignOnButton);
	
	SetCursorPos(rect_signon.right -10, rect_signon.bottom - 10);
	
	if (PostMessage(hSignOnButton, 
		WM_LBUTTONDOWN ,
		MK_LBUTTON,
//		(LPARAM)((rect_signon.left + 10) + ((rect_signon.top + 10)<<16))
		(LPARAM)((rect_signon.right- rect_signon.left)/2 + (((rect_signon.bottom - rect_signon.top)/2)<<16))
		) == 0) return ERR;


	PostMessage(hSignOnButton, 
		WM_LBUTTONUP ,
		0,
		(LPARAM)((rect_signon.right- rect_signon.left)/2 + (((rect_signon.bottom - rect_signon.top)/2)<<16))
		);
	
	return OK;

}


int AolSelectLocation(bool to_dial)
{
	HWND hLocationCombo = 0;
	DWORD dr;

	HWND hAolSignOnDlg = AolGetSignOnDlg();
	if (hAolSignOnDlg == 0) return ERR;
	
	int i;
	const int ICONS_NUMBER = 16;
	HWND hCombos[ICONS_NUMBER];
	memset(hCombos, 0, sizeof(hCombos));

	for (i = 0; i < ICONS_NUMBER ; i++)
	{
		hCombos[i] = FindWindowEx(
			hAolSignOnDlg,		// handle to parent window
			(i == 0)? 0: hCombos[i - 1],				// handle to a child window
			"_AOL_Combobox",		// pointer to class name
			NULL				// pointer to window name
			);

		if (hCombos[i] == 0) break;
		if (AOL_DEBUG) sysprintln("No." + i + "  " + hCombos[i]);

	}
	

	int rpos = -1;
	long rbottom = -1;

	for (i = 0; i < ICONS_NUMBER; i++)
	{
		if (hCombos[i] == 0) break;
		
		RECT rt;
		if (GetWindowRect(hCombos[i], &rt) != 0) 
		{
			if (rt.bottom > rbottom)
			{
				rpos = i; rbottom = rt.bottom;
			}
		
		}

	}
	if(AOL_DEBUG) sysprintln("rpos = " + rpos);

	// we got the hLocationCombo
	hLocationCombo = hCombos[rpos];
	
	int n = SendMessage(hLocationCombo, CB_GETCOUNT, 0, 0);
	if (n <= 0)
	{
		sysprintln("ACOM: invalid speciation.");
		return ERR;
	}

	if (n == 1) return OK;	// only one item on the combo box, so we don't need to choose.

	// In LocationCombo, here are two items, 
	// "HOME" or "ISP/LAN Connection", but we are not able to get the text
	// from it by LB_GETLBTEXT because it is "Owner_Drawing" style
	// and "HASSTRING" is not available.
	
	// We assume: the first item is "HOME", second one is "ISPXXX"
	// So it is a little rough, but we have no better choice so far.

	dr = SendMessage(hLocationCombo, 
		CB_SETCURSEL,
		!to_dial,
		0
		);


	dr = SendMessage(hAolSignOnDlg, 
		WM_COMMAND,
		((DWORD)CBN_SELCHANGE)<<16 ,
		(LPARAM)hLocationCombo
		);

	return OK;

}

BOOL AolGetSignOffId(UINT &hSignOffId)
{
	char strbuf[128];
	HWND hAolMain;

	hAolMain = AolGetMainFrame();

	if (hAolMain == 0) return FALSE;

	HMENU hAolMenu = GetMenu(hAolMain);
	
	if (hAolMenu == 0) return FALSE;

	// Try to get "SignOff"
	int signoff_pos = 0;
	strbuf[0] = 0;
	while(GetMenuString(hAolMenu, signoff_pos, strbuf, sizeof(strbuf), MF_BYPOSITION) != 0)
	{
		if(AOL_DEBUG)
		{
			sysprintln("The submenu name string: " + strbuf);
		}
		if(stricmp(strbuf, "&Sign Off") == 0 || stricmp(strbuf, "Sign &Off") == 0)
			break;
		signoff_pos++;
	}

	if (stricmp(strbuf, "&Sign Off") != 0 && stricmp(strbuf, "Sign &Off") != 0) 
	{
		if(AOL_DEBUG)sysprintln("failed to locate signoff submenu --- " + strbuf);
		return FALSE;
	}

	HMENU hSignoffSubMenu = GetSubMenu(hAolMenu, signoff_pos);

	if (hSignoffSubMenu == 0) return FALSE;

	const int signoff_item_pos = 1;

	hSignOffId = GetMenuItemID(hSignoffSubMenu, signoff_item_pos);

	strbuf[0] = 0;
	GetMenuString(hSignoffSubMenu, hSignOffId, strbuf, sizeof(strbuf), MF_BYCOMMAND);
	if(AOL_DEBUG)
	{
		sysprintln("The menu item string: " + strbuf);
	}
	
	if (stricmp(strbuf, "&Sign Off") != 0 && stricmp(strbuf, "Sign &Off") != 0)
	{
		if(AOL_DEBUG) sysprintln("failed to locate signoff item --- " + strbuf);
		return FALSE;
	}
	
	return TRUE;
}

static int AolChooseSignOffMenu(void)
{

	HWND hAolMain;

	hAolMain = AolGetMainFrame();

	if (hAolMain == 0) return ERR;

	UINT hSignOffId;

	if(AolGetSignOffId(hSignOffId) == FALSE)
		return ERR;

	if (PostMessage(hAolMain, WM_COMMAND, hSignOffId, 0)) return OK;

	return ERR;
}


int AolSetMiniWindow(bool enable_mini)
{
	HWND hAolMain;

	hAolMain = AolGetMainFrame();

	if (hAolMain == 0) return ERR;

	if (enable_mini)
	{
		Sleep (500);
		ShowWindow(hAolMain, SW_MINIMIZE);
	}
	else
		ShowWindow(hAolMain, SW_RESTORE); 

	return OK;
}


int AolMinimizeWindow(void)
{
	HWND hAolMain;

	hAolMain = AolGetMainFrame();

	if (hAolMain == 0) return ERR;

	//ShowWindow(hAolMain, SW_MINIMIZE);
	SendMessage(hAolMain, WM_SYSCOMMAND, SC_MINIMIZE, 0);

	return OK;
}



int AolGetAppPath(char *strbuf, DWORD len)
{
	HKEY	hKey;
	DWORD	dwStatus, RegType;

	// try to open AOL 5.0 first
	dwStatus = RegOpenKeyEx(
				HKEY_LOCAL_MACHINE,		// handle of open key
				"Software\\America Online\\America Online\\CurrentVersion",		// address of name of subkey to open
				0,				// reserved  
				KEY_READ,		// security access mask
				&hKey	// address of handle of open key
				);
	if (dwStatus == ERROR_SUCCESS)
	{
		dwStatus = RegQueryValueEx(
						hKey, 
						"AppPath",
						NULL,
						&RegType,
						(BYTE*) strbuf,
						&len
						);
		RegCloseKey(hKey);
		if (dwStatus == ERROR_SUCCESS && len != 0)
		{
			if (IsFileExist(strbuf, "Aol.exe"))
				return OK;
		}
	}

	// try this key
	dwStatus = RegOpenKeyEx(
				HKEY_LOCAL_MACHINE,		// handle of open key
				"Software\\America Online\\AOL\\CurrentVersion",		// address of name of subkey to open
				0,				// reserved  
				KEY_READ,		// security access mask
				&hKey	// address of handle of open key
				);
	if (dwStatus == ERROR_SUCCESS)
	{
		dwStatus = RegQueryValueEx(
						hKey, 
						"AppPath",
						NULL,
						&RegType,
						(BYTE*) strbuf,
						&len
						);
		RegCloseKey(hKey);
		if (dwStatus == ERROR_SUCCESS && len != 0)
		{
			if (IsFileExist(strbuf, "Aol.exe"))
				return OK;
		}
	}

	// try AOL 4.0
	dwStatus = RegOpenKeyEx(
				HKEY_LOCAL_MACHINE,		// handle of open key
				"Software\\America Online\\America Online\\4.0",		// address of name of subkey to open
				0,				// reserved  
				KEY_READ,		// security access mask
				&hKey	// address of handle of open key
				);
	if (dwStatus == ERROR_SUCCESS)
	{
		dwStatus = RegQueryValueEx(
						hKey, 
						"AppPath",
						NULL,
						&RegType,
						(BYTE*) strbuf,
						&len
						);
		RegCloseKey(hKey);
		if (dwStatus == ERROR_SUCCESS && len != 0)
		{
			if (IsFileExist(strbuf, "Aol.exe"))
				return OK;
		}
	}
	
	return ERR;
}


bool AolIsInstalled(void)
{
	char strbuf[300];
	if (AolGetAppPath(strbuf, sizeof(strbuf)) == OK) return true;
	return false;
}

bool AolIsRunning(void)
{
	if(AolGetMainFrame() != 0) return true;
	return false;
}


int AolStartProgram(bool wait_for_signon_button)
{
	if ( ! AolIsInstalled() )
	{
		sysprintln("ACOM:(ERROR) AOL 4.0 is not installed.");
		return ERR;
	}

	if ( AolIsRunning() ) return OK;

	char path[300];
	if (AolGetAppPath(path, sizeof(path)) != OK) return ERR;

	char AOL_BINARY_NAME[] = "aol.exe";
	strcat(path, "\\");
	strcat(path, AOL_BINARY_NAME);

	if (Exec(path, AOL_BINARY_NAME, false, true) != OK)
	{
		sysprintln("ACOM: failed to start AOL, path=" + path);
		return ERR;
	}

	int i;
	for (i = 10000/200; i > 0; i--)
	{
		if (AolIsRunning()) break;
		if (IsUrgentExit()) break;
		SleepEx(200, TRUE);
	}

	if ( AolIsRunning() == false ) 
	{
		sysprintln("ACOM: start AOL timeout.");
		return ERR;
	}

	if(AOL_DEBUG) sysprintln("ACOM: AOL started, waiting for signon.");
	
	if ( ! wait_for_signon_button) return OK;

	for (i = 20000/200; i > 0; i-- )
	{
		if (AolIsDialingReady()) return OK;
		if (AolIsRunning() == false)
		{
			sysprintln("ACOM: AOL was started but stopped again for unknown reason.");
			return ERR;
		}
		if (IsUrgentExit()) break;
		SleepEx(200, TRUE);
	}

	if ( i <= 0) 
		sysprintln("ACOM:Start(ERR) AOL ready timeout.");

	return ERR;
}


int AolStopProgram(bool wait_for_end)
{
	HWND hAolMain;

	hAolMain = AolGetMainFrame();

	if (hAolMain == 0) return OK;

	PostMessage(hAolMain, WM_CLOSE, 0, 0);

	if (wait_for_end == false) return OK;

	int i;
	for (i = 0; i < 7000/100; i++)
	{
		if (AolGetMainFrame() == 0)
		{
			SleepEx(200, TRUE);
			if(AOL_DEBUG)
				sysprintln("ACOM: AOL stopped.");
			return OK;
		}

		if (IsUrgentExit() && wait_for_end == false) break;

		SleepEx(100, TRUE);
	}

	return ERR;
}


bool AolIsOnlineByGui(void)
{
	char strbuf[128];

	HWND hAolMain;

	hAolMain = AolGetMainFrame();

	if (hAolMain == 0) return false;

	HMENU hAolMenu = GetMenu(hAolMain);
	
	if (hAolMenu == 0) return false;

	// Try to get "SignOff"
	int signoff_pos = 0;
	strbuf[0] = 0;
	while(GetMenuString(hAolMenu, signoff_pos, strbuf, sizeof(strbuf), MF_BYPOSITION) != 0)
	{
		if(AOL_DEBUG)
		{
			sysprintln("The submenu name string: " + strbuf);
		}
		if(stricmp(strbuf, "&Sign Off") == 0 || stricmp(strbuf, "Sign &Off") == 0)break;
		if(stricmp(strbuf, "&Sign On") == 0 || stricmp(strbuf, "Sign &On") == 0)break;
		signoff_pos ++;
	}
	
	if (stricmp(strbuf, "&Sign Off") == 0 || stricmp(strbuf, "Sign &Off") == 0) 
	{
		// try to find "Sign Off" sub-menu item.
		
		HMENU hAolSignOffMenu = GetSubMenu(hAolMenu, signoff_pos);
		if (hAolSignOffMenu == 0) return false;
		
		int n = GetMenuItemCount(hAolSignOffMenu);
		if (n < 0) return false;
		
		int i;
		for (i = 0; i < n; i++)
		{
			strbuf[0] = 0;
			GetMenuString(hAolSignOffMenu, i, strbuf, sizeof(strbuf), MF_BYPOSITION);
			if (stricmp(strbuf, "&Sign Off") == 0 || stricmp(strbuf, "Sign &Off") == 0) return true;
		}
		
		return false;
	}
	else if (stricmp(strbuf, "&Sign On") == 0 || stricmp(strbuf, "Sign &On") == 0)
	{
		return false;
	}

	if(AOL_DEBUG)sysprintln("ACOM: IsOnline(UNKNOWN) unknown sign.");

	return false;
}


bool AolIsOnlineWithLocalIpList(void)
{
	if (s_bLocalIpListValid == false)
	{
		sysprintln("ACOM: invalid enviroment for local");
		return false;
	}
	
	DWORD iplist[16];
	memset(iplist, 0, sizeof(iplist));
	AolGetLocalIP(iplist, sizeof(iplist), false);
	
	bool gotit = false;
	int i = -1;
	int j;
	while (iplist[++i])
	{
		j = -1;
		bool oldone = false;
		while (s_LocalIpList[++j])
		{
			if (iplist[i] == s_LocalIpList[j])
			{
				oldone = true;
				break;
			}
		}
		
		if (!oldone)
		{
			gotit = true;
			break;		
		}
	};
	
	if (gotit)
	{
		return true;
	}
	
	return false;
	
	
}

bool AolIsOnline(void)
{
	UINT	hsignoffid;
	if(AolGetSignOffId(hsignoffid) == FALSE) return false;

	if (s_bLocalIpListValid) return AolIsOnlineWithLocalIpList();
	
	if (AolIsOnlineByGui()) return true;
	
	DWORD iplist[16];
	memset(iplist, 0, sizeof(iplist));
	AolGetLocalIP(iplist, sizeof(iplist), false);
	
	bool gotit = false;
	int i = -1;
	while (iplist[++i])
	{
		if ((iplist[i] & 0x0000FFFFL) == 0x0000a8c0) continue;
		if ((iplist[i] & 0x000000FFL) == 0x0000000AL ) continue;
		if ((iplist[i] & 0x000000FFL) == 0x000000A9L ) continue;
		gotit = true;
		break;		
	};
	
	if (gotit)
	{
		return true;
	}

	return false;
}


EDialingDevLineStatus AolGetLineStatus(void)
// return EALS_XXXXXXX
{
	if (AolIsOnline())
	{
		if (s_LineStatus != ELS_ONLINE && s_LineStatus != ELS_DISCONNECTING)
		{
			s_LineStatus = ELS_ONLINE;
		}
	}
	else
	{
		if (s_LineStatus != ELS_OFFLINE && s_LineStatus != ELS_DIALING)
		{
			s_LineStatus = ELS_OFFLINE;
		}


		if (AolIsRunning() == false || ((GetTimeCount() - s_AolLastDialTime) > 60000)) s_LineStatus = ELS_OFFLINE;
		
	}

	return s_LineStatus;
}


int AolDial1(bool bAsyncDial = false)
// return OK if successfully
{
	AolSetMiniWindow(false);
	
	if (AolIsOnline())
	{
		AolReportActivity();
		return OK;
	}
	
	s_LineStatus = ELS_OFFLINE;

	// check if AOL has been installed.
	if ( ! AolIsInstalled() ) return ERR;

	// AOL is offline.

	if ( AolIsRunning() ) 
	{
		s_AolStartedByUs = false;
	}
	else
	{
		if(AOL_DEBUG) sysprintln("Starting AOL...");
		if (AolStartProgram(true) != OK)
		{
			sysprintln("ACOM: Dial1(ERROR) failed to start AOL.");
			AolStopProgram();
			return ERR;
		}

		s_AolStartedByUs = true;
	}
	

	AolSelectLocation(true);
	sysprintln("AOL Dialing...");

	if ( AolClickSignOnButton() != OK)
	{
		sysprintln("ACOM: Dial(ERROR) failed to sign on, close AOL.");
		AolStopProgram();
		return ERR;
	}

	if (s_AolStartedByUs) AolSetMiniWindow(true);

	s_LineStatus = ELS_DIALING;
	s_AolLastDialTime = GetTimeCount();
	if(bAsyncDial) return OK;

	int iret = ERR;
	int i;
	for (i = 120*1000/500; i > 0; i--)
	{
		if (AolIsOnline())
		{
			if(AOL_DEBUG) sysprintln("AOL is online.");
			
			SleepEx(1000, TRUE);	// for more reliablity
			if (IsUrgentExit()) break;
			SleepEx(1000, TRUE);	// for more reliablity
			if (IsUrgentExit()) break;
			SleepEx(1000, TRUE);	// for more reliablity
			if (IsUrgentExit()) break;
			
			s_LineStatus = ELS_ONLINE;

			AolReportActivity();

			if (s_AolStartedByUs) AolSetMiniWindow(true);
			iret = OK;
			break;
		}

		if (IsUrgentExit()) break;
		
		SleepEx(500, TRUE);
	}

	if(i == 0)
		s_LineStatus = ELS_OFFLINE;

	return iret;
}

int AolDial(bool bAsyncDial)
{
	const int r = AolDial1(bAsyncDial);
	
	if (r != OK) s_AolDialingFailedCount++;
	else s_AolDialingFailedCount = 0;
	
	return r;
}


int AolHangup(bool always_close_aol, bool wait_for_end)
// return OK if successfully
{
	if (s_AolStartedByUs || always_close_aol)
	{
powerful_close:
		// minimize window.
		AolSetMiniWindow(true);

		sysprintln("AOL hangup...");

		s_LineStatus = ELS_DISCONNECTING;
		
		AolStopProgram(wait_for_end);
		
		s_AolStartedByUs = false;

		s_LineStatus = ELS_OFFLINE;
		return OK;
	}
	
	// AOL is not started by us, so we should sign off only.
	if (AolIsOnline() == false) return OK;

	sysprintln("AOL hangup...");
	
	if (AolChooseSignOffMenu() != OK)
	{
		sysprintln("ACOM: Hangup(ERROR) failed to signoff.");
		
		s_LineStatus = ELS_OFFLINE;
		sysprintln("ACOM: Exec generic signoff.");
		goto powerful_close;
		// return ERR;
	}

	s_LineStatus = ELS_DISCONNECTING;

	int i;
	for (i = 20*1000/200; i > 0; i--)
	{
		if ( AolIsOnline() == false) break;

		SleepEx(200, TRUE);
	}

	if (i <= 0) 
	{
		sysprintln("ACOM: Hangup(ERROR) sign off timeout.");
		sysprintln("ACOM: Exec generic signoff.");
		goto powerful_close;
	}

	s_LineStatus = ELS_OFFLINE;

	// force to update the cached iplist
	DWORD iplist[16];
	memset(iplist, 0, sizeof(iplist));
	AolGetLocalIP(iplist, sizeof(iplist), true);

	return OK;
}


int AolInit(int autohangup_max_idletime, bool *urgent_exit_flag_ptr)
// return OK if successfully
{
	s_UrgentExitFlagPtr = urgent_exit_flag_ptr;
	s_MaxIdleTime = autohangup_max_idletime;

	sysprintln("Check AOL status...");
	if (AolIsRunning())
	{
		s_bLocalIpListValid = false;
		sysprintln("AOL is running");
	}
	else
	{
		AolGetLocalIP(s_LocalIpList, sizeof(s_LocalIpList), true);
		s_bLocalIpListValid = true;
		if(AOL_DEBUG) sysprintln("ACOM: Local IP is valid.");
	}
	sysprintln("end of check...");
	AolReportActivity();

	return OK;
}


int AolShut(void)
{
	s_UrgentExitFlagPtr = 0;

	memset(s_LocalIpList, 0, sizeof(s_LocalIpList));
	s_bLocalIpListValid = false;
	AolReportActivity();
	
	return OK;
}


void AolReportActivity(void)
{
	s_LastestPacketTime = GetTimeCount();
}


DWORD AolGetIdleTime(void)
{
	if (s_LineStatus == ELS_ONLINE && s_AolStartedByUs)
		return (GetTimeCount() - s_LastestPacketTime);

	return 0;
}



int AolProcMaintain(void)
{
	DWORD idletime;
	idletime = AolGetIdleTime();

	if (idletime > s_MaxIdleTime)
	{
		sysprintln("MaxIdleTime: " + s_MaxIdleTime/1000 +"(s) out, hang up AOL.");

		// do not hang up AOL at all since we do not know AOL's activities on server machine
		/*
		if (AolHangup() != OK)
		{
			serr<<"ACOM: ProcMaintain(ERROR) failed to hang up."<<endl;
		}
		*/
	}
	
	return OK;
}

DWORD AolGetDialingFailedCount(void)
/* return the failure count since the lastest success. */
{
	return s_AolDialingFailedCount;
}


// end of miniaol.cpp
