// Ras.cpp: implementation of the CMiniRas class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************

03/06/1998		Created by Yadong Zhang


--*/



#include "stdafx.h"
#include "winsvc.h"
#include "ipaddr.h"

#include "miniras.h"
#include "process.h"

#define MAX_ACTIVE_RAS	6

typedef DWORD APIENTRY DEF_RasEnumConnections( LPRASCONN, LPDWORD, LPDWORD );
typedef DWORD APIENTRY DEF_RasGetConnectStatus( HRASCONN, LPRASCONNSTATUS );
typedef DWORD APIENTRY DEF_RasGetProjectionInfo( HRASCONN, RASPROJECTION, LPVOID,
                   LPDWORD );
typedef DWORD APIENTRY DEF_RasDial( LPRASDIALEXTENSIONS, LPSTR, LPRASDIALPARAMS, DWORD,
                   LPVOID, LPHRASCONN );
typedef DWORD APIENTRY DEF_RasHangUp( HRASCONN );

typedef DWORD APIENTRY DEF_RasEnumEntries( LPSTR, LPSTR, LPSYGATERASENTRYNAME, LPDWORD,
                   LPDWORD );

typedef DWORD APIENTRY DEF_RasGetEntryDialParams( LPSTR, LPRASDIALPARAMSA, LPBOOL );

typedef DWORD APIENTRY DEF_RasGetEntryProperties(LPCTSTR, LPCTSTR, LPRASENTRY, LPDWORD, LPBYTE, LPDWORD);


// RASCREDENTIALS 'dwMask' values.
#define RASCM_UserName       0x00000001
#define RASCM_Password       0x00000002
#define RASCM_Domain         0x00000004
//	END OF COPY
/////////////////////////////////////////////////////////
#define RAS_MaxEntryName_9x	20
#define RAS_MaxEntryName_NT	256

#pragma	pack(push, 4)
typedef struct _sygaterasname9x{
	DWORD dwSize;
    CHAR  szEntryName[RAS_MaxEntryName_9x + 1];
}SYGATERASENTRYNAME9X, *LPSYGATERASENTRYNAME9X;

typedef struct _sygaterasnament{
	DWORD dwSize;
	CHAR  szEntryName[RAS_MaxEntryName_NT + 1];
}SYGATERASENTRYNAMENT,LPSYGATERASENTRYNAMENT;
#pragma	pack(pop, 4)

// for RASAPI32.DLL
static HINSTANCE				hRasDll = 0;
static unsigned					RasDllUsageCount = 0;

static DEF_RasEnumConnections	*ptr_RasEnumConnections = 0 ;
static DEF_RasGetConnectStatus	*ptr_RasGetConnectStatus = 0;
static DEF_RasGetProjectionInfo	*ptr_RasGetProjectionInfo = 0;
static DEF_RasDial				*ptr_RasDial = 0;
static DEF_RasHangUp			*ptr_RasHangUp = 0;
static DEF_RasEnumEntries		*ptr_RasEnumEntries = 0;
static DEF_RasGetEntryDialParams *ptr_RasGetEntryDialParams = 0;
static DEF_RasGetEntryProperties *ptr_RasGetEntryProperties = 0;

// Win95 OSR2 doesn't support RasGetCredentials.
// static DEF_RasGetCredentials	*ptr_RasGetCredentials = 0;

unsigned _stdcall AsycDial(void *ptr )
{
	CMiniRas * pMin = (CMiniRas *)ptr;
	return pMin->AsycDialing();
}
//**************** NOTICE !!!!!!! ********************
// These RasAPIs is for ***ANSI*** code 

int StartRasService(void)
{
    SC_HANDLE   schService = 0;
    SC_HANDLE   schSCManager = 0;
	int retcode;

	if (!IsWinNT()) return OK;	// RAS will be always started under win95.
	retcode = ERR;
	schSCManager = OpenSCManager(NULL, NULL, GENERIC_EXECUTE | GENERIC_READ);
	if(schSCManager)
	{
		SERVICE_STATUS ssStatus;

		schService = OpenService(schSCManager, TEXT("RasMan\0"), SERVICE_ALL_ACCESS);
		if(schService)
		{
			if(QueryServiceStatus(schService, &ssStatus))
			{
				if(ssStatus.dwCurrentState == SERVICE_STOPPED)
				{
					sysprintln("Start Ras Service...");
					StartService(schService, 0, NULL);
				}
			}
			CloseServiceHandle(schService);
			retcode = OK;
		}
		CloseServiceHandle(schSCManager);	
	}

	return retcode;
}
int TestRasService(bool wait_service_started)
// return OK if started
// return ERR if service wasn't installed or wait_time_out.
{
    SC_HANDLE   schService = 0;
    SC_HANDLE   schSCManager = 0;
	SERVICE_STATUS ssStatus;
	static bool bInstallRas = true;
	int retcode;

	if (!IsWinNT()) return OK;	// RAS will be always started under win95.
	
	retcode = ERR;
	schSCManager = OpenSCManager(
					NULL,                   // machine (NULL == local)
					NULL,                   // database (NULL == default)
					GENERIC_READ			// access required
					);
	if ( schSCManager ) 
	{
		schService = OpenService(schSCManager, TEXT("RasMan\0"), GENERIC_READ);
        
		if (schService)
		{
			sysprint("Waiting for RAS service  ");
			// wait for RAS starting, 180 seconds.
			int i;
			bool igotit = false;
			StartRasService();
			for (i = 0; i < 180*2; i++) 
			{
				if (QueryServiceStatus( schService, &ssStatus ))
				{
	                if ( ssStatus.dwCurrentState == SERVICE_RUNNING )
					{
						igotit = true;  break;
					}
				}
				if (!wait_service_started)
				{
					sysprintln("ras has not started, exit right away ");
					break;
				}
				
				if (i & 1) sysprint('.');		// every one second.
				SleepEx(500, TRUE);
			}
			
			if (!igotit) 
			{
				sysprintln("It is timeout to wait for ras started." +
					"NOTE: Under WinNT, Remote access autodial manager service shall" +
					" be Automatic in startup option. To do that, reboot and try again.");
			}
			else sysprintln("done");
			retcode = (igotit)?OK:ERR;
		}
		else
		{
			// RAS was not installed.
			if(bInstallRas) // control the error log only output once.
			{
				sysprintln("miniras: RAS was not installed."); 
				bInstallRas = false;
			}
		}

	}
	else
	{
		sysprintln("miniras: fail to open servicemanager.");
	}

	if (schService) CloseServiceHandle(schService);

	if (schSCManager) CloseServiceHandle(schSCManager);

	return retcode;
}


int LoadRas(bool wait_for_ras_started)
// return OK if successful.
{
	RasDllUsageCount++;
	if (hRasDll == 0)
	{
		// MODIFIED BY YADONG
		// In Windows 2000, LoadLibrary rasapi32.dll will start RAS service automatically
		// if it is stopped.
		if ( ! IsWinNT2000() )
		{
			if (TestRasService(wait_for_ras_started) != OK) return ERR;
		}

		hRasDll = LoadLibrary("rasapi32.dll");
		if (hRasDll == 0) return ERR;
		bool b;
		b = (  (ptr_RasEnumConnections = (DEF_RasEnumConnections*) GetProcAddress(hRasDll, "RasEnumConnectionsA"))
			&& (ptr_RasGetConnectStatus = (DEF_RasGetConnectStatus*) GetProcAddress(hRasDll, "RasGetConnectStatusA"))
			&& (ptr_RasGetProjectionInfo = (DEF_RasGetProjectionInfo*) GetProcAddress(hRasDll, "RasGetProjectionInfoA"))
			&& (ptr_RasDial = (DEF_RasDial*) GetProcAddress(hRasDll, "RasDialA"))
			&& (ptr_RasHangUp = (DEF_RasHangUp*) GetProcAddress(hRasDll, "RasHangUpA"))
			&& (ptr_RasEnumEntries = (DEF_RasEnumEntries*) GetProcAddress(hRasDll, "RasEnumEntriesA"))
			&& (ptr_RasGetEntryDialParams = (DEF_RasGetEntryDialParams*) GetProcAddress(hRasDll, "RasGetEntryDialParamsA"))
			&& (ptr_RasGetEntryProperties = (DEF_RasGetEntryProperties *) GetProcAddress(hRasDll, "RasGetEntryPropertiesA"))
//			&& (ptr_RasGetCredentials = (DEF_RasGetCredentials*) GetProcAddress(hRasDll, "RasGetCredentialsA"))
			);
		if (b == false) 
		{
			// some functions required cannot be located.
			return ERR;
		}

	}
	return OK;
}


int UnloadRas(void)
// return OK if successful.
{
	if (RasDllUsageCount == 0) goto unloaddll;
	
	RasDllUsageCount--;
	if (RasDllUsageCount) return OK;

unloaddll:
	if (hRasDll)
	{
		FreeLibrary(hRasDll); hRasDll = 0;
	}

	return OK;
}


DWORD MiniRas_RasEnumEntries ( 
			LPSTR reserved,       // reserved, must be NULL
			LPSTR lpszPhonebook,  // pointer to full path and filename of 
			LPSYGATERASENTRYNAME lprasentryname,		// buffer to receive phone-book entries
			LPDWORD lpcb,           // size in bytes of buffer
			LPDWORD lpcEntries      // number of entries written to buffer
			)
{
	DWORD ret;
	*lpcEntries = 0;
	
	if (LoadRas(FALSE) != OK) 
	{
		return 0;	// we still treat it as success.
	}
	
	if(IsWin9x())
	{
		DWORD dwNum	= (*lpcb/sizeof(SYGATERASENTRYNAME));
		DWORD dwNewcb = dwNum * sizeof(SYGATERASENTRYNAME9X);

		SYGATERASENTRYNAME9X *p = new SYGATERASENTRYNAME9X[dwNum];
		
		p->dwSize = sizeof(SYGATERASENTRYNAME9X);

		ret = (*ptr_RasEnumEntries)(reserved, lpszPhonebook, (SYGATERASENTRYNAME*)p, &dwNewcb, lpcEntries);

		memset(lprasentryname,0,*lpcb);

		for(DWORD i=0;i<*lpcEntries;i++)
		{
			memcpy(lprasentryname+i,p+i,sizeof(SYGATERASENTRYNAME9X)); 
		}

		delete p;

	}
	else if(IsWinNT4())
	{
		DWORD dwNum	= (*lpcb/sizeof(SYGATERASENTRYNAME));
		DWORD dwNewcb = dwNum * sizeof(SYGATERASENTRYNAMENT);

		SYGATERASENTRYNAMENT *p = new SYGATERASENTRYNAMENT[dwNum];
		
		p->dwSize = sizeof(SYGATERASENTRYNAMENT);

		ret = (*ptr_RasEnumEntries)(reserved, lpszPhonebook,(SYGATERASENTRYNAME*)p, &dwNewcb, lpcEntries);

		memset(lprasentryname,0,*lpcb);

		for(DWORD i=0;i<*lpcEntries;i++)
		{
			memcpy(lprasentryname+i,p+i,sizeof(SYGATERASENTRYNAMENT)); 
		}

		delete p;
	}
	else
		ret = (*ptr_RasEnumEntries)(reserved, lpszPhonebook, lprasentryname, lpcb, lpcEntries);

	UnloadRas();
	return ret;
}

DWORD MiniRas_RasEnumConnections(
			LPRASCONN lprasconn,   // buffer to receive connections data
			LPDWORD lpcb,          // size in bytes of buffer
			LPDWORD lpcConnections // number of connections written to buffer
			)
{
	DWORD ret;
	*lpcConnections = 0;
	
	if (LoadRas(false) != OK) return 0;	// we still treat it as success.
	
	ret = (*ptr_RasEnumConnections)(lprasconn, lpcb, lpcConnections);

	UnloadRas();
	return ret;
}
			
DWORD MiniRas_RasGetEntryDialParams(
			char *lpszPhonebook, 			// pointer to the full path and filename // of the phone-book file 
			RASDIALPARAMS* lprasdialparams,// pointer to a structure that receives // the connection parameters 
			BOOL *lpfPassword 				// indicates whether the user's password 
			)
{
	DWORD ret;
	*lpfPassword = false;
	
	if (LoadRas(false) != OK) return 0;	// we still treat it as success.
	
	ret = (*ptr_RasGetEntryDialParams)(
		lpszPhonebook, lprasdialparams, lpfPassword
			);

	UnloadRas();
	return ret;
}

ostream& operator<<(ostream& os, CMiniRas &mr)
{
	os<<"RAS Entry: "<<mr.RasEntryName<<", IdleTimeout: "<<mr.IdleTimeout;
	mr.Sync();
	if (mr.IsOnline()) os<<" On Line ";
	else os<<" Off Line ";
	
	return os;
}


CMiniRas::CMiniRas(char *entry_name, char *username, char *password, dword idle_timeout, bool *urgent_exit_flag_ptr )
{

	bItWorks = false;
	bIsDialing = false;
	bIsHanging = false;

	RasEntryName[0] = 0; RasUserName[0] = RasPassword[0] = 0;
	IdleTimeout = 0; Ip = 0; hRas = 0; pUrgentExitFlag = 0;
	
	if (LoadRas(true) != OK) return ;

	strcpy(RasEntryName, entry_name);

	if (username) strcpy(RasUserName, username);
	if (password) strcpy(RasPassword, password);
	
	IdleTimeout = idle_timeout;
	
	pUrgentExitFlag = urgent_exit_flag_ptr;

	LineStatus = ELS_OFFLINE;

	DialingFailedCount = 0;

	ReportActive();
	
	bItWorks = true;
}

CMiniRas::~CMiniRas(void)
{
//	if (hRas) Close();	// hang up.
	bItWorks = false;

	memset(RasEntryName, 0, sizeof(RasEntryName));
//	memset(RasUserName, 0, sizeof(RasUserName));
//	memset(RasPassword, 0, sizeof(RasPassword));
	IdleTimeout = 0;

	UnloadRas();
}

void CMiniRas::ResetEntryName(char *entry_name,BOOL bRefresh)
{

	if(!bItWorks) return;
	if(strcmp(RasEntryName, entry_name) != 0)
	{
		strcpy(RasEntryName, entry_name);
		DialingFailedCount = 0;
		LineStatus = ELS_OFFLINE;
		Ip = 0;
		ReportActive();
		hRas = NULL;
	}
	Sync(bRefresh);
}

void CMiniRas::CheckRas(void)
//Synchronize the special RAS entry.
// set hRas to zero if disconnected
// set hRas to non-zero if connected.
{
	if (hRasDll == 0) return ;
	
	RASCONNSTATUS	connstatus;
	DWORD			cConnections;
	DWORD			cb;
	unsigned		i;
	DWORD			r;
	RASCONN actras[MAX_ACTIVE_RAS];
	
	if (hRas)
	{
		connstatus.dwSize = sizeof(RASCONNSTATUS);
		r = (*ptr_RasGetConnectStatus)(
				hRas,			// handle to RAS connection of interest
				&connstatus				// buffer to receive status data
				);
		
		if (r == ERROR_INVALID_HANDLE) hRas = NULL;
	}
		
	// check if we have opened it.
	if (hRas == NULL)
	{
		// we need check the exist connection
		actras[0].dwSize = sizeof(RASCONN);
		cb = sizeof(actras);
		cConnections = 0;

		r = (*ptr_RasEnumConnections)(
				actras,			// buffer to receive connections data
				&cb,			// size in bytes of buffer
				&cConnections	// number of connections written to buffer
				);

		if ( r != 0)
		{
			sysprintln("Fail to RasEnumConnections, retcode=" + r);
			Ip = 0;
			return;
		}
		if(cConnections == 0)
		{
			Ip = 0;
			return;
		}

		strncpy(RasCurrentOnlineEntryName, actras[0].szEntryName, sizeof(RasCurrentOnlineEntryName));
		RasCurrentOnlineEntryName[sizeof(RasCurrentOnlineEntryName) - 1] = 0;

		if(strcmp(RasEntryName,"null") == 0)
			i = 0; //Set it as current online entry.
		else
			for (i = 0; i < cConnections; i++)
			{
				// detect wheather this EntryName is online.
				if (strcmp(actras[i].szEntryName, RasEntryName) == 0)
					break;
			}
		
		if (i >= cConnections) 
		{
			// There isn't connection we are care of.
			hRas = NULL; Ip = toipaddr(0);
			LineStatus = ELS_OFFLINE;
			return ;
		}

		if(RAS_DEBUG)
		{
			sysprint(actras[i].szEntryName + " connection was found, at no." + i);
			sysprintln(", device:" + actras[i].szDeviceName);
		}
		
		hRas = actras[i].hrasconn;
	}
}
int CMiniRas::Sync(BOOL bRefresh)
// Synchronize the special RAS entry.
// set hRas to zero if disconnected
// set hRas to non-zero if connected
// return OK if function succeeds.
// return ERR if failed.
{
	if (hRasDll == 0)
		return ERR;
	
	RASPPPIP		pppip;
	RASCONNSTATUS	connstatus;
	DWORD			cConnections;
	DWORD			cb;
	unsigned		i;
	DWORD			r=0;
	BOOL			bIpDirty;
	#define MAX_ACTIVE_RAS	6
	RASCONN actras[MAX_ACTIVE_RAS];
	static RASCONN LastRasConn[MAX_ACTIVE_RAS];
	static DWORD dwLastRasEntry;
	
	bIpDirty = FALSE;

	if (hRas)
	{
		connstatus.dwSize = sizeof(RASCONNSTATUS);
		r = (*ptr_RasGetConnectStatus)(
				hRas,			// handle to RAS connection of interest
				&connstatus				// buffer to receive status data
				);
		
		if (r == ERROR_INVALID_HANDLE) hRas = NULL;
	}
		
	// check if we have opened it.
	if (hRas == NULL)
	{
		// we need check the exist connection
		if (bRefresh)
		{
			actras[0].dwSize = sizeof(RASCONN);
			cb = sizeof(actras);
			cConnections = 0;

			r = (*ptr_RasEnumConnections)(
					actras,			// buffer to receive connections data
					&cb,			// size in bytes of buffer
					&cConnections	// number of connections written to buffer
					);

			if ( r != 0)
			{
				sysprintln("Fail to RasEnumConnections, retcode=" + r);
				Ip = 0;
				return ERR;
			}
			dwLastRasEntry=cConnections;
			if (cConnections)
			{
				memcpy(LastRasConn,actras,sizeof(actras));
			}
			
		}
		else
		{
			cConnections=dwLastRasEntry;
			if (cConnections)
			{
				memcpy(actras,LastRasConn,sizeof(actras));
			}
		}
		
		if(cConnections == 0)
		{
			Ip = 0;
			return OK;
		}
		strncpy(RasCurrentOnlineEntryName, actras[0].szEntryName, sizeof(RasCurrentOnlineEntryName));
		RasCurrentOnlineEntryName[sizeof(RasCurrentOnlineEntryName) - 1] = 0;

		if(strcmp(RasEntryName,"null") == 0)
			i = 0; //Set it as current online entry.
		else
			for (i = 0; i < cConnections; i++)
			{
				// detect wheather this EntryName is online.
				if (strcmp(actras[i].szEntryName, RasEntryName) == 0)
					break;
			}
		
		if (i >= cConnections) 
		{
			// There isn't connection we are care of.
			hRas = NULL; Ip = toipaddr(0);
			LineStatus = ELS_OFFLINE;
			return OK;
		}

		if(RAS_DEBUG)
		{
			sysprint(actras[i].szEntryName + " connection was found, at no." + i);
			sysprintln(", device:" + actras[i].szDeviceName);
		}
		
		hRas = actras[i].hrasconn;

		bIpDirty = TRUE;
		
	}

	/* let's check the status of Ras */
	do
	{
		
		connstatus.dwSize = sizeof(RASCONNSTATUS);
		r = (*ptr_RasGetConnectStatus)(
				hRas,			// handle to RAS connection of interest
				&connstatus				// buffer to receive status data
				);
		
		if (r == ERROR_INVALID_HANDLE)
		{
			// the hRas is invalid now, 
			//if (DEBUG)
			sysprintln("RAS handle invalid, disconnected.");
			
			Close();
			return OK;
		}

		if (r != 0)
		{
			sysprintln("Sync(ERR):FAIL to call RASGETConnectStatus, retcode=" + r);
			Close();
			return ERR;
		}
		// printf("RasGetConnectStatus ret:%u\n", r);
		// printf("status = %u\n", connstatus.rasconnstate);
		
		if (connstatus.rasconnstate == RASCS_Connected) break;

		if (connstatus.rasconnstate == RASCS_Disconnected)
		{
			// RasHangUp(pPrink->hRas);
			// pPrink->hRas = NULL;
			if(RAS_DEBUG) sysprintln("RAS_disconnected detected, ");
			
			//Close();
			return OK;
		}
		
		bIpDirty = TRUE;
		ReportActive();
		
		if (NeedToExitAtOnce()) { return OK; }
		
		// sdbg<<'.';
		SleepEx(100, TRUE);		// share CPU with others
	} while (connstatus.rasconnstate != RASCS_Connected);

	// printf("connected, last time, %lu\n", pPrink->PppPktLastestTime);
	
	pppip.dwSize = sizeof(RASPPPIP);
	cb = sizeof(pppip);
	
	r = (*ptr_RasGetProjectionInfo)(
			hRas,	// handle that specifies remote access connection of interest
			RASP_PppIp,		// specifies type of projection information to obtain
			&pppip,			// points to buffer that receives projection information
			&cb				// points to variable that specifies buffer size
			);
	if (r != 0) 
	{
		sysprintln("Sync(ERR): Failed to RasProjectionInfo");
		Close();
		return ERR;
	}
	
	if (pppip.dwError != 0)
	{
		sysprintln("Sync(WRN): The RAS connection doesn't support IP");
		Close();
		return ERR;
	}

	Ip = pppip.szIpAddress;

	if(RAS_DEBUG)
	{
		if (bIpDirty)
			sysprintln("RAS Connected, IP = " + Ip);	
	}

	LineStatus = ELS_ONLINE;

	return OK;
}



static bool s_bRasDailCompleted;
static DWORD s_dwError;

VOID WINAPI RasDialCallBackFunc1(
  HRASCONN hrasconn,  // handle to RAS connection
  UINT unMsg,         // type of event that has occurred
  RASCONNSTATE rascs, // connection state about to be entered
  DWORD dwError,      // error that may have occurred  
  DWORD dwExtendedError)
{
/*	if (dwError == ERROR_PORT_OR_DEVICE) PostEvent(EET_DD_OPEN_DEVICE_FAILURE, EEL_INFO);

	if (dwError == ERROR_AUTHENTICATION_FAILURE) PostEvent(EET_DD_AUTHETICATION_FAILURE, EEL_ERROR);

	if (dwError == ERROR_NO_DIALTONE) PostEvent(EET_DD_NO_DIALTONE, EEL_ERROR);

	if (dwError == ERROR_LINE_BUSY) PostEvent(EET_DD_LINE_BUSY, EEL_ERROR);

	if (dwError == ERROR_NO_ANSWER) PostEvent(EET_DD_NO_ANSWER, EEL_ERROR);

	switch (rascs)
	{

	case RASCS_ConnectDevice:
		if (dwError == 0) PostEvent(EET_DD_OPEN_DEVICE, EEL_INFO);
		break;

	case RASCS_DeviceConnected:
		if (dwError == 0) PostEvent(EET_DD_DEVICE_CONNECTED, EEL_INFO);
		break;

	case RASCS_Authenticate:
		if (dwError == 0) PostEvent(EET_DD_AUTHENTICATING, EEL_INFO);
		break;
	
	case RASCS_Connected:
		if (dwError == 0) PostEvent(EET_DD_CONNECTED, EEL_INFO);
		break;

	case RASCS_Disconnected:
		PostEvent(EET_DD_DISCONNECTED, EEL_INFO);
		break;
	}*/


	if(RAS_DEBUG) sysprintln("<RAS" + rascs + ',' + dwError + ',' + dwExtendedError + '>');
	
  	if (dwError) s_dwError = dwError;
  	
	if (rascs == RASCS_Connected || rascs == RASCS_Disconnected
		|| ( rascs == RASCS_AuthNotify && dwError )
		|| ( dwError )	
		)
	{
		s_bRasDailCompleted = true;
	}
}

int CMiniRas::AsycDialing(void)
{
	DWORD		r;
	RASDIALPARAMS	rasparm;

	memset(&rasparm, 0, sizeof(rasparm));
	
	rasparm.dwSize = sizeof(RASDIALPARAMS);
	strcpy(rasparm.szEntryName, RasEntryName);

	rasparm.szPhoneNumber[0]	= 0;
	rasparm.szCallbackNumber[0]	= 0;
	strcpy(rasparm.szUserName, RasUserName);
	strcpy(rasparm.szPassword, RasPassword);
	rasparm.szDomain[0]			= 0;

	// we have set the username and password?
	if(RasUserName[0] == 0 || RasPassword[0] == 0)
//	if (RasUserName[0] == 0 || RasPassword[0] == 0 || (!IsWinNT()) )
	{
		// for Win95, we always try to get username and password
		// dynamatically
		BOOL b_passwd_retrieved ;

		b_passwd_retrieved = true;
		r = (*ptr_RasGetEntryDialParams)(NULL, &rasparm, &b_passwd_retrieved);
		if (r != 0)
		{
			sysprintln("Failed to get primary entry's params");
			bIsDialing = false;
			return ERR;
		}
	}

	if (rasparm.szUserName[0] == 0) sysprintln("Warning: ras username may be invalid");
	if (rasparm.szPassword[0] == 0) sysprintln("Warning: ras password may be invalid");

//	PostEvent(EET_DD_DIALING, EEL_INFO, 0, rasparm.szEntryName);
	
	sysprintln("Dialing ... ");
	if(RAS_DEBUG) sysprintln("username: " + rasparm.szUserName + ", password: xxxxxx");
	
	s_bRasDailCompleted	= false;
	s_dwError = 0;
	HRASCONN	hTempRas;
	hTempRas = NULL;
	// we need dial
	r = (*ptr_RasDial)(
			NULL,				// pointer to function extensions data
			NULL,				// pointer to full path and filename of phone-book file
			&rasparm,			// pointer to calling parameters data
			1,					// specifies type of RasDial event handler 
			RasDialCallBackFunc1,// specifies a handler for RasDial events
			&hRas				// pointer to variable to receive connection handle
			);

	// wait for RasDial
	for (int i = 0; i < 120*10; i++)
	{
		if (s_bRasDailCompleted) break;
		// sdbg<<'.'<<flush;
		if (NeedToExitAtOnce())
		{
			bIsDialing = false;
			//leave the RAS dialing.
			return ERR;
		}
		SleepEx(100, TRUE);
	}

	bIsDialing = false;
	ReportActive();
	return OK;
}

int CMiniRas::DialRas(bool bAsycDial)
{
	int	iRet;
	if(bIsDialing) return ERR;
	bIsDialing = true;
	if(bAsycDial)
	{
		unsigned ThreadId;
		iRet = OK;
		if((HANDLE)_beginthreadex(NULL,	0, AsycDial,this,0,&ThreadId) == NULL)
			iRet = ERR;
	}
	else
	{
		iRet = GoOnline1();
		bIsDialing = false;
	}
	return iRet;
}

int CMiniRas::GoOnline1(void)
// return OK if it is connected.
// return ERR if failed.
{
	DWORD		r;
	int			a;
	RASDIALPARAMS	rasparm;
	int i;


	
	a = Sync();
	if (a != OK) 
	{
		sysprintln("GotoOnlineRas(ERR): fail to SyncMiniRas");
		return ERR;
	}
	if (hRas) 
	{
		return OK;
	}
	// we need check the exist connection

	// NOTICE: MUST fill rasparm with zero
	// because RasGetEntryDialParams only fill those 
	// items it know what should be filled in.
	
	// for instance, under Windows 95, RasGetEntryDialParams
	// won't fill PhoneNumber.
	// So RasDial won't work.

	memset(&rasparm, 0, sizeof(rasparm));
	
	rasparm.dwSize = sizeof(RASDIALPARAMS);
	strcpy(rasparm.szEntryName, RasEntryName);

	rasparm.szPhoneNumber[0]	= 0;
	rasparm.szCallbackNumber[0]	= 0;
	strcpy(rasparm.szUserName, RasUserName);
	strcpy(rasparm.szPassword, RasPassword);
	rasparm.szDomain[0]			= 0;

	// we have set the username and password?
	if(RasUserName[0] == 0 || RasPassword[0] == 0)
//	if (RasUserName[0] == 0 || RasPassword[0] == 0 || (!IsWinNT()) )
	{
		// for Win95, we always try to get username and password
		// dynamatically
		BOOL b_passwd_retrieved ;

		b_passwd_retrieved = false;
		r = (*ptr_RasGetEntryDialParams)(NULL, &rasparm, &b_passwd_retrieved);
		if (r != 0)
		{
			sysprintln("Failed to get primary entry's params");
			return ERR;
		}
		if (!b_passwd_retrieved)
		{
			sysprintln("CheckRasAccount: failed to recv p");
		}
	}

	if (rasparm.szUserName[0] == 0) sysprintln("Warning: ras username may be invalid");
	if (rasparm.szPassword[0] == 0) sysprintln("Warning: ras password may be invalid");

//	PostEvent(EET_DD_DIALING, EEL_INFO, 0, rasparm.szEntryName);
	
	sysprintln("Dialing ... ");
	if(RAS_DEBUG) sysprintln("username: " + rasparm.szUserName +", password: xxxxxx");
	
	ReportActive();
	
	LineStatus = ELS_DIALING;
	
	s_bRasDailCompleted	= false;
	s_dwError = 0;
	
	hRas = NULL;
	// we need dial
	r = (*ptr_RasDial)(
			NULL,				// pointer to function extensions data
			NULL,				// pointer to full path and filename of phone-book file
			&rasparm,			// pointer to calling parameters data
			1,					// specifies type of RasDial event handler 
			RasDialCallBackFunc1,// specifies a handler for RasDial events
			&hRas				// pointer to variable to receive connection handle
			);
	/*++if (r != 0)
	{
		serr<<"GotoOnlineRas(ERR): FAIL to call RasDial, with "<<rasparm.szEntryName
			<<endl;
		(*ptr_RasHangUp)(hRas);
		return ERR;
	}--*/
	
	// wait for RasDial
	for (i = 0; i < 120*10; i++)
	{
		if (s_bRasDailCompleted) break;
		// sdbg<<'.'<<flush;
		if (NeedToExitAtOnce())
		{
			Close();
			return ERR;
		}
		SleepEx(100, TRUE);
	}

	// sdbg<<s_bRasDailCompleted<<endl;
	
	ReportActive();

	// SynCMiniRas, it will do something for reinitialize
	if (Sync() != OK)
	{
		sysprintln("GoOnlineRas(ERR): Failed to Sync");
	}
	else
	{
		// Sync successfully
		if (hRas)
		{
			if(RAS_DEBUG) sysprintln("RAS Connected, IP = " + Ip);
			
			DialingFailedCount = 0;

//			PostEvent(EET_DD_CONNECTED, EEL_INFO, 0, rasparm.szEntryName);

			return OK;
		}
	
		sysprintln("MiniRas: dialing failed, errcode = " + s_dwError);
	}

	DialingFailedCount++;

	return ERR;
}


int CMiniRas::Close(bool bAsyncHangup)
// return OK if succeeds.
{
	DWORD r;
	RASCONNSTATUS	connstatus;
	int i;
	
	Ip = 0;
	if (hRas == NULL)
	{
		// printf("the RAS conn has been closed, nothing to do.\n");
		LineStatus = ELS_OFFLINE;
		return OK;
	}

	
//	PostEvent(EET_DD_DISCONNECTING, EEL_INFO);

	sysprint("Hanging up the RAS...");
	bIsHanging = true;
	LineStatus = ELS_DISCONNECTING;

	r = (*ptr_RasHangUp)(hRas); 
	if ( r != 0 && r != ERROR_INVALID_HANDLE)
	{
		sysprintln("CloseRas: Failure to RasHangUp.");
		hRas = 0, Ip = toipaddr(0);
		bIsHanging = false;
		return ERR;
	}

	Ip = toipaddr(0);

	if(bAsyncHangup)
	{
		hRas = NULL;
		sysprintln("done.(Async)");
		LineStatus = ELS_OFFLINE;
		bIsHanging = false;
		return OK;
	}

	// wait for a while 
	SleepEx(1000, TRUE);

	// The RAS connection state machine needs time to properly terminate.
	// wait 5 seconds at most.
	for (i = 0; i < 50; i++)
	{
		connstatus.dwSize = sizeof(RASCONNSTATUS);
		r = (*ptr_RasGetConnectStatus)(
				hRas,			// handle to RAS connection of interest
				&connstatus				// buffer to receive status data
				);
		
		// when ISP hang up the line or line error,
		// Windows 95 will pop up a "Reconnect?" box,
		// if the user press "reconnect", 
		// r won't be ERROR_INVALID_HANDLE until the line is down again.
		// So, we need to judge if this case occured.
		if (r == ERROR_INVALID_HANDLE) break;
		if (connstatus.rasconnstate == RASCS_Connected) break;
		if (NeedToExitAtOnce()) break;

		SleepEx(100, TRUE);
	}

	LineStatus = ELS_OFFLINE;
	bIsHanging = false;
	hRas = NULL; 
	sysprintln("done.");

//	PostEvent(EET_DD_DISCONNECTED, EEL_INFO);

	return OK;
}


void CMiniRas::ProcMaintain(void)
{
	if(bIsDialing)return;
	if (Sync() != OK) sysprintln("MiniRas:ProcMaintain(ERR) fail to sync"); 
	//sdbg<<"Idletime = "<<GetIdleTime()<<endl;
	if (IsOnline()) 
	{
		if (GetIdleTime() > IdleTimeout)
		{
			sysprintln("== Idle Timeout ==");
			if (Close() != OK) sysprintln("MiniRas:ProcMaintain(ERR) fail to close");
		}
	}
}

int MiniRas_GetRasAccount(char *entry_name, char *username_buf, char *password_buf, BOOL bAlert)
{
	// let's check if RAS username and password have any changes.
	RASDIALPARAMS	rasparm;
	memset(&rasparm, 0, sizeof(rasparm));
	rasparm.dwSize = sizeof(RASDIALPARAMS);
	strcpy(rasparm.szEntryName, entry_name);
	
	BOOL b_passwd_retrieved = false;
	int r;
	
	r = MiniRas_RasGetEntryDialParams(NULL, &rasparm, &b_passwd_retrieved);
	if (r != 0)
	{
		// some wrong occured, we failed to get.
		sysprintln("CheckRasAccount " + entry_name + " : failed to get rasparam");
		return ERR;
	}

	if (!b_passwd_retrieved) 
	{
		if(bAlert)//if caller pass the username in, we won't print below message again.
			sysprintln("CheckRasAccount " + entry_name + " : failed to recv p");
		return ERR;
	}
	
	if (username_buf) strcpy(username_buf, rasparm.szUserName);
	if (password_buf) strcpy(password_buf, rasparm.szPassword);
	
	return OK;
}

int MiniRas_GetRasDeviceType(char * entryname, char * devicetype_buf)
{
	int ret = ERR;
	if (entryname == NULL || devicetype_buf == NULL) return ret;
	if (LoadRas(false) != OK) return ret;	// we still treat it as success.

	if (ptr_RasGetEntryProperties != NULL)
	{
		RASENTRY *	rasentry;
		DWORD entrysize;

		entrysize = 0;
		ret = (*ptr_RasGetEntryProperties)(NULL, NULL, NULL , &entrysize, NULL, NULL);
		if(entrysize != 0)
		{
			rasentry = (RASENTRY *)new char[entrysize];
			
			// Added by Philip Qi
			// According to MSDN, dwSize MUST be set to sizeof(RASENTRY)
			// so that the raspapi32.dll could check version

			rasentry->dwSize = sizeof(RASENTRY);
			ret = (*ptr_RasGetEntryProperties)(NULL, entryname, rasentry, &entrysize, NULL, NULL);
			if(ret == 0)
			{
				strcpy(devicetype_buf, rasentry->szDeviceType);
				ret = OK;
			}
			else if(ret == ERROR_BUFFER_TOO_SMALL)
			{
				// If RasGetEntryProperties complain buffer is too small
				// create a new one use the required buffer size
				// return error if fail
				sysprintln("RasGetEntryProperties compain buffer is too small, buffer needed is "+entrysize);

				delete rasentry;

				rasentry = (RASENTRY *)new char[entrysize];
				rasentry->dwSize = sizeof(RASENTRY);
				ret = (*ptr_RasGetEntryProperties)(NULL, entryname, rasentry, &entrysize, NULL, NULL);
				if(ret == 0)
				{
					strcpy(devicetype_buf, rasentry->szDeviceType);
					ret = OK;
				}
				else
				{
					sysprintln("ERROR: when get device type again error code " + ret);
					ret = ERR;
				}
			}
			else
			{
				sysprintln("ERROR: when get device type error code " + ret);
				ret = ERR;
			}
			delete rasentry;
		}
	}

	UnloadRas();
	return ret;
}

int MiniRas_IsRasDllOK()
{
	if (IsWinNT4())
	{
		char caBuf[MAX_PATH];
		if (GetSystemDirectory(caBuf,MAX_PATH)==0) return ERR_NOT_FOUND;
		strcat(caBuf,"\\RasApi32.dll");
		HANDLE hf=CreateFile(caBuf,        // file name
			GENERIC_READ,                       // access mode
			FILE_SHARE_DELETE,					// share mode
			NULL,								// SD					
			OPEN_EXISTING,						// how to create
			FILE_ATTRIBUTE_NORMAL,				// file attributes
			NULL);
		if (hf==INVALID_HANDLE_VALUE) return ERR_NOT_FOUND;
		FILETIME ctime,atime,wtime;
		BOOL bRet=GetFileTime(
		  hf,                 // handle to file
		  &ctime,    // creation time
		  &atime,  // last access time
		  &wtime    // last write time
		);
		CloseHandle(hf);
		if (bRet==0) return ERR;
		SYSTEMTIME st;
		bRet=FileTimeToSystemTime(
			&wtime,  // file time to convert
			&st   // receives system time
		);
		if (bRet==0) return ERR;
		if (st.wYear>=1998 ) return OK;
		return ERR;

	}
	return OK;

  
}
// end of miniras.cpp
