// Ras.h: interface for the CMiniRas class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************

03/06/1998		Created by Yadong Zhang

--*/



#ifndef MINIRAS_H
#define MINIRAS_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "osutils.h"
#include "ras.h"
#include "raserror.h"
#include "sndtime.h"
#include "dunline.h"


typedef struct _sygaterasname{
	DWORD dwSize;
    CHAR  szEntryName[RAS_MaxEntryName + 1];
    DWORD dwFlags;
    CHAR  szPhonebookPath[MAX_PATH + 1];
}SYGATERASENTRYNAME, *LPSYGATERASENTRYNAME;

int LoadRas(bool wait_for_ras_started);
// return OK if successful.
int UnloadRas(void);
// return OK if successful.


class CMiniRas  
{
private:
	bool			bItWorks;
	
	char			RasEntryName[RAS_MaxEntryName + 1];
	char			RasUserName[UNLEN + 1];
	char			RasPassword[PWLEN + 1];
	dword			IdleTimeout; // second

	char			RasCurrentOnlineEntryName[RAS_MaxEntryName + 1];

	bool			*pUrgentExitFlag;

	//Below two variant is pathc, We need change these in future!!!!
	bool			bIsDialing;
	bool			bIsHanging;

	HRASCONN		hRas;
	ipaddr			Ip;

	dword			LastestPacketTime;	// ms

	EDialingDevLineStatus  LineStatus;

	dword			DialingFailedCount;

	int DialRas(bool bAsycDial);

	int Sync(BOOL bRefresh=FALSE);

	inline bool NeedToExitAtOnce(void) { return (pUrgentExitFlag)?(*pUrgentExitFlag):false; }

public:
	CMiniRas(char *entry_name, char *username, char *password, dword idle_timeout, bool *urgent_exit_flag_ptr = 0);
	// if username or password was set to null or empty string, 
	// The dial of miniras will try to get username and password from 
	// system automatically.
	// WARNNING: NT service uses a localsystem account generally,
	// it CAN NOT get a username and password automatically, when 
	// CMiniRas is used in a NT service, unless, the service is running
	// under a user account(like administrator) nor localsystem.
	
	~CMiniRas(void);

	inline bool ItWorks(void){return bItWorks;}
	void ResetEntryName(char *entry_name="null",BOOL bRefresh=FALSE);

	int GoOnline1(void);
	int	AsycDialing(void);
	
	void ProcMaintain(void);

	void CheckRas(void);

	int Close(bool bASyncHangup = false);
	inline bool IsOnline(void) { return (hRas != 0); };

	inline int GoOnline(bool bAsycDial = false) { return (IsOnline())?OK:DialRas(bAsycDial); }
	// return OK if it is connected.
	// return ERR if failed.
	
	inline int GoOffline(bool bAsycHangup = false) { return Close(bAsycHangup); }

	EDialingDevLineStatus GetLineStatus(void) { return LineStatus; }

	char *GetCurOnlnieEntryName(void) { return RasCurrentOnlineEntryName; }
	char *GetEntryName(void) { return RasEntryName; }

	inline void ReportActive(void); 
	// called by NetPort, report a packet arrived,
	// miniras decide if it shall hang up due to idle time out
	// account to it.

	inline dword GetIdleTime(void) { return(IsOnline())?(GetTimeCount() - LastestPacketTime )/1000 : 0; }
	// return seconds of idle.

	inline dword GetDialingFailedCount(void) { return DialingFailedCount; }

	inline ipaddr GetIp(void) { return Ip; }

	inline bool IsDialing(void) {return bIsDialing;}
	inline bool IsHanging(void)	{return bIsHanging;}

	friend inline bool operator!(CMiniRas &mr) { return !mr.bItWorks;}
	// return true if miniras doesn't work.

	friend ostream& operator<<(ostream& os, CMiniRas &mr);
};


void CMiniRas::ReportActive(void)
{
	LastestPacketTime = GetTimeCount();
}

DWORD MiniRas_RasEnumEntries ( 
			LPSTR reserved,       // reserved, must be NULL
			LPSTR lpszPhonebook,  // pointer to full path and filename of 
			LPSYGATERASENTRYNAME lprasentryname,		// buffer to receive phone-book entries
			LPDWORD lpcb,           // size in bytes of buffer
			LPDWORD lpcEntries      // number of entries written to buffer
			);
// the function is same as RasEnumEntries, except the RasDLL will be loaded 
// dynamaticlly.
DWORD MiniRas_RasEnumConnections(
			LPRASCONN lprasconn,   // buffer to receive connections data
			LPDWORD lpcb,          // size in bytes of buffer
			LPDWORD lpcConnections // number of connections written to buffer
			);


DWORD MiniRas_RasGetEntryDialParams(
			char *lpszPhonebook, 			// pointer to the full path and filename // of the phone-book file 
			RASDIALPARAMS* lprasdialparams,// pointer to a structure that receives // the connection parameters 
			BOOL *lpfPassword 				// indicates whether the user's password 
			);

int MiniRas_GetRasDeviceType(
			char * entry_name,
			char * devicetype_buf);
//This function only support Win95 OSR2 or later and winnt.
//return OK if success
//return ERR if failed.

int MiniRas_GetRasAccount(
			char *entry_name, 
			char *username_buf, 
			char *password_buf,
			BOOL	bAlert = TRUE
			);
// NOTICE: it MUST be called by a process with a user account
// Generally, a service own only a localsystem account, which
// cannot take effect.
// if bAlert is TRUE, this function will output when fail to get username and password.
// username_buf or password_buf may be null.
// return OK if no problem
// return ERR if failed
int MiniRas_IsRasDllOK();
//  in NT4 ,if you install NT4 without RAS , then install SP4 or high, then install RAS , RAS API will leak memory .
//  this function check NT4 , RASAPI32.dll modify date is late then 8/15/1998
//return OK if this DLL is ok or other OS
//return ERR if failed
//


#endif // !defined(AFX_RAS_H__DD822920_D2BA_11D1_946D_0020AF923BD8__INCLUDED_)
