// Pingpkt.h: interface for the CPingPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/



#if !defined(AFX_PINGPKT_H__92F39161_E51F_11D1_980D_00C026807807__INCLUDED_)
#define AFX_PINGPKT_H__92F39161_E51F_11D1_980D_00C026807807__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "hashobj.h"

#include "Packet.h"
#include "icmppkt.h"


//////////////////////////////////////////
//         ERROR MESSAGE
#define PING_ERR_ISNT_MINE		0x2
///////////////////////////////////////////


/*===================================================
 PING is ICMP message, type ECHO/ECHO REPLY
 It is defined in RFC 792

    0                   1                   2                   3
    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     Type      |     Code      |          Checksum             |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |           Identifier          |        Sequence Number        |
   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   |     Data ...
   +-+-+-+-+-

===================================================*/

#define PROTO_ICMP_TYPE_PING_REQUEST		0x8
#define PROTO_ICMP_TYPE_PING_REPLY			0x0

#include "packon1.h"
typedef struct
{
	word		Identifier;
	word		SeqNumber;
} SPing_FrameHead;
#include "packoff1.h"



class CPingPacket : public CPacket, public CObjForHash
{
public:
	CIcmpPacket		IcmpPacket;
	
	inline unsigned HashKey(void);	// deliving from CObjForHash

	inline CPingPacket(void) {};
	inline CPingPacket(CIcmpPacket &ipp);
	
	inline int Set(CIcmpPacket &ipp);

	inline SPing_FrameHead* GetFramePtr(void);
	// return the Ping frame pointer

//	inline unsigned GetHeadLen(void);
	// return the tcp header's length

//	inline unsigned GetDataLen(void);
	// return the tcp's data length;

//	inline memptr GetDataSection(void);
	// return the tcp's data section

	inline CIcmpPacket& GetIcmpPkt(void) { return IcmpPacket; }
	inline CIpPacket& GetIpPkt(void) { return GetIcmpPkt().GetIpPkt(); }

	friend ostream& operator<<(ostream& os, CPingPacket &tp);
	friend outputstr operator +(const outputstr& ops, CPingPacket &pp);

	inline bool IsRequest(void) { return GetIcmpPkt().GetType() == PROTO_ICMP_TYPE_PING_REQUEST; }
	// return true if the packet is a request packet.
	// return false if the packet is a reply packet.

	inline word GetIdentifier(void) { return GetFramePtr()->Identifier; }
	inline word GetSeqNumber(void) { return GetFramePtr()->SeqNumber; }
};


CPingPacket::CPingPacket(CIcmpPacket& icpp)
{
	Set(icpp);
}


int CPingPacket::Set(CIcmpPacket& icpp)
// return OK if it is our packet
// return ICMP_ERR_ISNT_MINE if not.
{
 	switch( icpp.GetType())
	{
	case PROTO_ICMP_TYPE_PING_REQUEST:
	case PROTO_ICMP_TYPE_PING_REPLY:
		{
			memptr p = icpp.GetDataSection();
			if (p.GetLen() < sizeof (SPing_FrameHead))
				return ICMP_ERR_ISNT_MINE;

			CPacket::Set(icpp, p);

			IcmpPacket = icpp;
			return OK;
		}	
	default:
		;
	}

	return ICMP_ERR_ISNT_MINE;
}

SPing_FrameHead* CPingPacket::GetFramePtr(void)
{
	return (SPing_FrameHead*)(GetPacketBinary().GetAddr());
}


unsigned CPingPacket::HashKey(void)
{
	// NOTICE: it is temporiate.
	// normally, the condition should be something look like following:
	// if (CEngineCore::IsOutgoingNet(GetNicId()) then ...
	if (GetIpPkt().GetEthiiPkt().GetNicId() != NICID_TOB)
	{
		// it is from the net outgoing
		return GetIpPkt().GetSourIp().getraw();
	}

	// it is from Intranet.
	return GetIpPkt().GetDestIp().getraw();
}


int MakePingReply(CPingPacket* ping_request);
// return OK if successfully
// the function modify the ping request packet to ping reply



#endif // !defined(AFX_PINGPKT_H__92F39161_E51F_11D1_980D_00C026807807__INCLUDED_)
