// file PKTFIFO.CPP

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/
/*++
History:

04/01/1999			Created by Yadong Zhang

--*/


#include "stdafx.h"
#include "pktfifo.h"

#define FIFO_SIGN	'PKFI'
#define PKT_SIGN	'ELPS'

void PktFifoReset(SPacketFifo *ppf)
{
	//char tempstr[128];
	ppf->FifoSign = 0;
	ppf->Bottom = ppf->BufLen;
	ppf->Head = ppf->Tail = 0;
	ppf->FreeBytes = ppf->BufLen;
	
	ppf->FifoResetCount++;
	/*
	sprintf(tempstr, "\nFifo RS:(%d), MiniFree(%d)\n", 
		ppf->FifoResetCount, ppf->MinFreeBytes);
	WriteSysLog(tempstr);
	*/

	ppf->FifoSign = FIFO_SIGN;
}

int PktFifoOpen(SPacketFifo *ppf, dword BufSize)
// return 0 if sucess
{
	ppf->FifoSign = 0;  ppf->BufLen = 0;
	ppf->pBuf = new char[BufSize];
	if (ppf->pBuf == 0) return -1;
	memset(ppf->pBuf, 0, BufSize);
	ppf->BufLen = ppf->Bottom = BufSize;
	ppf->Head = ppf->Tail = 0;
	ppf->PktId = 0;

	ppf->PutBytes = ppf->GetBytes = 0;
	ppf->FreeBytes = ppf->BufLen;
	ppf->MinFreeBytes = ppf->FreeBytes;
	ppf->PutCount = ppf->PutSuccessCount = 0;
	ppf->GetCount = ppf->GetSuccessCount = ppf->GetNoPktCount = 0;
	ppf->FifoResetCount = 0;
	
	ppf->FifoSign = FIFO_SIGN;
	return 0;
}


int PktFifoClose(SPacketFifo *ppf)
// return 0 if sucess
// return 1 if invalid SPacketFifo
{
	if (ppf->FifoSign != FIFO_SIGN) return 1;
	if (ppf->pBuf == 0) return 1;
	ppf->FifoSign = 0;

	ppf->Head = ppf->Tail = ppf->Bottom = ppf->BufLen = 0;
	ppf->BufLen = 0;
	
	delete ppf->pBuf; ppf->pBuf = 0;

	return 0;
}

static void WritePacket(SPacketFifo *ppf, void *pDest, 
						void *pPktData, dword PktLen)
{
	SFifoPacketHead* p;
	p = (SFifoPacketHead*)pDest;
	p->PktSign = PKT_SIGN;
	p->PktId = ppf->PktId++;
	p->PktLen = PktLen;
	memcpy( p + 1, pPktData, PktLen);
}

int PktFifoPut(SPacketFifo *ppf, void *pPktData, dword PktLen)
// return 0 if sucess
// return 1 if invalid SPacketFifo
// return 2 if no enough memory
{
	dword reallen;

	ppf->PutCount++;	ppf->PutBytes += PktLen;

	reallen = PktLen + sizeof(SFifoPacketHead);

	if (ppf->FifoSign != FIFO_SIGN) 
	{
		sysprintln("Fatel error: pipf pointer:" + ppf);
		return 1;
	}
	if (ppf->Tail >= ppf->Head)
	{
		ppf->Bottom = ppf->Tail;

		// it is [  XXXX  ]	
		if (ppf->BufLen - ppf->Tail >= reallen + 1)
		{
			// it is still [   XXXNN   ]
			WritePacket(ppf , ppf->pBuf + ppf->Tail, pPktData, PktLen);
			ppf->Tail += reallen;
			ppf->Bottom = ppf->Tail;
		}
		else
		{
			// try it as [NN  XXXXX ]
			if (ppf->Head > reallen )
			{
				// it will be [NN XXXXX ]
				WritePacket(ppf, ppf->pBuf, pPktData, PktLen);
				ppf->Tail = reallen;
			}
			else
			{
				if (DEBUG) 
				{
					sysprintln("PktFifoPut no enough buffer, at [ XXXX ]");
					PrintPktFifo(ppf);
				}
				// no enough buffer
				return 2;
			}
		}
	}
	else
	{
		// it is [XX  XXX]	
		if (ppf->Head - ppf->Tail > reallen + 1)
		// don't use "=", because tail can't equal with head
		{
			// ok, there are enough space
			WritePacket(ppf ,ppf->pBuf + ppf->Tail, pPktData, PktLen);
			ppf->Tail += reallen;
		}
		else
		{
			if (DEBUG) 
			{
				sysprintln("PktFifoPut no enough buffer, at [XX XXX]");
				PrintPktFifo(ppf);
			}
			// no enough buffer
			return 2;
		}
	}
	
	ppf->FreeBytes -= reallen;
	if (ppf->FreeBytes < ppf->MinFreeBytes) ppf->MinFreeBytes = ppf->FreeBytes;
	
	ppf->PutSuccessCount++;

	return 0;
}


int PktFifoGet(SPacketFifo *ppf, void *pBuf, dword bufsize, dword *RetLen)
// return 0 if sucess
// return 1 if invalid SPacketFifo
// return 2 if no packet
// return 3 if the Fifo is damaged
// return 4 if buf is too small
{
	SFifoPacketHead *phead;
	dword reallen;

	ppf->GetCount++;
	
	*RetLen = 0;
	if (ppf->FifoSign != FIFO_SIGN)
	{
		sysprintln("ERROR:(internal uncovered damage) PktFifoGet: invalid fifo pointer!");
		return 1;
	}
	
	if (ppf->Tail == ppf->Head)
	{
		ppf->GetNoPktCount++;
		return 2;
	}

	ppf->Head %= ppf->Bottom;
	
	if (ppf->Tail > ppf->Head)
	{
		// it is [ XXXXX ]
		phead = (SFifoPacketHead*)(ppf->pBuf + ppf->Head);
		if (phead->PktSign != PKT_SIGN)
		{
			// A terrible error
			sysprintln("ERROR: (Internal) PktFifoGet: PktElem sign is invalid at[ XXXX ]");
			sysprintln("the SPacketFifo is damaged. reset it");

			PktFifoReset(ppf);

			return 3;
		}
	
		reallen = phead->PktLen + sizeof (*phead);
		if (reallen > ppf->Tail - ppf->Head)
		{
			// Another terrible error
			sysprintln("ERROR: (Internal) PktFifoGet: pktlen is more than the data saved at[ XXXX ]");
			sysprintln("the SPacketFifo is damaged. destory it");
//			PktFifoDump(serr, ppf);

			PktFifoReset(ppf);

			return 3;
		}
		
		if (phead->PktLen > bufsize) return 4; // buf is too small
		
		memcpy(pBuf, ppf->pBuf + ppf->Head + sizeof(SFifoPacketHead) , phead->PktLen);
		
		phead->PktSign = 0;
    	*RetLen = phead->PktLen;
		// don't % ppf->buttom */
		ppf->Head += reallen;
	}
	else
	{
		// it is [XXX  XXXX]
		phead = (SFifoPacketHead*)(ppf->pBuf + ppf->Head);
	
		if (phead->PktSign != PKT_SIGN)
		{
			// A terrible error
			sysprintln("ERROR: (Internal) PktFifoGet: PktElem sign is invalid at[XX  XXXX]");
			sysprintln("the SPacketFifo is damaged. destory it");
//			PktFifoDump(serr, ppf);

			PktFifoReset(ppf);

			return 3;
		}
	
		reallen = phead->PktLen + sizeof (*phead);
		if (reallen > ppf->Bottom - ppf->Head)
		{
			// Another terrible error
			sysprintln("ERROR: (Internal) PktFifoGet: pktlen is more than the data saved at [XX  XXXX]");
			sysprintln("the SPacketFifo is damaged. destory it");
//			PktFifoDump(serr , ppf);

			PktFifoReset(ppf);

			return 3;
		}

		if (phead->PktLen > bufsize) return 4;	// buf is too small

		memcpy(pBuf, ppf->pBuf + ppf->Head + sizeof(SFifoPacketHead) , phead->PktLen);

		phead->PktSign = 0;
    	*RetLen = phead->PktLen;
		ppf->Head = (ppf->Head + reallen) % ppf->Bottom;
	}
		

	ppf->GetBytes += *RetLen;
	ppf->FreeBytes += reallen;
	ppf->GetSuccessCount++;
	
	return 0;
}


void PrintPktFifo(SPacketFifo *ppf)
{
	sysprint(	"Packet FIFO:   ");
	if (ppf->FifoSign != FIFO_SIGN)
	{
		sysprintln("invalid"); return;
	}
	sysprintln("valid");
	
	sysprint("FifoSize: " + ppf->BufLen);
	sysprint(" FreeBytes: " + ppf->FreeBytes);
	sysprintln(" MinFreeBytes: " + ppf->MinFreeBytes);
	sysprint("PutBytes: " + ppf->PutBytes);
	sysprintln(" GetBytes: " + ppf->GetBytes);
	sysprint("Head position:" + ppf->Head);
	sysprintln("  Tail position:" + ppf->Tail);
	sysprint("PutCount: " + ppf->PutCount);
	sysprint(" PutSuccessCount: " + ppf->PutSuccessCount);
	sysprintln(" PutFailCount: " + (ppf->PutCount - ppf->PutSuccessCount));
	sysprint("GetCount: " + ppf->GetCount);
	sysprint(" GetSuccessCount: " + ppf->GetSuccessCount);
	sysprint(" GetNoPktCount: " + ppf->GetNoPktCount);
	sysprintln(" GetFailCount: " + (ppf->GetCount - ppf->GetSuccessCount - ppf->GetNoPktCount));
	sysprintln("FifoResetCount: " + ppf->FifoResetCount);
}		
			

void PktFifoTest(void)
{
	SPacketFifo pf;
	char str1[] = "12345";
	char str2[] = "ABCD";
	PktFifoOpen(&pf, 10);
	//PktFifoPut(

}

void PktFifoDump(ostream &os, SPacketFifo *ppf)
{
	os	<<"===== PACKETFIFO ======="
		<<"\nppf addr = 0x"<<hex<<(void*)ppf
		<<", Fifo Sign = 0x"<<hex<<ppf->FifoSign
		<<"pBuf = 0x"<<hex<<(void*)ppf->pBuf
		<<", BufLen = "<<dec<<ppf->BufLen
		<<"\nHead = 0x"<<hex<<ppf->Head
		<<", Tail = 0x"<<hex<<ppf->Tail
		<<", Bottom = 0x"<<hex<<ppf->Bottom
		<<dec
		<<endl;
	// HexDump(0, ppf->pBuf, ppf->BufLen);

}

/* it is dump and stream out for C++
ostream& operator<<(ostream& os, CPktFifo &pf)
{
	os<<"Packet FIFO:   ";
	if (pf.FifoSign != FIFO_SIGN)
	{
		os<<"invalid\n"; return os;
	}
	
	os<<"valid\n";
	
	os	<<"FifoSize: "<<pf.BufLen
		<<", FreeBytes: "<<pf.FreeBytes
		<<", MinFreeBytes: "<<pf.MinFreeBytes
		<<endl;
	
	os	<<"PutBytes: "<<pf.PutBytes
		<<", GetBytes: "<<pf.GetBytes<<endl;
		
	os	<<"PutCount: "<< pf.PutCount
		<<", PutSuccessCount: "<< pf.PutSuccessCount
		<<", PutFailCount: "<< pf.PutCount - pf.PutSuccessCount
		<<endl;
	
	os	<<"GetCount: "<< pf.GetCount
		<<", GetSuccessCount: "<< pf.GetSuccessCount
		<<", GetNoPktCount"<< pf.GetNoPktCount
		<<", GetNoPktCount"<< pf.GetCount - pf.GetSuccessCount - pf.GetNoPktCount
		<<endl;
	
	os	<<"FifoResetCount: "<< pf.FifoResetCount<<endl;

		return os;
}
			

void CPktFifo::Dump(ostream &os)
{
	os	<<"CPktFifo addr = "<<hex<<this
		<<", Fifo Sign = "<<hex<<FifoSign
		<<"\npbuf   = "<<pFifoBuf
		<<"BufLen = "<<BufLen
		<<"\nHead = "<<Head
		<<" Tail = "<<Tail
		<<" Bottom = "<<Bottom
		<<endl;
	// HexDump(0, pbuf, BufLen);

}
*/

// end of PKTFIFO.CPP
