// TcpPkt.cpp: implementation of the CTcpPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

04/01/1999			Merged into SNDFC by Yadong Zhang
03/29/1998			Created by Yadong Zhang

--*/



#include "stdafx.h"
#include "TcpPkt.h"
#include "checksum.h"


odstream& operator<<(odstream& os, CTcpPacket &tp)
{
	os	<< "=========Tcp Segment, len: "<<dec<<tp.GetPacketBinary().GetLen()<<"====\n"
		<< tp.IpPacket.GetSourIp()<<":"<<tp.GetSourPort()<< " -> "
		<< tp.IpPacket.GetDestIp()<<":"<<tp.GetDestPort()
		<< "    ";

	if (tp.GetFlag(TCP_FLAGS_SYN)) os<<"SYN1 ";
		else os<<"SYN0 ";
	if (tp.GetFlag(TCP_FLAGS_ACK)) os<<"ACK1 ";
		else os<<"ACK0 ";
	if (tp.GetFlag(TCP_FLAGS_FIN)) os<<"FIN1 ";
		else os<<"FIN0 ";
	if (tp.GetFlag(TCP_FLAGS_RST)) os<<"RST1 ";
		else os<<"RST0 ";
	if (tp.GetFlag(TCP_FLAGS_PSH)) os<<"PSH1 ";
		else os<<"PSH0 ";
	if (tp.GetFlag(TCP_FLAGS_URG)) os<<"URG1 ";
		else os<<"URG0 ";

	os<<endl;

	os	<<"SeqNumber: "<<tp.GetSeqNumber()<<",  AckNumber:"<<tp.GetAckNumber()
		<<",  TcpDataLen: "<<tp.GetDataLen()
		<<", TcpHeadLen: "<<tp.GetHeadLen();

	os	<<"\nchecksum = 0x"<<hex<<tp.GetFramePtr()->Checksum
		<<" total sum = 0x"<<hex<<TcpChecksum(tp.IpPacket.GetPacketBinary().GetAddr(), tp.IpPacket.GetPacketBinary().GetLen())
		<<dec<<"\n";

	if (TCP_DATA_DEBUG)
	{
		os<<"Data: (len="<<tp.GetDataLen()<<")\n";
		os<<tp.GetDataSection();
	}
	
	return os;
}

outputstr operator+(const outputstr& ops, CTcpPacket &tp)
{
	ostrstream outbuf;
	outbuf<<tp<<(char)0;
	outputstr out = ops + outbuf.str();

	// unfreeze it, otherwise strstream won't release the memory 
	// and memory leak will occur.
	outbuf.freeze(false);
	return out;
}

word CTcpPacket::GetTcpChecksum(void)
{
    STcpSegment_FrameHead *ptcp;
    ptcp = (STcpSegment_FrameHead*)( ((char*)IpPacket.GetPacketBinary().GetAddr()) + GetIpPkt().GetHeadLen() );
    return ptcp->Checksum;
}

word CTcpPacket::CalculatedTcpChecksum(void)
{
    STcpSegment_FrameHead *ptcp;
    ptcp = (STcpSegment_FrameHead*)( ((char*)IpPacket.GetPacketBinary().GetAddr()) + GetIpPkt().GetHeadLen() );
    
    word oldchksum, oldtcpchksum, newchksum;
    
    oldchksum = IpPacket.GetFramePtr()->HeadChecksum;
    oldtcpchksum = ptcp->Checksum;
    
    RefillTcpIpChecksum();
    
    newchksum = ptcp->Checksum;
    IpPacket.GetFramePtr()->HeadChecksum = oldchksum;
    ptcp->Checksum = oldtcpchksum;
    return newchksum;
}

bool CTcpPacket::IsValidTcpChecksum(void)
{
	return( TcpChecksum(IpPacket.GetPacketBinary().GetAddr(), IpPacket.GetPacketBinary().GetLen()) == 0);
}


int CTcpPacket::RefillTcpIpChecksum(void)
// re-calculate the tcp and ip's checksum and write back.
// return OK if success,,, I think it should success always..  :-)
{
	CalcTcpipChecksum(IpPacket.GetPacketBinary().GetAddr());
	return OK;
}
