// TcpPkt.h: interface for the CTcpPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

04/01/1999			Merged into SNDFC by Yadong Zhang
03/29/1998			Created by Yadong Zhang

--*/




#ifndef SND_TCPPKT_H
#define SND_TCPPKT_H

#include "sndbasex.h"
#include "packet.h"
#include "ippkt.h"
#include "hashobj.h"

//////////////////////////////////////////
//         ERROR MESSAGE
#define TCP_ERR_ISNT_TCP			0x2
///////////////////////////////////////////


#define PROTO_IP_TYPE_TCP		0x6

#include "packon1.h"
struct STcpSegment_FrameHead
{
	nword		SourPort;
	nword		DestPort;
	ndword		SeqNumber;		// Sequence Number
	ndword		AckNumber;		// Acknowledgment Number
	word		Offset_Flags;	// Offset is number of TCP head (by 32bits)
	nword		Window;
	word		Checksum;
	nword		UrgentPointer;
};
#include "packoff1.h"

#define TCP_FLAGS_ECN_CWR	0x8000
#define TCP_FLAGS_ECN_ECHO	0x4000
#define TCP_FLAGS_URG	0x2000
#define TCP_FLAGS_ACK	0x1000
#define TCP_FLAGS_PSH   0x800
#define TCP_FLAGS_RST	0x400
#define TCP_FLAGS_SYN	0x200
#define TCP_FLAGS_FIN	0x100


class CTcpPacket;
inline odstream& operator<<(odstream& os, CTcpPacket &tp);

class CTcpPacket:	public CPacket, public CObjForHash
{
public:
	CIpPacket		IpPacket;
	
	inline unsigned HashKey(void);	// deliving from CObjForHash

	inline CTcpPacket(void) {};
	inline CTcpPacket(CIpPacket &ipp);
	
	inline int Set(CIpPacket &ipp);

	inline STcpSegment_FrameHead* GetFramePtr(void);
	// return the TcpSegment pointer

	inline unsigned GetHeadLen(void);
	// return the tcp header's length

	inline unsigned GetDataLen(void);
	// return the tcp's data length;

	inline memptr GetDataSection(void);
	// return the tcp's data section

	inline CIpPacket& GetIpPkt(void) { return IpPacket; }

	inline nword GetSourPort(void) { return GetFramePtr()->SourPort; }

	inline nword GetDestPort(void) { return GetFramePtr()->DestPort; }

	inline int GetFlag(int flag) { return (flag & GetFramePtr()->Offset_Flags); }
	// Get the tcp segment's flags
	// flag can be one or more of following, such as "TCP_FLAGS_ACK | TCP_FLAGS_SYN"
	// TCP_FLAGS_URG
	// TCP_FLAGS_ACK
	// TCP_FLAGS_PSH
	// TCP_FLAGS_RST
	// TCP_FLAGS_SYN
	// TCP_FLAGS_FIN

	inline void SetFlag(int flag) { GetFramePtr()->Offset_Flags |= flag; }
	// the flag definition is same as GetFlag
	inline void ClearFlag(int flag) { GetFramePtr()->Offset_Flags &= (flag ^ 0xFFFFFFFFL) ; }
	// the flag definition is same as GetFlag

	inline ndword GetSeqNumber(void) { return GetFramePtr()->SeqNumber; }

	inline ndword GetAckNumber(void) { return GetFramePtr()->AckNumber; }

    word GetTcpChecksum(void);
    
    word CalculatedTcpChecksum(void);

    bool IsValidTcpChecksum(void);
	// return true if the checksum is valid

	int RefillTcpIpChecksum(void);
	// re-calculate the tcp and ip's checksum and write back.
	// return OK if success,,, I think it should success always..  :-)

	friend odstream& operator<<(odstream& os, CTcpPacket &tp);
	friend outputstr operator+(const outputstr& ops, CTcpPacket &tp);
};


unsigned CTcpPacket::HashKey(void)
{
	// NOTICE: it is temporiate.
	// normally, the condition should be something look like following:
	// if (CEngineCore::IsOutgoingNet(GetNicId()) then ...
	if (GetIpPkt().GetEthiiPkt().GetNicId() != NICID_TOB)
	{
		// it is from the net outgoing
		return	IpPacket.GetSourIp().getraw() + GetSourPort().getraw();
	}

	// it is from Intranet.
	return IpPacket.GetDestIp().getraw() + GetDestPort().getraw();
}


CTcpPacket::CTcpPacket(CIpPacket& ipp)
{
	Set(ipp);
}


int CTcpPacket::Set(CIpPacket& ipp)
// return OK if it is our packet
// return TCP_ERR_ISNT_TCP if not.
{
 	if (ipp.GetProtocol() != PROTO_IP_TYPE_TCP) return TCP_ERR_ISNT_TCP;
	
	memptr p = ipp.GetDataSection();
	if (p.GetLen() < sizeof (STcpSegment_FrameHead))
		return TCP_ERR_ISNT_TCP;

	CPacket::Set(ipp, p);

	IpPacket = ipp;
	if ( GetPacketBinary().GetLen() >= GetHeadLen())
		return OK;
	else 
		return TCP_ERR_ISNT_TCP;
}


STcpSegment_FrameHead* CTcpPacket::GetFramePtr(void)
// return the TcpSegment pointer
{
	return (STcpSegment_FrameHead*)(GetPacketBinary().GetAddr());
}


unsigned CTcpPacket::GetHeadLen(void)
// return the tcp header's length
{
//	return ( (GetFramePtr()->Offset_Flags >> 2) & 0x3c );
	return  (GetFramePtr()->Offset_Flags & 0xF0)>>2;
}


unsigned CTcpPacket::GetDataLen(void)
// return the tcp's data length;
{
	return ( GetPacketBinary().GetLen() - GetHeadLen() );
}


memptr CTcpPacket::GetDataSection(void)
// return the tcp's data section
{
	memptr a(GetPacketBinary().GetAddr(GetHeadLen()), GetDataLen() );
	return a;
}


#endif // !defined(AFX_TCPPKT_H__22244340_9BDB_11D1_946D_0020AF923BD8__INCLUDED_)
