// TtlExPkt.h: interface for the CTtlExceededPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998                                    |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/



#if !defined(AFX_TTLEXPKT_H__B9008B52_E5D8_11D1_9810_00C026807807__INCLUDED_)
#define AFX_TTLEXPKT_H__B9008B52_E5D8_11D1_9810_00C026807807__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


//**************************************
// TTL(Time-To-Live) Exceeded protocol
// ICMP type: 0x0B


#include "Packet.h"
#include "ippkt.h"
#include "icmppkt.h"


//////////////////////////////////////////
//         ERROR MESSAGE
#define TTLEXCEEDED_ERR_ISNT_MINE		0x2
///////////////////////////////////////////

#define PROTO_ICMP_TYPE_TTL_EXCEEDED		0x0B


#include "packon1.h"
typedef struct
{
	dword				Mbz;			// unused, must be zero
	SIp4_FrameHead		OriginalIpHeader;	// Original IP Datagram header.
	
	// there are first 64 bits of original datagram data
	// NOTICE: the position should be associated with IP head's length;
	// byte aaaa[8];	8 bytes

} STtlExceeded_FrameHead;
#include "packoff1.h"


class CTtlExceededPacket : public CPacket
{
private:
	CIpPacket		OrgIpPacket;		// the original IP datagram included by this ICMP packet.

public:
	CIcmpPacket		IcmpPacket;

	//	inline unsigned HashKey(void);	// deliving from CObjForHash

	inline CTtlExceededPacket(void) {};
	inline CTtlExceededPacket(CIcmpPacket &ipp);
	
	inline int Set(CIcmpPacket &ipp);

	inline STtlExceeded_FrameHead* GetFramePtr(void);
	// return the TTL_EXCEEDED frame pointer

//	inline unsigned GetHeadLen(void);
	// return the tcp header's length

//	inline unsigned GetDataLen(void);
	// return the tcp's data length;

//	inline memptr GetDataSection(void);
	// return the tcp's data section

	inline CIcmpPacket& GetIcmpPkt(void) { return IcmpPacket; }
	inline CIpPacket& GetIpPkt(void) { return GetIcmpPkt().GetIpPkt(); }

	friend ostream& operator<<(ostream& os, CTtlExceededPacket &tp);
	friend outputstr operator +(const outputstr& ops, CTtlExceededPacket &tp);
	
	inline CIpPacket& GetOrgIpPkt(void);
	inline memptr GetOrgIpData(void);
	// TTL_EXCEEDED transports only 8 bytes original datagram data.

};


CTtlExceededPacket::CTtlExceededPacket(CIcmpPacket& icpp)
{
	Set(icpp);
}


int CTtlExceededPacket::Set(CIcmpPacket& icpp)
// return OK if it is our packet
// return ICMP_ERR_ISNT_MINE if not.
{
 	switch( icpp.GetType())
	{
	case PROTO_ICMP_TYPE_TTL_EXCEEDED:
		{
			memptr p = icpp.GetDataSection();
			if (p.GetLen() < sizeof (STtlExceeded_FrameHead))
				return TTLEXCEEDED_ERR_ISNT_MINE;

			CPacket::Set(icpp, p);

			IcmpPacket = icpp;

			OrgIpPacket.Set(&GetFramePtr()->OriginalIpHeader,
					GetPacketBinary().GetLen() - 
					(((char*)(&GetFramePtr()->OriginalIpHeader)) - 
					 ((char*)(&GetFramePtr()->Mbz) ) )
					);
			return OK;
		}	
	default:
		;
	}

	return TTLEXCEEDED_ERR_ISNT_MINE;
}

STtlExceeded_FrameHead* CTtlExceededPacket::GetFramePtr(void)
{
	return (STtlExceeded_FrameHead*)(GetPacketBinary().GetAddr());
}


CIpPacket& CTtlExceededPacket::GetOrgIpPkt(void)
{
	return OrgIpPacket;
}


memptr CTtlExceededPacket::GetOrgIpData(void)
// TTL_EXCEEDED transports only 8 bytes original datagram data.
{
	memptr a = GetOrgIpPkt().GetDataSection();
	memptr b(a.GetAddr(), 8);
	return b;
}



#endif // !defined(AFX_TTLEXPKT_H__B9008B52_E5D8_11D1_9810_00C026807807__INCLUDED_)
