// UdpPkt.cpp: implementation of the CUdpPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

04/02/1999			Merged into SNDFC by Yadong Zhang
04/21/1998			Created by Yadong Zhang

--*/



#include "stdafx.h"
#include "UdpPkt.h"
#include "checksum.h"


odstream& operator<<(odstream& os, CUdpPacket &up)
{
//	os	<<ep.GetSourIp()<<" --> "<<ep.GetDestIp()
//		<<endl;
	os	<< "========= UDP datagram, len: "<<up.IpPacket.GetPacketBinary().GetLen()<<"====\n"
		<< up.IpPacket.GetSourIp()<<":"<<up.GetSourPort()<< " -> "
		<< up.IpPacket.GetDestIp()<<":"<<up.GetDestPort()
		<< "    "
		<<",  DataLen: "<<up.GetDataLen()
		<<"\n";

	if (UDP_DATA_DEBUG)
	{
		os<<"Data: (len="<<up.GetDataLen()<<")\n";
		os<<up.GetDataSection();
	}
	
	return os;
}

outputstr operator+(const outputstr& ops, CUdpPacket &up)
{
	ostrstream outbuf;
	outbuf<<up<<(char)0;
	outputstr out = ops + outbuf.str();

	// unfreeze it, otherwise strstream won't release the memory 
	// and memory leak will occur.
	outbuf.freeze(false);
	return out;
}

word CUdpPacket::GetUdpChecksum(void)
{
    SUdp_FrameHead *pudp;
    pudp = (SUdp_FrameHead*)( ((char*)IpPacket.GetPacketBinary().GetAddr()) + GetIpPkt().GetHeadLen() );
    return pudp->Checksum;
}

word CUdpPacket::CalculatedUdpChecksum(void)
{
    SUdp_FrameHead *pudp;
    pudp = (SUdp_FrameHead*)( ((char*)IpPacket.GetPacketBinary().GetAddr()) + GetIpPkt().GetHeadLen() );

    if (pudp->Checksum==0)
        return 0;
    word oldchksum, oldudpchksum, newchksum;

    oldchksum = IpPacket.GetFramePtr()->HeadChecksum;
    oldudpchksum = pudp->Checksum;

    RefillUdpChecksum();

    newchksum = pudp->Checksum;
    IpPacket.GetFramePtr()->HeadChecksum = oldchksum;
    pudp->Checksum = oldudpchksum;
    return newchksum;
}

bool CUdpPacket::IsValidUdpChecksum(void)
{
	return( UdpChecksum(IpPacket.GetPacketBinary().GetAddr(), IpPacket.GetPacketBinary().GetLen()) == 0);
}


int CUdpPacket::RefillUdpChecksum(void)
// re-calculate the ipheader's checksum and write to ipheader.
// return OK if success,,, I think it should success always..  :-)
{
	CalcUdpChecksum(IpPacket.GetPacketBinary().GetAddr());
	return OK;
}
