// UdpPkt.h: interface for the CUdpPacket class.
//
//////////////////////////////////////////////////////////////////////

/*++
*************************************************************************
+-----------------------------------------------------------------------+
|  Copyright (C) SYBERGEN, INC. 1998,1999                               |
|      All Rights Reserved.                                             |
| Reproduction, adaptation, or translation without prior written        |
| permission is prohibited, except as allowed under Copyright laws.     |
|                                                                       |
| N O T I C E                                                           |
|                                                                       |
| THIS MATERIAL IS A TRADE SECRET OF SYBERGEN, INC.                     |
| UNAUTHORIZED ACCESS OR USE IS PROHIBITED.                             |
+-----------------------------------------------------------------------+
*************************************************************************
--*/

/*++
History:

04/02/1999			Merged into SNDFC by Yadong Zhang
04/21/1998			Created by Yadong Zhang

--*/


#ifndef SND_UDPPKT_H
#define SND_UDPPKT_H


#include "sndbasex.h"
#include "Packet.h"
#include "ippkt.h"

//////////////////////////////////////////
//         ERROR MESSAGE
#define UDP_ERR_ISNT_MINE		0x2
///////////////////////////////////////////

#define PROTO_IP_TYPE_UDP		0x11


#include "packon1.h"
typedef struct
{
	nword		SourPort;
	nword		DestPort;
	nword		Length;
	word		Checksum;
} SUdp_FrameHead;
#include "packoff1.h"


// WHY need we declare the friend function first? see KB:Q131066
class CUdpPacket;
odstream& operator<<(odstream& os, CUdpPacket &tp);

class CUdpPacket : public CPacket  
{
public:
	CIpPacket		IpPacket;

	CUdpPacket(void) {};
	inline CUdpPacket(CIpPacket &ipp);
	
	inline int Set(CIpPacket &ipp);

	inline CIpPacket& GetIpPkt(void) { return IpPacket; }

	inline SUdp_FrameHead* GetFramePtr(void);
	// return the Udp datagram head pointer

	inline unsigned GetHeadLen(void) { return sizeof(SUdp_FrameHead); }
	// return the udp header's length

	inline unsigned GetDataLen(void)
	// return the tcp's data length;
	{ return toword(GetFramePtr()->Length); }

	inline memptr GetDataSection(void)
	// return the tcp's data section
	{ memptr a(GetPacketBinary().GetAddr(GetHeadLen()), GetDataLen() ); return a;}

	inline nword GetSourPort(void) { return GetFramePtr()->SourPort; }

	inline nword GetDestPort(void) { return GetFramePtr()->DestPort; }

	inline unsigned HashKey(void);

    word CUdpPacket::GetUdpChecksum(void);

    word CUdpPacket::CalculatedUdpChecksum(void);
        
	bool IsValidUdpChecksum(void);
	// return true if the checksum is valid

	int RefillUdpChecksum(void);
	// re-calculate the ipheader's checksum and write to ipheader.
	// return OK if success,,, I think it should success always..  :-)

	friend odstream& operator<<(odstream& os, CUdpPacket &tp);
	friend outputstr operator +(const outputstr & ops, CUdpPacket &up);
};


CUdpPacket::CUdpPacket(CIpPacket& ipp)
{
	Set(ipp);
}

int CUdpPacket::Set(CIpPacket& ipp)
// return OK if it is our packet
//	IpPacket = ipp;
// return UDP_ERR_ISNT_MINE if not.
{
 	if (ipp.GetProtocol() != PROTO_IP_TYPE_UDP) return UDP_ERR_ISNT_MINE;

	memptr p = ipp.GetDataSection();
	if (p.GetLen() < sizeof (SUdp_FrameHead))
		return UDP_ERR_ISNT_MINE;

	CPacket::Set(ipp, p);
	IpPacket=ipp;
	if ( GetPacketBinary().GetLen() >= GetHeadLen())
		return OK;
	else 
		return UDP_ERR_ISNT_MINE;
}


SUdp_FrameHead* CUdpPacket::GetFramePtr(void)
// return the TcpSegment pointer
{
	return (SUdp_FrameHead*)(GetPacketBinary().GetAddr());
}


unsigned CUdpPacket::HashKey(void)
{
	// NOTICE: it is temporiate.
	// normally, the condition should be something look like following:
	// if (CEngineCore::IsOutgoingNet(GetNicId()) then ...
	if (GetIpPkt().GetEthiiPkt().GetNicId() != NICID_TOB)
	{
		// it is from the net outgoing
		return	IpPacket.GetSourIp().getraw() + GetSourPort().getraw();
	}

	// it is from Intranet.
	return IpPacket.GetDestIp().getraw() + GetDestPort().getraw();
}


#endif
// end of udppkt.h

